/*
 *  Alert.cc
 */

#include "my_be.h"
#include "rbAlert.h"
#include "rbPoint.h"


VALUE cAlert;


static VALUE
rb_alert_s_new(int argc, VALUE *argv, VALUE klass)
{
  VALUE vresult = my_class_s_new(argc, argv, klass, 0);
  return vresult;
}


static VALUE
rb_alert_initialize(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1, varg2, varg3, varg4, varg5, varg6, varg7, varg8;
  Alert *result;

  if (argc > 1) {
    switch (rb_scan_args(argc, argv, "35", &varg1, &varg2, &varg3, &varg4, &varg5, &varg6, &varg7, &varg8)) {
      case 3:
        {
          const char *arg1 = NULL, *arg2, *arg3;

          if (!NIL_P(varg1)) { arg1 = STR2CSTR(varg1); }
          arg2 = STR2CSTR(varg2);
          arg3 = STR2CSTR(varg3);

          result = new Alert(arg1, arg2, arg3);
        }
        break;

      case 4:
        {
          const char *arg1 = NULL, *arg2, *arg3, *arg4;

          if (!NIL_P(varg1)) { arg1 = STR2CSTR(varg1); }
          arg2 = STR2CSTR(varg2);
          arg3 = STR2CSTR(varg3);
          arg4 = STR2CSTR(varg4);

          result = new Alert(arg1, arg2, arg3, arg4);
        }
        break;

      case 5:
        {
          const char *arg1 = NULL, *arg2, *arg3, *arg4, *arg5;

          if (!NIL_P(varg1)) { arg1 = STR2CSTR(varg1); }
          arg2 = STR2CSTR(varg2);
          arg3 = STR2CSTR(varg3);
          arg4 = STR2CSTR(varg4);
          arg5 = STR2CSTR(varg5);

          result = new Alert(arg1, arg2, arg3, arg4, arg5);
        }
        break;

      case 6:
        {
          const char *arg1 = NULL, *arg2, *arg3, *arg4, *arg5;
          button_width arg6;

          if (!NIL_P(varg1)) { arg1 = STR2CSTR(varg1); }
          arg2 = STR2CSTR(varg2);
          arg3 = STR2CSTR(varg3);
          arg4 = STR2CSTR(varg4);
          arg5 = STR2CSTR(varg5);
          arg6 = (button_width)NUM2ULONG(varg6);

          result = new Alert(arg1, arg2, arg3, arg4, arg5, arg6);
        }
        break;

      case 7:
        {
          const char *arg1 = NULL, *arg2, *arg3, *arg4, *arg5;
          button_width arg6;
          alert_type arg7;

          if (!NIL_P(varg1)) { arg1 = STR2CSTR(varg1); }
          arg2 = STR2CSTR(varg2);
          arg3 = STR2CSTR(varg3);
          arg4 = STR2CSTR(varg4);
          arg5 = STR2CSTR(varg5);
          arg6 = (button_width)NUM2ULONG(varg6);
          arg7 = (alert_type)NUM2ULONG(varg7);

          result = new Alert(arg1, arg2, arg3, arg4, arg5, arg6, arg7);
        }
        break;

      case 8:
        {
          const char *arg1 = NULL, *arg2, *arg3, *arg4, *arg5;
          button_width arg6;
          button_spacing arg7;
          alert_type arg8;

          if (!NIL_P(varg1)) { arg1 = STR2CSTR(varg1); }
          arg2 = STR2CSTR(varg2);
          arg3 = STR2CSTR(varg3);
          arg4 = STR2CSTR(varg4);
          arg5 = STR2CSTR(varg5);
          arg6 = (button_width)NUM2ULONG(varg6);
          arg7 = (button_spacing)NUM2ULONG(varg7);
          arg8 = (alert_type)NUM2ULONG(varg8);

          result = new Alert(arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8);
        }
        break;

      default:
        rb_bug("rb_alert_initialize");
    }
  }
  else {
    BMessage *arg1;

    rb_scan_args(argc, argv, "1", &varg1);

    Get_BInstance(Message, varg1, arg1);
    result = new Alert(arg1);
  }
  result->myself = self;
  DATA_PTR(self) = result;
  return self;
}


static VALUE
rb_alert_SetShortcut(VALUE self, VALUE varg1, VALUE varg2)
{
  BAlert *arg0;
  int32 arg1;
  char arg2;

  Get_BInstance(Alert, self, arg0);
  arg1 = NUM2LONG(varg1);
  arg2 = NUM2CHR(varg2);
  arg0->SetShortcut(arg1,arg2);
  return Qnil;
}


static VALUE
rb_alert_Shortcut(VALUE self, VALUE varg1)
{
  BAlert *arg0;
  int32 arg1;
  char result;
  VALUE vresult;

  Get_BInstance(Alert, self, arg0);
  arg1 = NUM2LONG(varg1);
  result = arg0->Shortcut(arg1);
  vresult = rb_str_new(&result, 1);
  return vresult;
}


static VALUE
rb_alert_Go(int argc, VALUE *argv, VALUE self)
{
  BAlert *arg0;
  VALUE varg1, vresult = Qnil;

  Get_BInstance(Alert, self, arg0);
  switch (rb_scan_args(argc, argv, "01", &varg1)) {
    case 0:
      {
        int32 result;

        ((Alert *)arg0)->sync = true;
        result = arg0->Go();
        ((Alert *)arg0)->sync = false;
        vresult = LONG2NUM(result);
      }
      break;

    case 1:
      {
        BInvoker *arg1 = NULL;
        status_t result;

        if (!NIL_P(varg1)) {
          Get_BInstance(Invoker, varg1, arg1);
        }
        ((Alert *)arg0)->sync = false;
        result = arg0->Go(arg1);
        ((Alert *)arg0)->sync = false;
        if (result != B_OK) {
          DEBUG_TRACE();
          my_raise(result);
        }
      }
      break;

    default:
      rb_bug("rb_alert_Go");
  }
  return vresult;
}


static VALUE
rb_alert_MessageReceived(VALUE self, VALUE varg1)
{
  BAlert *arg0;
  BMessage *arg1;

  Get_BInstance(Alert, self, arg0);
  Get_BInstance(Message, varg1, arg1);
  arg0->BAlert::MessageReceived(arg1);
  return Qnil;
}


static VALUE
rb_alert_FrameResized(VALUE self, VALUE varg1, VALUE varg2)
{
  BAlert *arg0;
  float arg1;
  float arg2;

  Get_BInstance(Alert, self, arg0);
  arg1 = NUM2DBL(varg1);
  arg2 = NUM2DBL(varg2);
  arg0->BAlert::FrameResized(arg1, arg2);
  return Qnil;
}


static VALUE
rb_alert_ButtonAt(VALUE self, VALUE varg1)
{
  BAlert *arg0;
  int32 arg1;
  BButton *result;
  VALUE vresult;

  Get_BInstance(Alert, self, arg0);
  arg1 = NUM2LONG(varg1);
  result = arg0->ButtonAt(arg1);
  vresult = ((result) ? Data_Wrap_Struct(cButton, 0, 0, result):Qnil);
  return vresult;
}


static VALUE
rb_alert_TextView(VALUE self)
{
  BAlert *arg0;
  BTextView *result;
  VALUE vresult;

  Get_BInstance(Alert, self, arg0);
  result = arg0->TextView();
  vresult = ((result) ? Data_Wrap_Struct(cTextView, 0, 0, result):Qnil);
  return vresult;
}


static VALUE
rb_alert_ResolveSpecifier(VALUE self, VALUE varg1, VALUE varg2, VALUE varg3, VALUE varg4, VALUE varg5)
{
  BAlert *arg0;
  BMessage *arg1,  *arg3;
  int32 arg2, arg4;
  char *arg5;
  BHandler *result;
  VALUE vresult;

  Get_BInstance(Alert, self, arg0);
  Get_BInstance(Message, varg1, arg1);
  arg2 = NUM2LONG(varg2);
  Get_BInstance(Message, varg3, arg3);
  arg4 = NUM2LONG(varg4);
  arg5 = STR2CSTR(varg5);

  result = arg0->BAlert::ResolveSpecifier(arg1, arg2, arg3, arg4, arg5);
  vresult = ((result) ? Data_Wrap_Struct(cHandler, 0, 0, result):Qnil);
  return vresult;
}


static VALUE
rb_alert_GetSupportedSuites(VALUE self, VALUE varg1)
{
  BAlert *arg0;
  BMessage *arg1;
  status_t result;

  Get_BInstance(Alert, self, arg0);
  Get_BInstance(Message, varg1, arg1);
  result = arg0->BAlert::GetSupportedSuites(arg1);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_alert_DispatchMessage(VALUE self, VALUE varg1, VALUE varg2)
{
  BAlert *arg0;
  BMessage *arg1;
  BHandler *arg2;

  Get_BInstance(Alert, self, arg0);
  Get_BInstance(Message, varg1, arg1);
  Get_BInstance(Handler, varg2, arg2);
  arg0->BAlert::DispatchMessage(arg1, arg2);
  return Qnil;
}


static VALUE
rb_alert_Quit(VALUE self)
{
  BAlert *arg0;

  Get_BInstance(Alert, self, arg0);
  arg0->BAlert::Quit();
  return Qnil;
}


static VALUE
rb_alert_QuitRequested(VALUE self)
{
  BAlert *arg0;
  bool result;
  VALUE vresult;

  Get_BInstance(Alert, self, arg0);
  result = arg0->BAlert::QuitRequested();
  vresult = (result ? Qtrue:Qfalse);
  return vresult;
}


static VALUE
rb_alert_AlertPosition(VALUE self, VALUE varg1, VALUE varg2)
{
  float arg1, arg2;
  BPoint *result;
  VALUE vresult;

  arg1 = NUM2DBL(varg1);
  arg2 = NUM2DBL(varg2);
  result = new Point(BAlert::AlertPosition(arg1, arg2));
  vresult = Data_Wrap_Struct(cPoint, 0, delete_point, result);
  return vresult;
}


void
Init_Alert()
{
  cAlert = rb_define_class_under(mBe, "Alert", cWindow);
  rb_define_singleton_method(cAlert, "new", RUBY_METHOD_FUNC(rb_alert_s_new), -1);
  rb_define_method(cAlert, "initialize", RUBY_METHOD_FUNC(rb_alert_initialize), -1);

  rb_define_method(cAlert, "SetShortcut", RUBY_METHOD_FUNC(rb_alert_SetShortcut), 2);
  rb_define_method(cAlert, "Shortcut", RUBY_METHOD_FUNC(rb_alert_Shortcut), 1);

  rb_define_method(cAlert, "Go", RUBY_METHOD_FUNC(rb_alert_Go), -1);

  rb_define_method(cAlert, "MessageReceived", RUBY_METHOD_FUNC(rb_alert_MessageReceived), 1);
  rb_define_method(cAlert, "FrameResized", RUBY_METHOD_FUNC(rb_alert_FrameResized), 2);
  rb_define_method(cAlert, "ButtonAt", RUBY_METHOD_FUNC(rb_alert_ButtonAt), 1);
  rb_define_method(cAlert, "TextView", RUBY_METHOD_FUNC(rb_alert_TextView), 0);

  rb_define_method(cAlert, "ResolveSpecifier", RUBY_METHOD_FUNC(rb_alert_ResolveSpecifier), 5);
  rb_define_method(cAlert, "GetSupportedSuites", RUBY_METHOD_FUNC(rb_alert_GetSupportedSuites), 1);
  rb_define_method(cAlert, "DispatchMessage", RUBY_METHOD_FUNC(rb_alert_DispatchMessage), 2);
  rb_define_method(cAlert, "Quit", RUBY_METHOD_FUNC(rb_alert_Quit), 0);
  rb_define_method(cAlert, "QuitRequested", RUBY_METHOD_FUNC(rb_alert_QuitRequested), 0);
  rb_define_singleton_method(cAlert, "AlertPosition", RUBY_METHOD_FUNC(rb_alert_AlertPosition), 2);

  rb_define_const(mBe, "EMPTY_ALERT", ULONG2NUM(B_EMPTY_ALERT));
  rb_define_const(mBe, "INFO_ALERT", ULONG2NUM(B_INFO_ALERT));
  rb_define_const(mBe, "IDEA_ALERT", ULONG2NUM(B_IDEA_ALERT));
  rb_define_const(mBe, "WARNING_ALERT", ULONG2NUM(B_WARNING_ALERT));
  rb_define_const(mBe, "STOP_ALERT", ULONG2NUM(B_STOP_ALERT));

  rb_define_const(mBe, "EVEN_SPACING", ULONG2NUM(B_EVEN_SPACING));
  rb_define_const(mBe, "OFFSET_SPACING", ULONG2NUM(B_OFFSET_SPACING));
}
