/*
 *  Application.cc
 */

#include <Font.h>
#include <Roster.h>
#include "rbApplication.h"
#include "my_be.h"


VALUE cApplication;
VALUE my_app;           // $be_app
VALUE my_app_messenger; // $be_app_messenger


void
delete_application(BApplication *arg0)
{
  delete arg0;
}


static VALUE
rb_application_s_new(int argc, VALUE *argv, VALUE klass)
{
  VALUE vresult = my_class_s_new(argc, argv, klass, delete_application);
  return vresult;
}


static VALUE
rb_application_initialize(VALUE self, VALUE varg1)
{
  Data_To_Null(self, BApplication);

  if (TYPE(varg1) == T_STRING) {
    char *arg1;
    status_t arg2 = B_OK;
    BApplication *result;

    arg1 = STR2CSTR(varg1);

    result = new Application(arg1, &arg2);
    DATA_PTR(self) = result;
    if (arg2 != B_OK) {
      DEBUG_TRACE();
      my_raise(arg2);
    }
  }
  else {
    BMessage *arg1;
    BApplication *result;

    Get_BInstance(Message, varg1, arg1);
    result = new Application(arg1);
    DATA_PTR(self) = result;
  }
  rb_ivar_set(self, id_relatives, Qnil);

  /* global variables */
  my_app           = self;
  my_app_messenger = Data_Wrap_Struct(cMessenger, 0, 0, &be_app_messenger);
  my_roster        = Data_Wrap_Struct(cRoster,    0, 0, const_cast<BRoster *>(be_roster));
  my_clipboard     = Data_Wrap_Struct(cClipboard, 0, 0, be_clipboard);
  my_plain_font    = Data_Wrap_Struct(cFont,      0, 0, const_cast<BFont *>(be_plain_font));
  my_bold_font     = Data_Wrap_Struct(cFont,      0, 0, const_cast<BFont *>(be_bold_font));
  my_fixed_font    = Data_Wrap_Struct(cFont,      0, 0, const_cast<BFont *>(be_fixed_font));

  return self;
}


static VALUE
rb_application_InitCheck(VALUE self)
{
  BApplication *arg0;
  status_t result;

  Get_BInstance(Application, self, arg0);
  result = arg0->InitCheck();
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_application_Run(VALUE self)
{
  BApplication *arg0;
  thread_id result;
  VALUE vresult;

  Get_BInstance(Application, self, arg0);
  result = arg0->BApplication::Run();
  vresult = LONG2NUM(result);
  return vresult;
}


static VALUE
rb_application_Quit(VALUE self)
{
  BApplication *arg0;

  Get_BInstance(Application, self, arg0);
  arg0->BApplication::Quit();
  return Qnil;
}


static VALUE
rb_application_QuitRequested(VALUE self)
{
  BApplication *arg0;
  bool result;
  VALUE vresult;

  Get_BInstance(Application, self, arg0);
  result = arg0->BApplication::QuitRequested();
  vresult = (result ? Qtrue:Qfalse);
  return vresult;
}


static VALUE
rb_application_Pulse(VALUE self)
{
  BApplication *arg0;

  Get_BInstance(Application, self, arg0);
  arg0->BApplication::Pulse();
  return Qnil;
}


static VALUE
rb_application_ReadyToRun(VALUE self)
{
  BApplication *arg0;

  Get_BInstance(Application, self, arg0);
  arg0->BApplication::ReadyToRun();
  return Qnil;
}


static VALUE
rb_application_MessageReceived(VALUE self, VALUE varg1)
{
  BApplication *arg0;
  BMessage *arg1;

  Get_BInstance(Application, self, arg0);
  Get_BInstance(Message, varg1, arg1);
  arg0->BApplication::MessageReceived(arg1);
  return Qnil;
}


static VALUE
rb_application_ArgvReceived(VALUE self, VALUE varg1)
{
  BApplication *arg0;
  int32 arg1;
  char **arg2 = NULL;

  Get_BInstance(Application, self, arg0);
  Check_Type(varg1, T_ARRAY);

  arg1 = RARRAY(varg1)->len;
  arg2 = my_rarray2argv(varg1);
  arg0->BApplication::ArgvReceived(arg1, arg2);
  free(arg2);
  return Qnil;
}


static VALUE
rb_application_AppActivated(VALUE self, VALUE varg1)
{
  BApplication *arg0;
  bool arg1;

  Get_BInstance(Application, self, arg0);
  arg1 = RTEST(varg1);
  arg0->BApplication::AppActivated(arg1);
  return Qnil;
}


static VALUE
rb_application_RefsReceived(VALUE self, VALUE varg1)
{
  BApplication *arg0;
  BMessage *arg1;

  Get_BInstance(Application, self, arg0);
  Get_BInstance(Message, self, arg1);
  arg0->BApplication::RefsReceived(arg1);
  return Qnil;
}


static VALUE
rb_application_AboutRequested(VALUE self)
{
  BApplication *arg0;

  Get_BInstance(Application, self, arg0);
  arg0->BApplication::AboutRequested();
  return Qnil;
}


static VALUE
rb_application_ResolveSpecifier(VALUE self, VALUE varg1, VALUE varg2, VALUE varg3, VALUE varg4, VALUE varg5)
{
  BApplication *arg0;
  BMessage *arg1,  *arg3;
  int32 arg2, arg4;
  char *arg5;
  BHandler *result;
  VALUE vresult;

  Get_BInstance(Application, self, arg0);
  Get_BInstance(Message, varg1, arg1);
  arg2 = NUM2LONG(varg2);
  Get_BInstance(Message, varg3, arg3);
  arg4 = NUM2LONG(varg4);
  arg5 = STR2CSTR(varg5);

  result = arg0->BApplication::ResolveSpecifier(arg1, arg2, arg3, arg4, arg5);
  vresult = ((result) ? Data_Wrap_Struct(cHandler, 0, 0, result):Qnil);
  return vresult;
}


static VALUE
rb_application_ShowCursor(VALUE self)
{
  BApplication *arg0;

  Get_BInstance(Application, self, arg0);
  arg0->ShowCursor();
  return Qnil;
}


static VALUE
rb_application_HideCursor(VALUE self)
{
  BApplication *arg0;

  Get_BInstance(Application, self, arg0);
  arg0->HideCursor();
  return Qnil;
}


static VALUE
rb_application_ObscureCursor(VALUE self)
{
  BApplication *arg0;

  Get_BInstance(Application, self, arg0);
  arg0->ObscureCursor();
  return Qnil;
}


static VALUE
rb_application_IsCursorHidden(VALUE self)
{
  BApplication *arg0;
  bool result;
  VALUE vresult;

  Get_BInstance(Application, self, arg0);
  result = arg0->IsCursorHidden();
  vresult = (result ? Qtrue:Qfalse);
  return vresult;
}


static VALUE
rb_application_SetCursor(int argc, VALUE *argv, VALUE self)
{
  BApplication *arg0;
  VALUE varg1, varg2;

  Get_BInstance(Application, self, arg0);
  switch (rb_scan_args(argc, argv, "11", &varg1, &varg2)) {
    case 1:
      if (rb_obj_is_kind_of(varg1, cCursor)) {
        BCursor *arg1;

        Get_BInstance(Cursor, varg1, arg1);
        arg0->SetCursor(arg1);
      }
      else {
        char *arg1;

        arg1 = STR2CSTR(varg1);
        arg0->SetCursor(arg1);
      }
      break;

    case 2:
      {
        BCursor *arg1;
        bool arg2;

        Get_BInstance(Cursor, varg1, arg1);
        arg2 = RTEST(varg2);
        arg0->SetCursor(arg1, arg2);
      }
      break;

    default:
      rb_bug("rb_application_SetCursor");
  }
  return Qnil;
}


static VALUE
rb_application_CountWindows(VALUE self)
{
  BApplication *arg0;
  int32 result;
  VALUE vresult;

  Get_BInstance(Application, self, arg0);
  result = arg0->CountWindows();
  vresult = LONG2NUM(result);
  return vresult;
}


static VALUE
rb_application_WindowAt(VALUE self, VALUE varg1)
{
  BApplication *arg0;
  int32 arg1;
  BWindow *result;
  VALUE vresult;

  Get_BInstance(Application, self, arg0);
  arg1 = NUM2LONG(varg1);
  result = arg0->WindowAt(arg1);
  vresult = ((result) ? Data_Wrap_Struct(cWindow, 0, 0, result):Qnil);
  return vresult;
}


static VALUE
rb_application_CountLoopers(VALUE self)
{
  BApplication *arg0;
  int32 result;
  VALUE vresult;

  Get_BInstance(Application, self, arg0);
  result = arg0->CountLoopers();
  vresult = LONG2NUM(result);
  return vresult;
}


static VALUE
rb_application_LooperAt(VALUE self, VALUE varg1)
{
  BApplication *arg0;
  int32 arg1;
  BLooper *result;
  VALUE vresult;

  Get_BInstance(Application, self, arg0);
  arg1 = NUM2LONG(varg1);
  result = arg0->LooperAt(arg1);
  vresult = ((result) ? Data_Wrap_Struct(cLooper, 0, 0, result):Qnil);
  return vresult;
}


static VALUE
rb_application_IsLaunching(VALUE self)
{
  BApplication *arg0;
  bool result;
  VALUE vresult;

  Get_BInstance(Application, self, arg0);
  result = arg0->IsLaunching();
  vresult = (result ? Qtrue:Qfalse);
  return vresult;
}


static VALUE
rb_application_GetAppInfo(VALUE self)
{
  BApplication *arg0;
  app_info *arg1;
  status_t result;
  VALUE vresult;

  Get_BInstance(Application, self, arg0);
  arg1 = my_create_be_obj<app_info>();
  result = arg0->GetAppInfo(arg1);
  vresult = Data_Wrap_Struct(cApp_info, 0, delete_app_info, arg1);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return vresult;
}


static VALUE
rb_application_s_AppResources(VALUE klass)
{
  BResources *result;
  VALUE vresult;

  result = BApplication::AppResources();
  vresult = ((result) ? Data_Wrap_Struct(cResources, 0, 0, result):Qnil);
  return vresult;
}


static VALUE
rb_application_AppResources(VALUE self)
{
  BApplication *arg0;
  BResources *result;
  VALUE vresult;

  Get_BInstance(Application, self, arg0);
  result = arg0->AppResources();
  vresult = ((result) ? Data_Wrap_Struct(cResources, 0, 0, result):Qnil);
  return vresult;
}


static VALUE
rb_application_DispatchMessage(VALUE self, VALUE varg1, VALUE varg2)
{
  BApplication *arg0;
  BMessage *arg1;
  BHandler *arg2;

  Get_BInstance(Application, self, arg0);
  Get_BInstance(Message, varg1, arg1);
  Get_BInstance(Handler, varg2, arg2);
  arg0->BApplication::DispatchMessage(arg1, arg2);
  return Qnil;
}


static VALUE
rb_application_SetPulseRate(VALUE self, VALUE varg1)
{
  BApplication *arg0;
  bigtime_t arg1;

  Get_BInstance(Application, self, arg0);
  arg1 = NUM2LLONG(varg1);
  arg0->SetPulseRate(arg1);
  return Qnil;
}


static VALUE
rb_application_GetSupportedSuites(VALUE self, VALUE varg1)
{
  BApplication *arg0;
  BMessage *arg1;
  status_t result;

  Get_BInstance(Application, self, arg0);
  Get_BInstance(Message, varg1, arg1);
  result = arg0->BApplication::GetSupportedSuites(arg1);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


void
Init_Application()
{
  rb_global_variable(&my_app);
  rb_global_variable(&my_app_messenger);
  rb_define_readonly_variable("$be_app", &my_app);
  rb_define_readonly_variable("$be_app_messenger", &my_app_messenger);

  my_app           = Qnil;
  my_app_messenger = Qnil;

  cApplication = rb_define_class_under(mBe, "Application", cLooper);
  rb_define_singleton_method(cApplication, "new", RUBY_METHOD_FUNC(rb_application_s_new), -1);
  rb_define_method(cApplication, "initialize", RUBY_METHOD_FUNC(rb_application_initialize), 1);

  rb_define_method(cApplication, "InitCheck", RUBY_METHOD_FUNC(rb_application_InitCheck), 0);

  rb_define_method(cApplication, "Run", RUBY_METHOD_FUNC(rb_application_Run), 0);
  rb_define_method(cApplication, "Quit", RUBY_METHOD_FUNC(rb_application_Quit), 0);
  rb_define_method(cApplication, "QuitRequested", RUBY_METHOD_FUNC(rb_application_QuitRequested), 0);
  rb_define_method(cApplication, "Pulse", RUBY_METHOD_FUNC(rb_application_Pulse), 0);
  rb_define_method(cApplication, "ReadyToRun", RUBY_METHOD_FUNC(rb_application_ReadyToRun), 0);
  rb_define_method(cApplication, "MessageReceived", RUBY_METHOD_FUNC(rb_application_MessageReceived), 1);
  rb_define_method(cApplication, "ArgvReceived", RUBY_METHOD_FUNC(rb_application_ArgvReceived), -2);
  rb_define_method(cApplication, "AppActivated", RUBY_METHOD_FUNC(rb_application_AppActivated), 1);
  rb_define_method(cApplication, "RefsReceived", RUBY_METHOD_FUNC(rb_application_RefsReceived), 1);
  rb_define_method(cApplication, "AboutRequested", RUBY_METHOD_FUNC(rb_application_AboutRequested), 0);

  rb_define_method(cApplication, "ResolveSpecifier", RUBY_METHOD_FUNC(rb_application_ResolveSpecifier), 5);

  rb_define_method(cApplication, "ShowCursor", RUBY_METHOD_FUNC(rb_application_ShowCursor), 0);
  rb_define_method(cApplication, "HideCursor", RUBY_METHOD_FUNC(rb_application_HideCursor), 0);
  rb_define_method(cApplication, "ObscureCursor", RUBY_METHOD_FUNC(rb_application_ObscureCursor), 0);
  rb_define_method(cApplication, "IsCursorHidden", RUBY_METHOD_FUNC(rb_application_IsCursorHidden), 0);
  rb_define_method(cApplication, "SetCursor", RUBY_METHOD_FUNC(rb_application_SetCursor), -1);
  rb_define_method(cApplication, "CountWindows", RUBY_METHOD_FUNC(rb_application_CountWindows), 0);
  rb_define_method(cApplication, "WindowAt", RUBY_METHOD_FUNC(rb_application_WindowAt), 1);
  rb_define_method(cApplication, "CountLoopers", RUBY_METHOD_FUNC(rb_application_CountLoopers), 0);
  rb_define_method(cApplication, "LooperAt", RUBY_METHOD_FUNC(rb_application_LooperAt), 1);
  rb_define_method(cApplication, "IsLaunching", RUBY_METHOD_FUNC(rb_application_IsLaunching), 0);
  rb_define_method(cApplication, "GetAppInfo", RUBY_METHOD_FUNC(rb_application_GetAppInfo), 0);
  rb_define_singleton_method(cApplication, "AppResources", RUBY_METHOD_FUNC(rb_application_s_AppResources), 0);
  rb_define_method(cApplication, "AppResources", RUBY_METHOD_FUNC(rb_application_AppResources), 0);

  rb_define_method(cApplication, "DispatchMessage", RUBY_METHOD_FUNC(rb_application_DispatchMessage), 2);
  rb_define_method(cApplication, "SetPulseRate", RUBY_METHOD_FUNC(rb_application_SetPulseRate), 1);
  rb_define_method(cApplication, "GetSupportedSuites", RUBY_METHOD_FUNC(rb_application_GetSupportedSuites), 1);
}
