/*
 *  Archivable.cc
 */

#include <Archivable.h>
#include "rbMessage.h"
#include "my_be.h"


VALUE cArchivable;
VALUE cInstantiation_func;


void
delete_archivable(BArchivable *arg0)
{
  delete arg0;
}


static VALUE
rb_archivable_s_new(int argc, VALUE *argv, VALUE klass)
{
  VALUE vresult = my_class_s_new(argc, argv, klass, delete_archivable);
  return vresult;
}


static VALUE
rb_archivable_initialize(VALUE self)
{
  BArchivable *result;

  Data_To_Null(self, BArchivable);
  result = my_create_be_obj<BArchivable>();
  DATA_PTR(self) = result;
  return self;
}


static VALUE
rb_archivable_Archive(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1, varg2;
  BArchivable *arg0;
  status_t result;

  Get_BInstance(Archivable, self, arg0);
  switch (rb_scan_args(argc, argv, "11", &varg1, &varg2)) {
    case 1:
      {
        BMessage *arg1;

        Get_BInstance(Message, varg1, arg1);
        result = arg0->Archive(arg1);
      }
      break;

    case 2:
      {
        BMessage *arg1;
        bool arg2;

        Get_BInstance(Message, varg1, arg1);
        arg2 = RTEST(varg2);
        result = arg0->Archive(arg1, arg2);
      }
      break;

    default:
      rb_bug("rb_archivable_Archive");
  }
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_archivable_Instantiate(int argc, VALUE *argv, VALUE klass)
{
  BMessage *arg1;
  BArchivable *result;
  VALUE varg1, vresult;

  rb_scan_args(argc, argv, "1", &varg1);

  Get_BInstance(Message, varg1, arg1);
  result = BArchivable::Instantiate(arg1);
  vresult = Data_Wrap_Struct(klass, 0, delete_archivable, result);

  rb_obj_call_init(vresult, argc, argv);
  return vresult;
}


static VALUE
rb_archivable_dump(int argc, VALUE *argv, VALUE self)
{
  BArchivable *arg0;
  VALUE varg1, varg2, vresult;

  BMessage *arg1;
  arg1 = new Message();
  varg1 = Data_Wrap_Struct(cMessage, 0, delete_message, arg1);

  Get_BInstance(Archivable, self, arg0);
  switch (rb_scan_args(argc, argv, "01", &varg2)) {
    case 0:
      rb_funcall(self, rb_intern("Archive"), 2, varg1, Qtrue);
      break;

    case 1:
      {
        int32 limit;

        limit = NUM2LONG(varg2);
        rb_funcall(self, rb_intern("Archive"), 2, varg1, (limit ? Qtrue:Qfalse));
      }
      break;

    default:
      rb_bug("rb_archivable_dump");
  }
  vresult = rb_funcall2(varg1, rb_intern("Flatten"), 0, 0);
  return vresult;
}


static VALUE
rb_archivable_load(VALUE klass, VALUE varg1)
{
  BMessage *msg;
  VALUE self, vmsg;

  msg = new Message();
  vmsg = Data_Wrap_Struct(cMessage, 0, delete_message, msg);

  rb_funcall(vmsg, rb_intern("Unflatten"), 1, varg1);

  self = rb_funcall(klass, rb_intern("new"), 1, vmsg);
  return self;
}


static VALUE
rb_instantiate_object(int argc, VALUE *argv, VALUE klass)
{
  BArchivable *result;
  VALUE varg1, varg2, vresult;

  switch (rb_scan_args(argc, argv, "11", &varg1, &varg2)) {
    case 1:
      {
        BMessage *arg1;

        Get_BInstance(Message, varg1, arg1);
        result = instantiate_object(arg1);
      }
      break;

    case 2:
      {
        BMessage *arg1;
        image_id arg2;

        Get_BInstance(Message, varg1, arg1);
        arg2 = NUM2LONG(varg2);

        result = instantiate_object(arg1, &arg2);
      }
      break;

    default:
      rb_bug("rb_instantiate_object");
  }
  vresult = Data_Wrap_Struct(klass, 0, delete_archivable, result);
  return vresult;
}


static VALUE
rb_validate_instantiation(VALUE klass, VALUE varg1, VALUE varg2)
{
  BMessage *arg1;
  char *arg2;
  bool result;
  VALUE vresult;

  Get_BInstance(Message, varg1, arg1);
  arg2 = STR2CSTR(varg2);

  result = validate_instantiation(arg1, arg2);
  vresult = (result ? Qtrue:Qfalse);
  return vresult;
}


static VALUE
rb_find_instantiation_func(int argc, VALUE *argv, VALUE klass)
{
  instantiation_func result;
  VALUE varg1, varg2, vresult;

  switch (rb_scan_args(argc, argv, "11", &varg1, &varg2)) {
    case 1:
      if (rb_obj_is_kind_of(varg1, cMessage)) {
        BMessage *arg1;

        Get_BInstance(Message, varg1, arg1);
        result = find_instantiation_func(arg1);
      }
      else {
        char *arg1;

        arg1 = STR2CSTR(varg1);
        result = find_instantiation_func(arg1);
      }
      break;

    case 2:
      {
        char *arg1, *arg2;

        arg1 = STR2CSTR(varg1);
        arg2 = STR2CSTR(varg2);
        result = find_instantiation_func(arg1, arg2);
      }
      break;

    default:
      rb_bug("rb_find_instantiation_func");
  }
  vresult = ((result) ? Data_Wrap_Struct(cInstantiation_func, 0, 0, &result):Qnil);
  rb_ivar_set(vresult, rb_intern("__klass__"), klass);
  return vresult;
}


static VALUE
rb_instantiation_func_call(VALUE self, VALUE varg1)
{
  instantiation_func *arg0;
  BMessage *arg1;
  BArchivable *result;
  VALUE klass, vresult;

  Check_Is_Kind_Of(self, cInstantiation_func);
  Check_Type(self, T_DATA);
  arg0 = reinterpret_cast<instantiation_func *>(DATA_PTR(self));

  Get_BInstance(Message, varg1, arg1);

  result = (*arg0)(arg1);

  klass = rb_ivar_get(self, rb_intern("__klass__"));
  vresult = Data_Wrap_Struct(klass, 0, delete_archivable, result);
  return vresult;
}


void
Init_Archivable()
{
  cArchivable = rb_define_class_under(mBe, "Archivable", rb_cObject);
  rb_define_singleton_method(cArchivable, "new", RUBY_METHOD_FUNC(rb_archivable_s_new), -1);
  rb_define_method(cArchivable, "initialize", RUBY_METHOD_FUNC(rb_archivable_initialize), 0);

  rb_define_method(cArchivable, "Archive", RUBY_METHOD_FUNC(rb_archivable_Archive), -1);
  rb_define_singleton_method(cArchivable, "Instantiate", RUBY_METHOD_FUNC(rb_archivable_Instantiate), 1);
  rb_define_method(cArchivable, "_dump", RUBY_METHOD_FUNC(rb_archivable_dump), -1);
  rb_define_singleton_method(cArchivable, "_load", RUBY_METHOD_FUNC(rb_archivable_load), 1);

  cInstantiation_func = rb_define_class_under(mBe, "Instantiation_func", rb_cObject);
  rb_undef_method(CLASS_OF(cInstantiation_func), "new");
  rb_define_method(cInstantiation_func, "call", RUBY_METHOD_FUNC(rb_instantiation_func_call), 1);

  rb_define_module_function(mBe, "instantiate_object", RUBY_METHOD_FUNC(rb_instantiate_object), -1);
  rb_define_module_function(mBe, "validate_instantiation", RUBY_METHOD_FUNC(rb_validate_instantiation), 2);
  rb_define_module_function(mBe, "find_instantiation_func", RUBY_METHOD_FUNC(rb_find_instantiation_func), -1);
}
