/*
 *  Clipboard.cc
 */

#include <Clipboard.h>
#include "rbMessenger.h"
#include "my_be.h"


VALUE cClipboard;
VALUE my_clipboard; // $be_clipboard


void
delete_clipboard(BClipboard *arg0)
{
  delete arg0;
}


static VALUE
rb_clipboard_s_new(int argc, VALUE *argv, VALUE klass)
{
  VALUE vresult = my_class_s_new(argc, argv, klass, delete_clipboard);
  return vresult;
}


static VALUE
rb_clipboard_initialize(int argc, VALUE *argv, VALUE self)
{
  BClipboard *result;
  VALUE varg1, varg2;

  Data_To_Null(self, BClipboard);

  switch (rb_scan_args(argc, argv, "11", &varg1, &varg2)) {
    case 1:
      {
        char *arg1;

        arg1 = STR2CSTR(varg1);
        NEW(result, BClipboard(arg1));
      }
      break;

    case 2:
      {
        char *arg1;
        bool arg2;

        arg1 = STR2CSTR(varg1);
        arg2 = RTEST(varg2);

        NEW(result, BClipboard(arg1, arg2));
      }
      break;

    default:
      rb_bug("rb_clipboard_initialize");
  }
  DATA_PTR(self) = result;
  return self;
}


static VALUE
rb_clipboard_Name(VALUE self)
{
  BClipboard *arg0;
  const char *result;
  VALUE vresult;

  Get_BInstance(Clipboard, self, arg0);
  result = arg0->Name();
  vresult = rb_str_new2(result);
  return vresult;
}


static VALUE
rb_clipboard_LocalCount(VALUE self)
{
  BClipboard *arg0;
  uint32 result;
  VALUE vresult;

  Get_BInstance(Clipboard, self, arg0);
  result = arg0->LocalCount();
  vresult = ULONG2NUM(result);
  return vresult;
}


static VALUE
rb_clipboard_SystemCount(VALUE self)
{
  BClipboard *arg0;
  uint32 result;
  VALUE vresult;

  Get_BInstance(Clipboard, self, arg0);
  result = arg0->SystemCount();
  vresult = ULONG2NUM(result);
  return vresult;
}


static VALUE
rb_clipboard_StartWatching(VALUE self, VALUE varg1)
{
  BClipboard *arg0;
  BMessenger *arg1;
  status_t result;

  Get_BInstance(Clipboard, self, arg0);
  Get_BInstance(Messenger, varg1, arg1);
  result = arg0->StartWatching(*arg1);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_clipboard_StopWatching(VALUE self, VALUE varg1)
{
  BClipboard *arg0;
  BMessenger *arg1;
  status_t result;

  Get_BInstance(Clipboard, self, arg0);
  Get_BInstance(Messenger, varg1, arg1);
  result = arg0->StopWatching(*arg1);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_clipboard_Unlock(VALUE self)
{
  BClipboard *arg0;

  Get_BInstance(Clipboard, self, arg0);
  arg0->Unlock();
  return Qnil;
}


static VALUE
yld_clipboard_lock(VALUE self)
{
  BClipboard *arg0;
  VALUE vresult = Qnil;

  Get_BInstance(Clipboard, self, arg0);
  if (arg0->Lock()) {
    vresult = rb_yield(self);
  }
  return vresult;
}


static VALUE
rb_clipboard_Lock(VALUE self)
{
  BClipboard *arg0;
  VALUE vresult;

  if (rb_block_given_p()) {
    vresult = rb_ensure(RUBY_METHOD_FUNC(yld_clipboard_lock), self,
                        RUBY_METHOD_FUNC(rb_clipboard_Unlock), self);
  }
  else {
    bool result;

    Get_BInstance(Clipboard, self, arg0);
    result = arg0->Lock();
    vresult = (result ? Qtrue:Qfalse);
  }
  return vresult;
}


static VALUE
rb_clipboard_IsLocked(VALUE self)
{
  BClipboard *arg0;
  bool result;
  VALUE vresult;

  Get_BInstance(Clipboard, self, arg0);
  result = arg0->IsLocked();
  vresult = (result ? Qtrue:Qfalse);
  return vresult;
}


static VALUE
rb_clipboard_Clear(VALUE self)
{
  BClipboard *arg0;
  status_t result;

  Get_BInstance(Clipboard, self, arg0);
  result = arg0->Clear();
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_clipboard_Commit(VALUE self)
{
  BClipboard *arg0;
  status_t result;

  Get_BInstance(Clipboard, self, arg0);
  result = arg0->Commit();
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_clipboard_Revert(VALUE self)
{
  BClipboard *arg0;
  status_t result;

  Get_BInstance(Clipboard, self, arg0);
  result = arg0->Revert();
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_clipboard_DataSource(VALUE self)
{
  BClipboard *arg0;
  BMessenger *result;
  VALUE vresult;

  Get_BInstance(Clipboard, self, arg0);
  result = new Messenger(arg0->DataSource());
  vresult = Data_Wrap_Struct(cMessenger, 0, delete_messenger, result);
  return vresult;
}


static VALUE
rb_clipboard_Data(VALUE self)
{
  BClipboard *arg0;
  BMessage *result;
  VALUE vresult;

  Get_BInstance(Clipboard, self, arg0);
  result = arg0->Data();
  vresult = ((result) ? Data_Wrap_Struct(cMessage, 0, 0, result):Qnil);
  return vresult;
}


void
Init_Clipboard()
{
  rb_global_variable(&my_clipboard);
  rb_define_readonly_variable("$be_clipboard", &my_clipboard);

  my_clipboard = Qnil;

  cClipboard = rb_define_class_under(mBe, "Clipboard", rb_cObject);
  rb_define_singleton_method(cClipboard, "new", RUBY_METHOD_FUNC(rb_clipboard_s_new), -1);
  rb_define_method(cClipboard, "initialize", RUBY_METHOD_FUNC(rb_clipboard_initialize), -1);

  rb_define_method(cClipboard, "Name", RUBY_METHOD_FUNC(rb_clipboard_Name), 0);

  rb_define_method(cClipboard, "LocalCount", RUBY_METHOD_FUNC(rb_clipboard_LocalCount), 0);
  rb_define_method(cClipboard, "SystemCount", RUBY_METHOD_FUNC(rb_clipboard_SystemCount), 0);
  rb_define_method(cClipboard, "StartWatching", RUBY_METHOD_FUNC(rb_clipboard_StartWatching), 1);
  rb_define_method(cClipboard, "StopWatching", RUBY_METHOD_FUNC(rb_clipboard_StopWatching), 1);

  rb_define_method(cClipboard, "Lock", RUBY_METHOD_FUNC(rb_clipboard_Lock), 0);
  rb_define_method(cClipboard, "Unlock", RUBY_METHOD_FUNC(rb_clipboard_Unlock), 0);
  rb_define_method(cClipboard, "IsLocked", RUBY_METHOD_FUNC(rb_clipboard_IsLocked), 0);

  rb_define_method(cClipboard, "Clear", RUBY_METHOD_FUNC(rb_clipboard_Clear), 0);
  rb_define_method(cClipboard, "Commit", RUBY_METHOD_FUNC(rb_clipboard_Commit), 0);
  rb_define_method(cClipboard, "Revert", RUBY_METHOD_FUNC(rb_clipboard_Revert), 0);

  rb_define_method(cClipboard, "DataSource", RUBY_METHOD_FUNC(rb_clipboard_DataSource), 0);
  rb_define_method(cClipboard, "Data", RUBY_METHOD_FUNC(rb_clipboard_Data), 0);

  rb_define_const(mBe, "CLIPBOARD_CHANGED", ULONG2NUM(B_CLIPBOARD_CHANGED));
}
