/*
 *  Entry.cc
 */

#include <Entry.h>
#include "my_be.h"


VALUE cEntry_ref;
VALUE cEntry;


static VALUE
rb_get_ref_for_path(VALUE self, VALUE varg1)
{
  char *arg1;
  entry_ref arg2, *copy;
  status_t result;
  VALUE vresult;

  arg1 = STR2CSTR(varg1);
  result = get_ref_for_path(arg1, &arg2);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  NEW(copy, entry_ref(arg2));
  vresult = Data_Wrap_Struct(cEntry_ref, 0, delete_entry_ref, copy);
  return vresult;
}


void
delete_entry_ref(entry_ref *arg0)
{
  delete arg0;
}


static VALUE
rb_entry_ref_s_new(int argc, VALUE *argv, VALUE klass)
{
  VALUE vresult = my_class_s_new(argc, argv, klass, delete_entry_ref);
  return vresult;
}


static VALUE
rb_entry_ref_initialize(int argc, VALUE *argv, VALUE self)
{
  entry_ref *result;
  VALUE varg1, varg2, varg3;

  Data_To_Null(self, entry_ref);

  switch (rb_scan_args(argc, argv, "03", &varg1, &varg2, &varg3)) {
    case 0:
      result = my_create_be_obj<entry_ref>();
      break;

    case 1:
      {
        entry_ref *arg1;

        Get_BInstance2(Entry_ref, entry_ref, varg1, arg1);
        NEW(result, entry_ref(*arg1));
      }
      break;

    case 2:
      DEBUG_TRACE();
      rb_raise(rb_eArgError, "wrong # of arguments");
      break;

    case 3:
      {
        dev_t arg1;
        ino_t arg2;
        char *arg3;

        arg1 = NUM2LONG(varg1);
        arg2 = NUM2LLONG(varg2);
        arg3 = STR2CSTR(varg3);

        NEW(result, entry_ref(arg1, arg2, arg3));
      }
      break;

    default:
      rb_bug("rb_entry_ref_initialize");
  }
  DATA_PTR(self) = result;
  return self;
}


static VALUE
rb_entry_ref_set_name(VALUE self, VALUE varg1)
{
  entry_ref *arg0;
  char *arg1;
  status_t result;
  VALUE vresult;

  Get_BInstance2(Entry_ref, entry_ref, self, arg0);
  arg1 = STR2CSTR(varg1);
  result = arg0->set_name(arg1);
  vresult = LONG2NUM(result);
  return vresult;
}


static VALUE
rb_entry_ref_equal(VALUE self, VALUE varg1)
{
  entry_ref *arg0, *arg1;

  if (!rb_obj_is_kind_of(varg1, cEntry_ref)) {
    return Qfalse;
  }

  Get_BInstance2(Entry_ref, entry_ref, self, arg0);
  Get_BInstance2(Entry_ref, entry_ref, varg1, arg1);

  if (*arg0 == *arg1) {
    return Qtrue;
  }
  return Qfalse;
}


static VALUE
rb_entry_ref_not_equal(VALUE self, VALUE varg1)
{
  entry_ref *arg0, *arg1;

  if (!rb_obj_is_kind_of(varg1, cEntry_ref)) {
    return Qtrue;
  }

  Get_BInstance2(Entry_ref, entry_ref, self, arg0);
  Get_BInstance2(Entry_ref, entry_ref, varg1, arg1);

  if (*arg0 != *arg1) {
    return Qtrue;
  }
  return Qfalse;
}


static VALUE
rb_entry_ref_device_set(VALUE self, VALUE varg1)
{
  entry_ref *arg0;
  dev_t arg1;

  Get_BInstance2(Entry_ref, entry_ref, self, arg0);
  arg1 = NUM2LONG(varg1);
  arg0->device = arg1;
  return varg1;
}


static VALUE
rb_entry_ref_device_ref(VALUE self)
{
  entry_ref *arg0;
  dev_t result;
  VALUE vresult;

  Get_BInstance2(Entry_ref, entry_ref, self, arg0);
  result = arg0->device;
  vresult = LONG2NUM(result);
  return vresult;
}


static VALUE
rb_entry_ref_directory_set(VALUE self, VALUE varg1)
{
  entry_ref *arg0;
  ino_t arg1;

  Get_BInstance2(Entry_ref, entry_ref, self, arg0);
  arg1 = NUM2LLONG(varg1);
  arg0->directory = arg1;
  return varg1;
}


static VALUE
rb_entry_ref_directory_ref(VALUE self)
{
  entry_ref *arg0;
  ino_t result;
  VALUE vresult;

  Get_BInstance2(Entry_ref, entry_ref, self, arg0);
  result = arg0->directory;
  vresult = LLONG2NUM(result);
  return vresult;
}


static VALUE
rb_entry_ref_name_set(VALUE self, VALUE varg1)
{
  entry_ref *arg0;
  char *arg1;

  Get_BInstance2(Entry_ref, entry_ref, self, arg0);
  arg1 = STR2CSTR(varg1);
  arg0->name = arg1;
  return varg1;
}


static VALUE
rb_entry_ref_name_ref(VALUE self)
{
  entry_ref *arg0;
  char *result;
  VALUE vresult;

  Get_BInstance2(Entry_ref, entry_ref, self, arg0);
  result = arg0->name;
  vresult = rb_str_new2(result);
  return vresult;
}


static VALUE
rb_entry_ref_inspect(VALUE self)
{
  entry_ref *arg0;
  VALUE str;
  char *c;

  Get_BInstance2(Entry_ref, entry_ref, self, arg0);

  c = rb_class2name(CLASS_OF(self));
  str = rb_str_new2("#<");
  str = rb_str_cat2(str, c);

  str = rb_str_cat2(str, " device=");
  str = rb_str_concat(str, rb_obj_as_string(LONG2NUM(arg0->device)));
  str = rb_str_cat2(str, " directory=");
  str = rb_str_concat(str, rb_obj_as_string(LLONG2NUM(arg0->directory)));
  str = rb_str_cat2(str, " name=");
  str = rb_str_cat2(str, arg0->name);

  str = rb_str_cat2(str, ">");
  return str;
}


void
delete_entry(BEntry *arg0)
{
  delete arg0;
}


void
Init_Entry()
{
  rb_define_module_function(mBe, "get_ref_for_path", RUBY_METHOD_FUNC(rb_get_ref_for_path), 1);

  cEntry_ref = rb_define_class_under(mBe, "Entry_ref", rb_cObject);
  rb_define_singleton_method(cEntry_ref, "new", RUBY_METHOD_FUNC(rb_entry_ref_s_new), -1);
  rb_define_method(cEntry_ref, "initialize", RUBY_METHOD_FUNC(rb_entry_ref_initialize), -1);

  rb_define_method(cEntry_ref, "set_name", RUBY_METHOD_FUNC(rb_entry_ref_set_name), 1);

  rb_define_method(cEntry_ref, "==", RUBY_METHOD_FUNC(rb_entry_ref_equal), 1);
  rb_define_method(cEntry_ref, "!=", RUBY_METHOD_FUNC(rb_entry_ref_not_equal), 1);

  rb_define_method(cEntry_ref, "device=", RUBY_METHOD_FUNC(rb_entry_ref_device_set), 1);
  rb_define_method(cEntry_ref, "device", RUBY_METHOD_FUNC(rb_entry_ref_device_ref), 0);
  rb_define_method(cEntry_ref, "directory=", RUBY_METHOD_FUNC(rb_entry_ref_directory_set), 1);
  rb_define_method(cEntry_ref, "directory", RUBY_METHOD_FUNC(rb_entry_ref_directory_ref), 0);
  rb_define_method(cEntry_ref, "name=", RUBY_METHOD_FUNC(rb_entry_ref_name_set), 1);
  rb_define_method(cEntry_ref, "name", RUBY_METHOD_FUNC(rb_entry_ref_name_ref), 0);
  rb_define_method(cEntry_ref, "inspect", RUBY_METHOD_FUNC(rb_entry_ref_inspect), 0);

  cEntry = rb_define_class_under(mBe, "Entry", cStatable);
}
