/*
 *  EntryList.cc
 */

#include <EntryList.h>
#include <Entry.h>
#include "my_be.h"


VALUE cEntryList;


void
delete_entrylist(BEntryList *arg0)
{
  delete arg0;
}


static VALUE
rb_entrylist_GetNextEntry(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1, vresult = Qnil;
  BEntryList *arg0;
  BEntry arg1, *copy;
  status_t result;

  Get_BInstance(EntryList, self, arg0);
  switch (rb_scan_args(argc, argv, "01", &varg1)) {
    case 0:
      if (rb_block_given_p()) {
        for (int32 i=0; arg0->GetNextEntry(&arg1) == B_OK; i++) {
          NEW(copy, BEntry(arg1));
          vresult = Data_Wrap_Struct(cEntry, 0, delete_entry, copy);
          vresult = rb_yield(vresult);
        }
      }
      else {
        result = arg0->GetNextEntry(&arg1);
        if (result != B_OK) {
          DEBUG_TRACE();
          my_raise(result);
        }
        NEW(copy, BEntry(arg1));
        vresult = Data_Wrap_Struct(cEntry, 0, delete_entry, copy);
      }
      break;

    case 1:
      if (rb_block_given_p()) {
        bool arg2;
        arg2 = RTEST(varg1);

        for (int32 i=0; arg0->GetNextEntry(&arg1, arg2) == B_OK; i++) {
          NEW(copy, BEntry(arg1));
          vresult = Data_Wrap_Struct(cEntry, 0, delete_entry, copy);
          vresult = rb_yield(vresult);
        }
      }
      else {
        bool arg2;

        arg2 = RTEST(varg1);
        result = arg0->GetNextEntry(&arg1, arg2);
        if (result != B_OK) {
          DEBUG_TRACE();
          my_raise(result);
        }
        NEW(copy, BEntry(arg1));
        vresult = Data_Wrap_Struct(cEntry, 0, delete_entry, copy);
      }
      break;

    default:
      rb_bug("rb_entrylist_GetNextEntry");
  }
  return vresult;
}


static VALUE
rb_entrylist_GetNextRef(VALUE self)
{
  BEntryList *arg0;
  entry_ref arg1, *copy;
  status_t result;
  VALUE vresult = Qnil;

  Get_BInstance(EntryList, self, arg0);
  if (rb_block_given_p()) {
    for (int32 i=0; arg0->GetNextRef(&arg1) == B_OK; i++) {
      NEW(copy, entry_ref(arg1));
      vresult = Data_Wrap_Struct(cEntry_ref, 0, delete_entry_ref, copy);
      vresult = rb_yield(vresult);
    }
  }
  else {
    result = arg0->GetNextRef(&arg1);
    if (result != B_OK) {
      DEBUG_TRACE();
      my_raise(result);
    }
    NEW(copy, entry_ref(arg1));
    vresult = Data_Wrap_Struct(cEntry_ref, 0, delete_entry_ref, copy);
  }
  return vresult;
}


static VALUE
rb_entrylist_GetNextDirents(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1, vresult = Qnil;
  BEntryList *arg0;
  dirent_t *arg1;
  int32 result;

  Get_BInstance(EntryList, self, arg0);
  switch (rb_scan_args(argc, argv, "01", &varg1)) {
    case 0:
      if (rb_block_given_p()) {
        arg1 = ALLOCA_N(dirent_t, 2);

        for (int32 i=0; arg0->GetNextDirents(arg1, sizeof(dirent_t)*2, 1) != 0; i++) {
          vresult = my_dirent_t2hash(*arg1);
          vresult = rb_yield(vresult);
        }
      }
      else {
        arg1 = ALLOCA_N(dirent_t, 2);
        result = arg0->GetNextDirents(arg1, sizeof(dirent_t)*2, 1);
        vresult = my_dirent_t2hash(*arg1);
      }
      break;

    case 1:
      if (rb_block_given_p()) {
        int32 arg3;

        arg3 = NUM2LONG(varg1);
        arg1 = ALLOCA_N(dirent_t, arg3 + 1);

        for (int32 i=0; arg0->GetNextDirents(arg1, sizeof(dirent_t)*arg3 + 1, arg3) != 0; i++) {
          vresult = my_dirent_t2hash(*arg1);
          vresult = rb_yield(vresult);
        }
      }
      else {
        int32 arg3;

        arg3 = NUM2LONG(varg1);
        arg1 = ALLOCA_N(dirent_t, arg3 + 1);
        result = arg0->GetNextDirents(arg1, sizeof(dirent_t)*arg3 + 1, arg3);
        vresult = my_dirent_t2hash(*arg1);
      }
      break;

    default:
      rb_bug("rb_entrylist_GetNextDirents");
  }
  return vresult;
}


static VALUE
rb_entrylist_Rewind(VALUE self)
{
  BEntryList *arg0;
  status_t result;
  VALUE vresult;

  Get_BInstance(EntryList, self, arg0);
  result = arg0->Rewind();
  vresult = LONG2NUM(result);
  return vresult;
}


static VALUE
rb_entrylist_CountEntries(VALUE self)
{
  BEntryList *arg0;
  int32 result;
  VALUE vresult;

  Get_BInstance(EntryList, self, arg0);
  result = arg0->CountEntries();
  vresult = LONG2NUM(result);
  return vresult;
}


void
Init_EntryList()
{
  cEntryList = rb_define_class_under(mBe, "EntryList", rb_cObject);
  rb_undef_method(CLASS_OF(cEntryList), "new");

  rb_define_method(cEntryList, "GetNextEntry", RUBY_METHOD_FUNC(rb_entrylist_GetNextEntry), -1);
  rb_define_method(cEntryList, "GetNextRef", RUBY_METHOD_FUNC(rb_entrylist_GetNextRef), 0);
  rb_define_method(cEntryList, "GetNextDirents", RUBY_METHOD_FUNC(rb_entrylist_GetNextDirents), -1);

  rb_define_method(cEntryList, "Rewind", RUBY_METHOD_FUNC(rb_entrylist_Rewind), 0);
  rb_define_method(cEntryList, "CountEntries", RUBY_METHOD_FUNC(rb_entrylist_CountEntries), 0);
}
