/*
 *  Errors.cc
 */

#include <Errors.h>
#include <MediaDefs.h>
#include <MidiDefs.h>
#include <TranslationErrors.h>
#include "my_be.h"


/* Error baselines */
VALUE eGeneralError;
VALUE eOsError;
VALUE eAppError;
VALUE eInterfaceError;
VALUE eMediaError;
VALUE eTranslationError;
VALUE eMidiError;
VALUE eStrageError;
VALUE eMailError;
VALUE ePrintError;
VALUE eDeviceError;

/* General Errors */
VALUE eNoMemory;
VALUE eIoError;
VALUE ePermissionDenied;
VALUE eBadIndex;
VALUE eBadType;
VALUE eBadValue;
VALUE eMismatchedValue;
VALUE eNameNotFound;
VALUE eNameInUse;
VALUE eTimedOut;
VALUE eInterrupted;
VALUE eWouldBlock;
VALUE eCanceled;
VALUE eNoInit;
VALUE eBusy;
VALUE eNotAllowed;
VALUE eError;
VALUE eOk;
VALUE eNoError;
/* Kernel Kit Errors */
VALUE eBadSemId;
VALUE eNoMoreSems;
VALUE eBadThreadId;
VALUE eNoMoreThreads;
VALUE eBadThreadState;
VALUE eBadTeamId;
VALUE eNoMoreTeams;
VALUE eBadPortId;
VALUE eNoMorePorts;
VALUE eBadImageId;
VALUE eBadAddress;
VALUE eNotAnExecutable;
VALUE eMissingLibrary;
VALUE eMissingSymbol;
VALUE eDebuggerAlreadyInstalled;
/* Application Kit Errors */
VALUE eBadReply;
VALUE eDuplicatReply;
VALUE eMessageToSelf;
VALUE eBadHandler;
VALUE eAlreadyRunning;
VALUE eLaunchFailed;
VALUE eAmbiguousAppLaunch;
VALUE eUnknownMimeType;
VALUE eBadScriptSyntax;
VALUE eLaunchFailedNoResolveLink;
VALUE eLaunchFailedExecutable;
VALUE eLaunchFailedAppNotFound;
VALUE eLaunchFailedAppInTrash;
VALUE eLaunchFailedNoPreferredApp;
VALUE eLaunchFailedFilesAppNotFound;
VALUE eBadMimeSnifferRule;
/* Media Kit Errors */
VALUE eStreamNotFound;
VALUE eServerNotFound;
VALUE eResouceNotFound;
VALUE eResouceUnavailable;
VALUE eBadSubscriber;
VALUE eSubscriberNotEntered;
VALUE eBufferNotAvailable;
VALUE eLastBufferError;
VALUE eMediaSystemFailure;
VALUE eMediaBadNode;
VALUE eMediaNodeBusy;
VALUE eMediaBadFormat;
VALUE eMediaBadBuffer;
VALUE eMediaTooManyNodes;
VALUE eMediaTooManyBuffers;
VALUE eMediaNodeAlreadyExists;
VALUE eMediaBufferAlreadyExists;
VALUE eMediaCannotSeek;
VALUE eMediaCannotChangeRunMode;
VALUE eMediaAppAlreadyRegistered;
VALUE eMediaAppNotRegistered;
VALUE eMediaCannotReclaimBuffers;
VALUE eMediaBuffersNotReclaimed;
VALUE eMediaTimeSourceStopped;
VALUE eMediaTimeSourceBusy;
VALUE eMediaBadSource;
VALUE eMediaBadDestination;
VALUE eMediaAlreadyConnected;
VALUE eMediaNotConnected;
VALUE eMediaBadClipFormat;
VALUE eMediaAddonFailed;
VALUE eMediaAddonDisabled;
VALUE eMediaChangeInProgress;
VALUE eMediaStaleChangeCount;
VALUE eMediaAddonRestricted;
VALUE eMediaNoHandler;
VALUE eMediaDuplicateFormat;
VALUE eMediaRealtimeDisabled;
VALUE eMediaRealtimeUnavailable;
/* Error codes for the Translation Kit */
VALUE eNoTranslator;
VALUE eIllegalData;
VALUE eNotInitialized;
/* Midi Kit Errors */
VALUE eBadInstrument;
VALUE eBadMidiData;
VALUE eAlreadyPaused;
VALUE eAlreadyResumed;
VALUE eNoSongPlaying;
VALUE eTooManySongsPlaying;
/* Storage Kit/File System Errors */
VALUE eFileError;
VALUE eFileExists;
VALUE eEntryNotFound;
VALUE eNameTooLong;
VALUE eNotADirectory;
VALUE eDirectoryNotEmpty;
VALUE eDeviceFull;
VALUE eReadOnlyDevice;
VALUE eIsADirectory;
VALUE eNoMoreFds;
VALUE eCrossDeviceLink;
VALUE eLinkLimit;
VALUE eBustedPipe;
VALUE eUnsupported;
VALUE ePartitionTooSmall;
/* Mail Kit Errors */
VALUE eMailNoDaemon;
VALUE eMailUnknownUser;
VALUE eMailWrongPassword;
VALUE eMailUnknownHost;
VALUE eMailAccessError;
VALUE eMailUnknownField;
VALUE eMailNoRecipient;
VALUE eMailInvalidMail;
/* Printing Errors */
VALUE eNoPrintServer;
/* Device Kit Errors */
VALUE eDevInvalidIoctl;
VALUE eDevNoMemory;
VALUE eDevBadDriveNum;
VALUE eDevNoMedia;
VALUE eDevUnreadable;
VALUE eDevFormatError;
VALUE eDevTimeout;
VALUE eDevRecalibrateError;
VALUE eDevSeekError;
VALUE eDevIdError;
VALUE eDevReadError;
VALUE eDevWriteError;
VALUE eDevNotReady;
VALUE eDevMediaChanged;
VALUE eDevMediaChangeRequested;
VALUE eDevResouceConflict;
VALUE eDevConfigurationError;
VALUE eDevDisabledByUser;
VALUE eDevDoorOpen;


struct error_base {
  VALUE error;
  int   errno;
};


static VALUE
error_base2obj(int no, struct error_base *errs)
{
  for (; errs->errno; errs++) {
    if (errs->errno == no) {
      return errs->error;
    }
  }
  return errs->error;
}


void
my_raise(int err)
{
  struct error_base general_errors [] = {
    { eNoMemory,         B_NO_MEMORY },
    { eIoError,          B_IO_ERROR },
    { ePermissionDenied, B_PERMISSION_DENIED },
    { eBadIndex,         B_BAD_INDEX },
    { eBadType,          B_BAD_TYPE },
    { eBadValue,         B_BAD_VALUE },
    { eMismatchedValue,  B_MISMATCHED_VALUES },
    { eNameNotFound,     B_NAME_NOT_FOUND },
    { eNameInUse,        B_NAME_IN_USE },
    { eTimedOut,         B_TIMED_OUT },
    { eInterrupted,      B_INTERRUPTED },
    { eWouldBlock,       B_WOULD_BLOCK },
    { eCanceled,         B_CANCELED },
    { eNoInit,           B_NO_INIT },
    { eBusy,             B_BUSY },
    { eNotAllowed,       B_NOT_ALLOWED },
    { eGeneralError,     0 }
  };
  struct error_base os_errors [] = {
    { eBadSemId,        B_BAD_SEM_ID },
    { eNoMoreSems,      B_NO_MORE_SEMS },
    { eBadThreadId,     B_BAD_THREAD_ID },
    { eNoMoreThreads,   B_NO_MORE_THREADS },
    { eBadThreadState,  B_BAD_THREAD_STATE },
    { eBadTeamId,       B_BAD_TEAM_ID },
    { eNoMoreTeams,     B_NO_MORE_TEAMS },
    { eBadPortId,       B_BAD_PORT_ID },
    { eNoMorePorts,     B_NO_MORE_PORTS },
    { eBadImageId,      B_BAD_IMAGE_ID },
    { eBadAddress,      B_BAD_ADDRESS },
    { eNotAnExecutable, B_NOT_AN_EXECUTABLE },
    { eMissingLibrary,  B_MISSING_LIBRARY },
    { eMissingSymbol,   B_MISSING_SYMBOL },
    { eDebuggerAlreadyInstalled, B_DEBUGGER_ALREADY_INSTALLED },
    { eOsError,         0 }
  };
  struct error_base app_errors [] = {
    { eBadReply,                     B_BAD_REPLY },
    { eDuplicatReply,                B_DUPLICATE_REPLY },
    { eMessageToSelf,                B_MESSAGE_TO_SELF },
    { eBadHandler,                   B_BAD_HANDLER },
    { eAlreadyRunning,               B_ALREADY_RUNNING },
    { eLaunchFailed,                 B_LAUNCH_FAILED },
    { eAmbiguousAppLaunch,           B_AMBIGUOUS_APP_LAUNCH },
    { eUnknownMimeType,              B_UNKNOWN_MIME_TYPE },
    { eBadScriptSyntax,              B_BAD_SCRIPT_SYNTAX },
    { eLaunchFailedNoResolveLink,    B_LAUNCH_FAILED_NO_RESOLVE_LINK },
    { eLaunchFailedExecutable,       B_LAUNCH_FAILED_EXECUTABLE },
    { eLaunchFailedAppNotFound,      B_LAUNCH_FAILED_APP_NOT_FOUND },
    { eLaunchFailedAppInTrash,       B_LAUNCH_FAILED_APP_IN_TRASH },
    { eLaunchFailedNoPreferredApp,   B_LAUNCH_FAILED_NO_PREFERRED_APP },
    { eLaunchFailedFilesAppNotFound, B_LAUNCH_FAILED_FILES_APP_NOT_FOUND },
    { eBadMimeSnifferRule,           B_BAD_MIME_SNIFFER_RULE },
    { eAppError,                     0 }
  };
  struct error_base media_errors [] = {
    { eStreamNotFound,            B_STREAM_NOT_FOUND },
    { eServerNotFound,            B_SERVER_NOT_FOUND },
    { eResouceNotFound,           B_RESOURCE_NOT_FOUND },
    { eResouceUnavailable,        B_RESOURCE_UNAVAILABLE },
    { eBadSubscriber,             B_BAD_SUBSCRIBER },
    { eSubscriberNotEntered,      B_SUBSCRIBER_NOT_ENTERED },
    { eBufferNotAvailable,        B_BUFFER_NOT_AVAILABLE },
    { eLastBufferError,           B_LAST_BUFFER_ERROR },
    { eMediaSystemFailure,        B_MEDIA_SYSTEM_FAILURE },
    { eMediaBadNode,              B_MEDIA_BAD_NODE },
    { eMediaNodeBusy,             B_MEDIA_NODE_BUSY },
    { eMediaBadFormat,            B_MEDIA_BAD_FORMAT },
    { eMediaBadBuffer,            B_MEDIA_BAD_BUFFER },
    { eMediaTooManyNodes,         B_MEDIA_TOO_MANY_NODES },
    { eMediaTooManyBuffers,       B_MEDIA_TOO_MANY_BUFFERS },
    { eMediaNodeAlreadyExists,    B_MEDIA_NODE_ALREADY_EXISTS },
    { eMediaBufferAlreadyExists,  B_MEDIA_BUFFER_ALREADY_EXISTS },
    { eMediaCannotSeek,           B_MEDIA_CANNOT_SEEK },
    { eMediaCannotChangeRunMode,  B_MEDIA_CANNOT_CHANGE_RUN_MODE },
    { eMediaAppAlreadyRegistered, B_MEDIA_APP_ALREADY_REGISTERED },
    { eMediaAppNotRegistered,     B_MEDIA_APP_NOT_REGISTERED },
    { eMediaCannotReclaimBuffers, B_MEDIA_CANNOT_RECLAIM_BUFFERS },
    { eMediaBuffersNotReclaimed,  B_MEDIA_BUFFERS_NOT_RECLAIMED },
    { eMediaTimeSourceStopped,    B_MEDIA_TIME_SOURCE_STOPPED },
    { eMediaTimeSourceBusy,       B_MEDIA_TIME_SOURCE_BUSY },
    { eMediaBadSource,            B_MEDIA_BAD_SOURCE },
    { eMediaBadDestination,       B_MEDIA_BAD_DESTINATION },
    { eMediaAlreadyConnected,     B_MEDIA_ALREADY_CONNECTED },
    { eMediaNotConnected,         B_MEDIA_NOT_CONNECTED },
    { eMediaBadClipFormat,        B_MEDIA_BAD_CLIP_FORMAT },
    { eMediaAddonFailed,          B_MEDIA_ADDON_FAILED },
    { eMediaAddonDisabled,        B_MEDIA_ADDON_DISABLED },
    { eMediaChangeInProgress,     B_MEDIA_CHANGE_IN_PROGRESS },
    { eMediaStaleChangeCount,     B_MEDIA_STALE_CHANGE_COUNT },
    { eMediaAddonRestricted,      B_MEDIA_ADDON_RESTRICTED },
    { eMediaNoHandler,            B_MEDIA_NO_HANDLER },
    { eMediaDuplicateFormat,      B_MEDIA_DUPLICATE_FORMAT },
    { eMediaRealtimeDisabled,     B_MEDIA_REALTIME_DISABLED },
    { eMediaRealtimeUnavailable,  B_MEDIA_REALTIME_UNAVAILABLE },
    { eMediaError,                0 }
  };
  struct error_base translation_errors [] = {
    { eNoTranslator,     B_NO_TRANSLATOR },
    { eIllegalData,      B_ILLEGAL_DATA },
    { eNotInitialized,   B_NOT_INITIALIZED },
    { eTranslationError, 0 }
  };
  struct error_base midi_errors [] = {
    { eBadInstrument,       B_BAD_INSTRUMENT },
    { eBadMidiData,         B_BAD_MIDI_DATA },
    { eAlreadyPaused,       B_ALREADY_PAUSED },
    { eAlreadyResumed,      B_ALREADY_RESUMED },
    { eNoSongPlaying,       B_NO_SONG_PLAYING },
    { eTooManySongsPlaying, B_TOO_MANY_SONGS_PLAYING },
    { eMidiError,           0 }
  };
  struct error_base storage_errors [] = {
    { eFileError,         B_FILE_ERROR },
    { eEntryNotFound,     B_FILE_NOT_FOUND },
    { eFileExists,        B_FILE_EXISTS },
    { eEntryNotFound,     B_ENTRY_NOT_FOUND },
    { eNameTooLong,       B_NAME_TOO_LONG },
    { eNotADirectory,     B_NOT_A_DIRECTORY },
    { eDirectoryNotEmpty, B_DIRECTORY_NOT_EMPTY },
    { eDeviceFull,        B_DEVICE_FULL },
    { eReadOnlyDevice,    B_READ_ONLY_DEVICE },
    { eIsADirectory,      B_IS_A_DIRECTORY },
    { eNoMoreFds,         B_NO_MORE_FDS },
    { eCrossDeviceLink,   B_CROSS_DEVICE_LINK },
    { eLinkLimit,         B_LINK_LIMIT },
    { eBustedPipe,        B_BUSTED_PIPE },
    { eUnsupported,       B_UNSUPPORTED },
    { ePartitionTooSmall, B_PARTITION_TOO_SMALL },
    { eStrageError,       0 }
  };
  struct error_base mail_errors [] = {
    { eMailNoDaemon,      B_MAIL_NO_DAEMON },
    { eMailUnknownUser,   B_MAIL_UNKNOWN_USER },
    { eMailWrongPassword, B_MAIL_WRONG_PASSWORD },
    { eMailUnknownHost,   B_MAIL_UNKNOWN_HOST },
    { eMailAccessError,   B_MAIL_ACCESS_ERROR },
    { eMailUnknownField,  B_MAIL_UNKNOWN_FIELD },
    { eMailNoRecipient,   B_MAIL_NO_RECIPIENT },
    { eMailInvalidMail,   B_MAIL_INVALID_MAIL },
    { eMailError,         0 }
  };
  struct error_base print_errors [] = {
    { eNoPrintServer, B_NO_PRINT_SERVER },
    { ePrintError,    0 }
  };
  struct error_base device_errors [] = {
    { eDevInvalidIoctl,         B_DEV_INVALID_IOCTL },
    { eDevNoMemory,             B_DEV_NO_MEMORY },
    { eDevBadDriveNum,          B_DEV_BAD_DRIVE_NUM },
    { eDevNoMedia,              B_DEV_NO_MEDIA },
    { eDevUnreadable,           B_DEV_UNREADABLE },
    { eDevFormatError,          B_DEV_FORMAT_ERROR },
    { eDevTimeout,              B_DEV_TIMEOUT },
    { eDevRecalibrateError,     B_DEV_RECALIBRATE_ERROR },
    { eDevSeekError,            B_DEV_SEEK_ERROR },
    { eDevIdError,              B_DEV_ID_ERROR },
    { eDevReadError,            B_DEV_READ_ERROR },
    { eDevWriteError,           B_DEV_WRITE_ERROR },
    { eDevNotReady,             B_DEV_NOT_READY },
    { eDevMediaChanged,         B_DEV_MEDIA_CHANGED },
    { eDevMediaChangeRequested, B_DEV_MEDIA_CHANGE_REQUESTED },
    { eDevResouceConflict,      B_DEV_RESOURCE_CONFLICT },
    { eDevConfigurationError,   B_DEV_CONFIGURATION_ERROR },
    { eDevDisabledByUser,       B_DEV_DISABLED_BY_USER },
    { eDevDoorOpen,             B_DEV_DOOR_OPEN },
    { eDeviceError,             0 }
  };


#define ERROR_MASK 0x8000f800

  VALUE eClass = rb_eStandardError;

  if (err == B_ERROR) {
    rb_raise(eError, "%s", strerror(err));
  }
  else if (err == B_NO_ERROR) {
    rb_raise(eNoError, "%s", strerror(err));
  }

  switch (err & ERROR_MASK) {
    case B_GENERAL_ERROR_BASE:
      eClass = error_base2obj(err, general_errors);
      break;
    case B_OS_ERROR_BASE:
      eClass = error_base2obj(err, os_errors);
      break;
    case B_APP_ERROR_BASE:
      eClass = error_base2obj(err, app_errors);
      break;
    case B_INTERFACE_ERROR_BASE:
      eClass = eInterfaceError;
      break;
    case B_MEDIA_ERROR_BASE:
      eClass = error_base2obj(err, media_errors);
      break;
    case B_TRANSLATION_ERROR_BASE:
      eClass = error_base2obj(err, translation_errors);
      break;
    case B_MIDI_ERROR_BASE:
      eClass = error_base2obj(err, midi_errors);;
      break;
    case B_STORAGE_ERROR_BASE:
      eClass = error_base2obj(err, storage_errors);
      break;
    case B_MAIL_ERROR_BASE:
      eClass = error_base2obj(err, mail_errors);
      break;
    case B_PRINT_ERROR_BASE:
      eClass = error_base2obj(err, print_errors);
      break;
    case B_DEVICE_ERROR_BASE:
      eClass = error_base2obj(err, device_errors);
      break;
    default:
      rb_sys_fail(0);
      break;
  }
  rb_raise(eClass, "%s", strerror(err));
}


static VALUE
rb_strerror(VALUE self, VALUE varg1)
{
  int arg1;
  arg1 = NUM2LONG(varg1);
  return rb_str_new2(strerror(arg1));
}


void
Init_Errors()
{
  rb_define_module_function(mBe, "strerror", RUBY_METHOD_FUNC(rb_strerror), 1);

#define define_error(name, error) rb_define_class_under(mBe, #name, error)

#define define_standard_error(name) e##name = define_error(name, rb_eStandardError)
  define_standard_error(GeneralError);
  define_standard_error(OsError);
  define_standard_error(AppError);
  define_standard_error(InterfaceError);
  define_standard_error(MediaError);
  define_standard_error(TranslationError);
  define_standard_error(MidiError);
  define_standard_error(StrageError);
  define_standard_error(MailError);
  define_standard_error(PrintError);
  define_standard_error(DeviceError);

#define define_general_error(name) e##name = define_error(name, eGeneralError)
  define_general_error(NoMemory);
  define_general_error(IoError);
  define_general_error(PermissionDenied);
  define_general_error(BadIndex);
  define_general_error(BadType);
  define_general_error(BadValue);
  define_general_error(MismatchedValue);
  define_general_error(NameNotFound);
  define_general_error(NameInUse);
  define_general_error(TimedOut);
  define_general_error(Interrupted);
  define_general_error(WouldBlock);
  define_general_error(Canceled);
  define_general_error(NoInit);
  define_general_error(Busy);
  define_general_error(NotAllowed);
  define_general_error(Error);
  define_general_error(Ok);
  define_general_error(NoError);

#define define_os_error(name) e##name = define_error(name, eOsError)
  define_os_error(BadSemId);
  define_os_error(NoMoreSems);
  define_os_error(BadThreadId);
  define_os_error(NoMoreThreads);
  define_os_error(BadThreadState);
  define_os_error(BadTeamId);
  define_os_error(NoMoreTeams);
  define_os_error(BadPortId);
  define_os_error(NoMorePorts);
  define_os_error(BadImageId);
  define_os_error(BadAddress);
  define_os_error(NotAnExecutable);
  define_os_error(MissingLibrary);
  define_os_error(MissingSymbol);
  define_os_error(DebuggerAlreadyInstalled);

#define define_app_error(name) e##name = define_error(name, eAppError)
  define_os_error(BadReply);
  define_os_error(DuplicatReply);
  define_os_error(MessageToSelf);
  define_os_error(BadHandler);
  define_os_error(AlreadyRunning);
  define_os_error(LaunchFailed);
  define_os_error(AmbiguousAppLaunch);
  define_os_error(UnknownMimeType);
  define_os_error(BadScriptSyntax);
  define_os_error(LaunchFailedNoResolveLink);
  define_os_error(LaunchFailedExecutable);
  define_os_error(LaunchFailedAppNotFound);
  define_os_error(LaunchFailedAppInTrash);
  define_os_error(LaunchFailedNoPreferredApp);
  define_os_error(LaunchFailedFilesAppNotFound);
  define_os_error(BadMimeSnifferRule);

#define define_media_error(name) e##name = define_error(name, eMediaError)
  define_media_error(StreamNotFound);
  define_media_error(ServerNotFound);
  define_media_error(ResouceNotFound);
  define_media_error(ResouceUnavailable);
  define_media_error(BadSubscriber);
  define_media_error(SubscriberNotEntered);
  define_media_error(BufferNotAvailable);
  define_media_error(LastBufferError);
  define_media_error(MediaSystemFailure);
  define_media_error(MediaBadNode);
  define_media_error(MediaNodeBusy);
  define_media_error(MediaBadFormat);
  define_media_error(MediaBadBuffer);
  define_media_error(MediaTooManyNodes);
  define_media_error(MediaTooManyBuffers);
  define_media_error(MediaNodeAlreadyExists);
  define_media_error(MediaBufferAlreadyExists);
  define_media_error(MediaCannotSeek);
  define_media_error(MediaCannotChangeRunMode);
  define_media_error(MediaAppAlreadyRegistered);
  define_media_error(MediaAppNotRegistered);
  define_media_error(MediaCannotReclaimBuffers);
  define_media_error(MediaBuffersNotReclaimed);
  define_media_error(MediaTimeSourceStopped);
  define_media_error(MediaTimeSourceBusy);
  define_media_error(MediaBadSource);
  define_media_error(MediaBadDestination);
  define_media_error(MediaAlreadyConnected);
  define_media_error(MediaNotConnected);
  define_media_error(MediaBadClipFormat);
  define_media_error(MediaAddonFailed);
  define_media_error(MediaAddonDisabled);
  define_media_error(MediaChangeInProgress);
  define_media_error(MediaStaleChangeCount);
  define_media_error(MediaAddonRestricted);
  define_media_error(MediaNoHandler);
  define_media_error(MediaDuplicateFormat);
  define_media_error(MediaRealtimeDisabled);
  define_media_error(MediaRealtimeUnavailable);

#define define_translation_error(name) e##name = define_error(name, eTranslationError)
  define_translation_error(NoTranslator);
  define_translation_error(IllegalData);
  define_translation_error(NotInitialized);

#define define_midi_error(name) e##name = define_error(name, eMidiError)
  define_midi_error(BadInstrument);
  define_midi_error(BadMidiData);
  define_midi_error(AlreadyPaused);
  define_midi_error(AlreadyResumed);
  define_midi_error(NoSongPlaying);
  define_midi_error(TooManySongsPlaying);

#define define_strage_error(name) e##name = define_error(name, eStrageError)
  define_strage_error(FileError);
  define_strage_error(FileExists);
  define_strage_error(EntryNotFound);
  define_strage_error(NameTooLong);
  define_strage_error(NotADirectory);
  define_strage_error(DirectoryNotEmpty);
  define_strage_error(DeviceFull);
  define_strage_error(ReadOnlyDevice);
  define_strage_error(IsADirectory);
  define_strage_error(NoMoreFds);
  define_strage_error(CrossDeviceLink);
  define_strage_error(LinkLimit);
  define_strage_error(BustedPipe);
  define_strage_error(Unsupported);
  define_strage_error(PartitionTooSmall);

#define define_mail_error(name) e##name = define_error(name, eMailError)
  define_mail_error(MailNoDaemon);
  define_mail_error(MailUnknownUser);
  define_mail_error(MailWrongPassword);
  define_mail_error(MailUnknownHost);
  define_mail_error(MailAccessError);
  define_mail_error(MailUnknownField);
  define_mail_error(MailNoRecipient);
  define_mail_error(MailInvalidMail);

#define define_print_error(name) e##name = define_error(name, ePrintError)
  define_print_error(NoPrintServer);

#define define_device_error(name) e##name = define_error(name, eDeviceError)
  define_device_error(DevInvalidIoctl);
  define_device_error(DevNoMemory);
  define_device_error(DevBadDriveNum);
  define_device_error(DevNoMedia);
  define_device_error(DevUnreadable);
  define_device_error(DevFormatError);
  define_device_error(DevTimeout);
  define_device_error(DevRecalibrateError);
  define_device_error(DevSeekError);
  define_device_error(DevIdError);
  define_device_error(DevReadError);
  define_device_error(DevWriteError);
  define_device_error(DevNotReady);
  define_device_error(DevMediaChanged);
  define_device_error(DevMediaChangeRequested);
  define_device_error(DevResouceConflict);
  define_device_error(DevConfigurationError);
  define_device_error(DevDisabledByUser);
  define_device_error(DevDoorOpen);


  rb_define_const(mBe, "GENERAL_ERROR_BASE", LONG2NUM(B_GENERAL_ERROR_BASE));
  rb_define_const(mBe, "OS_ERROR_BASE", LONG2NUM(B_OS_ERROR_BASE));
  rb_define_const(mBe, "APP_ERROR_BASE", LONG2NUM(B_APP_ERROR_BASE));
  rb_define_const(mBe, "INTERFACE_ERROR_BASE", LONG2NUM(B_INTERFACE_ERROR_BASE));
  rb_define_const(mBe, "MEDIA_ERROR_BASE", LONG2NUM(B_MEDIA_ERROR_BASE));
  rb_define_const(mBe, "TRANSLATION_ERROR_BASE", LONG2NUM(B_TRANSLATION_ERROR_BASE));
  rb_define_const(mBe, "MIDI_ERROR_BASE", LONG2NUM(B_MIDI_ERROR_BASE));
  rb_define_const(mBe, "STORAGE_ERROR_BASE", LONG2NUM(B_STORAGE_ERROR_BASE));
  rb_define_const(mBe, "POSIX_ERROR_BASE", LONG2NUM(B_POSIX_ERROR_BASE));
  rb_define_const(mBe, "MAIL_ERROR_BASE", LONG2NUM(B_MAIL_ERROR_BASE));
  rb_define_const(mBe, "PRINT_ERROR_BASE", LONG2NUM(B_PRINT_ERROR_BASE));
  rb_define_const(mBe, "DEVICE_ERROR_BASE", LONG2NUM(B_DEVICE_ERROR_BASE));

  rb_define_const(mBe, "ERRORS_END", LONG2NUM(B_ERRORS_END));

  rb_define_const(mBe, "NO_MEMORY", LONG2NUM(B_NO_MEMORY));
  rb_define_const(mBe, "IO_ERROR", LONG2NUM(B_IO_ERROR));
  rb_define_const(mBe, "PERMISSION_DENIED", LONG2NUM(B_PERMISSION_DENIED));
  rb_define_const(mBe, "BAD_INDEX", LONG2NUM(B_BAD_INDEX));
  rb_define_const(mBe, "BAD_TYPE", LONG2NUM(B_BAD_TYPE));
  rb_define_const(mBe, "BAD_VALUE", LONG2NUM(B_BAD_VALUE));
  rb_define_const(mBe, "MISMATCHED_VALUES", LONG2NUM(B_MISMATCHED_VALUES));
  rb_define_const(mBe, "NAME_NOT_FOUND", LONG2NUM(B_NAME_NOT_FOUND));
  rb_define_const(mBe, "NAME_IN_USE", LONG2NUM(B_NAME_IN_USE));
  rb_define_const(mBe, "TIMED_OUT", LONG2NUM(B_TIMED_OUT));
  rb_define_const(mBe, "INTERRUPTED", LONG2NUM(B_INTERRUPTED));
  rb_define_const(mBe, "WOULD_BLOCK", LONG2NUM(B_WOULD_BLOCK));
  rb_define_const(mBe, "CANCELED", LONG2NUM(B_CANCELED));
  rb_define_const(mBe, "NO_INIT", LONG2NUM(B_NO_INIT));
  rb_define_const(mBe, "BUSY", LONG2NUM(B_BUSY));
  rb_define_const(mBe, "NOT_ALLOWED", LONG2NUM(B_NOT_ALLOWED));
  rb_define_const(mBe, "ERROR", LONG2NUM(B_ERROR));
  rb_define_const(mBe, "OK", LONG2NUM(B_OK));
  rb_define_const(mBe, "NO_ERROR", LONG2NUM(B_NO_ERROR));

  rb_define_const(mBe, "BAD_SEM_ID", LONG2NUM(B_BAD_SEM_ID));
  rb_define_const(mBe, "NO_MORE_SEMS", LONG2NUM(B_NO_MORE_SEMS));
  rb_define_const(mBe, "BAD_THREAD_ID", LONG2NUM(B_BAD_THREAD_ID));
  rb_define_const(mBe, "NO_MORE_THREADS", LONG2NUM(B_NO_MORE_THREADS));
  rb_define_const(mBe, "BAD_THREAD_STATE", LONG2NUM(B_BAD_THREAD_STATE));
  rb_define_const(mBe, "BAD_TEAM_ID", LONG2NUM(B_BAD_TEAM_ID));
  rb_define_const(mBe, "NO_MORE_TEAMS", LONG2NUM(B_NO_MORE_TEAMS));
  rb_define_const(mBe, "BAD_PORT_ID", LONG2NUM(B_BAD_PORT_ID));
  rb_define_const(mBe, "NO_MORE_PORTS", LONG2NUM(B_NO_MORE_PORTS));
  rb_define_const(mBe, "BAD_IMAGE_ID", LONG2NUM(B_BAD_IMAGE_ID));
  rb_define_const(mBe, "BAD_ADDRESS", LONG2NUM(B_BAD_ADDRESS));
  rb_define_const(mBe, "NOT_AN_EXECUTABLE", LONG2NUM(B_NOT_AN_EXECUTABLE));
  rb_define_const(mBe, "MISSING_LIBRARY", LONG2NUM(B_MISSING_LIBRARY));
  rb_define_const(mBe, "MISSING_SYMBOL", LONG2NUM(B_MISSING_SYMBOL));
  rb_define_const(mBe, "DEBUGGER_ALREADY_INSTALLED", LONG2NUM(B_DEBUGGER_ALREADY_INSTALLED));

  rb_define_const(mBe, "BAD_REPLY", LONG2NUM(B_BAD_REPLY));
  rb_define_const(mBe, "DUPLICATE_REPLY", LONG2NUM(B_DUPLICATE_REPLY));
  rb_define_const(mBe, "MESSAGE_TO_SELF", LONG2NUM(B_MESSAGE_TO_SELF));
  rb_define_const(mBe, "BAD_HANDLER", LONG2NUM(B_BAD_HANDLER));
  rb_define_const(mBe, "ALREADY_RUNNING", LONG2NUM(B_ALREADY_RUNNING));
  rb_define_const(mBe, "LAUNCH_FAILED", LONG2NUM(B_LAUNCH_FAILED));
  rb_define_const(mBe, "AMBIGUOUS_APP_LAUNCH", LONG2NUM(B_AMBIGUOUS_APP_LAUNCH));
  rb_define_const(mBe, "UNKNOWN_MIME_TYPE", LONG2NUM(B_UNKNOWN_MIME_TYPE));
  rb_define_const(mBe, "BAD_SCRIPT_SYNTAX", LONG2NUM(B_BAD_SCRIPT_SYNTAX));
  rb_define_const(mBe, "LAUNCH_FAILED_NO_RESOLVE_LINK", LONG2NUM(B_LAUNCH_FAILED_NO_RESOLVE_LINK));
  rb_define_const(mBe, "LAUNCH_FAILED_EXECUTABLE", LONG2NUM(B_LAUNCH_FAILED_EXECUTABLE));
  rb_define_const(mBe, "LAUNCH_FAILED_APP_NOT_FOUND", LONG2NUM(B_LAUNCH_FAILED_APP_NOT_FOUND));
  rb_define_const(mBe, "LAUNCH_FAILED_APP_IN_TRASH", LONG2NUM(B_LAUNCH_FAILED_APP_IN_TRASH));
  rb_define_const(mBe, "LAUNCH_FAILED_NO_PREFERRED_APP", LONG2NUM(B_LAUNCH_FAILED_NO_PREFERRED_APP));
  rb_define_const(mBe, "LAUNCH_FAILED_FILES_APP_NOT_FOUND", LONG2NUM(B_LAUNCH_FAILED_FILES_APP_NOT_FOUND));
  rb_define_const(mBe, "BAD_MIME_SNIFFER_RULE", LONG2NUM(B_BAD_MIME_SNIFFER_RULE));

  rb_define_const(mBe, "FILE_ERROR", LONG2NUM(B_FILE_ERROR));
  rb_define_const(mBe, "FILE_EXISTS", LONG2NUM(B_FILE_EXISTS));
  rb_define_const(mBe, "ENTRY_NOT_FOUND", LONG2NUM(B_ENTRY_NOT_FOUND));
  rb_define_const(mBe, "NAME_TOO_LONG", LONG2NUM(B_NAME_TOO_LONG));
  rb_define_const(mBe, "NOT_A_DIRECTORY", LONG2NUM(B_NOT_A_DIRECTORY));
  rb_define_const(mBe, "DIRECTORY_NOT_EMPTY", LONG2NUM(B_DIRECTORY_NOT_EMPTY));
  rb_define_const(mBe, "DEVICE_FULL", LONG2NUM(B_DEVICE_FULL));
  rb_define_const(mBe, "READ_ONLY_DEVICE", LONG2NUM(B_READ_ONLY_DEVICE));
  rb_define_const(mBe, "IS_A_DIRECTORY", LONG2NUM(B_IS_A_DIRECTORY));
  rb_define_const(mBe, "NO_MORE_FDS", LONG2NUM(B_NO_MORE_FDS));
  rb_define_const(mBe, "CROSS_DEVICE_LINK", LONG2NUM(B_CROSS_DEVICE_LINK));
  rb_define_const(mBe, "LINK_LIMIT", LONG2NUM(B_LINK_LIMIT));
  rb_define_const(mBe, "BUSTED_PIPE", LONG2NUM(B_BUSTED_PIPE));
  rb_define_const(mBe, "UNSUPPORTED", LONG2NUM(B_UNSUPPORTED));
  rb_define_const(mBe, "PARTITION_TOO_SMALL", LONG2NUM(B_PARTITION_TOO_SMALL));

  rb_define_const(mBe, "E2BIG", LONG2NUM(E2BIG));
  rb_define_const(mBe, "ECHILD", LONG2NUM(ECHILD));
  rb_define_const(mBe, "EDEADLK", LONG2NUM(EDEADLK));
  rb_define_const(mBe, "EFBIG", LONG2NUM(EFBIG));
  rb_define_const(mBe, "EMLINK", LONG2NUM(EMLINK));
  rb_define_const(mBe, "ENFILE", LONG2NUM(ENFILE));
  rb_define_const(mBe, "ENODEV", LONG2NUM(ENODEV));
  rb_define_const(mBe, "ENOLCK", LONG2NUM(ENOLCK));
  rb_define_const(mBe, "ENOSYS", LONG2NUM(ENOSYS));
  rb_define_const(mBe, "ENOTTY", LONG2NUM(ENOTTY));
  rb_define_const(mBe, "ENXIO", LONG2NUM(ENXIO));
  rb_define_const(mBe, "ESPIPE", LONG2NUM(ESPIPE));
  rb_define_const(mBe, "ESRCH", LONG2NUM(ESRCH));
  rb_define_const(mBe, "EFPOS", LONG2NUM(EFPOS));
  rb_define_const(mBe, "ESIGPARM", LONG2NUM(ESIGPARM));
  rb_define_const(mBe, "EDOM", LONG2NUM(EDOM));
  rb_define_const(mBe, "ERANGE", LONG2NUM(ERANGE));
  rb_define_const(mBe, "EPROTOTYPE", LONG2NUM(EPROTOTYPE));
  rb_define_const(mBe, "EPROTONOSUPPORT", LONG2NUM(EPROTONOSUPPORT));
  rb_define_const(mBe, "EPFNOSUPPORT", LONG2NUM(EPFNOSUPPORT));
  rb_define_const(mBe, "EAFNOSUPPORT", LONG2NUM(EAFNOSUPPORT));
  rb_define_const(mBe, "EADDRINUSE", LONG2NUM(EADDRINUSE));
  rb_define_const(mBe, "EADDRNOTAVAIL", LONG2NUM(EADDRNOTAVAIL));
  rb_define_const(mBe, "ENETDOWN", LONG2NUM(ENETDOWN));
  rb_define_const(mBe, "ENETUNREACH", LONG2NUM(ENETUNREACH));
  rb_define_const(mBe, "ENETRESET", LONG2NUM(ENETRESET));
  rb_define_const(mBe, "ECONNABORTED", LONG2NUM(ECONNABORTED));
  rb_define_const(mBe, "ECONNRESET", LONG2NUM(ECONNRESET));
  rb_define_const(mBe, "EISCONN", LONG2NUM(EISCONN));
  rb_define_const(mBe, "ENOTCONN", LONG2NUM(ENOTCONN));
  rb_define_const(mBe, "ESHUTDOWN", LONG2NUM(ESHUTDOWN));
  rb_define_const(mBe, "ECONNREFUSED", LONG2NUM(ECONNREFUSED));
  rb_define_const(mBe, "EHOSTUNREACH", LONG2NUM(EHOSTUNREACH));
  rb_define_const(mBe, "ENOPROTOOPT", LONG2NUM(ENOPROTOOPT));
  rb_define_const(mBe, "ENOBUFS", LONG2NUM(ENOBUFS));
  rb_define_const(mBe, "EINPROGRESS", LONG2NUM(EINPROGRESS));
  rb_define_const(mBe, "EALREADY", LONG2NUM(EALREADY));
  rb_define_const(mBe, "EILSEQ", LONG2NUM(EILSEQ));
  rb_define_const(mBe, "ENOMSG", LONG2NUM(ENOMSG));
  rb_define_const(mBe, "ESTALE", LONG2NUM(ESTALE));
  rb_define_const(mBe, "EOVERFLOW", LONG2NUM(EOVERFLOW));
  rb_define_const(mBe, "EMSGSIZE", LONG2NUM(EMSGSIZE));
  rb_define_const(mBe, "EOPNOTSUPP", LONG2NUM(EOPNOTSUPP));
  rb_define_const(mBe, "ENOTSOCK", LONG2NUM(ENOTSOCK));

  rb_define_const(mBe, "ENOMEM", LONG2NUM(ENOMEM));
  rb_define_const(mBe, "EACCES", LONG2NUM(EACCES));
  rb_define_const(mBe, "EINTR", LONG2NUM(EINTR));
  rb_define_const(mBe, "EIO", LONG2NUM(EIO));
  rb_define_const(mBe, "EBUSY", LONG2NUM(EBUSY));
  rb_define_const(mBe, "EFAULT", LONG2NUM(EFAULT));
  rb_define_const(mBe, "ETIMEDOUT", LONG2NUM(ETIMEDOUT));
  rb_define_const(mBe, "EAGAIN", LONG2NUM(EAGAIN));
  rb_define_const(mBe, "EWOULDBLOCK", LONG2NUM(EWOULDBLOCK));
  rb_define_const(mBe, "EBADF", LONG2NUM(EBADF));
  rb_define_const(mBe, "EEXIST", LONG2NUM(EEXIST));
  rb_define_const(mBe, "EINVAL", LONG2NUM(EINVAL));
  rb_define_const(mBe, "ENAMETOOLONG", LONG2NUM(ENAMETOOLONG));
  rb_define_const(mBe, "ENOENT", LONG2NUM(ENOENT));
  rb_define_const(mBe, "EPERM", LONG2NUM(EPERM));
  rb_define_const(mBe, "ENOTDIR", LONG2NUM(ENOTDIR));
  rb_define_const(mBe, "EISDIR", LONG2NUM(EISDIR));
  rb_define_const(mBe, "ENOTEMPTY", LONG2NUM(ENOTEMPTY));
  rb_define_const(mBe, "ENOSPC", LONG2NUM(ENOSPC));
  rb_define_const(mBe, "EROFS", LONG2NUM(EROFS));
  rb_define_const(mBe, "EMFILE", LONG2NUM(EMFILE));
  rb_define_const(mBe, "EXDEV", LONG2NUM(EXDEV));
  rb_define_const(mBe, "ELOOP", LONG2NUM(ELOOP));
  rb_define_const(mBe, "ENOEXEC", LONG2NUM(ENOEXEC));
  rb_define_const(mBe, "EPIPE", LONG2NUM(EPIPE));

  rb_define_const(mBe, "STREAM_NOT_FOUND", LONG2NUM(B_STREAM_NOT_FOUND));
  rb_define_const(mBe, "SERVER_NOT_FOUND", LONG2NUM(B_SERVER_NOT_FOUND));
  rb_define_const(mBe, "RESOURCE_NOT_FOUND", LONG2NUM(B_RESOURCE_NOT_FOUND));
  rb_define_const(mBe, "RESOURCE_UNAVAILABLE", LONG2NUM(B_RESOURCE_UNAVAILABLE));
  rb_define_const(mBe, "BAD_SUBSCRIBER", LONG2NUM(B_BAD_SUBSCRIBER));
  rb_define_const(mBe, "SUBSCRIBER_NOT_ENTERED", LONG2NUM(B_SUBSCRIBER_NOT_ENTERED));
  rb_define_const(mBe, "BUFFER_NOT_AVAILABLE", LONG2NUM(B_BUFFER_NOT_AVAILABLE));
  rb_define_const(mBe, "LAST_BUFFER_ERROR", LONG2NUM(B_LAST_BUFFER_ERROR));

  rb_define_const(mBe, "MAIL_NO_DAEMON", LONG2NUM(B_MAIL_NO_DAEMON));
  rb_define_const(mBe, "MAIL_UNKNOWN_USER", LONG2NUM(B_MAIL_UNKNOWN_USER));
  rb_define_const(mBe, "MAIL_WRONG_PASSWORD", LONG2NUM(B_MAIL_WRONG_PASSWORD));
  rb_define_const(mBe, "MAIL_UNKNOWN_HOST", LONG2NUM(B_MAIL_UNKNOWN_HOST));
  rb_define_const(mBe, "MAIL_ACCESS_ERROR", LONG2NUM(B_MAIL_ACCESS_ERROR));
  rb_define_const(mBe, "MAIL_UNKNOWN_FIELD", LONG2NUM(B_MAIL_UNKNOWN_FIELD));
  rb_define_const(mBe, "MAIL_NO_RECIPIENT", LONG2NUM(B_MAIL_NO_RECIPIENT));
  rb_define_const(mBe, "MAIL_INVALID_MAIL", LONG2NUM(B_MAIL_INVALID_MAIL));

  rb_define_const(mBe, "NO_PRINT_SERVER", LONG2NUM(B_NO_PRINT_SERVER));

  rb_define_const(mBe, "DEV_INVALID_IOCTL", LONG2NUM(B_DEV_INVALID_IOCTL));
  rb_define_const(mBe, "DEV_NO_MEMORY", LONG2NUM(B_DEV_NO_MEMORY));
  rb_define_const(mBe, "DEV_BAD_DRIVE_NUM", LONG2NUM(B_DEV_BAD_DRIVE_NUM));
  rb_define_const(mBe, "DEV_NO_MEDIA", LONG2NUM(B_DEV_NO_MEDIA));
  rb_define_const(mBe, "DEV_UNREADABLE", LONG2NUM(B_DEV_UNREADABLE));
  rb_define_const(mBe, "DEV_FORMAT_ERROR", LONG2NUM(B_DEV_FORMAT_ERROR));
  rb_define_const(mBe, "DEV_TIMEOUT", LONG2NUM(B_DEV_TIMEOUT));
  rb_define_const(mBe, "DEV_RECALIBRATE_ERROR", LONG2NUM(B_DEV_RECALIBRATE_ERROR));
  rb_define_const(mBe, "DEV_SEEK_ERROR", LONG2NUM(B_DEV_SEEK_ERROR));
  rb_define_const(mBe, "DEV_ID_ERROR", LONG2NUM(B_DEV_ID_ERROR));
  rb_define_const(mBe, "DEV_READ_ERROR", LONG2NUM(B_DEV_READ_ERROR));
  rb_define_const(mBe, "DEV_WRITE_ERROR", LONG2NUM(B_DEV_WRITE_ERROR));
  rb_define_const(mBe, "DEV_NOT_READY", LONG2NUM(B_DEV_NOT_READY));
  rb_define_const(mBe, "DEV_MEDIA_CHANGED", LONG2NUM(B_DEV_MEDIA_CHANGED));
  rb_define_const(mBe, "DEV_MEDIA_CHANGE_REQUESTED", LONG2NUM(B_DEV_MEDIA_CHANGE_REQUESTED));
  rb_define_const(mBe, "DEV_RESOURCE_CONFLICT", LONG2NUM(B_DEV_RESOURCE_CONFLICT));
  rb_define_const(mBe, "DEV_CONFIGURATION_ERROR", LONG2NUM(B_DEV_CONFIGURATION_ERROR));
  rb_define_const(mBe, "DEV_DISABLED_BY_USER", LONG2NUM(B_DEV_DISABLED_BY_USER));
  rb_define_const(mBe, "DEV_DOOR_OPEN", LONG2NUM(B_DEV_DOOR_OPEN));
}
