/*
 *  Handler.cc
 */

#include <Messenger.h>
#include "rbHandler.h"
#include "my_be.h"


VALUE cHandler;
ID id_MessageReceived;


void
delete_handler(BHandler *arg0)
{
  delete arg0;
}


static VALUE
rb_handler_s_new(int argc, VALUE *argv, VALUE klass)
{
  VALUE vresult = my_class_s_new(argc, argv, klass, delete_handler);
  return vresult;
}


static VALUE
rb_handler_initialize(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1;
  Handler *result;

  Data_To_Null(self, BHandler);

  switch (rb_scan_args(argc, argv, "01", &varg1)) {
    case 0:
      result = new Handler();
      break;

    case 1:
      if (rb_obj_is_kind_of(varg1, cMessage)) {
        BMessage *arg1;

        Get_BInstance(Message, varg1, arg1);
        result = new Handler(arg1);
      }
      else {
        char *arg1;

        arg1 = STR2CSTR(varg1);
        result = new Handler(arg1);
      }
      break;

    default:
      rb_bug("rb_handler_initialize");
  }
  result->myself = self;
  DATA_PTR(self) = result;
  return self;
}


static VALUE
rb_handler_MessageReceived(VALUE self, VALUE varg1)
{
  BHandler *arg0;
  BMessage *arg1;

  Get_BInstance(Handler, self, arg0);
  Get_BInstance(Message, varg1, arg1);
  arg0->BHandler::MessageReceived(arg1);
  return Qnil;
}


static VALUE
rb_handler_Looper(VALUE self)
{
  BHandler *arg0;
  BLooper *result;
  VALUE vresult;

  Get_BInstance(Handler, self, arg0);
  result = arg0->Looper();
  vresult = ((result) ? Data_Wrap_Struct(cLooper, 0, 0, result):Qnil);
  return vresult;
}


static VALUE
rb_handler_SetName(VALUE self, VALUE varg1)
{
  BHandler *arg0;
  char *arg1;

  Get_BInstance(Handler, self, arg0);
  arg1 = STR2CSTR(varg1);
  arg0->SetName(arg1);
  return Qnil;
}


static VALUE
rb_handler_Name(VALUE self)
{
  BHandler *arg0;
  const char *result;
  VALUE vresult;

  Get_BInstance(Handler, self, arg0);
  result = arg0->Name();
  vresult = rb_str_new2(result);
  return vresult;
}


static VALUE
rb_handler_SetNextHandler(VALUE self, VALUE varg1)
{
  BHandler *arg0, *arg1;

  Get_BInstance(Handler, self, arg0);
  Get_BInstance(Handler, varg1, arg1);
  arg0->BHandler::SetNextHandler(arg1);
  return Qnil;
}


static VALUE
rb_handler_NextHandler(VALUE self)
{
  BHandler *arg0, *result;
  VALUE vresult;

  Get_BInstance(Handler, self, arg0);
  result = arg0->NextHandler();
  vresult = ((result) ? Data_Wrap_Struct(cHandler, 0, 0, result):Qnil);
  return vresult;
}


static VALUE
rb_handler_AddFilter(VALUE self, VALUE varg1)
{
  BHandler *arg0;
  BMessageFilter *arg1;

  Get_BInstance(Handler, self, arg0);
  Get_BInstance(MessageFilter, varg1, arg1);
  arg0->BHandler::AddFilter(arg1);
  return Qnil;
}


static VALUE
rb_handler_RemoveFilter(VALUE self, VALUE varg1)
{
  BHandler *arg0;
  BMessageFilter *arg1;
  bool result;
  VALUE vresult;

  Get_BInstance(Handler, self, arg0);
  Get_BInstance(MessageFilter, varg1, arg1);
  result = arg0->BHandler::RemoveFilter(arg1);
  vresult = (result ? Qtrue:Qfalse);
  return vresult;
}


static VALUE
rb_handler_SetFilterList(VALUE self, VALUE varg1)
{
  BHandler *arg0;
  BList *arg1;

  Get_BInstance(Handler, self, arg0);
  Get_BInstance(List, varg1, arg1);
  arg0->BHandler::SetFilterList(arg1);
  return Qnil;
}


static VALUE
rb_handler_FilterList(VALUE self)
{
  BHandler *arg0;
  BList *result;
  VALUE vresult;

  Get_BInstance(Handler, self, arg0);
  result = arg0->FilterList();

  vresult = my_filter_list2rarry(*result);
  return vresult;
}


static VALUE
rb_handler_LockLooper(VALUE self)
{
  BHandler *arg0;
  bool result;
  VALUE vresult;

  Get_BInstance(Handler, self, arg0);
  result = arg0->LockLooper();
  vresult = (result ? Qtrue:Qfalse);
  return vresult;
}


static VALUE
rb_handler_LockLooperWithTimeout(VALUE self, VALUE varg1)
{
  BHandler *arg0;
  bigtime_t arg1;
  status_t result;

  Get_BInstance(Handler, self, arg0);
  arg1 = NUM2LLONG(varg1);
  result = arg0->LockLooperWithTimeout(arg1);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_handler_UnlockLooper(VALUE self)
{
  BHandler *arg0;

  Get_BInstance(Handler, self, arg0);
  arg0->UnlockLooper();
  return Qnil;
}


static VALUE
rb_handler_ResolveSpecifier(VALUE self, VALUE varg1, VALUE varg2, VALUE varg3, VALUE varg4, VALUE varg5)
{
  BHandler *arg0;
  BMessage *arg1,  *arg3;
  int32 arg2, arg4;
  char *arg5;
  BHandler *result;
  VALUE vresult;

  Get_BInstance(Handler, self, arg0);
  Get_BInstance(Message, varg1, arg1);
  arg2 = NUM2LONG(varg2);
  Get_BInstance(Message, varg3, arg3);
  arg4 = NUM2LONG(varg4);
  arg5 = STR2CSTR(varg5);

  result = arg0->BHandler::ResolveSpecifier(arg1, arg2, arg3, arg4, arg5);
  vresult = ((result) ? Data_Wrap_Struct(cHandler, 0, 0, result):Qnil);
  return vresult;
}


static VALUE
rb_handler_GetSupportedSuites(VALUE self, VALUE varg1)
{
  BHandler *arg0;
  BMessage *arg1;
  status_t result;

  Get_BInstance(Handler, self, arg0);
  Get_BInstance(Message, varg1, arg1);
  result = arg0->BHandler::GetSupportedSuites(arg1);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_handler_StartWatching(VALUE self, VALUE varg1, VALUE varg2)
{
  BHandler *arg0;
  status_t result;

  Get_BInstance(Handler, self, arg0);
  if (rb_obj_is_kind_of(varg1, cMessenger)) {
    BMessenger *arg1;
    uint32 arg2;

    Get_BInstance(Messenger, varg1, arg1);
    arg2 = NUM2ULONG(varg2);
    result = arg0->StartWatching(*arg1, arg2);
  }
  else {
    BHandler *arg1;
    uint32 arg2;

    Get_BInstance(Handler, varg1, arg1);
    arg2 = NUM2ULONG(varg2);
    result = arg0->StartWatching(arg1, arg2);
  }
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_handler_StartWatchingAll(VALUE self, VALUE varg1)
{
  BHandler *arg0;
  status_t result;

  Get_BInstance(Handler, self, arg0);
  if (rb_obj_is_kind_of(varg1, cMessenger)) {
    BMessenger *arg1;

    Get_BInstance(Messenger, varg1, arg1);
    result = arg0->StartWatchingAll(*arg1);
  }
  else {
    BHandler *arg1;

    Get_BInstance(Handler, varg1, arg1);
    result = arg0->StartWatchingAll(arg1);
  }
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_handler_StopWatching(VALUE self, VALUE varg1, VALUE varg2)
{
  BHandler *arg0;
  status_t result;

  Get_BInstance(Handler, self, arg0);
  if (rb_obj_is_kind_of(varg1, cMessenger)) {
    BMessenger *arg1;
    uint32 arg2;

    Get_BInstance(Messenger, varg1, arg1);
    arg2 = NUM2ULONG(varg2);
    result = arg0->StopWatching(*arg1, arg2);
  }
  else {
    BHandler *arg1;
    uint32 arg2;

    Get_BInstance(Handler, varg1, arg1);
    arg2 = NUM2ULONG(varg2);
    result = arg0->StopWatching(arg1, arg2);
  }
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_handler_StopWatchingAll(VALUE self, VALUE varg1)
{
  BHandler *arg0;
  status_t result;

  Get_BInstance(Handler, self, arg0);
  if (rb_obj_is_kind_of(varg1, cMessenger)) {
    BMessenger *arg1;

    Get_BInstance(Messenger, varg1, arg1);
    result = arg0->StartWatchingAll(*arg1);
  }
  else {
    BHandler *arg1;

    Get_BInstance(Handler, varg1, arg1);
    result = arg0->StartWatchingAll(arg1);
  }
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_handler_SendNotices(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1, varg2;
  BHandler *arg0;

  Get_BInstance(Handler, self, arg0);
  switch (rb_scan_args(argc, argv, "11", &varg1, &varg2)) {
    case 1:
      {
        uint32 arg1;

        arg1 = NUM2ULONG(varg1);
        arg0->BHandler::SendNotices(arg1);
      }
      break;

    case 2:
      {
        uint32 arg1;
        BMessage *arg2;

        arg1 = NUM2ULONG(varg1);
        Get_BInstance(Message, varg2, arg2);

        arg0->BHandler::SendNotices(arg1, arg2);
      }
      break;

    default:
      rb_bug("rb_handler_SendNotices");
  }
  return Qnil;
}


static VALUE
rb_handler_IsWatched(VALUE self)
{
  BHandler *arg0;
  bool result;
  VALUE vresult;

  Get_BInstance(Handler, self, arg0);
  result = arg0->IsWatched();
  vresult = (result ? Qtrue:Qfalse);
  return vresult;
}


void
Init_Handler()
{
  id_MessageReceived = rb_intern("MessageReceived");

  cHandler = rb_define_class_under(mBe, "Handler", cArchivable);
  rb_define_singleton_method(cHandler, "new", RUBY_METHOD_FUNC(rb_handler_s_new), -1);
  rb_define_method(cHandler, "initialize", RUBY_METHOD_FUNC(rb_handler_initialize), -1);

  rb_define_method(cHandler, "MessageReceived", RUBY_METHOD_FUNC(rb_handler_MessageReceived), 1);
  rb_define_method(cHandler, "Looper", RUBY_METHOD_FUNC(rb_handler_Looper), 0);
  rb_define_method(cHandler, "SetName", RUBY_METHOD_FUNC(rb_handler_SetName), 1);
  rb_define_method(cHandler, "Name", RUBY_METHOD_FUNC(rb_handler_Name), 0);
  rb_define_method(cHandler, "SetNextHandler", RUBY_METHOD_FUNC(rb_handler_SetNextHandler), 1);
  rb_define_method(cHandler, "NextHandler", RUBY_METHOD_FUNC(rb_handler_NextHandler), 0);

  rb_define_method(cHandler, "AddFilter", RUBY_METHOD_FUNC(rb_handler_AddFilter), 1);
  rb_define_method(cHandler, "RemoveFilter", RUBY_METHOD_FUNC(rb_handler_RemoveFilter), 1);
  rb_define_method(cHandler, "SetFilterList", RUBY_METHOD_FUNC(rb_handler_SetFilterList), 1);
  rb_define_method(cHandler, "FilterList", RUBY_METHOD_FUNC(rb_handler_FilterList), 0);

  rb_define_method(cHandler, "LockLooper", RUBY_METHOD_FUNC(rb_handler_LockLooper), 0);
  rb_define_method(cHandler, "LockLooperWithTimeout", RUBY_METHOD_FUNC(rb_handler_LockLooperWithTimeout), 1);
  rb_define_method(cHandler, "UnlockLooper", RUBY_METHOD_FUNC(rb_handler_UnlockLooper), 0);

  rb_define_method(cHandler, "ResolveSpecifier", RUBY_METHOD_FUNC(rb_handler_ResolveSpecifier), 5);
  rb_define_method(cHandler, "GetSupportedSuites", RUBY_METHOD_FUNC(rb_handler_GetSupportedSuites), 1);

  rb_define_method(cHandler, "StartWatching", RUBY_METHOD_FUNC(rb_handler_StartWatching), 2);
  rb_define_method(cHandler, "StartWatchingAll", RUBY_METHOD_FUNC(rb_handler_StartWatchingAll), 1);
  rb_define_method(cHandler, "StopWatching", RUBY_METHOD_FUNC(rb_handler_StopWatching), 2);
  rb_define_method(cHandler, "StopWatchingAll", RUBY_METHOD_FUNC(rb_handler_StopWatchingAll), 1);

  rb_define_method(cHandler, "SendNotices", RUBY_METHOD_FUNC(rb_handler_SendNotices), -1);
  rb_define_method(cHandler, "IsWatched", RUBY_METHOD_FUNC(rb_handler_IsWatched), 0);

  rb_define_const(mBe, "OBSERVE_WHAT_CHANGE", rb_str_new2(B_OBSERVE_WHAT_CHANGE));
  rb_define_const(mBe, "OBSERVE_ORIGINAL_WHAT", rb_str_new2(B_OBSERVE_ORIGINAL_WHAT));
  rb_define_const(mBe, "OBSERVER_OBSERVE_ALL", ULONG2NUM(B_OBSERVER_OBSERVE_ALL));
}
