/*
 *  InterfaceDefs.cc
 */

#include "my_be.h"
#include <InterfaceDefs.h>
#include "rbRect.h"


static VALUE
rb_get_deskbar_frame(VALUE self)
{
  BRect arg1, *copy;
  status_t result;
  VALUE vresult;

  result = get_deskbar_frame(&arg1);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  copy = new Rect(arg1);
  vresult = Data_Wrap_Struct(cRect, 0, delete_rect, copy);
  return vresult;
}


static VALUE
rb_set_screen_space(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1, varg2, varg3;
  status_t result;

  switch (rb_scan_args(argc, argv, "21", &varg1, &varg2, &varg3)) {
    case 2:
      {
        int32 arg1;
        uint32 arg2;

        arg1 = LONG2NUM(varg1);
        arg2 = ULONG2NUM(varg2);
        result = set_screen_space(arg1, arg2);
      }
      break;

    case 3:
      {
        int32 arg1;
        uint32 arg2;
        bool arg3;

        arg1 = LONG2NUM(varg1);
        arg2 = ULONG2NUM(varg2);
        arg3 = RTEST(varg3);
        result = set_screen_space(arg1, arg2, arg3);
      }
      break;

    default:
      rb_bug("rb_set_screen_space");
  }
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
my_scroll_bar_info2hash(scroll_bar_info &scl)
{
  VALUE vresult = rb_hash_new();
  rb_hash_aset(vresult, rb_str_new2("proportional"),  (scl.proportional ? Qtrue:Qfalse));
  rb_hash_aset(vresult, rb_str_new2("double_arrows"), (scl.double_arrows ? Qtrue:Qfalse));
  rb_hash_aset(vresult, rb_str_new2("knob"),          LONG2NUM(scl.knob));
  rb_hash_aset(vresult, rb_str_new2("min_knob_size"), LONG2NUM(scl.min_knob_size));
  return vresult;
}


static VALUE
rb_get_scroll_bar_info(VALUE self)
{
  scroll_bar_info arg1;
  status_t result;
  VALUE vresult;

  result = get_scroll_bar_info(&arg1);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  vresult = my_scroll_bar_info2hash(arg1);
  return vresult;
}


static scroll_bar_info
my_hash2scroll_bar_info(VALUE hash)
{
  scroll_bar_info scl;

  Check_Type(hash, T_HASH);

  scl.proportional  = RTEST(rb_hash_aref(hash, rb_str_new2("proportional")));
  scl.double_arrows = RTEST(rb_hash_aref(hash, rb_str_new2("double_arrows")));
  scl.knob          = NUM2LONG(rb_hash_aref(hash, rb_str_new2("knob")));
  scl.min_knob_size = NUM2LONG(rb_hash_aref(hash, rb_str_new2("min_knob_size")));

  return scl;
}


static VALUE
rb_set_scroll_bar_info(VALUE self, VALUE varg1)
{
  scroll_bar_info arg1;
  status_t result;

  arg1 = my_hash2scroll_bar_info(varg1);

  result = set_scroll_bar_info(&arg1);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_get_mouse_type(VALUE self)
{
  int32 arg1 = 0;
  status_t result;
  VALUE vresult;

  result = get_mouse_type(&arg1);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  vresult = LONG2NUM(arg1);
  return vresult;
}


static VALUE
rb_set_mouse_type(VALUE self, VALUE varg1)
{
  int32 arg1;
  status_t result;

  arg1 = NUM2LONG(varg1);
  result = set_mouse_type(arg1);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
my_mouse_map2hash(mouse_map &mous)
{
  VALUE vresult = rb_hash_new();
  rb_hash_aset(vresult, rb_str_new2("left"),   ULONG2NUM(mous.left));
  rb_hash_aset(vresult, rb_str_new2("right"),  ULONG2NUM(mous.right));
  rb_hash_aset(vresult, rb_str_new2("middle"), ULONG2NUM(mous.middle));
  return vresult;
}


static VALUE
rb_get_mouse_map(VALUE self)
{
  mouse_map arg1;
  status_t result;
  VALUE vresult;

  result = get_mouse_map(&arg1);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  vresult = my_mouse_map2hash(arg1);
  return vresult;
}


static mouse_map
my_hash2mouse_map(VALUE hash)
{
  mouse_map mous;

  Check_Type(hash, T_HASH);

  mous.left   = NUM2ULONG(rb_hash_aref(hash, rb_str_new2("left")));
  mous.right  = NUM2ULONG(rb_hash_aref(hash, rb_str_new2("right")));
  mous.middle = NUM2ULONG(rb_hash_aref(hash, rb_str_new2("middle")));

  return mous;
}


static VALUE
rb_set_mouse_map(VALUE self, VALUE varg1)
{
  mouse_map arg1;
  status_t result;

  arg1 = my_hash2mouse_map(varg1);
  result = set_mouse_map(&arg1);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_get_click_speed(VALUE self)
{
  bigtime_t arg1 = 0;
  status_t result;
  VALUE vresult;

  result = get_click_speed(&arg1);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  vresult = LLONG2NUM(arg1);
  return vresult;
}


static VALUE
rb_set_click_speed(VALUE self, VALUE varg1)
{
  bigtime_t arg1;
  status_t result;

  arg1 = NUM2LLONG(varg1);
  result = set_click_speed(arg1);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_get_mouse_speed(VALUE self)
{
  int32 arg1 = 0;
  status_t result;
  VALUE vresult;

  result = get_mouse_speed(&arg1);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  vresult = LONG2NUM(arg1);
  return vresult;
}


static VALUE
rb_set_mouse_speed(VALUE self, VALUE varg1)
{
  int32 arg1;
  status_t result;

  arg1 = NUM2LONG(varg1);
  result = set_mouse_speed(arg1);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_get_mouse_acceleration(VALUE self)
{
  int32 arg1 = 0;
  status_t result;
  VALUE vresult;

  result = get_mouse_acceleration(&arg1);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  vresult = LONG2NUM(arg1);
  return vresult;
}


static VALUE
rb_set_mouse_acceleration(VALUE self, VALUE varg1)
{
  int32 arg1;
  status_t result;

  arg1 = NUM2LONG(varg1);
  result = set_mouse_acceleration(arg1);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_get_key_repeat_rate(VALUE self)
{
  int32 arg1 = 0;
  status_t result;
  VALUE vresult;

  result = get_key_repeat_rate(&arg1);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  vresult = LONG2NUM(arg1);
  return vresult;
}


static VALUE
rb_set_key_repeat_rate(VALUE self, VALUE varg1)
{
  int32 arg1;
  status_t result;

  arg1 = NUM2LONG(varg1);
  result = set_key_repeat_rate(arg1);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_get_key_repeat_delay(VALUE self)
{
  bigtime_t arg1 = 0;
  status_t result;
  VALUE vresult;

  result = get_key_repeat_delay(&arg1);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  vresult = LLONG2NUM(arg1);
  return vresult;
}


static VALUE
rb_set_key_repeat_delay(VALUE self, VALUE varg1)
{
  bigtime_t arg1;
  status_t result;

  arg1 = NUM2LLONG(varg1);
  result = set_key_repeat_delay(arg1);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_modifiers(VALUE self)
{
  uint32 result;
  VALUE vresult;

  result = modifiers();
  vresult = ULONG2NUM(result);
  return vresult;
}


static VALUE
rb_get_keyboard_id(VALUE self)
{
  uint16 arg1 = 0;
  status_t result;
  VALUE vresult;

  result = get_keyboard_id(&arg1);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  vresult = ULONG2NUM(arg1);
  return vresult;
}


static VALUE
rb_set_modifier_key(VALUE self, VALUE varg1, VALUE varg2)
{
  uint32 arg1, arg2;

  arg1 = NUM2ULONG(varg1);
  arg2 = NUM2ULONG(varg2);
  set_modifier_key(arg1, arg2);
  return Qnil;
}


static VALUE
rb_set_keyboard_locks(VALUE self, VALUE varg1)
{
  uint32 arg1;

  arg1 = NUM2ULONG(varg1);
  set_keyboard_locks(arg1);
  return Qnil;
}


static VALUE
rb_keyboard_navigation_color(VALUE self)
{
  rgb_color result;
  VALUE vresult;

  result = keyboard_navigation_color();
  vresult = my_rgb_color2hash(result);
  return vresult;
}


static VALUE
rb_count_workspaces(VALUE self)
{
  int32 result;
  VALUE vresult;

  result = count_workspaces();
  vresult = LONG2NUM(result);
  return vresult;
}


static VALUE
rb_set_workspace_count(VALUE self, VALUE varg1)
{
  int32 arg1;

  arg1 = NUM2LONG(varg1);
  set_workspace_count(arg1);
  return Qnil;
}


static VALUE
rb_current_workspace(VALUE self)
{
  int32 result;
  VALUE vresult;

  result = current_workspace();
  vresult = LONG2NUM(result);
  return vresult;
}


static VALUE
rb_activate_workspace(VALUE self, VALUE varg1)
{
  int32 arg1;

  arg1 = NUM2LONG(varg1);
  activate_workspace(arg1);
  return Qnil;
}


static VALUE
rb_idle_time(VALUE self)
{
  bigtime_t result;
  VALUE vresult;

  result = idle_time();
  vresult = LLONG2NUM(result);
  return vresult;
}


static VALUE
rb_run_select_printer_panel(VALUE self)
{
  run_select_printer_panel();
  return Qnil;
}


static VALUE
rb_run_add_printer_panel(VALUE self)
{
  run_add_printer_panel();
  return Qnil;
}


static VALUE
rb_run_be_about(VALUE self)
{
  run_be_about();
  return Qnil;
}


static VALUE
rb_set_focus_follows_mouse(VALUE self, VALUE varg1)
{
  bool arg1;

  arg1 = RTEST(varg1);
  set_focus_follows_mouse(arg1);
  return Qnil;
}


static VALUE
rb_focus_follows_mouse(VALUE self)
{
  bool result;
  VALUE vresult;

  result = focus_follows_mouse();
  vresult = (result ? Qtrue:Qfalse);
  return vresult;
}


static VALUE
rb_set_mouse_mode(VALUE self, VALUE varg1)
{
  mode_mouse arg1;

  arg1 = (mode_mouse)NUM2LONG(varg1);
  set_mouse_mode(arg1);
  return Qnil;
}


static VALUE
rb_mouse_mode(VALUE self)
{
  mode_mouse result;
  VALUE vresult;

  result = mouse_mode();
  vresult = LONG2NUM(result);
  return vresult;
}


static VALUE
rb_ui_color(VALUE self, VALUE varg1)
{
  color_which arg1;
  rgb_color result;
  VALUE vresult;

  arg1 = (color_which)NUM2LONG(varg1);
  result = ui_color(arg1);
  vresult = my_rgb_color2hash(result);
  return vresult;
}


static VALUE
rb_tint_color(VALUE self, VALUE varg1, VALUE varg2)
{
  rgb_color arg1, result;
  float arg2;
  VALUE vresult;

  arg1 = my_hash2rgb_color(varg1);
  arg2 = NUM2DBL(varg2);
  result = tint_color(arg1, arg2);
  vresult = my_rgb_color2hash(result);
  return vresult;
}


void
Init_InterfaceDefs()
{
  rb_define_module_function(mBe, "get_deskbar_frame", RUBY_METHOD_FUNC(rb_get_deskbar_frame), 0);
  rb_define_module_function(mBe, "set_screen_space", RUBY_METHOD_FUNC(rb_set_screen_space), -1);
  rb_define_module_function(mBe, "get_scroll_bar_info", RUBY_METHOD_FUNC(rb_get_scroll_bar_info), 0);
  rb_define_module_function(mBe, "set_scroll_bar_info", RUBY_METHOD_FUNC(rb_set_scroll_bar_info), 1);

  rb_define_module_function(mBe, "get_mouse_type", RUBY_METHOD_FUNC(rb_get_mouse_type), 0);
  rb_define_module_function(mBe, "set_mouse_type", RUBY_METHOD_FUNC(rb_set_mouse_type), 1);
  rb_define_module_function(mBe, "get_mouse_map", RUBY_METHOD_FUNC(rb_get_mouse_map), 0);
  rb_define_module_function(mBe, "set_mouse_map", RUBY_METHOD_FUNC(rb_set_mouse_map), 1);
  rb_define_module_function(mBe, "get_click_speed", RUBY_METHOD_FUNC(rb_get_click_speed), 0);
  rb_define_module_function(mBe, "set_click_speed", RUBY_METHOD_FUNC(rb_set_click_speed), 1);
  rb_define_module_function(mBe, "get_mouse_speed", RUBY_METHOD_FUNC(rb_get_mouse_speed), 0);
  rb_define_module_function(mBe, "set_mouse_speed", RUBY_METHOD_FUNC(rb_set_mouse_speed), 1);
  rb_define_module_function(mBe, "get_mouse_acceleration", RUBY_METHOD_FUNC(rb_get_mouse_acceleration), 0);
  rb_define_module_function(mBe, "set_mouse_acceleration", RUBY_METHOD_FUNC(rb_set_mouse_acceleration), 1);

  rb_define_module_function(mBe, "get_key_repeat_rate", RUBY_METHOD_FUNC(rb_get_key_repeat_rate), 0);
  rb_define_module_function(mBe, "set_key_repeat_rate", RUBY_METHOD_FUNC(rb_set_key_repeat_rate), 1);
  rb_define_module_function(mBe, "get_key_repeat_delay", RUBY_METHOD_FUNC(rb_get_key_repeat_delay), 0);
  rb_define_module_function(mBe, "set_key_repeat_delay", RUBY_METHOD_FUNC(rb_set_key_repeat_delay), 1);

  rb_define_module_function(mBe, "modifiers", RUBY_METHOD_FUNC(rb_modifiers), 0);
  rb_define_module_function(mBe, "get_keyboard_id", RUBY_METHOD_FUNC(rb_get_keyboard_id), 0);
  rb_define_module_function(mBe, "set_modifier_key", RUBY_METHOD_FUNC(rb_set_modifier_key), 2);
  rb_define_module_function(mBe, "set_keyboard_locks", RUBY_METHOD_FUNC(rb_set_keyboard_locks), 1);

  rb_define_module_function(mBe, "keyboard_navigation_color", RUBY_METHOD_FUNC(rb_keyboard_navigation_color), 0);

  rb_define_module_function(mBe, "count_workspaces", RUBY_METHOD_FUNC(rb_count_workspaces), 0);
  rb_define_module_function(mBe, "set_workspace_count", RUBY_METHOD_FUNC(rb_set_workspace_count), 1);
  rb_define_module_function(mBe, "current_workspace", RUBY_METHOD_FUNC(rb_current_workspace), 0);
  rb_define_module_function(mBe, "activate_workspace", RUBY_METHOD_FUNC(rb_activate_workspace), 1);
  rb_define_module_function(mBe, "idle_time", RUBY_METHOD_FUNC(rb_idle_time), 0);
  rb_define_module_function(mBe, "run_select_printer_panel", RUBY_METHOD_FUNC(rb_run_select_printer_panel), 0);
  rb_define_module_function(mBe, "run_add_printer_panel", RUBY_METHOD_FUNC(rb_run_add_printer_panel), 0);
  rb_define_module_function(mBe, "run_be_about", RUBY_METHOD_FUNC(rb_run_be_about), 0);
  rb_define_module_function(mBe, "set_focus_follows_mouse", RUBY_METHOD_FUNC(rb_set_focus_follows_mouse), 1);
  rb_define_module_function(mBe, "focus_follows_mouse", RUBY_METHOD_FUNC(rb_focus_follows_mouse), 0);

  rb_define_module_function(mBe, "set_mouse_mode", RUBY_METHOD_FUNC(rb_set_mouse_mode), 1);
  rb_define_module_function(mBe, "mouse_mode", RUBY_METHOD_FUNC(rb_mouse_mode), 0);

  rb_define_module_function(mBe, "ui_color", RUBY_METHOD_FUNC(rb_ui_color), 1);
  rb_define_module_function(mBe, "tint_color", RUBY_METHOD_FUNC(rb_tint_color), 2);

  rb_define_const(mBe, "UTF8_ELLIPSIS", rb_str_new2(B_UTF8_ELLIPSIS));
  rb_define_const(mBe, "UTF8_OPEN_QUOTE", rb_str_new2(B_UTF8_OPEN_QUOTE));
  rb_define_const(mBe, "UTF8_CLOSE_QUOTE", rb_str_new2(B_UTF8_CLOSE_QUOTE));
  rb_define_const(mBe, "UTF8_COPYRIGHT", rb_str_new2(B_UTF8_COPYRIGHT));
  rb_define_const(mBe, "UTF8_REGISTERED", rb_str_new2(B_UTF8_REGISTERED));
  rb_define_const(mBe, "UTF8_TRADEMARK", rb_str_new2(B_UTF8_TRADEMARK));
  rb_define_const(mBe, "UTF8_SMILING_FACE", rb_str_new2(B_UTF8_SMILING_FACE));
  rb_define_const(mBe, "UTF8_HIROSHI", rb_str_new2(B_UTF8_HIROSHI));

  rb_define_const(mBe, "BACKSPACE", ULONG2NUM(B_BACKSPACE));
  rb_define_const(mBe, "RETURN", ULONG2NUM(B_RETURN));
  rb_define_const(mBe, "ENTER", ULONG2NUM(B_ENTER));
  rb_define_const(mBe, "SPACE", ULONG2NUM(B_SPACE));
  rb_define_const(mBe, "TAB", ULONG2NUM(B_TAB));
  rb_define_const(mBe, "ESCAPE", ULONG2NUM(B_ESCAPE));
  rb_define_const(mBe, "SUBSTITUTE", ULONG2NUM(B_SUBSTITUTE));
  rb_define_const(mBe, "LEFT_ARROW", ULONG2NUM(B_LEFT_ARROW));
  rb_define_const(mBe, "RIGHT_ARROW", ULONG2NUM(B_RIGHT_ARROW));
  rb_define_const(mBe, "UP_ARROW", ULONG2NUM(B_UP_ARROW));
  rb_define_const(mBe, "DOWN_ARROW", ULONG2NUM(B_DOWN_ARROW));
  rb_define_const(mBe, "INSERT", ULONG2NUM(B_INSERT));
  rb_define_const(mBe, "DELETE", ULONG2NUM(B_DELETE));
  rb_define_const(mBe, "HOME", ULONG2NUM(B_HOME));
  rb_define_const(mBe, "END", ULONG2NUM(B_END));
  rb_define_const(mBe, "PAGE_UP", ULONG2NUM(B_PAGE_UP));
  rb_define_const(mBe, "PAGE_DOWN", ULONG2NUM(B_PAGE_DOWN));
  rb_define_const(mBe, "FUNCTION_KEY", ULONG2NUM(B_FUNCTION_KEY));

  rb_define_const(mBe, "F1_KEY", ULONG2NUM(B_F1_KEY));
  rb_define_const(mBe, "F2_KEY", ULONG2NUM(B_F2_KEY));
  rb_define_const(mBe, "F3_KEY", ULONG2NUM(B_F3_KEY));
  rb_define_const(mBe, "F4_KEY", ULONG2NUM(B_F4_KEY));
  rb_define_const(mBe, "F5_KEY", ULONG2NUM(B_F5_KEY));
  rb_define_const(mBe, "F6_KEY", ULONG2NUM(B_F6_KEY));
  rb_define_const(mBe, "F7_KEY", ULONG2NUM(B_F7_KEY));
  rb_define_const(mBe, "F8_KEY", ULONG2NUM(B_F8_KEY));
  rb_define_const(mBe, "F9_KEY", ULONG2NUM(B_F9_KEY));
  rb_define_const(mBe, "F10_KEY", ULONG2NUM(B_F10_KEY));
  rb_define_const(mBe, "F11_KEY", ULONG2NUM(B_F11_KEY));
  rb_define_const(mBe, "F12_KEY", ULONG2NUM(B_F12_KEY));
  rb_define_const(mBe, "PRINT_KEY", ULONG2NUM(B_PRINT_KEY));
  rb_define_const(mBe, "SCROLL_KEY", ULONG2NUM(B_SCROLL_KEY));
  rb_define_const(mBe, "PAUSE_KEY", ULONG2NUM(B_PAUSE_KEY));

  rb_define_const(mBe, "PLAIN_BORDER", ULONG2NUM(B_PLAIN_BORDER));
  rb_define_const(mBe, "FANCY_BORDER", ULONG2NUM(B_FANCY_BORDER));
  rb_define_const(mBe, "NO_BORDER", ULONG2NUM(B_NO_BORDER));

  rb_define_const(mBe, "HORIZONTAL", ULONG2NUM(B_HORIZONTAL));
  rb_define_const(mBe, "VERTICAL", ULONG2NUM(B_VERTICAL));

  rb_define_const(mBe, "WIDTH_AS_USUAL", ULONG2NUM(B_WIDTH_AS_USUAL));
  rb_define_const(mBe, "WIDTH_FROM_WIDEST", ULONG2NUM(B_WIDTH_FROM_WIDEST));
  rb_define_const(mBe, "WIDTH_FROM_LABEL", ULONG2NUM(B_WIDTH_FROM_LABEL));

  rb_define_const(mBe, "ROUND_JOIN", ULONG2NUM(B_ROUND_JOIN));
  rb_define_const(mBe, "MITER_JOIN", ULONG2NUM(B_MITER_JOIN));
  rb_define_const(mBe, "BEVEL_JOIN", ULONG2NUM(B_BEVEL_JOIN));
  rb_define_const(mBe, "BUTT_JOIN", ULONG2NUM(B_BUTT_JOIN));
  rb_define_const(mBe, "SQUARE_JOIN", ULONG2NUM(B_SQUARE_JOIN));

  rb_define_const(mBe, "ROUND_CAP", ULONG2NUM(B_ROUND_CAP));
  rb_define_const(mBe, "BUTT_CAP", ULONG2NUM(B_BUTT_CAP));
  rb_define_const(mBe, "SQUARE_CAP", ULONG2NUM(B_SQUARE_CAP));

  rb_define_const(mBe, "DEFAULT_MITER_LIMIT", rb_float_new(B_DEFAULT_MITER_LIMIT));

  rb_define_const(mBe, "ALIGN_LEFT", ULONG2NUM(B_ALIGN_LEFT));
  rb_define_const(mBe, "ALIGN_RIGHT", ULONG2NUM(B_ALIGN_RIGHT));
  rb_define_const(mBe, "ALIGN_CENTER", ULONG2NUM(B_ALIGN_CENTER));

  rb_define_const(mBe, "ALIGN_TOP", LONG2NUM(B_ALIGN_TOP));
  rb_define_const(mBe, "ALIGN_MIDDLE", LONG2NUM(B_ALIGN_MIDDLE));
  rb_define_const(mBe, "ALIGN_BOTTOM", LONG2NUM(B_ALIGN_BOTTOM));
  rb_define_const(mBe, "ALIGN_NO_VERTICAL", LONG2NUM(B_ALIGN_NO_VERTICAL));

  rb_define_const(mBe, "CONTROL_TABLE", ULONG2NUM(B_CONTROL_TABLE));
  rb_define_const(mBe, "OPTION_CAPS_SHIFT_TABLE", ULONG2NUM(B_OPTION_CAPS_SHIFT_TABLE));
  rb_define_const(mBe, "OPTION_CAPS_TABLE", ULONG2NUM(B_OPTION_CAPS_TABLE));
  rb_define_const(mBe, "OPTION_SHIFT_TABLE", ULONG2NUM(B_OPTION_SHIFT_TABLE));
  rb_define_const(mBe, "OPTION_TABLE", ULONG2NUM(B_OPTION_TABLE));
  rb_define_const(mBe, "CAPS_SHIFT_TABLE", ULONG2NUM(B_CAPS_SHIFT_TABLE));
  rb_define_const(mBe, "CAPS_TABLE", ULONG2NUM(B_CAPS_TABLE));
  rb_define_const(mBe, "SHIFT_TABLE", ULONG2NUM(B_SHIFT_TABLE));
  rb_define_const(mBe, "NORMAL_TABLE", ULONG2NUM(B_NORMAL_TABLE));

  rb_define_const(mBe, "SHIFT_KEY", ULONG2NUM(B_SHIFT_KEY));
  rb_define_const(mBe, "COMMAND_KEY", ULONG2NUM(B_COMMAND_KEY));
  rb_define_const(mBe, "CONTROL_KEY", ULONG2NUM(B_CONTROL_KEY));
  rb_define_const(mBe, "CAPS_LOCK", ULONG2NUM(B_CAPS_LOCK));
  rb_define_const(mBe, "SCROLL_LOCK", ULONG2NUM(B_SCROLL_LOCK));
  rb_define_const(mBe, "NUM_LOCK", ULONG2NUM(B_NUM_LOCK));
  rb_define_const(mBe, "OPTION_KEY", ULONG2NUM(B_OPTION_KEY));
  rb_define_const(mBe, "MENU_KEY", ULONG2NUM(B_MENU_KEY));
  rb_define_const(mBe, "LEFT_SHIFT_KEY", ULONG2NUM(B_LEFT_SHIFT_KEY));
  rb_define_const(mBe, "RIGHT_SHIFT_KEY", ULONG2NUM(B_RIGHT_SHIFT_KEY));
  rb_define_const(mBe, "LEFT_COMMAND_KEY", ULONG2NUM(B_LEFT_COMMAND_KEY));
  rb_define_const(mBe, "RIGHT_COMMAND_KEY", ULONG2NUM(B_RIGHT_COMMAND_KEY));
  rb_define_const(mBe, "LEFT_CONTROL_KEY", ULONG2NUM(B_LEFT_CONTROL_KEY));
  rb_define_const(mBe, "RIGHT_CONTROL_KEY", ULONG2NUM(B_RIGHT_CONTROL_KEY));
  rb_define_const(mBe, "LEFT_OPTION_KEY", ULONG2NUM(B_LEFT_OPTION_KEY));
  rb_define_const(mBe, "RIGHT_OPTION_KEY", ULONG2NUM(B_RIGHT_OPTION_KEY));

  rb_define_const(mBe, "TILE_BITMAP_X", ULONG2NUM(B_TILE_BITMAP_X));
  rb_define_const(mBe, "TILE_BITMAP_Y", ULONG2NUM(B_TILE_BITMAP_Y));
  rb_define_const(mBe, "TILE_BITMAP", ULONG2NUM(B_TILE_BITMAP));

  rb_define_const(mBe, "OVERLAY_FILTER_HORIZONTAL", ULONG2NUM(B_OVERLAY_FILTER_HORIZONTAL));
  rb_define_const(mBe, "OVERLAY_FILTER_VERTICAL", ULONG2NUM(B_OVERLAY_FILTER_VERTICAL));
  rb_define_const(mBe, "OVERLAY_MIRROR", ULONG2NUM(B_OVERLAY_MIRROR));
  rb_define_const(mBe, "OVERLAY_TRANSFER_CHANNEL", ULONG2NUM(B_OVERLAY_TRANSFER_CHANNEL));

  rb_define_const(mBe, "NORMAL_MOUSE", ULONG2NUM(B_NORMAL_MOUSE));
  rb_define_const(mBe, "FOCUS_FOLLOWS_MOUSE", ULONG2NUM(B_FOCUS_FOLLOWS_MOUSE));
  rb_define_const(mBe, "WARP_MOUSE", ULONG2NUM(B_WARP_MOUSE));
  rb_define_const(mBe, "INSTANT_WARP_MOUSE", ULONG2NUM(B_INSTANT_WARP_MOUSE));

  rb_define_const(mBe, "PANEL_BACKGROUND_COLOR", ULONG2NUM(B_PANEL_BACKGROUND_COLOR));
  rb_define_const(mBe, "MENU_BACKGROUND_COLOR", ULONG2NUM(B_MENU_BACKGROUND_COLOR));
  rb_define_const(mBe, "MENU_SELECTION_BACKGROUND_COLOR", ULONG2NUM(B_MENU_SELECTION_BACKGROUND_COLOR));
  rb_define_const(mBe, "MENU_ITEM_TEXT_COLOR", ULONG2NUM(B_MENU_ITEM_TEXT_COLOR));
  rb_define_const(mBe, "MENU_SELECTED_ITEM_TEXT_COLOR", ULONG2NUM(B_MENU_SELECTED_ITEM_TEXT_COLOR));
  rb_define_const(mBe, "WINDOW_TAB_COLOR", ULONG2NUM(B_WINDOW_TAB_COLOR));
  rb_define_const(mBe, "KEYBOARD_NAVIGATION_COLOR", ULONG2NUM(B_KEYBOARD_NAVIGATION_COLOR));
  rb_define_const(mBe, "DESKTOP_COLOR", ULONG2NUM(B_DESKTOP_COLOR));

  rb_define_const(mBe, "LIGHTEN_MAX_TINT", rb_float_new(B_LIGHTEN_MAX_TINT));
  rb_define_const(mBe, "LIGHTEN_2_TINT", rb_float_new(B_LIGHTEN_2_TINT));
  rb_define_const(mBe, "LIGHTEN_1_TINT", rb_float_new(B_LIGHTEN_1_TINT));
  rb_define_const(mBe, "NO_TINT", rb_float_new(B_NO_TINT));
  rb_define_const(mBe, "DARKEN_1_TINT", rb_float_new(B_DARKEN_1_TINT));
  rb_define_const(mBe, "DARKEN_2_TINT", rb_float_new(B_DARKEN_2_TINT));
  rb_define_const(mBe, "DARKEN_3_TINT", rb_float_new(B_DARKEN_3_TINT));
  rb_define_const(mBe, "DARKEN_4_TINT", rb_float_new(B_DARKEN_4_TINT));
  rb_define_const(mBe, "DARKEN_MAX_TINT", rb_float_new(B_DARKEN_MAX_TINT));
  rb_define_const(mBe, "DISABLED_LABEL_TINT", rb_float_new(B_DISABLED_LABEL_TINT));
  rb_define_const(mBe, "HIGHLIGHT_BACKGROUND_TINT", rb_float_new(B_HIGHLIGHT_BACKGROUND_TINT));
  rb_define_const(mBe, "DISABLED_MARK_TINT", rb_float_new(B_DISABLED_MARK_TINT));
}
