/*
 *  Looper.cc
 */

#include "rbLooper.h"
#include "my_be.h"


VALUE cLooper;
ID id_DispatchMessage;


void
delete_looper(BLooper *arg0)
{
  if (arg0->Lock()) {
    arg0->Quit();
  }
}


static VALUE
rb_looper_s_new(int argc, VALUE *argv, VALUE klass)
{
  VALUE vresult = my_class_s_new(argc, argv, klass, delete_looper);
  return vresult;
}


static VALUE
rb_looper_initialize(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1, varg2, varg3;
  Looper *result;

  Data_To_Null(self, BLooper);

  switch (rb_scan_args(argc, argv, "03", &varg1, &varg2, &varg3)) {
    case 0:
      result = new Looper();
      break;

    case 1:
      if (rb_obj_is_kind_of(varg1, cMessage)) {
        BMessage *arg1;

        Get_BInstance(Message, varg1, arg1);
        result = new Looper(arg1);
      }
      else {
        char *arg1;

        arg1 = STR2CSTR(varg1);
        result = new Looper(arg1);
      }
      break;

    case 2:
      {
        char *arg1;
        int32 arg2;

        arg1 = STR2CSTR(varg1);
        arg2 = NUM2LONG(varg2);
        result = new Looper(arg1, arg2);
      }
      break;

    case 3:
      {
        char *arg1;
        int32 arg2, arg3;

        arg1 = STR2CSTR(varg1);
        arg2 = NUM2LONG(varg2);
        arg3 = NUM2LONG(varg3);
        result = new Looper(arg1, arg2, arg3);
      }
      break;

    default:
      rb_bug("rb_looper_initialize");
  }
  result->myself = self;
  DATA_PTR(self) = result;
  return self;
}


static VALUE
rb_looper_PostMessage(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1, varg2, varg3;
  BLooper *arg0;
  status_t result;

  Get_BInstance(Looper, self, arg0);
  switch(rb_scan_args(argc, argv, "12", &varg1, &varg2, &varg3)){
    case 1:
      if (rb_obj_is_kind_of(varg1, cMessage)) {
        BMessage *arg1;

        Get_BInstance(Message, varg1, arg1);
        result = arg0->PostMessage(arg1);
      }
      else {
        uint32 arg1;

        arg1 = NUM2ULONG(varg1);
        result = arg0->PostMessage(arg1);
      }
      break;

    case 2:
      if (rb_obj_is_kind_of(varg1, cMessage)) {
        BMessage *arg1;
        BHandler *arg2;

        Get_BInstance(Message, varg1, arg1);
        Get_BInstance(Handler, varg2, arg2);

        result = arg0->PostMessage(arg1, arg2);
      }
      else {
        uint32 arg1;
        BHandler *arg2;

        arg1 = NUM2ULONG(varg1);
        Get_BInstance(Handler, varg2, arg2);

        result = arg0->PostMessage(arg1, arg2);
      }
      break;

    case 3:
      if (rb_obj_is_kind_of(varg1, cMessage)) {
        BMessage *arg1;
        BHandler *arg2, *arg3;

        Get_BInstance(Message, varg1, arg1);
        Get_BInstance(Handler, varg2, arg2);
        Get_BInstance(Handler, varg3, arg3);

        result = arg0->PostMessage(arg1, arg2, arg3);
      }
      else {
        uint32 arg1;
        BHandler *arg2, *arg3;

        arg1 = NUM2ULONG(varg1);
        Get_BInstance(Handler, varg2, arg2);
        Get_BInstance(Handler, varg3, arg3);

        result = arg0->PostMessage(arg1, arg2, arg3);
      }
      break;

    default:
      rb_bug("rb_looper_PostMessage");
  }
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_looper_DispatchMessage(VALUE self, VALUE varg1, VALUE varg2)
{
  BLooper *arg0;
  BMessage *arg1;
  BHandler *arg2;

  Get_BInstance(Looper, self, arg0);
  Get_BInstance(Message, varg1, arg1);
  Get_BInstance(Handler, varg2, arg2);
  arg0->BLooper::DispatchMessage(arg1, arg2);
  return Qnil;
}


static VALUE
rb_looper_MessageReceived(VALUE self, VALUE varg1)
{
  BLooper *arg0;
  BMessage *arg1;

  Get_BInstance(Looper, self, arg0);
  Get_BInstance(Message, varg1, arg1);
  arg0->BLooper::MessageReceived(arg1);
  return Qnil;
}


static VALUE
rb_looper_CurrentMessage(VALUE self)
{
  BLooper *arg0;
  BMessage *result;
  VALUE vresult;

  Get_BInstance(Looper, self, arg0);
  result = arg0->CurrentMessage();
  vresult = ((result) ? Data_Wrap_Struct(cMessage, 0, 0, result):Qnil);
  return vresult;
}


static VALUE
rb_looper_DetachCurrentMessage(VALUE self)
{
  BLooper *arg0;
  BMessage *result;
  VALUE vresult;

  Get_BInstance(Looper, self, arg0);
  result = arg0->DetachCurrentMessage();
  vresult = ((result) ? Data_Wrap_Struct(cMessage, 0, delete_message, result):Qnil);
  return vresult;
}


static VALUE
rb_looper_MessageQueue(VALUE self)
{
  BLooper *arg0;
  BMessageQueue *result;
  VALUE vresult;

  Get_BInstance(Looper, self, arg0);
  result = arg0->MessageQueue();
  vresult = ((result) ? Data_Wrap_Struct(cMessageQueue, 0, delete_messagequeue, result):Qnil);
  return vresult;
}


static VALUE
rb_looper_IsMessageWaiting(VALUE self)
{
  BLooper *arg0;
  bool result;
  VALUE vresult;

  Get_BInstance(Looper, self, arg0);
  result = arg0->IsMessageWaiting();
  vresult = (result ? Qtrue:Qfalse);
  return vresult;
}


static VALUE
rb_looper_AddHandler(VALUE self, VALUE varg1)
{
  BLooper *arg0;
  BHandler *arg1;

  Get_BInstance(Looper, self, arg0);
  Get_BInstance(Handler, varg1, arg1);
  arg0->AddHandler(arg1);
  return Qnil;
}


static VALUE
rb_looper_RemoveHandler(VALUE self, VALUE varg1)
{
  BLooper *arg0;
  BHandler *arg1;
  bool result;
  VALUE vresult;

  Get_BInstance(Looper, self, arg0);
  Get_BInstance(Handler, varg1, arg1);
  result = arg0->RemoveHandler(arg1);
  vresult = (result ? Qtrue:Qfalse);
  return vresult;
}


static VALUE
rb_looper_CountHandlers(VALUE self)
{
  BLooper *arg0;
  int32 result;
  VALUE vresult;

  Get_BInstance(Looper, self, arg0);
  result = arg0->CountHandlers();
  vresult = LONG2NUM(result);
  return vresult;
}


static VALUE
rb_looper_HandlerAt(VALUE self, VALUE varg1)
{
  BLooper *arg0;
  int32 arg1;
  BHandler *result;
  VALUE vresult;

  Get_BInstance(Looper, self, arg0);
  arg1 = NUM2LONG(varg1);
  result = arg0->HandlerAt(arg1);
  vresult = ((result) ? Data_Wrap_Struct(cHandler, 0, 0, result):Qnil);
  return vresult;
}


static VALUE
rb_looper_IndexOf(VALUE self, VALUE varg1)
{
  BLooper *arg0;
  BHandler *arg1;
  int32 result;
  VALUE vresult;

  Get_BInstance(Looper, self, arg0);
  Get_BInstance(Handler, varg1, arg1);
  result = arg0->IndexOf(arg1);
  vresult = LONG2NUM(result);
  return vresult;
}


static VALUE
rb_looper_PreferredHandler(VALUE self)
{
  BLooper *arg0;
  BHandler *result;
  VALUE vresult;

  Get_BInstance(Looper, self, arg0);
  result = arg0->PreferredHandler();
  vresult = ((result) ? Data_Wrap_Struct(cHandler, 0, 0, result):Qnil);
  return vresult;
}


static VALUE
rb_looper_SetPreferredHandler(VALUE self, VALUE varg1)
{
  BLooper *arg0;
  BHandler *arg1;

  Get_BInstance(Looper, self, arg0);
  Get_BInstance(Handler, varg1, arg1);
  arg0->SetPreferredHandler(arg1);
  return Qnil;
}


static VALUE
rb_looper_Run(VALUE self)
{
  BLooper *arg0;
  thread_id result;
  VALUE vresult;

  Get_BInstance(Looper, self, arg0);
  result = arg0->BLooper::Run();
  vresult = LONG2NUM(result);
  return vresult;
}


static VALUE
rb_looper_Quit(VALUE self)
{
  BLooper *arg0;

  Get_BInstance(Looper, self, arg0);
  arg0->BLooper::Quit();
  return Qnil;
}


static VALUE
rb_looper_QuitRequested(VALUE self)
{
  BLooper *arg0;
  bool result;
  VALUE vresult;

  Get_BInstance(Looper, self, arg0);
  result = arg0->BLooper::QuitRequested();
  vresult = (result ? Qtrue:Qfalse);
  return vresult;
}


static VALUE
rb_looper_Unlock(VALUE self)
{
  BLooper *arg0;

  Get_BInstance(Looper, self, arg0);
  arg0->Unlock();
  return Qnil;
}


static VALUE
yld_looper_lock(VALUE self)
{
  BLooper *arg0;
  VALUE vresult = Qnil;

  Get_BInstance(Looper, self, arg0);
  if (arg0->Lock()) {
    vresult = rb_yield(self);
  }
  return vresult;
}


static VALUE
rb_looper_Lock(VALUE self)
{
  BLooper *arg0;
  VALUE vresult;

  if (rb_block_given_p()) {
    vresult = rb_ensure(RUBY_METHOD_FUNC(yld_looper_lock), self,
                        RUBY_METHOD_FUNC(rb_looper_Unlock), self);
  }
  else {
    bool result;

    Get_BInstance(Looper, self, arg0);
    result = arg0->Lock();
    vresult = (result ? Qtrue:Qfalse);
  }
  return vresult;
}


static VALUE
rb_looper_IsLocked(VALUE self)
{
  BLooper *arg0;
  bool result;
  VALUE vresult;

  Get_BInstance(Looper, self, arg0);
  result = arg0->IsLocked();
  vresult = (result ? Qtrue:Qfalse);
  return vresult;
}


static VALUE
rb_looper_LockWithTimeout(VALUE self, VALUE varg1)
{
  BLooper *arg0;
  bigtime_t arg1;
  status_t result;

  Get_BInstance(Looper, self, arg0);
  arg1 = NUM2LLONG(varg1);
  result = arg0->LockWithTimeout(arg1);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_looper_Thread(VALUE self)
{
  BLooper *arg0;
  thread_id result;
  VALUE vresult;

  Get_BInstance(Looper, self, arg0);
  result = arg0->Thread();
  vresult = LONG2NUM(result);
  return vresult;
}


static VALUE
rb_looper_Team(VALUE self)
{
  BLooper *arg0;
  team_id result;
  VALUE vresult;

  Get_BInstance(Looper, self, arg0);
  result = arg0->Team();
  vresult = LONG2NUM(result);
  return vresult;
}


static VALUE
rb_looper_LooperForThread(VALUE self, VALUE varg1)
{
  thread_id arg1;
  BLooper *result;
  VALUE vresult;

  arg1 = NUM2LONG(varg1);
  result = BLooper::LooperForThread(arg1);
  vresult = ((result) ? Data_Wrap_Struct(cLooper, 0, 0, result):Qnil);
  return vresult;
}


static VALUE
rb_looper_LockingThread(VALUE self)
{
  BLooper *arg0;
  thread_id result;
  VALUE vresult;

  Get_BInstance(Looper, self, arg0);
  result = arg0->LockingThread();
  vresult = LONG2NUM(result);
  return vresult;
}


static VALUE
rb_looper_CountLocks(VALUE self)
{
  BLooper *arg0;
  int32 result;
  VALUE vresult;

  Get_BInstance(Looper, self, arg0);
  result = arg0->CountLocks();
  vresult = LONG2NUM(result);
  return vresult;
}


static VALUE
rb_looper_CountLockRequests(VALUE self)
{
  BLooper *arg0;
  int32 result;
  VALUE vresult;

  Get_BInstance(Looper, self, arg0);
  result = arg0->CountLockRequests();
  vresult = LONG2NUM(result);
  return vresult;
}


static VALUE
rb_looper_Sem(VALUE self)
{
  BLooper *arg0;
  sem_id result;
  VALUE vresult;

  Get_BInstance(Looper, self, arg0);
  result = arg0->Sem();
  vresult = LONG2NUM(result);
  return vresult;
}


static VALUE
rb_looper_ResolveSpecifier(VALUE self, VALUE varg1, VALUE varg2, VALUE varg3, VALUE varg4, VALUE varg5)
{
  BLooper *arg0;
  BMessage *arg1,  *arg3;
  int32 arg2, arg4;
  char *arg5;
  BHandler *result;
  VALUE vresult;

  Get_BInstance(Looper, self, arg0);
  Get_BInstance(Message, varg1, arg1);
  arg2 = NUM2LONG(varg2);
  Get_BInstance(Message, varg3, arg3);
  arg4 = NUM2LONG(varg4);
  arg5 = STR2CSTR(varg5);

  result = arg0->BLooper::ResolveSpecifier(arg1, arg2, arg3, arg4, arg5);
  vresult = ((result) ? Data_Wrap_Struct(cHandler, 0, 0, result):Qnil);
  return vresult;
}


static VALUE
rb_looper_GetSupportedSuites(VALUE self, VALUE varg1)
{
  BLooper *arg0;
  BMessage *arg1;
  status_t result;

  Get_BInstance(Looper, self, arg0);
  Get_BInstance(Message, varg1, arg1);
  result = arg0->BLooper::GetSupportedSuites(arg1);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_looper_AddCommonFilter(VALUE self, VALUE varg1)
{
  BLooper *arg0;
  BMessageFilter *arg1;

  Get_BInstance(Looper, self, arg0);
  Get_BInstance(MessageFilter, varg1, arg1);
  arg0->BLooper::AddCommonFilter(arg1);
  return Qnil;
}


static VALUE
rb_looper_RemoveCommonFilter(VALUE self, VALUE varg1)
{
  BLooper *arg0;
  BMessageFilter *arg1;
  bool result;
  VALUE vresult;

  Get_BInstance(Looper, self, arg0);
  Get_BInstance(MessageFilter, varg1, arg1);
  result = arg0->BLooper::RemoveCommonFilter(arg1);
  vresult = (result ? Qtrue:Qfalse);
  return vresult;
}


static VALUE
rb_looper_SetCommonFilterList(VALUE self, VALUE varg1)
{
  BLooper *arg0;
  BList *arg1;

  Get_BInstance(Looper, self, arg0);
  Get_BInstance(List, varg1, arg1);
  arg0->BLooper::SetCommonFilterList(arg1);
  return Qnil;
}


static VALUE
rb_looper_CommonFilterList(VALUE self)
{
  BLooper *arg0;
  BList *result;
  VALUE vresult;

  Get_BInstance(Looper, self, arg0);
  result = arg0->CommonFilterList();
  vresult = my_filter_list2rarry(*result);
  return vresult;
}


void
Init_Looper()
{
  id_DispatchMessage = rb_intern("DispatchMessage");

  cLooper = rb_define_class_under(mBe, "Looper", cHandler);
  rb_define_singleton_method(cLooper, "new", RUBY_METHOD_FUNC(rb_looper_s_new), -1);
  rb_define_method(cLooper, "initialize", RUBY_METHOD_FUNC(rb_looper_initialize), -1);

  rb_define_method(cLooper, "PostMessage", RUBY_METHOD_FUNC(rb_looper_PostMessage), -1);
  rb_define_method(cLooper, "DispatchMessage", RUBY_METHOD_FUNC(rb_looper_DispatchMessage), 2);
  rb_define_method(cLooper, "MessageReceived", RUBY_METHOD_FUNC(rb_looper_MessageReceived), 1);
  rb_define_method(cLooper, "CurrentMessage", RUBY_METHOD_FUNC(rb_looper_CurrentMessage), 0);
  rb_define_method(cLooper, "DetachCurrentMessage", RUBY_METHOD_FUNC(rb_looper_DetachCurrentMessage), 0);
  rb_define_method(cLooper, "MessageQueue", RUBY_METHOD_FUNC(rb_looper_MessageQueue), 0);
  rb_define_method(cLooper, "IsMessageWaiting", RUBY_METHOD_FUNC(rb_looper_IsMessageWaiting), 0);

  rb_define_method(cLooper, "AddHandler", RUBY_METHOD_FUNC(rb_looper_AddHandler), 1);
  rb_define_method(cLooper, "RemoveHandler", RUBY_METHOD_FUNC(rb_looper_RemoveHandler), 1);
  rb_define_method(cLooper, "CountHandlers", RUBY_METHOD_FUNC(rb_looper_CountHandlers), 0);
  rb_define_method(cLooper, "HandlerAt", RUBY_METHOD_FUNC(rb_looper_HandlerAt), 1);
  rb_define_method(cLooper, "IndexOf", RUBY_METHOD_FUNC(rb_looper_IndexOf), 1);

  rb_define_method(cLooper, "PreferredHandler", RUBY_METHOD_FUNC(rb_looper_PreferredHandler), 0);
  rb_define_method(cLooper, "SetPreferredHandler", RUBY_METHOD_FUNC(rb_looper_SetPreferredHandler), 1);

  rb_define_method(cLooper, "Run", RUBY_METHOD_FUNC(rb_looper_Run), 0);
  rb_define_method(cLooper, "Quit", RUBY_METHOD_FUNC(rb_looper_Quit), 0);
  rb_define_method(cLooper, "QuitRequested", RUBY_METHOD_FUNC(rb_looper_QuitRequested), 0);
  rb_define_method(cLooper, "Lock", RUBY_METHOD_FUNC(rb_looper_Lock), 0);
  rb_define_method(cLooper, "Unlock", RUBY_METHOD_FUNC(rb_looper_Unlock), 0);
  rb_define_method(cLooper, "IsLocked", RUBY_METHOD_FUNC(rb_looper_IsLocked), 0);
  rb_define_method(cLooper, "LockWithTimeout", RUBY_METHOD_FUNC(rb_looper_LockWithTimeout), 1);
  rb_define_method(cLooper, "Thread", RUBY_METHOD_FUNC(rb_looper_Thread), 0);
  rb_define_method(cLooper, "Team", RUBY_METHOD_FUNC(rb_looper_Team), 0);
  rb_define_singleton_method(cLooper, "LooperForThread", RUBY_METHOD_FUNC(rb_looper_LooperForThread), 1);

  rb_define_method(cLooper, "LockingThread", RUBY_METHOD_FUNC(rb_looper_LockingThread), 0);
  rb_define_method(cLooper, "CountLocks", RUBY_METHOD_FUNC(rb_looper_CountLocks), 0);
  rb_define_method(cLooper, "CountLockRequests", RUBY_METHOD_FUNC(rb_looper_CountLockRequests), 0);
  rb_define_method(cLooper, "Sem", RUBY_METHOD_FUNC(rb_looper_Sem), 0);

  rb_define_method(cLooper, "ResolveSpecifier", RUBY_METHOD_FUNC(rb_looper_ResolveSpecifier), 5);
  rb_define_method(cLooper, "GetSupportedSuites", RUBY_METHOD_FUNC(rb_looper_GetSupportedSuites), 1);

  rb_define_method(cLooper, "AddCommonFilter", RUBY_METHOD_FUNC(rb_looper_AddCommonFilter), 1);
  rb_define_method(cLooper, "RemoveCommonFilter", RUBY_METHOD_FUNC(rb_looper_RemoveCommonFilter), 1);
  rb_define_method(cLooper, "SetCommonFilterList", RUBY_METHOD_FUNC(rb_looper_SetCommonFilterList), 1);
  rb_define_method(cLooper, "CommonFilterList", RUBY_METHOD_FUNC(rb_looper_CommonFilterList), 0);

  rb_define_const(mBe, "LOOPER_PORT_DEFAULT_CAPACITY", ULONG2NUM(B_LOOPER_PORT_DEFAULT_CAPACITY));
}
