/*
 *  Menu.cc
 */

#include "rbMenu.h"
#include "rbMenuItem.h"
#include "my_be.h"


VALUE cMenu;


void
delete_menu(BMenu *arg0)
{
  delete arg0;
}


static VALUE
rb_menu_s_new(int argc, VALUE *argv, VALUE klass)
{
  VALUE vresult = my_class_s_new(argc, argv, klass, delete_menu);
  return vresult;
}


static VALUE
rb_menu_initialize(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1, varg2, varg3, varg4, varg5, varg6;
  Menu *result;

  Data_To_Null(self, BMenu);

  if (argc < 4) {
    switch (rb_scan_args(argc, argv, "12", &varg1, &varg2, &varg3)) {
      case 1:
        if (rb_obj_is_kind_of(varg1, cMessage)) {
          BMessage *arg1;

          Get_BInstance(Message, varg1, arg1);
          result = new Menu(arg1);
        }
        else {
          char *arg1;

          arg1 = STR2CSTR(varg1);
          result = new Menu(arg1);
        }
        break;

      case 2:
        {
          char *arg1;
          menu_layout arg2;

          arg1 = STR2CSTR(varg1);
          arg2 = (menu_layout)NUM2ULONG(varg2);
          result = new Menu(arg1, arg2);
        }
        break;

      case 3:
        {
          char *arg1;
          float arg2, arg3;

          arg1 = STR2CSTR(varg1);
          arg2 = NUM2DBL(varg2);
          arg3 = NUM2DBL(varg3);
          result = new Menu(arg1, arg2, arg3);
        }
        break;

      default:
        rb_bug("rb_menu_initialize");
    }
  }
  else {
    BRect *arg1;
    char *arg2;
    uint32 arg3, arg4;
    menu_layout arg5;
    bool arg6;

    rb_scan_args(argc, argv, "6", &varg1, &varg2, &varg3, &varg4, &varg5, &varg6);

    Get_BInstance(Rect, varg1, arg1);
    arg2 = STR2CSTR(varg2);
    arg3 = NUM2ULONG(varg3);
    arg4 = NUM2ULONG(varg4);
    arg5 = (menu_layout)NUM2ULONG(varg5);
    arg6 = RTEST(varg6);

    result = new Menu(*arg1, arg2, arg3, arg4, arg5, arg6);
  }
  rb_ivar_set(self, id_relatives, Qnil);
  result->myself = self;
  DATA_PTR(self) = result;
  return self;
}


static VALUE
rb_menu_AttachedToWindow(VALUE self)
{
  BMenu *arg0;

  Get_BInstance(Menu, self, arg0);
  arg0->BMenu::AttachedToWindow();
  return Qnil;
}


static VALUE
rb_menu_DetachedFromWindow(VALUE self)
{
  BMenu *arg0;

  Get_BInstance(Menu, self, arg0);
  arg0->BMenu::DetachedFromWindow();
  return Qnil;
}


static VALUE
rb_menu_AddItem(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1, varg2, vresult;
  BMenu *arg0;
  bool result;

  Get_BInstance(Menu, self, arg0);
  switch (rb_scan_args(argc, argv, "11", &varg1, &varg2)) {
    case 1:
      if (rb_obj_is_kind_of(varg1, cMenu)) {
        BMenu *arg1;

        Get_BInstance(Menu, varg1, arg1);
        result = arg0->AddItem(arg1);
      }
      else {
        BMenuItem *arg1;

        Get_BInstance(MenuItem, varg1, arg1);
        result = arg0->AddItem(arg1);
      }
      break;

    case 2:
      if (rb_obj_is_kind_of(varg1, cMenu)) {
        BMenu *arg1;
        Get_BInstance(Menu, varg1, arg1);

        if (rb_obj_is_kind_of(varg2, cRect)) {
          BRect *arg2;

          Get_BInstance(Rect, varg2, arg2);
          result = arg0->AddItem(arg1, *arg2);
        }
        else {
          int32 arg2;

          arg2 = NUM2LONG(varg2);
          result = arg0->AddItem(arg1, arg2);
        }
      }
      else {
        BMenuItem *arg1;
        Get_BInstance(MenuItem, varg1, arg1);

        if (rb_obj_is_kind_of(varg2, cRect)) {
          BRect *arg2;

          Get_BInstance(Rect, varg2, arg2);
          result = arg0->AddItem(arg1, *arg2);
        }
        else {
          int32 arg2;

          arg2 = NUM2LONG(varg2);
          result = arg0->AddItem(arg1, arg2);
        }
      }
      break;

    default:
      rb_bug("rb_menu_AddItem");
  }
  RDATA(varg1)->dfree = 0;
  my_add_relative(self, varg1);

  vresult = (result ? Qtrue:Qfalse);
  return vresult;
}


static VALUE
rb_menu_AddSeparatorItem(VALUE self)
{
  BMenu *arg0;
  bool result;
  VALUE vresult;

  Get_BInstance(Menu, self, arg0);
  result = arg0->AddSeparatorItem();
  vresult = (result ? Qtrue:Qfalse);
  return vresult;
}


void
Init_Menu()
{
  cMenu = rb_define_class_under(mBe, "Menu", cView);
  rb_define_singleton_method(cMenu, "new", RUBY_METHOD_FUNC(rb_menu_s_new), -1);
  rb_define_method(cMenu, "initialize", RUBY_METHOD_FUNC(rb_menu_initialize), -1);

  rb_define_method(cMenu, "AttachedToWindow", RUBY_METHOD_FUNC(rb_menu_AttachedToWindow), 0);
  rb_define_method(cMenu, "DetachedFromWindow", RUBY_METHOD_FUNC(rb_menu_DetachedFromWindow), 0);

  rb_define_method(cMenu, "AddItem", RUBY_METHOD_FUNC(rb_menu_AddItem), -1);
  rb_define_method(cMenu, "AddSeparatorItem", RUBY_METHOD_FUNC(rb_menu_AddSeparatorItem), 0);

  rb_define_const(mBe, "ITEMS_IN_ROW", ULONG2NUM(B_ITEMS_IN_ROW));
  rb_define_const(mBe, "ITEMS_IN_COLUMN", ULONG2NUM(B_ITEMS_IN_COLUMN));
  rb_define_const(mBe, "ITEMS_IN_MATRIX", ULONG2NUM(B_ITEMS_IN_MATRIX));
}
