/*
 *  MenuItem.cc
 */

#include "rbMenuItem.h"
#include "rbMenu.h"
#include "rbRect.h"
#include "my_be.h"


VALUE cMenuItem;
VALUE cSeparatorItem;


void
delete_menuitem(BMenuItem *arg0)
{
  delete arg0;
}


static VALUE
rb_menuitem_s_new(int argc, VALUE *argv, VALUE klass)
{
  VALUE vresult = my_class_s_new(argc, argv, klass, delete_menuitem);
  return vresult;
}


static VALUE
rb_menuitem_initialize(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1, varg2, varg3, varg4;
  MenuItem *result;

  Data_To_Null(self, BMenuItem);

  switch (rb_scan_args(argc, argv, "13", &varg1, &varg2, &varg3, &varg4)) {
    case 1:
      if (rb_obj_is_kind_of(varg1, cMenu)) {
        BMenu *arg1;

        Get_BInstance(Menu, varg1, arg1);
        result = new MenuItem(arg1);
      }
      else {
        BMessage *arg1;

        Get_BInstance(Message, varg1, arg1);
        result = new MenuItem(arg1);
      }
      break;

    case 2:
      if (rb_obj_is_kind_of(varg1, cMenu)) {
        BMenu *arg1;
        BMessage *arg2;

        Get_BInstance(Menu, varg1, arg1);
        Get_BInstance(Message, varg2, arg2);
        result = new MenuItem(arg1, arg2);
      }
      else {
        char *arg1;
        BMessage *arg2;

        arg1 = STR2CSTR(varg1);
        Get_BInstance(Message, varg2, arg2);
        result = new MenuItem(arg1, arg2);
      }
      break;

    case 3:
      {
        char *arg1, arg3;
        BMessage *arg2;

        arg1 = STR2CSTR(varg1);
        Get_BInstance(Message, varg2, arg2);
        arg3 = NUM2CHR(varg3);
        result = new MenuItem(arg1, new BMessage(*arg2), arg3);
      }
      break;

    case 4:
      {
        char *arg1, arg3;
        BMessage *arg2;
        uint32 arg4;

        arg1 = STR2CSTR(varg1);
        Get_BInstance(Message, varg2, arg2);
        arg3 = NUM2CHR(varg3);
        arg4 = NUM2ULONG(varg4);
        result = new MenuItem(arg1, arg2, arg3, arg4);
      }
      break;

    default:
      rb_bug("rb_menuitem_initialize");
  }
  DATA_PTR(self) = result;
  return self;
}


static VALUE
rb_menuitem_Instantiate(int argc, VALUE *argv, VALUE klass)
{
  BMessage *arg1;
  BArchivable *result;
  VALUE varg1, vresult;

  rb_scan_args(argc, argv, "1", &varg1);

  Get_BInstance(Message, varg1, arg1);
  result = BMenuItem::Instantiate(arg1);
  vresult = Data_Wrap_Struct(klass, 0, delete_archivable, result);

  rb_obj_call_init(vresult, argc, argv);
  return vresult;
}


static VALUE
rb_menuitem_Archive(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1, varg2;
  BMenuItem *arg0;
  status_t result;

  Get_BInstance(MenuItem, self, arg0);
  switch (rb_scan_args(argc, argv, "11", &varg1, &varg2)) {
    case 1:
      {
        BMessage *arg1;

        Get_BInstance(Message, varg1, arg1);
        result = arg0->Archive(arg1);
      }
      break;

    case 2:
      {
        BMessage *arg1;
        bool arg2;

        Get_BInstance(Message, varg1, arg1);
        arg2 = RTEST(varg2);
        result = arg0->Archive(arg1, arg2);
      }
      break;

    default:
      rb_bug("rb_menuitem_Archive");
  }
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_menuitem_SetLabel(VALUE self, VALUE varg1)
{
  BMenuItem *arg0;
  char *arg1;

  Get_BInstance(MenuItem, self, arg0);
  arg1 = STR2CSTR(varg1);
  arg0->BMenuItem::SetLabel(arg1);
  return Qnil;
}


static VALUE
rb_menuitem_SetEnabled(VALUE self, VALUE varg1)
{
  BMenuItem *arg0;
  bool arg1;

  Get_BInstance(MenuItem, self, arg0);
  arg1 = RTEST(varg1);
  arg0->BMenuItem::SetEnabled(arg1);
  return Qnil;
}


static VALUE
rb_menuitem_SetMarked(VALUE self, VALUE varg1)
{
  BMenuItem *arg0;
  bool arg1;

  Get_BInstance(MenuItem, self, arg0);
  arg1 = RTEST(varg1);
  arg0->BMenuItem::SetMarked(arg1);
  return Qnil;
}


static VALUE
rb_menuitem_SetTrigger(VALUE self, VALUE varg1)
{
  BMenuItem *arg0;
  char arg1;

  Get_BInstance(MenuItem, self, arg0);
  arg1 = NUM2CHR(varg1);
  arg0->BMenuItem::SetTrigger(arg1);
  return Qnil;
}


static VALUE
rb_menuitem_SetShortcut(VALUE self, VALUE varg1, VALUE varg2)
{
  BMenuItem *arg0;
  char arg1;
  uint32 arg2;

  Get_BInstance(MenuItem, self, arg0);
  arg1 = NUM2CHR(varg1);
  arg2 = NUM2ULONG(varg2);
  arg0->BMenuItem::SetShortcut(arg1, arg2);
  return Qnil;
}


static VALUE
rb_menuitem_Label(VALUE self)
{
  BMenuItem *arg0;
  const char *result;
  VALUE vresult;

  Get_BInstance(MenuItem, self, arg0);
  result = arg0->Label();
  vresult = rb_str_new2(result);
  return vresult;
}


static VALUE
rb_menuitem_IsEnabled(VALUE self)
{
  BMenuItem *arg0;
  bool result;
  VALUE vresult;

  Get_BInstance(MenuItem, self, arg0);
  result = arg0->IsEnabled();
  vresult = (result ? Qtrue:Qfalse);
  return vresult;
}


static VALUE
rb_menuitem_IsMarked(VALUE self)
{
  BMenuItem *arg0;
  bool result;
  VALUE vresult;

  Get_BInstance(MenuItem, self, arg0);
  result = arg0->IsMarked();
  vresult = (result ? Qtrue:Qfalse);
  return vresult;
}


static VALUE
rb_menuitem_Trigger(VALUE self)
{
  BMenuItem *arg0;
  char result;
  VALUE vresult;

  Get_BInstance(MenuItem, self, arg0);
  result = arg0->Trigger();
  vresult = rb_str_new(&result, 1);
  return vresult;
}


static VALUE
rb_menuitem_Shortcut(VALUE self)
{
  BMenuItem *arg0;
  uint32 arg1;
  char result;
  VALUE vresult;

  Get_BInstance(MenuItem, self, arg0);
  result = arg0->Shortcut(&arg1);
  vresult = rb_ary_new3(2, rb_str_new(&result, 1), ULONG2NUM(arg1));
  return vresult;
}


static VALUE
rb_menuitem_Submenu(VALUE self)
{
  BMenuItem *arg0;
  BMenu *result;
  VALUE vresult;

  Get_BInstance(MenuItem, self, arg0);
  result = arg0->Submenu();
  vresult = ((result) ? Data_Wrap_Struct(cMenu, 0, 0, result):Qnil);
  return vresult;
}


static VALUE
rb_menuitem_Menu(VALUE self)
{
  BMenuItem *arg0;
  BMenu *result;
  VALUE vresult = Qnil;

  Get_BInstance(MenuItem, self, arg0);
  result = arg0->Menu();
  vresult = ((result) ? Data_Wrap_Struct(cMenu, 0, 0, result):Qnil);
  return vresult;
}


static VALUE
rb_menuitem_Frame(VALUE self)
{
  BMenuItem *arg0;
  BRect *result;
  VALUE vresult;

  Get_BInstance(MenuItem, self, arg0);
  result = new Rect(arg0->Frame());
  vresult = ((result) ? Data_Wrap_Struct(cRect, 0, delete_rect, result):Qnil);
  return vresult;
}


void
Init_MenuItem()
{
  cMenuItem = rb_define_class_under(mBe, "MenuItem", cInvoker);
  rb_define_singleton_method(cMenuItem, "new", RUBY_METHOD_FUNC(rb_menuitem_s_new), -1);
  rb_define_method(cMenuItem, "initialize", RUBY_METHOD_FUNC(rb_menuitem_initialize), -1);

  rb_define_singleton_method(cMenuItem, "Instantiate", RUBY_METHOD_FUNC(rb_menuitem_Instantiate), -1);
  rb_define_method(cMenuItem, "Archive", RUBY_METHOD_FUNC(rb_menuitem_Archive), -1);

  rb_define_method(cMenuItem, "SetLabel", RUBY_METHOD_FUNC(rb_menuitem_SetLabel), 1);
  rb_define_method(cMenuItem, "SetEnabled", RUBY_METHOD_FUNC(rb_menuitem_SetEnabled), 1);
  rb_define_method(cMenuItem, "SetMarked", RUBY_METHOD_FUNC(rb_menuitem_SetMarked), 1);
  rb_define_method(cMenuItem, "SetTrigger", RUBY_METHOD_FUNC(rb_menuitem_SetTrigger), 1);
  rb_define_method(cMenuItem, "SetShortcut", RUBY_METHOD_FUNC(rb_menuitem_SetShortcut), 2);
  rb_define_method(cMenuItem, "Label", RUBY_METHOD_FUNC(rb_menuitem_Label), 0);
  rb_define_method(cMenuItem, "IsEnabled", RUBY_METHOD_FUNC(rb_menuitem_IsEnabled), 0);
  rb_define_method(cMenuItem, "IsMarked", RUBY_METHOD_FUNC(rb_menuitem_IsMarked), 0);
  rb_define_method(cMenuItem, "Trigger", RUBY_METHOD_FUNC(rb_menuitem_Trigger), 0);
  rb_define_method(cMenuItem, "Shortcut", RUBY_METHOD_FUNC(rb_menuitem_Shortcut), 0);
  rb_define_method(cMenuItem, "Submenu", RUBY_METHOD_FUNC(rb_menuitem_Submenu), 0);
  rb_define_method(cMenuItem, "Menu", RUBY_METHOD_FUNC(rb_menuitem_Menu), 0);
  rb_define_method(cMenuItem, "Frame", RUBY_METHOD_FUNC(rb_menuitem_Frame), 0);
}


void
delete_separatoritem(BSeparatorItem *arg0)
{
  delete arg0;
}


static VALUE
rb_separatoritem_s_new(int argc, VALUE *argv, VALUE klass)
{
  VALUE vresult = my_class_s_new(argc, argv, klass, delete_separatoritem);
  return vresult;
}


static VALUE
rb_separatoritem_initialize(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1;
  SeparatorItem *result;

  Data_To_Null(self, BSeparatorItem);

  switch (rb_scan_args(argc, argv, "01", &varg1)) {
    case 0:
      result = new SeparatorItem();
      break;

    case 1:
      {
        BMessage *arg1;

        Get_BInstance(Message, varg1, arg1);
        result = new SeparatorItem(arg1);
      }
      break;

    default:
      rb_bug("rb_separatoritem_initialize");
  }
  DATA_PTR(self) = result;
  return self;
}


static VALUE
rb_separatoritem_SetEnabled(VALUE self, VALUE varg1)
{
  BSeparatorItem *arg0;
  bool arg1;

  Get_BInstance(SeparatorItem, self, arg0);
  arg1 = RTEST(varg1);
  arg0->SetEnabled(arg1);
  return Qnil;
}


void
Init_SeparatorItem()
{
  cSeparatorItem = rb_define_class_under(mBe, "SeparatorItem", cMenuItem);
  rb_define_singleton_method(cSeparatorItem, "new", RUBY_METHOD_FUNC(rb_separatoritem_s_new), -1);
  rb_define_method(cSeparatorItem, "initialize", RUBY_METHOD_FUNC(rb_separatoritem_initialize), -1);

  rb_define_method(cSeparatorItem, "SetEnabled", RUBY_METHOD_FUNC(rb_separatoritem_SetEnabled), 1);
}
