/*
 *  Message.cc
 */

#include <String.h>
#include <Entry.h>
#include "rbMessage.h"
#include "rbMessenger.h"
#include "rbPoint.h"
#include "rbRect.h"
#include "my_be.h"


VALUE cMessage;


void
delete_message(BMessage *arg0)
{
  delete arg0;
}


static VALUE
rb_message_s_new(int argc, VALUE *argv, VALUE klass)
{
  VALUE vresult = my_class_s_new(argc, argv, klass, delete_message);
  return vresult;
}


static VALUE
rb_message_initialize(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1;
  BMessage *result;

  Data_To_Null(self, BMessage);

  switch (rb_scan_args(argc, argv, "01", &varg1)) {
    case 0:
      result = new Message();
      break;

    case 1:
      if (rb_obj_is_kind_of(varg1, rb_cInteger)) {
        uint32 arg1;

        arg1 = NUM2ULONG(varg1);
        result = new Message(arg1);
      }
      else {
        BMessage *arg1;

        Get_BInstance(Message, varg1, arg1);
        result = new Message(*arg1);
      }
      break;

    default:
      rb_bug("rb_message_initialize");
  }
  DATA_PTR(self) = result;
  return self;
}


static VALUE
rb_message_what_set(VALUE self, VALUE varg1)
{
  BMessage *arg0;
  uint32 arg1;

  Get_BInstance(Message, self, arg0);
  arg1 = NUM2ULONG(varg1);
  arg0->what = arg1;
  return Qnil;
}


static VALUE
rb_message_what_ref(VALUE self)
{
  BMessage *arg0;
  uint32 result;
  VALUE vresult;

  Get_BInstance(Message, self, arg0);
  result = arg0->what;
  vresult = ULONG2NUM(result);
  return vresult;
}


static VALUE
rb_message_GetInfo(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1, varg2, vresult;
  BMessage *arg0;

  Get_BInstance(Message, self, arg0);
  switch (rb_scan_args(argc, argv, "11", &varg1, &varg2)) {
    case 1:
      {
        char *arg1;
        type_code arg2=0;
        bool arg3;
        status_t result;

        arg1 = STR2CSTR(varg1);

        result = arg0->GetInfo(arg1, &arg2, &arg3);
        if (result != B_OK) {
          DEBUG_TRACE();
          my_raise(result);
        }
        vresult = rb_ary_new3(2, ULONG2NUM(arg2), (arg3 ? Qtrue:Qfalse));
      }
      break;

    case 2:
      {
        type_code arg1, arg4=0;
        int32 arg2, arg5=0;
        char *arg3;
        status_t result;

        arg1 = NUM2ULONG(varg1);
        arg2 = NUM2LONG(varg2);

        result = arg0->GetInfo(arg1, arg2, &arg3, &arg4, &arg5);
        if (result != B_OK) {
          DEBUG_TRACE();
          my_raise(result);
        }
        vresult = rb_ary_new3(3, rb_str_new2(arg3),
                                 ULONG2NUM(arg4),
                                 LONG2NUM(arg5));
      }
      break;

    default:
      rb_bug("rb_message_GetInfo");
  }
  return vresult;
}


static VALUE
rb_message_CountNames(VALUE self, VALUE varg1)
{
  BMessage *arg0;
  type_code arg1;
  int32 result;
  VALUE vresult;

  Get_BInstance(Message, self, arg0);
  arg1 = NUM2ULONG(varg1);
  result = arg0->CountNames(arg1);
  vresult = LONG2NUM(result);
  return vresult;
}


static VALUE
rb_message_IsEmpty(VALUE self)
{
  BMessage *arg0;
  bool result;
  VALUE vresult;

  Get_BInstance(Message, self, arg0);
  result = arg0->IsEmpty();
  vresult = (result ? Qtrue:Qfalse);
  return vresult;
}


static VALUE
rb_message_IsSystem(VALUE self)
{
  BMessage *arg0;
  bool result;
  VALUE vresult;

  Get_BInstance(Message, self, arg0);
  result = arg0->IsSystem();
  vresult = (result ? Qtrue:Qfalse);
  return vresult;
}


static VALUE
rb_message_IsReply(VALUE self)
{
  BMessage *arg0;
  bool result;
  VALUE vresult;

  Get_BInstance(Message, self, arg0);
  result = arg0->IsReply();
  vresult = (result ? Qtrue:Qfalse);
  return vresult;
}


static VALUE
rb_message_PrintToStream(VALUE self)
{
  BMessage *arg0;

  Get_BInstance(Message, self, arg0);
  arg0->PrintToStream();
  return Qnil;
}


static VALUE
rb_message_Rename(VALUE self, VALUE varg1, VALUE varg2)
{
  BMessage *arg0;
  char *arg1, *arg2;
  status_t result;

  Get_BInstance(Message, self, arg0);
  arg1 = STR2CSTR(varg1);
  arg2 = STR2CSTR(varg2);
  result = arg0->Rename(arg1, arg2);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_message_WasDelivered(VALUE self)
{
  BMessage *arg0;
  bool result;
  VALUE vresult;

  Get_BInstance(Message, self, arg0);
  result = arg0->WasDelivered();
  vresult = (result ? Qtrue:Qfalse);
  return vresult;
}


static VALUE
rb_message_IsSourceWaiting(VALUE self)
{
  BMessage *arg0;
  bool result;
  VALUE vresult;

  Get_BInstance(Message, self, arg0);
  result = arg0->IsSourceWaiting();
  vresult = (result ? Qtrue:Qfalse);
  return vresult;
}


static VALUE
rb_message_IsSourceRemote(VALUE self)
{
  BMessage *arg0;
  bool result;
  VALUE vresult;

  Get_BInstance(Message, self, arg0);
  result = arg0->IsSourceRemote();
  vresult = (result ? Qtrue:Qfalse);
  return vresult;
}


static VALUE
rb_message_ReturnAddress(VALUE self)
{
  BMessage *arg0;
  BMessenger *result;
  VALUE vresult;

  Get_BInstance(Message, self, arg0);
  result = new Messenger(arg0->ReturnAddress());
  vresult = Data_Wrap_Struct(cMessenger, 0, delete_messenger, result);
  return vresult;
}


static VALUE
rb_message_Previous(VALUE self)
{
  BMessage *arg0, *result;
  VALUE vresult;

  Get_BInstance(Message, self, arg0);
  result = const_cast<BMessage *>(arg0->Previous());
  vresult = ((result) ? Data_Wrap_Struct(cMessage, 0, 0, result):Qnil);
  return vresult;
}


static VALUE
rb_message_WasDropped(VALUE self)
{
  BMessage *arg0;
  bool result;
  VALUE vresult;

  Get_BInstance(Message, self, arg0);
  result = arg0->WasDropped();
  vresult = (result ? Qtrue:Qfalse);
  return vresult;
}


static VALUE
rb_message_DropPoint(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1, vresult;
  BMessage *arg0;
  BPoint *result;

  Get_BInstance(Message, self, arg0);
  switch (rb_scan_args(argc, argv, "01", &varg1)) {
    case 0:
      result = new Point(arg0->DropPoint());
      break;

    case 1:
      {
        BPoint *arg1;

        Get_BInstance(Point, varg1, arg1);
        result = new Point(arg0->DropPoint(arg1));
      }
      break;

    default:
      rb_bug("rb_message_DropPoint");
  }
  vresult = Data_Wrap_Struct(cPoint, 0, delete_point, result);
  return vresult;
}


static VALUE
rb_message_SendReply(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1, varg2, varg3, varg4;
  BMessage *arg0;
  status_t result;

  Get_BInstance(Message, self, arg0);
  switch (rb_scan_args(argc, argv, "13", &varg1, &varg2, &varg3, &varg4)) {
    case 1:
      if (rb_obj_is_kind_of(varg1, cMessage)) {
        BMessage *arg1;
        Get_BInstance(Message, varg1, arg1);
        result = arg0->SendReply(arg1);
      }
      else {
        uint32 arg1;
        arg1 = NUM2ULONG(varg1);
        result = arg0->SendReply(arg1);
      }
      break;

    case 2:
      if (rb_obj_is_kind_of(varg1, cMessage)) {
        BMessage *arg1;
        Get_BInstance(Message, varg1, arg1);

        if (rb_obj_is_kind_of(varg2, cHandler)) {
          BHandler *arg2;
          Get_BInstance(Handler, varg2, arg2);
          result = arg0->SendReply(arg1, arg2);
        }
        else if (rb_obj_is_kind_of(varg2, cMessage)) {
          BMessage *arg2;
          Get_BInstance(Message, varg2, arg2);
          result = arg0->SendReply(arg1, arg2);
        }
        else {
          BMessenger *arg2;
          Get_BInstance(Messenger, varg2, arg2);
          result = arg0->SendReply(arg1, *arg2);
        }
      }
      else {
        uint32 arg1;
        arg1 = NUM2ULONG(varg1);

        if (rb_obj_is_kind_of(varg2, cMessage)) {
          BMessage *arg2;
          Get_BInstance(Message, varg2, arg2);
          result = arg0->SendReply(arg1, arg2);
        }
        else {
          BHandler *arg2;
          Get_BInstance(Handler, varg2, arg2);
          result = arg0->SendReply(arg1, arg2);
        }
      }
      break;

    case 3:
      {
        BMessage *arg1;
        Get_BInstance(Message, varg1, arg1);

        if (rb_obj_is_kind_of(varg2, cHandler)) {
          BHandler *arg2;
          bigtime_t arg3;

          Get_BInstance(Handler, varg2, arg2);
          arg3 = NUM2LLONG(varg3);
          result = arg0->SendReply(arg1, arg2, arg3);
        }
        else if (rb_obj_is_kind_of(varg2, cMessage)) {
          BMessage *arg2;
          bigtime_t arg3;

          Get_BInstance(Message, varg2, arg2);
          arg3 = NUM2LLONG(varg3);
          result = arg0->SendReply(arg1, arg2, arg3);
        }
        else {
          BMessenger *arg2;
          bigtime_t arg3;

          Get_BInstance(Messenger, varg2, arg2);
          arg3 = NUM2LLONG(varg3);
          result = arg0->SendReply(arg1, *arg2, arg3);
        }
      }
      break;

    case 4:
      {
        BMessage *arg1, *arg2;
        bigtime_t arg3, arg4;

        Get_BInstance(Message, varg1, arg1);
        Get_BInstance(Message, varg2, arg2);
        arg3 = NUM2LLONG(varg3);
        arg4 = NUM2LLONG(varg4);
        result = arg0->SendReply(arg1, arg2, arg3, arg4);
      }
      break;

    default:
      rb_bug("rb_message_SendReply");
  }
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_message_FlattenedSize(VALUE self)
{
  BMessage *arg0;
  ssize_t result;
  VALUE vresult;

  Get_BInstance(Message, self, arg0);
  result = arg0->FlattenedSize();
  vresult = LONG2NUM(result);
  return vresult;
}


static VALUE
rb_message_Flatten(VALUE self)
{
  char *arg1;
  ssize_t arg2;
  status_t result;
  VALUE vresult;
  BMessage *arg0;

  Get_BInstance(Message, self, arg0);

  arg2 = arg0->FlattenedSize();
  arg1 = ALLOCA_N(char, arg2);

  result = arg0->Flatten(arg1, arg2);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }

  vresult = rb_str_new(arg1, arg2);
  return vresult;
}


static VALUE
rb_message_Unflatten(VALUE self, VALUE varg1)
{
  BMessage *arg0;
  status_t result;

  Get_BInstance(Message, self, arg0);

  if (rb_obj_is_kind_of(varg1, cDataIO)) {
    BDataIO *arg1;

    Get_BInstance(DataIO, varg1, arg1);
    result = arg0->Unflatten(arg1);
    if (result == B_NO_MEMORY) {
      rb_gc();
      result = arg0->Unflatten(arg1);
    }
  }
  else {
    char *arg1;

    arg1 = STR2CSTR(varg1);
    result = arg0->Unflatten(arg1);
    if (result == B_NO_MEMORY) {
      rb_gc();
      result = arg0->Unflatten(arg1);
    }
  }
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_message_AddSpecifier(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1, varg2, varg3;
  BMessage *arg0;
  status_t result;

  Get_BInstance(Message, self, arg0);
  switch (rb_scan_args(argc, argv, "12", &varg1, &varg2, &varg3)) {
    case 1:
      if (TYPE(varg1) == T_STRING) {
        char *arg1;

        arg1 = STR2CSTR(varg1);
        result = arg0->AddSpecifier(arg1);
      }
      else {
        BMessage *arg1;

        Get_BInstance(Message, varg1, arg1);
        result = arg0->AddSpecifier(arg1);
      }
      break;

    case 2:
      {
        char *arg1;
        arg1 = STR2CSTR(varg1);

        if (TYPE(varg2) == T_STRING) {
          char *arg2;

          arg2 = STR2CSTR(varg2);
          result = arg0->AddSpecifier(arg1, arg2);
        }
        else {
          int32 arg2;

          arg2 = NUM2LONG(varg2);
          result = arg0->AddSpecifier(arg1, arg2);
        }
      }
      break;

    case 3:
      {
        char *arg1;
        int32 arg2, arg3;

        arg1 = STR2CSTR(varg1);
        arg2 = NUM2LONG(varg2);
        arg3 = NUM2LONG(varg3);

        result = arg0->AddSpecifier(arg1, arg2, arg3);
      }
      break;

    default:
      rb_bug("rb_message_AddSpecifier");
  }
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_message_SetCurrentSpecifier(VALUE self, VALUE varg1)
{
  BMessage *arg0;
  int32 arg1;
  status_t result;

  Get_BInstance(Message, self, arg0);
  arg1 = NUM2LONG(varg1);
  result = arg0->SetCurrentSpecifier(arg1);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_message_GetCurrentSpecifier(VALUE self)
{
  BMessage *arg0, arg2;
  int32 arg1, arg3;
  const char *arg4;
  status_t result;
  VALUE vresult;

  Get_BInstance(Message, self, arg0);

  result = arg0->GetCurrentSpecifier(&arg1, &arg2, &arg3, &arg4);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }

  BMessage *msg;
  msg = new Message(arg2);
  vresult = rb_ary_new3(4, LONG2NUM(arg1),
                           Data_Wrap_Struct(cMessage, 0, delete_message, msg),
                           LONG2NUM(arg3),
                           rb_str_new2(arg4));
  return vresult;
}


static VALUE
rb_message_HasSpecifiers(VALUE self)
{
  BMessage *arg0;
  bool result;
  VALUE vresult;

  Get_BInstance(Message, self, arg0);
  result = arg0->HasSpecifiers();
  vresult = (result ? Qtrue:Qfalse);
  return vresult;
}


static VALUE
rb_message_PopSpecifier(VALUE self)
{
  BMessage *arg0;
  status_t result;

  Get_BInstance(Message, self, arg0);
  result = arg0->PopSpecifier();
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_message_AddRect(VALUE self, VALUE varg1, VALUE varg2)
{
  BMessage *arg0;
  char *arg1;
  BRect *arg2;
  status_t result;

  Get_BInstance(Message, self, arg0);
  arg1 = STR2CSTR(varg1);
  Get_BInstance(Rect, varg2, arg2);
  result = arg0->AddRect(arg1, *arg2);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_message_AddPoint(VALUE self, VALUE varg1, VALUE varg2)
{
  BMessage *arg0;
  char *arg1;
  BPoint *arg2;
  status_t result;

  Get_BInstance(Message, self, arg0);
  arg1 = STR2CSTR(varg1);
  Get_BInstance(Point, varg2, arg2);
  result = arg0->AddPoint(arg1, *arg2);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_message_AddString(VALUE self, VALUE varg1, VALUE varg2)
{
  BMessage *arg0;
  char *arg1;
  status_t result;

  Get_BInstance(Message, self, arg0);
  arg1 = STR2CSTR(varg1);
  if (TYPE(varg2) == T_STRING) {
    char *arg2;

    arg2 = STR2CSTR(varg2);
    result = arg0->AddString(arg1, arg2);
  }
  else {
    BString *arg2;

    Get_BInstance(String, varg2, arg2);
    result = arg0->AddString(arg1, *arg2);
  }
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_message_AddInt8(VALUE self, VALUE varg1, VALUE varg2)
{
  BMessage *arg0;
  char *arg1;
  int8 arg2;
  status_t result;

  Get_BInstance(Message, self, arg0);
  arg1 = STR2CSTR(varg1);
  arg2 = NUM2CHR(varg2);
  result = arg0->AddInt8(arg1, arg2);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_message_AddInt16(VALUE self, VALUE varg1, VALUE varg2)
{
  BMessage *arg0;
  char *arg1;
  int16 arg2;
  status_t result;

  Get_BInstance(Message, self, arg0);
  arg1 = STR2CSTR(varg1);
  arg2 = NUM2SHRT(varg2);
  result = arg0->AddInt16(arg1, arg2);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_message_AddInt32(VALUE self, VALUE varg1, VALUE varg2)
{
  BMessage *arg0;
  char *arg1;
  int32 arg2;
  status_t result;

  Get_BInstance(Message, self, arg0);
  arg1 = STR2CSTR(varg1);
  arg2 = NUM2LONG(varg2);
  result = arg0->AddInt32(arg1, arg2);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_message_AddInt64(VALUE self, VALUE varg1, VALUE varg2)
{
  BMessage *arg0;
  char *arg1;
  int64 arg2;
  status_t result;

  Get_BInstance(Message, self, arg0);

  arg1 = STR2CSTR(varg1);
  arg2 = NUM2LLONG(varg2);

  result = arg0->AddInt64(arg1, arg2);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_message_AddBool(VALUE self, VALUE varg1, VALUE varg2)
{
  BMessage *arg0;
  char *arg1;
  bool arg2;
  status_t result;

  Get_BInstance(Message, self, arg0);
  arg1 = STR2CSTR(varg1);
  arg2 = RTEST(varg2);
  result = arg0->AddBool(arg1, arg2);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_message_AddFloat(VALUE self, VALUE varg1, VALUE varg2)
{
  BMessage *arg0;
  char *arg1;
  float arg2;
  status_t result;

  Get_BInstance(Message, self, arg0);
  arg1 = STR2CSTR(varg1);
  arg2 = NUM2DBL(varg2);
  result = arg0->AddFloat(arg1, arg2);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_message_AddDouble(VALUE self, VALUE varg1, VALUE varg2)
{
  BMessage *arg0;
  char *arg1;
  double arg2;
  status_t result;

  Get_BInstance(Message, self, arg0);
  arg1 = STR2CSTR(varg1);
  arg2 = NUM2DBL(varg2);
  result = arg0->AddDouble(arg1, arg2);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_message_AddPointer(VALUE self, VALUE varg1, VALUE varg2)
{
  BMessage *arg0;
  char *arg1;
  void *arg2;
  status_t result;

  Get_BInstance(Message, self, arg0);
  arg1 = STR2CSTR(varg1);
  Data_Get_Struct(varg2, void, arg2);
  result = arg0->AddPointer(arg1, arg2);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_message_AddMessenger(VALUE self, VALUE varg1, VALUE varg2)
{
  BMessage *arg0;
  char *arg1;
  BMessenger *arg2;
  status_t result;

  Get_BInstance(Message, self, arg0);
  arg1 = STR2CSTR(varg1);
  Get_BInstance(Messenger, varg2, arg2);
  result = arg0->AddMessenger(arg1, *arg2);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_message_AddRef(VALUE self, VALUE varg1, VALUE varg2)
{
  BMessage *arg0;
  char *arg1;
  entry_ref *arg2;
  status_t result;

  Get_BInstance(Message, self, arg0);
  arg1 = STR2CSTR(varg1);
  Get_BInstance2(Entry_ref, entry_ref, varg2, arg2);
  result = arg0->AddRef(arg1, arg2);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_message_AddMessage(VALUE self, VALUE varg1, VALUE varg2)
{
  BMessage *arg0;
  char *arg1;
  BMessage *arg2;
  status_t result;

  Get_BInstance(Message, self, arg0);
  arg1 = STR2CSTR(varg1);
  Get_BInstance(Message, varg2, arg2);
  result = arg0->AddMessage(arg1, arg2);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_message_AddFlat(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1, varg2, varg3;
  BMessage *arg0;
  status_t result;

  Get_BInstance(Message, self, arg0);
  switch (rb_scan_args(argc, argv, "21", &varg1, &varg2, &varg3)) {
    case 2:
      {
        char *arg1;
        BFlattenable *arg2;

        arg1 = STR2CSTR(varg1);
        Get_BInstance(Flattenable, varg2, arg2);

        result = arg0->AddFlat(arg1, arg2);
      }
      break;

    case 3:
      {
        char *arg1;
        BFlattenable *arg2;
        int32 arg3;

        arg1 = STR2CSTR(varg1);
        Get_BInstance(Flattenable, varg2, arg2);
        arg3 = NUM2LONG(varg3);

        result = arg0->AddFlat(arg1, arg2, arg3);
      }
      break;

    default:
      rb_bug("rb_message_AddFlat");
  }
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_message_AddData(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1, varg2, varg3, varg5, varg6;
  BMessage *arg0;
  status_t result;

  Get_BInstance(Message, self, arg0);
  switch (rb_scan_args(argc, argv, "32", &varg1, &varg2, &varg3, &varg5, &varg6)) {
    case 3:
      {
        char *arg1, *arg3;
        type_code arg2;
        int arg4;

        arg1 = STR2CSTR(varg1);
        arg2 = NUM2ULONG(varg2);
        arg3 = rb_str2cstr(varg3, &arg4);

        result = arg0->AddData(arg1, arg2, arg3, arg4);
      }
      break;

    case 4:
      {
        char *arg1, *arg3;
        type_code arg2;
        int arg4;
        bool arg5;

        arg1 = STR2CSTR(varg1);
        arg2 = NUM2ULONG(varg2);
        arg3 = rb_str2cstr(varg3, &arg4);
        arg5 = RTEST(varg5);

        result = arg0->AddData(arg1, arg2, arg3, arg4, arg5);
      }
      break;

    case 5:
      {
        char *arg1, *arg3;
        type_code arg2;
        int arg4;
        bool arg5;
        int32 arg6;

        arg1 = STR2CSTR(varg1);
        arg2 = NUM2ULONG(varg2);
        arg3 = rb_str2cstr(varg3, &arg4);
        arg5 = RTEST(varg5);
        arg6 = NUM2LONG(varg6);

        result = arg0->AddData(arg1, arg2, arg3, arg4, arg5, arg6);
      }
      break;

    default:
      rb_bug("rb_message_AddData");
  }
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_message_RemoveData(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1, varg2;
  BMessage *arg0;
  status_t result;

  Get_BInstance(Message, self, arg0);
  switch (rb_scan_args(argc, argv, "11", &varg1, &varg2)) {
    case 1:
      {
        char *arg1;

        arg1 = STR2CSTR(varg1);
        result = arg0->RemoveData(arg1);
      }
      break;

    case 2:
      {
        char *arg1;
        int32 arg2;

        arg1 = STR2CSTR(varg1);
        arg2 = NUM2LONG(varg2);

        result = arg0->RemoveData(arg1, arg2);
      }
      break;

    default:
      rb_bug("rb_message_RemoveData");
  }
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_message_RemoveName(VALUE self, VALUE varg1)
{
  BMessage *arg0;
  char *arg1;
  status_t result;

  Get_BInstance(Message, self, arg0);
  arg1 = STR2CSTR(varg1);
  result = arg0->RemoveName(arg1);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_message_MakeEmpty(VALUE self)
{
  BMessage *arg0;
  status_t result;

  Get_BInstance(Message, self, arg0);
  result = arg0->MakeEmpty();
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_message_FindRect(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1, varg2, vresult = Qnil;
  BMessage *arg0;
  status_t result;
  BRect *copy;

  Get_BInstance(Message, self, arg0);
  switch (rb_scan_args(argc, argv, "11", &varg1, &varg2)) {
    case 1:
      {
        char *arg1;
        BRect arg2;

        arg1 = STR2CSTR(varg1);
        result = arg0->FindRect(arg1, &arg2);
        if (result != B_OK) {
          DEBUG_TRACE();
          my_raise(result);
        }

        if (rb_block_given_p()) {
          for (int32 i=0; arg0->FindRect(arg1, i, &arg2) == B_OK; i++) {
            copy = new Rect(arg2);
            vresult = Data_Wrap_Struct(cRect, 0, delete_rect, copy);
            vresult = rb_yield(vresult);
          }
        }
        else {
          copy = new Rect(arg2);
          vresult = Data_Wrap_Struct(cRect, 0, delete_rect, copy);
        }
      }
      break;

    case 2:
      {
        char *arg1;
        int32 arg2;
        BRect arg3;

        arg1 = STR2CSTR(varg1);
        arg2 = NUM2LONG(varg2);

        result = arg0->FindRect(arg1, arg2, &arg3);
        if (result != B_OK) {
          DEBUG_TRACE();
          my_raise(result);
        }
        copy = new Rect(arg3);
        vresult = Data_Wrap_Struct(cRect, 0, delete_rect, copy);
      }
      break;

    default:
      rb_bug("rb_message_FindRect");
  }
  return vresult;
}


static VALUE
rb_message_FindPoint(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1, varg2, vresult = Qnil;
  BMessage *arg0;
  status_t result;
  BPoint *copy;

  Get_BInstance(Message, self, arg0);
  switch (rb_scan_args(argc, argv, "11", &varg1, &varg2)) {
    case 1:
      {
        char *arg1;
        BPoint arg2;

        arg1 = STR2CSTR(varg1);
        result = arg0->FindPoint(arg1, &arg2);
        if (result != B_OK) {
          DEBUG_TRACE();
          my_raise(result);
        }

        if (rb_block_given_p()) {
          for (int32 i=0; arg0->FindPoint(arg1, i, &arg2) == B_OK; i++) {
            copy = new Point(arg2);
            vresult = Data_Wrap_Struct(cPoint, 0, delete_point, copy);
            vresult = rb_yield(vresult);
          }
        }
        else {
          copy = new Point(arg2);
          vresult = Data_Wrap_Struct(cPoint, 0, delete_point, copy);
        }
      }
      break;

    case 2:
      {
        char *arg1;
        int32 arg2;
        BPoint arg3;

        arg1 = STR2CSTR(varg1);
        arg2 = NUM2LONG(varg2);

        result = arg0->FindPoint(arg1, arg2, &arg3);
        if (result != B_OK) {
          DEBUG_TRACE();
          my_raise(result);
        }
        copy = new Point(arg3);
        vresult = Data_Wrap_Struct(cPoint, 0, delete_point, copy);
      }
      break;

    default:
      rb_bug("rb_message_FindPoint");
  }
  return vresult;
}


static VALUE
rb_message_FindString(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1, varg2, vresult = Qnil;
  BMessage *arg0;
  status_t result;

  Get_BInstance(Message, self, arg0);
  switch (rb_scan_args(argc, argv, "11", &varg1, &varg2)) {
    case 1:
      {
        char *arg1;
        const char *arg2;

        arg1 = STR2CSTR(varg1);
        result = arg0->FindString(arg1, &arg2);
        if (result != B_OK) {
          DEBUG_TRACE();
          my_raise(result);
        }

        if (rb_block_given_p()) {
          for (int32 i=0; arg0->FindString(arg1, i, &arg2) == B_OK; i++) {
            vresult = rb_str_new2(arg2);
            vresult = rb_yield(vresult);
          }
        }
        else {
          vresult = rb_str_new2(arg2);
        }
      }
      break;

    case 2:
      {
        char *arg1;
        int32 arg2;
        const char *arg3;

        arg1 = STR2CSTR(varg1);
        arg2 = NUM2LONG(varg2);

        result = arg0->FindString(arg1, arg2, &arg3);
        if (result != B_OK) {
          DEBUG_TRACE();
          my_raise(result);
        }
        vresult = rb_str_new2(arg3);
      }
      break;

    default:
      rb_bug("rb_message_FindString");
  }
  return vresult;
}


static VALUE
rb_message_FindBString(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1, varg2, vresult = Qnil;
  BMessage *arg0;
  BString *copy;
  status_t result;

  Get_BInstance(Message, self, arg0);
  switch (rb_scan_args(argc, argv, "11", &varg1, &varg2)) {
    case 1:
      {
        char *arg1;
        BString arg2;

        arg1 = STR2CSTR(varg1);

        result = arg0->FindString(arg1, &arg2);
        if (result != B_OK) {
          DEBUG_TRACE();
          my_raise(result);
        }

        if (rb_block_given_p()) {
          for (int32 i=0; arg0->FindString(arg1, i, &arg2) == B_OK; i++) {
            NEW(copy, BString(arg2));
            vresult = Data_Wrap_Struct(cString, 0, delete_string, copy);
            vresult = rb_yield(vresult);
          }
        }
        else {
          NEW(copy, BString(arg2));
          vresult = Data_Wrap_Struct(cString, 0, delete_string, copy);
        }
      }
      break;

    case 2:
      {
        char *arg1;
        int32 arg2;
        BString arg3;

        arg1 = STR2CSTR(varg1);
        arg2 = NUM2LONG(varg2);

        result = arg0->FindString(arg1, arg2, &arg3);
        if (result != B_OK) {
          DEBUG_TRACE();
          my_raise(result);
        }
        NEW(copy, BString(arg3));
        vresult = Data_Wrap_Struct(cString, 0, delete_string, copy);
      }
      break;

    default:
      rb_bug("rb_message_FindBString");
  }
  return vresult;
}


static VALUE
rb_message_FindInt8(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1, varg2, vresult = Qnil;
  BMessage *arg0;
  status_t result;

  Get_BInstance(Message, self, arg0);
  switch (rb_scan_args(argc, argv, "11", &varg1, &varg2)) {
    case 1:
      {
        char *arg1;
        int8 arg2;

        arg1 = STR2CSTR(varg1);

        result = arg0->FindInt8(arg1, &arg2);
        if (result != B_OK) {
          DEBUG_TRACE();
          my_raise(result);
        }

        if (rb_block_given_p()) {
          for (int32 i=0; arg0->FindInt8(arg1, i, &arg2) == B_OK; i++) {
            vresult = CHR2FIX(arg2);
            vresult = rb_yield(vresult);
          }
        }
        else {
          vresult = CHR2FIX(arg2);
        }
      }
      break;

    case 2:
      {
        char *arg1;
        int32 arg2;
        int8 arg3;

        arg1 = STR2CSTR(varg1);
        arg2 = NUM2LONG(varg2);

        result = arg0->FindInt8(arg1, arg2, &arg3);
        if (result != B_OK) {
          DEBUG_TRACE();
          my_raise(result);
        }
        vresult = CHR2FIX(arg3);
      }
      break;

    default:
      rb_bug("rb_message_FindInt8");
  }
  return vresult;
}


static VALUE
rb_message_FindInt16(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1, varg2, vresult = Qnil;
  BMessage *arg0;
  status_t result;

  Get_BInstance(Message, self, arg0);
  switch (rb_scan_args(argc, argv, "11", &varg1, &varg2)) {
    case 1:
      {
        char *arg1;
        int16 arg2;

        arg1 = STR2CSTR(varg1);

        result = arg0->FindInt16(arg1, &arg2);
        if (result != B_OK) {
          DEBUG_TRACE();
          my_raise(result);
        }

        if (rb_block_given_p()) {
          for (int32 i=0; arg0->FindInt16(arg1, i, &arg2) == B_OK; i++) {
            vresult = INT2FIX(arg2);
            vresult = rb_yield(vresult);
          }
        }
        else {
          vresult = INT2FIX(arg2);
        }
      }
      break;

    case 2:
      {
        char *arg1;
        int32 arg2;
        int16 arg3;

        arg1 = STR2CSTR(varg1);
        arg2 = NUM2LONG(varg2);

        result = arg0->FindInt16(arg1, arg2, &arg3);
        if (result != B_OK) {
          DEBUG_TRACE();
          my_raise(result);
        }
        vresult = INT2FIX(arg3);
      }
      break;

    default:
      rb_bug("rb_message_FindInt16");
  }
  return vresult;
}


static VALUE
rb_message_FindInt32(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1, varg2, vresult = Qnil;
  BMessage *arg0;
  status_t result;

  Get_BInstance(Message, self, arg0);
  switch (rb_scan_args(argc, argv, "11", &varg1, &varg2)) {
    case 1:
      {
        char *arg1;
        int32 arg2;

        arg1 = STR2CSTR(varg1);

        result = arg0->FindInt32(arg1, &arg2);
        if (result != B_OK) {
          DEBUG_TRACE();
          my_raise(result);
        }

        if (rb_block_given_p()) {
          for (int32 i=0; arg0->FindInt32(arg1, i, &arg2) == B_OK; i++) {
            vresult = LONG2NUM(arg2);
            vresult = rb_yield(vresult);
          }
        }
        else {
          vresult = LONG2NUM(arg2);
        }
      }
      break;

    case 2:
      {
        char *arg1;
        int32 arg2, arg3;

        arg1 = STR2CSTR(varg1);
        arg2 = NUM2LONG(varg2);

        result = arg0->FindInt32(arg1, arg2, &arg3);
        if (result != B_OK) {
          DEBUG_TRACE();
          my_raise(result);
        }
        vresult = LONG2NUM(arg3);
      }
      break;

    default:
      rb_bug("rb_message_FindInt32");
  }
  return vresult;
}


static VALUE
rb_message_FindInt64(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1, varg2, vresult = Qnil;
  BMessage *arg0;
  status_t result;

  Get_BInstance(Message, self, arg0);
  switch (rb_scan_args(argc, argv, "11", &varg1, &varg2)) {
    case 1:
      {
        char *arg1;
        int64 arg2;

        arg1 = STR2CSTR(varg1);

        result = arg0->FindInt64(arg1, &arg2);
        if (result != B_OK) {
          DEBUG_TRACE();
          my_raise(result);
        }

        if (rb_block_given_p()) {
          for (int32 i=0; arg0->FindInt64(arg1, i, &arg2) == B_OK; i++) {
            vresult = LLONG2NUM(arg2);
            vresult = rb_yield(vresult);
          }
        }
        else {
          vresult = LLONG2NUM(arg2);
        }
      }
      break;

    case 2:
      {
        char *arg1;
        int32 arg2;
        int64 arg3;

        arg1 = STR2CSTR(varg1);
        arg2 = NUM2LONG(varg2);

        result = arg0->FindInt64(arg1, arg2, &arg3);
        if (result != B_OK) {
          DEBUG_TRACE();
          my_raise(result);
        }
        vresult = LLONG2NUM(arg3);
      }
      break;

    default:
      rb_bug("rb_message_FindInt64");
  }
  return vresult;
}


static VALUE
rb_message_FindBool(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1, varg2, vresult = Qnil;
  BMessage *arg0;
  status_t result;

  Get_BInstance(Message, self, arg0);
  switch (rb_scan_args(argc, argv, "11", &varg1, &varg2)) {
    case 1:
      {
        char *arg1;
        bool arg2;

        arg1 = STR2CSTR(varg1);

        result = arg0->FindBool(arg1, &arg2);
        if (result != B_OK) {
          DEBUG_TRACE();
          my_raise(result);
        }

        if (rb_block_given_p()) {
          for (int32 i=0; arg0->FindBool(arg1, i, &arg2) == B_OK; i++) {
            vresult = (arg2 ? Qtrue:Qfalse);
            vresult = rb_yield(vresult);
          }
        }
        else {
          vresult = (arg2 ? Qtrue:Qfalse);
        }
      }
      break;

    case 2:
      {
        char *arg1;
        int32 arg2;
        bool arg3;

        arg1 = STR2CSTR(varg1);
        arg2 = NUM2LONG(varg2);

        result = arg0->FindBool(arg1, arg2, &arg3);
        if (result != B_OK) {
          DEBUG_TRACE();
          my_raise(result);
        }
        vresult = (arg3 ? Qtrue:Qfalse);
      }
      break;

    default:
      rb_bug("rb_message_FindBool");
  }
  return vresult;
}


static VALUE
rb_message_FindFloat(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1, varg2, vresult = Qnil;
  BMessage *arg0;
  status_t result;

  Get_BInstance(Message, self, arg0);
  switch (rb_scan_args(argc, argv, "11", &varg1, &varg2)) {
    case 1:
      {
        char *arg1;
        float arg2;

        arg1 = STR2CSTR(varg1);

        result = arg0->FindFloat(arg1, &arg2);
        if (result != B_OK) {
          DEBUG_TRACE();
          my_raise(result);
        }

        if (rb_block_given_p()) {
          for (int32 i=0; arg0->FindFloat(arg1, i, &arg2) == B_OK; i++) {
            vresult = rb_float_new(static_cast<double>(arg2));
            vresult = rb_yield(vresult);
          }
        }
        else {
          vresult = rb_float_new(static_cast<double>(arg2));
        }
      }
      break;

    case 2:
      {
        char *arg1;
        int32 arg2;
        float arg3;

        arg1 = STR2CSTR(varg1);
        arg2 = NUM2LONG(varg2);

        result = arg0->FindFloat(arg1, arg2, &arg3);
        if (result != B_OK) {
          DEBUG_TRACE();
          my_raise(result);
        }
        vresult = rb_float_new(static_cast<double>(arg3));
      }
      break;

    default:
      rb_bug("rb_message_FindFloat");
  }
  return vresult;
}


static VALUE
rb_message_FindDouble(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1, varg2, vresult = Qnil;
  BMessage *arg0;
  status_t result;

  Get_BInstance(Message, self, arg0);
  switch (rb_scan_args(argc, argv, "11", &varg1, &varg2)) {
    case 1:
      {
        char *arg1;
        double arg2;

        arg1 = STR2CSTR(varg1);

        result = arg0->FindDouble(arg1, &arg2);
        if (result != B_OK) {
          DEBUG_TRACE();
          my_raise(result);
        }

        if (rb_block_given_p()) {
          for (int32 i=0; arg0->FindDouble(arg1, i, &arg2) == B_OK; i++) {
            vresult = rb_float_new(arg2);
            vresult = rb_yield(vresult);
          }
        }
        else {
          vresult = rb_float_new(arg2);
        }
      }
      break;

    case 2:
      {
        char *arg1;
        int32 arg2;
        double arg3;

        arg1 = STR2CSTR(varg1);
        arg2 = NUM2LONG(varg2);

        result = arg0->FindDouble(arg1, arg2, &arg3);
        if (result != B_OK) {
          DEBUG_TRACE();
          my_raise(result);
        }
        vresult = rb_float_new(arg3);
      }
      break;

    default:
      rb_bug("rb_message_FindDouble");
  }
  return vresult;
}


static VALUE
rb_message_FindPointer(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1, varg2, varg3, vresult = Qnil;
  BMessage *arg0;
  status_t result;

  Get_BInstance(Message, self, arg0);
  switch (rb_scan_args(argc, argv, "21", &varg1, &varg2, &varg3)) {
    case 2:
      {
        char *arg1;
        void *arg2 = NULL;

        arg1 = STR2CSTR(varg1);
        Check_Type(varg2, T_CLASS);
        result = arg0->FindPointer(arg1, &arg2);
        if (result != B_OK) {
          DEBUG_TRACE();
          my_raise(result);
        }

        if (rb_block_given_p()) {
          for (int32 i=0; arg0->FindPointer(arg1, i, &arg2) == B_OK; i++) {
            vresult = my_data_wrap_object(varg2, arg2);
            vresult = rb_yield(vresult);
          }
        }
        else {
          vresult = my_data_wrap_object(varg2, arg2);
        }
      }
      break;

    case 3:
      {
        char *arg1;
        int32 arg2;
        void *arg3 = NULL;

        arg1 = STR2CSTR(varg1);
        arg2 = NUM2LONG(varg2);
        Check_Type(varg3, T_CLASS);

        result = arg0->FindPointer(arg1, arg2, &arg3);
        if (result != B_OK) {
          DEBUG_TRACE();
          my_raise(result);
        }
        vresult = my_data_wrap_object(varg3, arg3);
      }
      break;

    default:
      rb_bug("rb_message_FindPointer");
  }
  return vresult;
}


static VALUE
rb_message_FindMessenger(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1, varg2, vresult = Qnil;
  BMessage *arg0;
  status_t result;
  BMessenger *copy;

  Get_BInstance(Message, self, arg0);
  switch (rb_scan_args(argc, argv, "11", &varg1, &varg2)) {
    case 1:
      {
        char *arg1;
        BMessenger arg2;

        arg1 = STR2CSTR(varg1);

        result = arg0->FindMessenger(arg1, &arg2);
        if (result != B_OK) {
          DEBUG_TRACE();
          my_raise(result);
        }

        if (rb_block_given_p()) {
          for (int32 i=0; arg0->FindMessenger(arg1, i, &arg2) == B_OK; i++) {
            copy = new Messenger(arg2);
            vresult = Data_Wrap_Struct(cMessenger, 0, delete_messenger, copy);
            vresult = rb_yield(vresult);
          }
        }
        else {
          copy = new Messenger(arg2);
          vresult = Data_Wrap_Struct(cMessenger, 0, delete_messenger, copy);
        }
      }
      break;

    case 2:
      {
        char *arg1;
        int32 arg2;
        BMessenger arg3;

        arg1 = STR2CSTR(varg1);
        arg2 = NUM2LONG(varg2);

        result = arg0->FindMessenger(arg1, arg2, &arg3);
        if (result != B_OK) {
          DEBUG_TRACE();
          my_raise(result);
        }
        copy = new Messenger(arg3);
        vresult = Data_Wrap_Struct(cMessenger, 0, delete_messenger, copy);
      }
      break;

    default:
      rb_bug("rb_message_FindMessenger");
  }
  return vresult;
}


static VALUE
rb_message_FindRef(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1, varg2, vresult = Qnil;
  BMessage *arg0;
  status_t result;
  entry_ref *copy;

  Get_BInstance(Message, self, arg0);
  switch (rb_scan_args(argc, argv, "11", &varg1, &varg2)) {
    case 1:
      {
        char *arg1;
        entry_ref arg2;

        arg1 = STR2CSTR(varg1);
        result = arg0->FindRef(arg1, &arg2);
        if (result != B_OK) {
          DEBUG_TRACE();
          my_raise(result);
        }

        if (rb_block_given_p()) {
          for (int32 i=0; arg0->FindRef(arg1, i, &arg2) == B_OK; i++) {
            NEW(copy, entry_ref(arg2));
            vresult = Data_Wrap_Struct(cEntry_ref, 0, delete_entry_ref, copy);
            vresult = rb_yield(vresult);
          }
        }
        else {
          NEW(copy, entry_ref(arg2));
          vresult = Data_Wrap_Struct(cEntry_ref, 0, delete_entry_ref, copy);
        }
      }
      break;

    case 2:
      {
        char *arg1;
        int32 arg2;
        entry_ref arg3;

        arg1 = STR2CSTR(varg1);
        arg2 = NUM2LONG(varg2);

        result = arg0->FindRef(arg1, arg2, &arg3);
        if (result != B_OK) {
          DEBUG_TRACE();
          my_raise(result);
        }
        NEW(copy, entry_ref(arg3));
        vresult = Data_Wrap_Struct(cEntry_ref, 0, delete_entry_ref, copy);
      }
      break;

    default:
      rb_bug("rb_message_FindRef");
  }
  return vresult;
}


static VALUE
rb_message_FindMessage(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1, varg2, vresult = Qnil;
  BMessage *arg0;
  status_t result;
  BMessage *copy;

  Get_BInstance(Message, self, arg0);
  switch (rb_scan_args(argc, argv, "11", &varg1, &varg2)) {
    case 1:
      {
        char *arg1;
        BMessage arg2;

        arg1 = STR2CSTR(varg1);

        result = arg0->FindMessage(arg1, &arg2);
        if (result != B_OK) {
          DEBUG_TRACE();
          my_raise(result);
        }

        if (rb_block_given_p()) {
          for (int32 i=0; arg0->FindMessage(arg1, i, &arg2) == B_OK; i++) {
            copy = new Message(arg2);
            vresult = Data_Wrap_Struct(cMessage, 0, delete_message, copy);
            vresult = rb_yield(vresult);
          }
        }
        else {
          copy = new Message(arg2);
          vresult = Data_Wrap_Struct(cMessage, 0, delete_message, copy);
        }
      }
      break;

    case 2:
      {
        char *arg1;
        int32 arg2;
        BMessage arg3;

        arg1 = STR2CSTR(varg1);
        arg2 = NUM2LONG(varg2);

        result = arg0->FindMessage(arg1, arg2, &arg3);
        if (result != B_OK) {
          DEBUG_TRACE();
          my_raise(result);
        }
        copy = new Message(arg3);
        vresult = Data_Wrap_Struct(cMessage, 0, delete_message, copy);
      }
      break;

    default:
      rb_bug("rb_message_FindMessage");
  }
  return vresult;
}


static VALUE
rb_message_FindFlat(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1, varg2, varg3;
  BMessage *arg0;
  status_t result;

  Get_BInstance(Message, self, arg0);
  switch (rb_scan_args(argc, argv, "21", &varg1, &varg2, &varg3)) {
    case 2:
      {
        char *arg1;
        BFlattenable *arg2;

        arg1 = STR2CSTR(varg1);
        Get_BInstance(Flattenable, varg2, arg2);

        result = arg0->FindFlat(arg1, arg2);
        if (result != B_OK) {
          DEBUG_TRACE();
          my_raise(result);
        }
      }
      break;

    case 3:
      {
        char *arg1;
        int32 arg2;
        BFlattenable *arg3;

        arg1 = STR2CSTR(varg1);
        arg2 = NUM2LONG(varg2);
        Get_BInstance(Flattenable, varg3, arg3);

        result = arg0->FindFlat(arg1, arg2, arg3);
        if (result != B_OK) {
          DEBUG_TRACE();
          my_raise(result);
        }
      }
      break;

    default:
      rb_bug("rb_message_FindFlat");
  }
  return Qnil;
}


static VALUE
rb_message_FindData(int argc, VALUE *argv, VALUE self)
{
  BMessage *arg0;
  status_t result;
  VALUE varg1, varg2, varg3, vresult = Qnil;

  Get_BInstance(Message, self, arg0);
  switch (rb_scan_args(argc, argv, "21", &varg1, &varg2, &varg3)) {
    case 2:
      {
        char *arg1;
        int32 arg2;
        const char *arg3;
        ssize_t arg4=0;

        arg1 = STR2CSTR(varg1);
        arg2 = NUM2LONG(varg2);

        result = arg0->FindData(arg1, arg2, reinterpret_cast<const void **>(&arg3), &arg4);
        if (result != B_OK) {
          DEBUG_TRACE();
          my_raise(result);
        }

        if (rb_block_given_p()) {
          for (int32 i=0; arg0->FindData(arg1, arg2, i, reinterpret_cast<const void **>(&arg3), &arg4) == B_OK; i++) {
            vresult = rb_str_new(arg3, arg4);
            vresult = rb_yield(vresult);
          }
        }
        else {
          vresult = rb_str_new(arg3, arg4);
        }
      }
      break;

    case 3:
      {
        char *arg1;
        int32 arg2, arg3;
        const char *arg4;
        ssize_t arg5=0;

        arg1 = STR2CSTR(varg1);
        arg2 = NUM2LONG(varg2);
        arg3 = NUM2LONG(varg3);

        result = arg0->FindData(arg1, arg2, arg3, reinterpret_cast<const void **>(&arg4), &arg5);
        if (result != B_OK) {
          DEBUG_TRACE();
          my_raise(result);
        }
        vresult = rb_str_new(arg4, arg5);
      }
      break;

    default:
      rb_bug("rb_message_FindData");
  }
  return vresult;
}


static VALUE
rb_message_ReplaceRect(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1, varg2, varg3;
  BMessage *arg0;
  status_t result;

  Get_BInstance(Message, self, arg0);
  switch (rb_scan_args(argc, argv, "21", &varg1, &varg2, &varg3)) {
    case 2:
      {
        char *arg1;
        BRect *arg2;

        arg1 = STR2CSTR(varg1);
        Get_BInstance(Rect, varg2, arg2);

        result = arg0->ReplaceRect(arg1, *arg2);
      }
      break;

    case 3:
      {
        char *arg1;
        int32 arg2;
        BRect *arg3;

        arg1 = STR2CSTR(varg1);
        arg2 = NUM2LONG(varg2);
        Get_BInstance(Rect, varg3, arg3);

        result = arg0->ReplaceRect(arg1, arg2, *arg3);
      }
      break;

    default:
      rb_bug("rb_message_ReplaceRect");
  }
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_message_ReplacePoint(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1, varg2, varg3;
  BMessage *arg0;
  status_t result;

  Get_BInstance(Message, self, arg0);
  switch (rb_scan_args(argc, argv, "21", &varg1, &varg2, &varg3)) {
    case 2:
      {
        char *arg1;
        BPoint *arg2;

        arg1 = STR2CSTR(varg1);
        Get_BInstance(Point, varg2, arg2);

        result = arg0->ReplacePoint(arg1, *arg2);
      }
      break;

    case 3:
      {
        char *arg1;
        int32 arg2;
        BPoint *arg3;

        arg1 = STR2CSTR(varg1);
        arg2 = NUM2LONG(varg2);
        Get_BInstance(Point, varg3, arg3);

        result = arg0->ReplacePoint(arg1, arg2, *arg3);
      }
      break;

    default:
      rb_bug("rb_message_ReplacePoint");
  }
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_message_ReplaceString(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1, varg2, varg3;
  BMessage *arg0;
  status_t result;

  Get_BInstance(Message, self, arg0);
  switch (rb_scan_args(argc, argv, "21", &varg1, &varg2, &varg3)) {
    case 2:
      {
        char *arg1;
        arg1 = STR2CSTR(varg1);

        if (TYPE(varg2) == T_STRING) {
          char *arg2;

          arg2 = STR2CSTR(varg2);
          result = arg0->ReplaceString(arg1, arg2);
        }
        else {
          BString *arg2;

          Get_BInstance(String, varg2, arg2);
          result = arg0->ReplaceString(arg1, *arg2);
        }
      }
      break;

    case 3:
      {
        char *arg1;
        int32 arg2;

        arg1 = STR2CSTR(varg1);
        arg2 = NUM2LONG(varg2);

        if (TYPE(varg3) == T_STRING) {
          char *arg3;

          arg3 = STR2CSTR(varg3);
          result = arg0->ReplaceString(arg1, arg2, arg3);
        }
        else {
          BString *arg3;

          Get_BInstance(String, varg3, arg3);
          result = arg0->ReplaceString(arg1, arg2, *arg3);
        }
      }
      break;

    default:
      rb_bug("rb_message_ReplaceString");
  }
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_message_ReplaceInt8(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1, varg2, varg3;
  BMessage *arg0;
  status_t result;

  Get_BInstance(Message, self, arg0);
  switch (rb_scan_args(argc, argv, "21", &varg1, &varg2, &varg3)) {
    case 2:
      {
        char *arg1;
        int8 arg2;

        arg1 = STR2CSTR(varg1);
        arg2 = NUM2CHR(varg2);

        result = arg0->ReplaceInt8(arg1, arg2);
      }
      break;

    case 3:
      {
        char *arg1;
        int32 arg2;
        int8 arg3;

        arg1 = STR2CSTR(varg1);
        arg2 = NUM2LONG(varg2);
        arg3 = NUM2CHR(varg3);

        result = arg0->ReplaceInt8(arg1, arg2, arg3);
      }
      break;

    default:
      rb_bug("rb_message_ReplaceInt8");
  }
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_message_ReplaceInt16(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1, varg2, varg3;
  BMessage *arg0;
  status_t result;

  Get_BInstance(Message, self, arg0);
  switch (rb_scan_args(argc, argv, "21", &varg1, &varg2, &varg3)) {
    case 2:
      {
        char *arg1;
        int16 arg2;

        arg1 = STR2CSTR(varg1);
        arg2 = NUM2SHRT(varg2);

        result = arg0->ReplaceInt16(arg1, arg2);
      }
      break;

    case 3:
      {
        char *arg1;
        int32 arg2;
        int16 arg3;

        arg1 = STR2CSTR(varg1);
        arg2 = NUM2LONG(varg2);
        arg3 = NUM2SHRT(varg3);

        result = arg0->ReplaceInt16(arg1, arg2, arg3);
      }
      break;

    default:
      rb_bug("rb_message_ReplaceInt16");
  }
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_message_ReplaceInt32(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1, varg2, varg3;
  BMessage *arg0;
  status_t result;

  Get_BInstance(Message, self, arg0);
  switch (rb_scan_args(argc, argv, "21", &varg1, &varg2, &varg3)) {
    case 2:
      {
        char *arg1;
        int32 arg2;

        arg1 = STR2CSTR(varg1);
        arg2 = NUM2LONG(varg2);

        result = arg0->ReplaceInt32(arg1, arg2);
      }
      break;

    case 3:
      {
        char *arg1;
        int32 arg2, arg3;

        arg1 = STR2CSTR(varg1);
        arg2 = NUM2LONG(varg2);
        arg3 = NUM2LONG(varg3);

        result = arg0->ReplaceInt32(arg1, arg2, arg3);
      }
      break;

    default:
      rb_bug("rb_message_ReplaceInt32");
  }
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_message_ReplaceInt64(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1, varg2, varg3;
  BMessage *arg0;
  status_t result;

  Get_BInstance(Message, self, arg0);
  switch (rb_scan_args(argc, argv, "21", &varg1, &varg2, &varg3)) {
    case 2:
      {
        char *arg1;
        int64 arg2;

        arg1 = STR2CSTR(varg1);
        arg2 = NUM2LLONG(varg2);

        result = arg0->ReplaceInt64(arg1, arg2);
      }
      break;

    case 3:
      {
        char *arg1;
        int32 arg2;
        int64 arg3;

        arg1 = STR2CSTR(varg1);
        arg2 = NUM2LONG(varg2);
        arg3 = NUM2LLONG(varg3);

        result = arg0->ReplaceInt64(arg1, arg2, arg3);
      }
      break;

    default:
      rb_bug("rb_message_ReplaceInt64");
  }
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_message_ReplaceBool(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1, varg2, varg3;
  BMessage *arg0;
  status_t result;

  Get_BInstance(Message, self, arg0);
  switch (rb_scan_args(argc, argv, "21", &varg1, &varg2, &varg3)) {
    case 2:
      {
        char *arg1;
        bool arg2;

        arg1 = STR2CSTR(varg1);
        arg2 = RTEST(varg2);

        result = arg0->ReplaceBool(arg1, arg2);
      }
      break;

    case 3:
      {
        char *arg1;
        int32 arg2;
        bool arg3;

        arg1 = STR2CSTR(varg1);
        arg2 = NUM2LONG(varg2);
        arg3 = RTEST(varg3);

        result = arg0->ReplaceBool(arg1, arg2, arg3);
      }
      break;

    default:
      rb_bug("rb_message_ReplaceBool");
  }
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_message_ReplaceFloat(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1, varg2, varg3;
  BMessage *arg0;
  status_t result;

  Get_BInstance(Message, self, arg0);
  switch (rb_scan_args(argc, argv, "21", &varg1, &varg2, &varg3)) {
    case 2:
      {
        char *arg1;
        float arg2;

        arg1 = STR2CSTR(varg1);
        arg2 = NUM2DBL(varg2);

        result = arg0->ReplaceFloat(arg1, arg2);
      }
      break;

    case 3:
      {
        char *arg1;
        int32 arg2;
        float arg3;

        arg1 = STR2CSTR(varg1);
        arg2 = NUM2LONG(varg2);
        arg3 = NUM2DBL(varg3);

        result = arg0->ReplaceFloat(arg1, arg2, arg3);
      }
      break;

    default:
      rb_bug("rb_message_ReplaceFloat");
  }
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_message_ReplaceDouble(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1, varg2, varg3;
  BMessage *arg0;
  status_t result;

  Get_BInstance(Message, self, arg0);
  switch (rb_scan_args(argc, argv, "21", &varg1, &varg2, &varg3)) {
    case 2:
      {
        char *arg1;
        double arg2;

        arg1 = STR2CSTR(varg1);
        arg2 = NUM2DBL(varg2);

        result = arg0->ReplaceDouble(arg1, arg2);
      }
      break;

    case 3:
      {
        char *arg1;
        int32 arg2;
        double arg3;

        arg1 = STR2CSTR(varg1);
        arg2 = NUM2LONG(varg2);
        arg3 = NUM2DBL(varg3);

        result = arg0->ReplaceDouble(arg1, arg2, arg3);
      }
      break;

    default:
      rb_bug("rb_message_ReplaceDouble");
  }
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_message_ReplacePointer(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1, varg2, varg3;
  BMessage *arg0;
  status_t result;

  Get_BInstance(Message, self, arg0);
  switch (rb_scan_args(argc, argv, "21", &varg1, &varg2, &varg3)) {
    case 2:
      {
        char *arg1;
        void *arg2;

        arg1 = STR2CSTR(varg1);
        Data_Get_Struct(varg2, void, arg2);

        result = arg0->ReplacePointer(arg1, arg2);
      }
      break;

    case 3:
      {
        char *arg1;
        int32 arg2;
        void *arg3;

        arg1 = STR2CSTR(varg1);
        arg2 = NUM2LONG(varg2);
        Data_Get_Struct(varg3, void, arg3);

        result = arg0->ReplacePointer(arg1, arg2, arg3);
      }
      break;

    default:
      rb_bug("rb_message_ReplacePointer");
  }
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_message_ReplaceMessenger(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1, varg2, varg3;
  BMessage *arg0;
  status_t result;

  Get_BInstance(Message, self, arg0);
  switch (rb_scan_args(argc, argv, "21", &varg1, &varg2, &varg3)) {
    case 2:
      {
        char *arg1;
        BMessenger *arg2;

        arg1 = STR2CSTR(varg1);
        Get_BInstance(Messenger, varg2, arg2);

        result = arg0->ReplaceMessenger(arg1, *arg2);
      }
      break;

    case 3:
      {
        char *arg1;
        int32 arg2;
        BMessenger *arg3;

        arg1 = STR2CSTR(varg1);
        arg2 = NUM2LONG(varg2);
        Get_BInstance(Messenger, varg3, arg3);

        result = arg0->ReplaceMessenger(arg1, arg2, *arg3);
      }
      break;

    default:
      rb_bug("rb_message_ReplaceMessenger");
  }
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_message_ReplaceRef(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1, varg2, varg3;
  BMessage *arg0;
  status_t result;

  Get_BInstance(Message, self, arg0);
  switch (rb_scan_args(argc, argv, "21", &varg1, &varg2, &varg3)) {
    case 2:
      {
        char *arg1;
        entry_ref *arg2;

        arg1 = STR2CSTR(varg1);
        Get_BInstance2(Entry_ref, entry_ref, varg2, arg2);

        result = arg0->ReplaceRef(arg1, arg2);
      }
      break;

    case 3:
      {
        char *arg1;
        int32 arg2;
        entry_ref *arg3;

        arg1 = STR2CSTR(varg1);
        arg2 = NUM2LONG(varg2);
        Get_BInstance2(Entry_ref, entry_ref, varg3, arg3);

        result = arg0->ReplaceRef(arg1, arg2, arg3);
      }
      break;

    default:
      rb_bug("rb_message_ReplaceRef");
  }
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_message_ReplaceMessage(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1, varg2, varg3;
  BMessage *arg0;
  status_t result;

  Get_BInstance(Message, self, arg0);
  switch (rb_scan_args(argc, argv, "21", &varg1, &varg2, &varg3)) {
    case 2:
      {
        char *arg1;
        BMessage *arg2;

        arg1 = STR2CSTR(varg1);
        Get_BInstance(Message, varg2, arg2);

        result = arg0->ReplaceMessage(arg1, arg2);
      }
      break;

    case 3:
      {
        char *arg1;
        int32 arg2;
        BMessage *arg3;

        arg1 = STR2CSTR(varg1);
        arg2 = NUM2LONG(varg2);
        Get_BInstance(Message, varg3, arg3);

        result = arg0->ReplaceMessage(arg1, arg2, arg3);
      }
      break;

    default:
      rb_bug("rb_message_ReplaceMessage");
  }
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_message_ReplaceFlat(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1, varg2, varg3;
  BMessage *arg0;
  status_t result;

  Get_BInstance(Message, self, arg0);
  switch (rb_scan_args(argc, argv, "21", &varg1, &varg2, &varg3)) {
    case 2:
      {
        char *arg1;
        BFlattenable *arg2;

        arg1 = STR2CSTR(varg1);
        Get_BInstance(Flattenable, varg2, arg2);

        result = arg0->ReplaceFlat(arg1, arg2);
      }
      break;

    case 3:
      {
        char *arg1;
        int32 arg2;
        BFlattenable *arg3;

        arg1 = STR2CSTR(varg1);
        arg2 = NUM2LONG(varg2);
        Get_BInstance(Flattenable, varg3, arg3);

        result = arg0->ReplaceFlat(arg1, arg2, arg3);
      }
      break;

    default:
      rb_bug("rb_message_ReplaceFlat");
  }
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_message_ReplaceData(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1, varg2, varg3, varg4;
  BMessage *arg0;
  status_t result;

  Get_BInstance(Message, self, arg0);
  switch (rb_scan_args(argc, argv, "31", &varg1, &varg2, &varg3, &varg4)) {
    case 3:
      {
        char *arg1;
        type_code arg2;
        const char *arg3;
        int arg4;

        arg1 = STR2CSTR(varg1);
        arg2 = NUM2ULONG(varg2);
        arg3 = rb_str2cstr(varg3, &arg4);

        result = arg0->ReplaceData(arg1, arg2, arg3, arg4);
      }
      break;

    case 4:
      {
        char *arg1;
        type_code arg2;
        int32 arg3;
        const char *arg4;
        int arg5;

        arg1 = STR2CSTR(varg1);
        arg2 = NUM2ULONG(varg2);
        arg3 = NUM2LONG(varg3);
        arg4 = rb_str2cstr(varg4, &arg5);

        result = arg0->ReplaceData(arg1, arg2, arg3, arg4, arg5);
      }
      break;

    default:
      rb_bug("rb_message_ReplaceData");
  }
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_message_dump(int argc, VALUE *argv, VALUE self)
{
  VALUE dummy, vresult;

  rb_scan_args(argc, argv, "01", &dummy);
  vresult = rb_message_Flatten(self);

  return vresult;
}


static VALUE
rb_message_load(VALUE klass, VALUE varg1)
{
  BMessage *msg;
  VALUE self;

  msg = new Message();
  self = Data_Wrap_Struct(klass, 0, delete_message, msg);

  rb_message_Unflatten(self, varg1);
  return self;
}


void
Init_Message()
{
  cMessage = rb_define_class_under(mBe, "Message", rb_cObject);
  rb_define_singleton_method(cMessage, "new", RUBY_METHOD_FUNC(rb_message_s_new), -1);
  rb_define_method(cMessage, "initialize", RUBY_METHOD_FUNC(rb_message_initialize), -1);

  rb_define_method(cMessage, "what=", RUBY_METHOD_FUNC(rb_message_what_set), 1);
  rb_define_method(cMessage, "what", RUBY_METHOD_FUNC(rb_message_what_ref), 0);

  rb_define_method(cMessage, "GetInfo", RUBY_METHOD_FUNC(rb_message_GetInfo), -1);

  rb_define_method(cMessage, "CountNames", RUBY_METHOD_FUNC(rb_message_CountNames), 1);
  rb_define_method(cMessage, "IsEmpty", RUBY_METHOD_FUNC(rb_message_IsEmpty), 0);
  rb_define_method(cMessage, "IsSystem", RUBY_METHOD_FUNC(rb_message_IsSystem), 0);
  rb_define_method(cMessage, "IsReply", RUBY_METHOD_FUNC(rb_message_IsReply), 0);
  rb_define_method(cMessage, "PrintToStream", RUBY_METHOD_FUNC(rb_message_PrintToStream), 0);

  rb_define_method(cMessage, "Rename", RUBY_METHOD_FUNC(rb_message_Rename), 2);

  rb_define_method(cMessage, "WasDelivered", RUBY_METHOD_FUNC(rb_message_WasDelivered), 0);
  rb_define_method(cMessage, "IsSourceWaiting", RUBY_METHOD_FUNC(rb_message_IsSourceWaiting), 0);
  rb_define_method(cMessage, "IsSourceRemote", RUBY_METHOD_FUNC(rb_message_IsSourceRemote), 0);
  rb_define_method(cMessage, "ReturnAddress", RUBY_METHOD_FUNC(rb_message_ReturnAddress), 0);
  rb_define_method(cMessage, "Previous", RUBY_METHOD_FUNC(rb_message_Previous), 0);
  rb_define_method(cMessage, "WasDropped", RUBY_METHOD_FUNC(rb_message_WasDropped), 0);
  rb_define_method(cMessage, "DropPoint", RUBY_METHOD_FUNC(rb_message_DropPoint), -1);

  rb_define_method(cMessage, "SendReply", RUBY_METHOD_FUNC(rb_message_SendReply), -1);

  rb_define_method(cMessage, "FlattenedSize", RUBY_METHOD_FUNC(rb_message_FlattenedSize), 0);
  rb_define_method(cMessage, "Flatten", RUBY_METHOD_FUNC(rb_message_Flatten), 0);
  rb_define_method(cMessage, "Unflatten", RUBY_METHOD_FUNC(rb_message_Unflatten), 1);

  rb_define_method(cMessage, "AddSpecifier", RUBY_METHOD_FUNC(rb_message_AddSpecifier), -1);

  rb_define_method(cMessage, "SetCurrentSpecifier", RUBY_METHOD_FUNC(rb_message_SetCurrentSpecifier), 1);
  rb_define_method(cMessage, "GetCurrentSpecifier", RUBY_METHOD_FUNC(rb_message_GetCurrentSpecifier), 0);
  rb_define_method(cMessage, "HasSpecifiers", RUBY_METHOD_FUNC(rb_message_HasSpecifiers), 0);
  rb_define_method(cMessage, "PopSpecifier", RUBY_METHOD_FUNC(rb_message_PopSpecifier), 0);

  rb_define_method(cMessage, "AddRect", RUBY_METHOD_FUNC(rb_message_AddRect), 2);
  rb_define_method(cMessage, "AddPoint", RUBY_METHOD_FUNC(rb_message_AddPoint), 2);
  rb_define_method(cMessage, "AddString", RUBY_METHOD_FUNC(rb_message_AddString), 2);
  rb_define_method(cMessage, "AddInt8", RUBY_METHOD_FUNC(rb_message_AddInt8), 2);
  rb_define_method(cMessage, "AddInt16", RUBY_METHOD_FUNC(rb_message_AddInt16), 2);
  rb_define_method(cMessage, "AddInt32", RUBY_METHOD_FUNC(rb_message_AddInt32), 2);
  rb_define_method(cMessage, "AddInt64", RUBY_METHOD_FUNC(rb_message_AddInt64), 2);
  rb_define_method(cMessage, "AddBool", RUBY_METHOD_FUNC(rb_message_AddBool), 2);
  rb_define_method(cMessage, "AddFloat", RUBY_METHOD_FUNC(rb_message_AddFloat), 2);
  rb_define_method(cMessage, "AddDouble", RUBY_METHOD_FUNC(rb_message_AddDouble), 2);
  rb_define_method(cMessage, "AddPointer", RUBY_METHOD_FUNC(rb_message_AddPointer), 2);
  rb_define_method(cMessage, "AddMessenger", RUBY_METHOD_FUNC(rb_message_AddMessenger), 2);
  rb_define_method(cMessage, "AddRef", RUBY_METHOD_FUNC(rb_message_AddRef), 2);
  rb_define_method(cMessage, "AddMessage", RUBY_METHOD_FUNC(rb_message_AddMessage), 2);
  rb_define_method(cMessage, "AddFlat", RUBY_METHOD_FUNC(rb_message_AddFlat), -1);
  rb_define_method(cMessage, "AddData", RUBY_METHOD_FUNC(rb_message_AddData), -1);

  rb_define_method(cMessage, "RemoveData", RUBY_METHOD_FUNC(rb_message_RemoveData), -1);
  rb_define_method(cMessage, "RemoveName", RUBY_METHOD_FUNC(rb_message_RemoveName), 1);
  rb_define_method(cMessage, "MakeEmpty", RUBY_METHOD_FUNC(rb_message_MakeEmpty), 0);

  rb_define_method(cMessage, "FindRect", RUBY_METHOD_FUNC(rb_message_FindRect), -1);
  rb_define_method(cMessage, "FindPoint", RUBY_METHOD_FUNC(rb_message_FindPoint), -1);
  rb_define_method(cMessage, "FindString", RUBY_METHOD_FUNC(rb_message_FindString), -1);
  rb_define_method(cMessage, "FindBString", RUBY_METHOD_FUNC(rb_message_FindBString), -1);
  rb_define_method(cMessage, "FindInt8", RUBY_METHOD_FUNC(rb_message_FindInt8), -1);
  rb_define_method(cMessage, "FindInt16", RUBY_METHOD_FUNC(rb_message_FindInt16), -1);
  rb_define_method(cMessage, "FindInt32", RUBY_METHOD_FUNC(rb_message_FindInt32), -1);
  rb_define_method(cMessage, "FindInt64", RUBY_METHOD_FUNC(rb_message_FindInt64), -1);
  rb_define_method(cMessage, "FindBool", RUBY_METHOD_FUNC(rb_message_FindBool), -1);
  rb_define_method(cMessage, "FindFloat", RUBY_METHOD_FUNC(rb_message_FindFloat), -1);
  rb_define_method(cMessage, "FindDouble", RUBY_METHOD_FUNC(rb_message_FindDouble), -1);
  rb_define_method(cMessage, "FindPointer", RUBY_METHOD_FUNC(rb_message_FindPointer), -1);
  rb_define_method(cMessage, "FindMessenger", RUBY_METHOD_FUNC(rb_message_FindMessenger), -1);
  rb_define_method(cMessage, "FindRef", RUBY_METHOD_FUNC(rb_message_FindRef), -1);
  rb_define_method(cMessage, "FindMessage", RUBY_METHOD_FUNC(rb_message_FindMessage), -1);
  rb_define_method(cMessage, "FindFlat", RUBY_METHOD_FUNC(rb_message_FindFlat), -1);
  rb_define_method(cMessage, "FindData", RUBY_METHOD_FUNC(rb_message_FindData), -1);

  rb_define_method(cMessage, "ReplaceRect", RUBY_METHOD_FUNC(rb_message_ReplaceRect), -1);
  rb_define_method(cMessage, "ReplacePoint", RUBY_METHOD_FUNC(rb_message_ReplacePoint), -1);
  rb_define_method(cMessage, "ReplaceString", RUBY_METHOD_FUNC(rb_message_ReplaceString), -1);
  rb_define_method(cMessage, "ReplaceInt8", RUBY_METHOD_FUNC(rb_message_ReplaceInt8), -1);
  rb_define_method(cMessage, "ReplaceInt16", RUBY_METHOD_FUNC(rb_message_ReplaceInt16), -1);
  rb_define_method(cMessage, "ReplaceInt32", RUBY_METHOD_FUNC(rb_message_ReplaceInt32), -1);
  rb_define_method(cMessage, "ReplaceInt64", RUBY_METHOD_FUNC(rb_message_ReplaceInt64), -1);
  rb_define_method(cMessage, "ReplaceBool", RUBY_METHOD_FUNC(rb_message_ReplaceBool), -1);
  rb_define_method(cMessage, "ReplaceFloat", RUBY_METHOD_FUNC(rb_message_ReplaceFloat), -1);
  rb_define_method(cMessage, "ReplaceDouble", RUBY_METHOD_FUNC(rb_message_ReplaceDouble), -1);
  rb_define_method(cMessage, "ReplacePointer", RUBY_METHOD_FUNC(rb_message_ReplacePointer), -1);
  rb_define_method(cMessage, "ReplaceMessenger", RUBY_METHOD_FUNC(rb_message_ReplaceMessenger), -1);
  rb_define_method(cMessage, "ReplaceRef", RUBY_METHOD_FUNC(rb_message_ReplaceRef), -1);
  rb_define_method(cMessage, "ReplaceMessage", RUBY_METHOD_FUNC(rb_message_ReplaceMessage), -1);
  rb_define_method(cMessage, "ReplaceFlat", RUBY_METHOD_FUNC(rb_message_ReplaceFlat), -1);
  rb_define_method(cMessage, "ReplaceData", RUBY_METHOD_FUNC(rb_message_ReplaceData), -1);

  rb_define_method(cMessage, "_dump", RUBY_METHOD_FUNC(rb_message_dump), -1);
  rb_define_singleton_method(cMessage, "_load", RUBY_METHOD_FUNC(rb_message_load), 1);

  rb_define_const(mBe, "FIELD_NAME_LENGTH", ULONG2NUM(B_FIELD_NAME_LENGTH));
  rb_define_const(mBe, "PROPERTY_NAME_LENGTH", ULONG2NUM(B_PROPERTY_NAME_LENGTH));

  rb_define_const(mBe, "NO_SPECIFIER", ULONG2NUM(B_NO_SPECIFIER));
  rb_define_const(mBe, "DIRECT_SPECIFIER", ULONG2NUM(B_DIRECT_SPECIFIER));
  rb_define_const(mBe, "INDEX_SPECIFIER", ULONG2NUM(B_INDEX_SPECIFIER));
  rb_define_const(mBe, "REVERSE_INDEX_SPECIFIER", ULONG2NUM(B_REVERSE_INDEX_SPECIFIER));
  rb_define_const(mBe, "RANGE_SPECIFIER", ULONG2NUM(B_RANGE_SPECIFIER));
  rb_define_const(mBe, "REVERSE_RANGE_SPECIFIER", ULONG2NUM(B_REVERSE_RANGE_SPECIFIER));
  rb_define_const(mBe, "NAME_SPECIFIER", ULONG2NUM(B_NAME_SPECIFIER));
  rb_define_const(mBe, "ID_SPECIFIER", ULONG2NUM(B_ID_SPECIFIER));
  rb_define_const(mBe, "SPECIFIERS_END", ULONG2NUM(B_SPECIFIERS_END));
}
