/*
 *  MessageQueue.cc
 */

#include <MessageQueue.h>
#include "my_be.h"


VALUE cMessageQueue;


void
delete_messagequeue(BMessageQueue *arg0)
{
  delete arg0;
}


static VALUE
rb_messagequeue_s_new(int argc, VALUE *argv, VALUE klass)
{
  VALUE vresult = my_class_s_new(argc, argv, klass, delete_messagequeue);
  return vresult;
}


static VALUE
rb_messagequeue_initialize(VALUE self)
{
  BMessageQueue *result;

  Data_To_Null(self, BMessageQueue);
  result = my_create_be_obj<BMessageQueue>();
  DATA_PTR(self) = result;
  return self;
}


static VALUE
rb_messagequeue_AddMessage(VALUE self, VALUE varg1)
{
  BMessageQueue *arg0;
  BMessage *arg1;

  Get_BInstance(MessageQueue, self, arg0);
  Get_BInstance(Message, varg1, arg1);
  arg0->AddMessage(arg1);
  return Qnil;
}


static VALUE
rb_messagequeue_RemoveMessage(VALUE self, VALUE varg1)
{
  BMessageQueue *arg0;
  BMessage *arg1;
  bool result;
  VALUE vresult;

  Get_BInstance(MessageQueue, self, arg0);
  Get_BInstance(Message, varg1, arg1);
  result = arg0->RemoveMessage(arg1);
  vresult = (result ? Qtrue:Qfalse);
  return vresult;
}


static VALUE
rb_messagequeue_NextMessage(VALUE self)
{
  BMessageQueue *arg0;
  BMessage *result;
  VALUE vresult;

  Get_BInstance(MessageQueue, self, arg0);
  result = arg0->NextMessage();
  vresult = Data_Wrap_Struct(cMessage, 0, 0, result);
  return vresult;
}


static VALUE
rb_messagequeue_FindMessage(int argc, VALUE *argv, VALUE self)
{
  BMessageQueue *arg0;
  BMessage *result;
  VALUE varg1, varg2, vresult;

  Get_BInstance(MessageQueue, self, arg0);
  switch (rb_scan_args(argc, argv, "11", &varg1, &varg2)) {
    case 1:
      {
        int32 arg1;

        arg1 = NUM2LONG(varg1);
        result = arg0->FindMessage(arg1);
      }
      break;

    case 2:
      {
        uint32 arg1;
        int32 arg2;

        arg1 = NUM2LONG(varg1);
        arg2 = NUM2ULONG(varg2);
        result = arg0->FindMessage(arg1, arg2);
      }
      break;

    default:
      rb_bug("rb_messagequeue_FindMessage");
  }
  vresult = Data_Wrap_Struct(cMessage, 0, 0, result);
  return vresult;
}


static VALUE
rb_messagequeue_CountMessages(VALUE self)
{
  BMessageQueue *arg0;
  int32 result;
  VALUE vresult;

  Get_BInstance(MessageQueue, self, arg0);
  result = arg0->CountMessages();
  vresult = LONG2NUM(result);
  return vresult;
}


static VALUE
rb_messagequeue_IsEmpty(VALUE self)
{
  BMessageQueue *arg0;
  bool result;
  VALUE vresult;

  Get_BInstance(MessageQueue, self, arg0);
  result = arg0->IsEmpty();
  vresult = (result ? Qtrue:Qfalse);
  return vresult;
}


static VALUE
rb_messagequeue_Unlock(VALUE self)
{
  BMessageQueue *arg0;

  Get_BInstance(MessageQueue, self, arg0);
  arg0->Unlock();
  return Qnil;
}


static VALUE
yld_messagequeue_Lock(VALUE self)
{
  BMessageQueue *arg0;
  VALUE vresult = Qnil;

  Get_BInstance(MessageQueue, self, arg0);
  if (arg0->Lock()) {
    vresult = rb_yield(self);
  }
  return vresult;
}


static VALUE
rb_messagequeue_Lock(VALUE self)
{
  BMessageQueue *arg0;
  VALUE vresult;

  if (rb_block_given_p()) {
    vresult = rb_ensure(RUBY_METHOD_FUNC(yld_messagequeue_Lock), self,
                        RUBY_METHOD_FUNC(rb_messagequeue_Unlock), self);
  }
  else {
    bool result;

    Get_BInstance(MessageQueue, self, arg0);
    result = arg0->Lock();
    vresult = (result ? Qtrue:Qfalse);
  }
  return vresult;
}


void
Init_MessageQueue()
{
  cMessageQueue = rb_define_class_under(mBe, "MessageQueue", rb_cObject);
  rb_define_singleton_method(cMessageQueue, "new", RUBY_METHOD_FUNC(rb_messagequeue_s_new), -1);
  rb_define_method(cMessageQueue, "initialize", RUBY_METHOD_FUNC(rb_messagequeue_initialize), 0);

  rb_define_method(cMessageQueue, "AddMessage", RUBY_METHOD_FUNC(rb_messagequeue_AddMessage), 1);
  rb_define_method(cMessageQueue, "RemoveMessage", RUBY_METHOD_FUNC(rb_messagequeue_RemoveMessage), 1);
  rb_define_method(cMessageQueue, "NextMessage", RUBY_METHOD_FUNC(rb_messagequeue_NextMessage), 0);
  rb_define_method(cMessageQueue, "FindMessage", RUBY_METHOD_FUNC(rb_messagequeue_FindMessage), -1);
  rb_define_method(cMessageQueue, "CountMessages", RUBY_METHOD_FUNC(rb_messagequeue_CountMessages), 0);
  rb_define_method(cMessageQueue, "IsEmpty", RUBY_METHOD_FUNC(rb_messagequeue_IsEmpty), 0);

  rb_define_method(cMessageQueue, "Lock", RUBY_METHOD_FUNC(rb_messagequeue_Lock), 0);
  rb_define_method(cMessageQueue, "Unlock", RUBY_METHOD_FUNC(rb_messagequeue_Unlock), 0);
}
