/*
 *  Messenger.cc
 */

#include "rbMessenger.h"
#include "my_be.h"


VALUE cMessenger;


void
delete_messenger(BMessenger *arg0)
{
  delete arg0;
}


static VALUE
rb_messenger_s_new(int argc, VALUE *argv, VALUE klass)
{
  VALUE vresult = my_class_s_new(argc, argv, klass, delete_messenger);
  return vresult;
}


static VALUE
rb_messenger_initialize(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1, varg2;
  BMessenger *result;
  status_t arg3 = B_OK;

  Data_To_Null(self, BMessenger);

  switch (rb_scan_args(argc, argv, "02", &varg1, &varg2)) {
    case 0:
      result = new Messenger();
      break;

    case 1:
      if (rb_obj_is_kind_of(varg1, cMessenger)) {
        BMessenger *arg1;

        Get_BInstance(Messenger, varg1, arg1);
        result = new Messenger(*arg1);
      }
      else if (rb_obj_is_kind_of(varg1, cHandler)) {
        BHandler *arg1;

        Get_BInstance(Handler, varg1, arg1);
        result = new Messenger(arg1, NULL, &arg3);
      }
      else {
        char *arg1;

        arg1 = STR2CSTR(varg1);
        result = new Messenger(arg1, -1, &arg3);
      }
      break;

    case 2:
      if (rb_obj_is_kind_of(varg1, cHandler)) {
        BHandler *arg1 = NULL;
        BLooper *arg2;

        if (!NIL_P(varg1)) {
          Get_BInstance(Handler, varg1, arg1);
        }
        Get_BInstance(Looper, varg2, arg2);
        result = new Messenger(arg1, arg2, &arg3);
      }
      else {
        char *arg1 = NULL;
        team_id arg2;

        if (!NIL_P(varg1)) {
          arg1 = STR2CSTR(varg1);
        }
        arg2 = NUM2LONG(varg2);
        result = new Messenger(arg1, arg2, &arg3);
      }
      break;

    default:
      rb_bug("rb_messenger_initialize");
  }
  DATA_PTR(self) = result;
  if (arg3 != B_OK) {
    DEBUG_TRACE();
    my_raise(arg3);
  }
  return self;
}


static VALUE
rb_messenger_IsTargetLocal(VALUE self)
{
  BMessenger *arg0;
  bool result;
  VALUE vresult;

  Get_BInstance(Messenger, self, arg0);
  result = arg0->IsTargetLocal();
  vresult = (result ? Qtrue:Qfalse);
  return vresult;
}


static VALUE
rb_messenger_Target(VALUE self)
{
  BMessenger *arg0;
  BLooper *arg1;
  BHandler *result;
  VALUE vresult;

  Get_BInstance(Messenger, self, arg0);
  result = arg0->Target(&arg1);
  vresult = rb_ary_new3(2, ((result) ? Data_Wrap_Struct(cHandler, 0, 0, result):Qnil),
                           ((arg1) ? Data_Wrap_Struct(cLooper, 0, 0, arg1):Qnil));
  return vresult;
}


static VALUE
rb_messenger_LockTarget(VALUE self)
{
  BMessenger *arg0;
  bool result;
  VALUE vresult;

  Get_BInstance(Messenger, self, arg0);
  result = arg0->LockTarget();
  vresult = (result ? Qtrue:Qfalse);
  return vresult;
}


static VALUE
rb_messenger_LockTargetWithTimeout(VALUE self, VALUE varg1)
{
  BMessenger *arg0;
  bigtime_t arg1;
  status_t result;

  Get_BInstance(Messenger, self, arg0);
  arg1 = NUM2LLONG(varg1);
  result = arg0->LockTargetWithTimeout(arg1);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_messenger_SendMessage(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1, varg2, varg3, varg4;
  BMessenger *arg0;
  status_t result;

  Get_BInstance(Messenger, self, arg0);
  switch (rb_scan_args(argc, argv, "13", &varg1, &varg2, &varg3, &varg4)) {
    case 1:
      if (rb_obj_is_kind_of(varg1, cMessage)) {
        BMessage *arg1;

        Get_BInstance(Message, varg1, arg1);
        result = arg0->SendMessage(arg1);
      }
      else {
        uint32 arg1;

        arg1 = NUM2ULONG(varg1);
        result = arg0->SendMessage(arg1);
      }
      break;

    case 2:
      if (rb_obj_is_kind_of(varg1, cMessage)) {
        BMessage *arg1;
        Get_BInstance(Message, varg1, arg1);

        if (rb_obj_is_kind_of(varg2, cHandler)) {
          BHandler *arg2;

          Get_BInstance(Handler, varg2, arg2);
          result = arg0->SendMessage(arg1, arg2);
        }
        else if (rb_obj_is_kind_of(varg2, cMessage)) {
          BMessage *arg2;

          Get_BInstance(Message, varg2, arg2);
          result = arg0->SendMessage(arg1, arg2);
        }
        else {
          BMessenger *arg2;

          Get_BInstance(Messenger, varg2, arg2);
          result = arg0->SendMessage(arg1, *arg2);
        }
      }
      else {
        uint32 arg1;
        arg1 = NUM2ULONG(varg1);

        if (rb_obj_is_kind_of(varg2, cMessage)) {
          BMessage *arg2;

          Get_BInstance(Message, varg2, arg2);
          result = arg0->SendMessage(arg1, arg2);
        }
        else {
          BHandler *arg2;

          Get_BInstance(Handler, varg2, arg2);
          result = arg0->SendMessage(arg1, arg2);
        }
      }
      break;

    case 3:
      {
        BMessage *arg1;
        Get_BInstance(Message, varg1, arg1);

        if (rb_obj_is_kind_of(varg2, cHandler)) {
          BHandler *arg2;
          bigtime_t arg3;

          Get_BInstance(Handler, varg2, arg2);
          arg3 = NUM2LLONG(varg3);
          result = arg0->SendMessage(arg1, arg2, arg3);
        }
        else if (rb_obj_is_kind_of(varg2, cMessage)) {
          BMessage *arg2;
          bigtime_t arg3;

          Get_BInstance(Message, varg2, arg2);
          arg3 = NUM2LLONG(varg3);
          result = arg0->SendMessage(arg1, arg2, arg3);
        }
        else {
          BMessenger *arg2;
          bigtime_t arg3;

          Get_BInstance(Messenger, varg2, arg2);
          arg3 = NUM2LLONG(varg3);
          result = arg0->SendMessage(arg1, *arg2, arg3);
        }
      }
      break;

    case 4:
      {
        BMessage *arg1, *arg2;
        bigtime_t arg3, arg4;

        Get_BInstance(Message, varg1, arg1);
        Get_BInstance(Message, varg2, arg2);
        arg3 = NUM2LLONG(varg3);
        arg4 = NUM2LLONG(varg4);
        result = arg0->SendMessage(arg1, arg2, arg3, arg4);
      }
      break;

    default:
      rb_bug("rb_messenger_SendMessage");
  }
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_messenger_equal(VALUE self, VALUE varg1)
{
  BMessenger *arg0, *arg1;

  if (!rb_obj_is_kind_of(varg1, cMessenger)) {
    return Qfalse;
  }

  Get_BInstance(Messenger, self, arg0);
  Get_BInstance(Messenger, varg1, arg1);

  if (*arg0 == *arg1) {
    return Qtrue;
  }
  return Qfalse;
}


static VALUE
rb_messenger_IsValid(VALUE self)
{
  BMessenger *arg0;
  bool result;
  VALUE vresult = Qnil;

  Get_BInstance(Messenger, self, arg0);
  result = arg0->IsValid();
  vresult = (result ? Qtrue:Qfalse);
  return vresult;
}


static VALUE
rb_messenger_Team(VALUE self)
{
  BMessenger *arg0;
  team_id result;
  VALUE vresult;

  Get_BInstance(Messenger, self, arg0);
  result = arg0->Team();
  vresult = LONG2NUM(result);
  return vresult;
}


void
Init_Messenger()
{
  cMessenger = rb_define_class_under(mBe, "Messenger", rb_cObject);
  rb_define_singleton_method(cMessenger, "new", RUBY_METHOD_FUNC(rb_messenger_s_new), -1);
  rb_define_method(cMessenger, "initialize", RUBY_METHOD_FUNC(rb_messenger_initialize), -1);

  rb_define_method(cMessenger, "IsTargetLocal", RUBY_METHOD_FUNC(rb_messenger_IsTargetLocal), 0);
  rb_define_method(cMessenger, "Target", RUBY_METHOD_FUNC(rb_messenger_Target), 0);
  rb_define_method(cMessenger, "LockTarget", RUBY_METHOD_FUNC(rb_messenger_LockTarget), 0);
  rb_define_method(cMessenger, "LockTargetWithTimeout", RUBY_METHOD_FUNC(rb_messenger_LockTargetWithTimeout), 1);

  rb_define_method(cMessenger, "SendMessage", RUBY_METHOD_FUNC(rb_messenger_SendMessage), -1);

  rb_define_method(cMessenger, "==", RUBY_METHOD_FUNC(rb_messenger_equal), 1);

  rb_define_method(cMessenger, "IsValid", RUBY_METHOD_FUNC(rb_messenger_IsValid), 0);
  rb_define_method(cMessenger, "Team", RUBY_METHOD_FUNC(rb_messenger_Team), 0);
}
