/*
 *  NodeMonitor.cc
 */

#include <NodeMonitor.h>
#include "my_be.h"


static VALUE
rb_watch_node(int argc, VALUE *argv, VALUE self)
{
  status_t result;
  VALUE varg1, varg2, varg3, varg4;

  switch (rb_scan_args(argc, argv, "31", &varg1, &varg2, &varg3, &varg4)) {
    case 3:
      if (rb_obj_is_kind_of(varg3, cMessenger)) {
        node_ref *arg1;
        uint32 arg2;
        BMessenger *arg3;

        Get_BInstance2(Node_ref, node_ref, varg1, arg1);
        arg2 = NUM2ULONG(varg2);
        Get_BInstance(Messenger, varg3, arg3);
        result = watch_node(arg1, arg2, *arg3);
      }
      else {
        node_ref *arg1;
        uint32 arg2;
        BHandler *arg3;

        Get_BInstance2(Node_ref, node_ref, varg1, arg1);
        arg2 = NUM2ULONG(varg2);
        Get_BInstance(Handler, varg3, arg3);
        result = watch_node(arg1, arg2, arg3);
      }
      break;

    case 4:
      {
        node_ref *arg1;
        uint32 arg2;
        BHandler *arg3;
        BLooper *arg4 = NULL;

        Get_BInstance2(Node_ref, node_ref, varg1, arg1);
        arg2 = NUM2ULONG(varg2);
        Get_BInstance(Handler, varg3, arg3);
        if (!NIL_P(varg4)) {
          Get_BInstance(Looper, varg4, arg4);
        }
        result = watch_node(arg1, arg2, arg3, arg4);
      }
      break;

    default:
      rb_bug("rb_watch_node");
  }
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_stop_watching(int argc, VALUE *argv, VALUE self)
{
  status_t result;
  VALUE varg1, varg2;

  switch (rb_scan_args(argc, argv, "11", &varg1, &varg2)) {
    case 1:
      if (rb_obj_is_kind_of(varg1, cMessenger)) {
        BMessenger *arg1;

        Get_BInstance(Messenger, varg1, arg1);
        result = stop_watching(*arg1);
      }
      else {
        BHandler *arg1;

        Get_BInstance(Handler, varg1, arg1);
        result = stop_watching(arg1);
      }
      break;

    case 2:
      {
        BHandler *arg1;
        BLooper *arg2 = NULL;

        Get_BInstance(Handler, varg1, arg1);
        if (!NIL_P(varg2)) {
          Get_BInstance(Looper, varg2, arg2);
        }
        result = stop_watching(arg1, arg2);
      }
      break;

    default:
      rb_bug("rb_stop_watching");
  }
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


void
Init_NodeMonitor()
{
  rb_define_module_function(mBe, "watch_node", RUBY_METHOD_FUNC(rb_watch_node), -1);
  rb_define_module_function(mBe, "stop_watching", RUBY_METHOD_FUNC(rb_stop_watching), -1);

  rb_define_const(mBe, "STOP_WATCHING", ULONG2NUM(B_STOP_WATCHING));
  rb_define_const(mBe, "WATCH_NAME", ULONG2NUM(B_WATCH_NAME));
  rb_define_const(mBe, "WATCH_STAT", ULONG2NUM(B_WATCH_STAT));
  rb_define_const(mBe, "WATCH_ATTR", ULONG2NUM(B_WATCH_ATTR));
  rb_define_const(mBe, "WATCH_DIRECTORY", ULONG2NUM(B_WATCH_DIRECTORY));
  rb_define_const(mBe, "WATCH_ALL", ULONG2NUM(B_WATCH_ALL));
  rb_define_const(mBe, "WATCH_MOUNT", ULONG2NUM(B_WATCH_MOUNT));

  rb_define_const(mBe, "ENTRY_CREATED", ULONG2NUM(B_ENTRY_CREATED));
  rb_define_const(mBe, "ENTRY_REMOVED", ULONG2NUM(B_ENTRY_REMOVED));
  rb_define_const(mBe, "ENTRY_MOVED", ULONG2NUM(B_ENTRY_MOVED));
  rb_define_const(mBe, "STAT_CHANGED", ULONG2NUM(B_STAT_CHANGED));
  rb_define_const(mBe, "ATTR_CHANGED", ULONG2NUM(B_ATTR_CHANGED));
  rb_define_const(mBe, "DEVICE_MOUNTED", ULONG2NUM(B_DEVICE_MOUNTED));
  rb_define_const(mBe, "DEVICE_UNMOUNTED", ULONG2NUM(B_DEVICE_UNMOUNTED));
}
