/*
 *  Path.cc
 */

#include <Path.h>
#include "my_be.h"


VALUE cPath;


void
delete_path(BPath *arg0)
{
  delete arg0;
}


static VALUE
rb_path_s_new(int argc, VALUE *argv, VALUE klass)
{
  VALUE vresult = my_class_s_new(argc, argv, klass, delete_path);
  return vresult;
}


static VALUE
rb_path_initialize(int argc, VALUE *argv, VALUE self)
{
  BPath *result;
  VALUE varg1, varg2, varg3;

  Data_To_Null(self, BPath);

  switch (rb_scan_args(argc, argv, "03", &varg1, &varg2, &varg3)) {
    case 0:
      result = my_create_be_obj<BPath>();
      break;

    case 1:
      if (rb_obj_is_kind_of(varg1, cPath)) {
        BPath *arg1;

        Get_BInstance(Path, varg1, arg1);
        NEW(result, BPath(*arg1));
      }
      else if (rb_obj_is_kind_of(varg1, cEntry)) {
        BEntry *arg1;

        Get_BInstance(Entry, varg1, arg1);
        NEW(result, BPath(arg1));
      }
      else if (rb_obj_is_kind_of(varg1, cEntry_ref)) {
        entry_ref *arg1;

        Get_BInstance2(Entry_ref, entry_ref, varg1, arg1);
        NEW(result, BPath(arg1));
      }
      else {
        char *arg1;

        arg1 = STR2CSTR(varg1);
        NEW(result, BPath(arg1));
      }
      break;

    case 2:
      if (rb_obj_is_kind_of(varg1, cDirectory)) {
        BDirectory *arg1;
        char *arg2;

        Get_BInstance(Directory, varg1, arg1);
        arg2 = STR2CSTR(varg2);

        NEW(result, BPath(arg1, arg2));
      }
      else {
        char *arg1, *arg2;

        arg1 = STR2CSTR(varg1);
        arg2 = STR2CSTR(varg2);

        NEW(result, BPath(arg1, arg2));
      }
      break;

    case 3:
      if (rb_obj_is_kind_of(varg1, cDirectory)) {
        BDirectory *arg1;
        char *arg2;
        bool arg3;

        Get_BInstance(Directory, varg1, arg1);
        arg2 = STR2CSTR(varg2);
        arg3 = RTEST(varg3);

        NEW(result, BPath(arg1, arg2, arg3));
      }
      else {
        char *arg1, *arg2;
        bool arg3;

        arg1 = STR2CSTR(varg1);
        arg2 = STR2CSTR(varg2);
        arg3 = RTEST(varg3);

        NEW(result, BPath(arg1, arg2, arg3));
      }
      break;

    default:
      rb_bug("rb_path_initialize");
  }
  DATA_PTR(self) = result;
  return self;
}


static VALUE
rb_path_InitCheck(VALUE self)
{
  BPath *arg0;
  status_t result;

  Get_BInstance(Path, self, arg0);
  result = arg0->InitCheck();
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_path_SetTo(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1, varg2, varg3;
  BPath *arg0;
  status_t result;

  Get_BInstance(Path, self, arg0);
  switch (rb_scan_args(argc, argv, "12", &varg1, &varg2, &varg3)) {
    case 1:
      if (rb_obj_is_kind_of(varg1, cEntry)) {
        BEntry *arg1;

        Get_BInstance(Entry, varg1, arg1);
        result = arg0->SetTo(arg1);
      }
      else if (rb_obj_is_kind_of(varg1, cEntry_ref)) {
        entry_ref *arg1;

        Get_BInstance2(Entry_ref, entry_ref, varg1, arg1);
        result = arg0->SetTo(arg1);
      }
      else {
        char *arg1;

        arg1 = STR2CSTR(varg1);
        result = arg0->SetTo(arg1);
      }
      break;

    case 2:
      if (rb_obj_is_kind_of(varg1, cDirectory)) {
        BDirectory *arg1;
        char *arg2;

        Get_BInstance(Directory, varg1, arg1);
        arg2 = STR2CSTR(varg2);

        result = arg0->SetTo(arg1, arg2);
      }
      else {
        char *arg1, *arg2;

        arg1 = STR2CSTR(varg1);
        arg2 = STR2CSTR(varg2);

        result = arg0->SetTo(arg1, arg2);
      }
      break;

    case 3:
      if (rb_obj_is_kind_of(varg1, cDirectory)) {
        BDirectory *arg1;
        char *arg2;
        bool arg3;

        Get_BInstance(Directory, varg1, arg1);
        arg2 = STR2CSTR(varg2);
        arg3 = RTEST(varg3);

        result = arg0->SetTo(arg1, arg2, arg3);
      }
      else {
        char *arg1, *arg2;
        bool arg3;

        arg1 = STR2CSTR(varg1);
        arg2 = STR2CSTR(varg2);
        arg3 = RTEST(varg3);

        result = arg0->SetTo(arg1, arg2, arg3);
      }
      break;

    default:
      rb_bug("rb_path_SetTo");
  }
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_path_Append(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1, varg2;
  BPath *arg0;
  status_t result;

  Get_BInstance(Path, self, arg0);
  switch (rb_scan_args(argc, argv, "11", &varg1, &varg2)) {
    case 1:
      {
        char *arg1;

        arg1 = STR2CSTR(varg1);
        result = arg0->Append(arg1);
      }
      break;

    case 2:
      {
        char *arg1;
        bool arg2;

        arg1 = STR2CSTR(varg1);
        arg2 = RTEST(varg2);

        result = arg0->Append(arg1, arg2);
      }
      break;

    default:
      rb_bug("rb_path_Append");
  }
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_path_Unset(VALUE self)
{
  BPath *arg0;

  Get_BInstance(Path, self, arg0);
  arg0->Unset();
  return Qnil;
}


static VALUE
rb_path_Path(VALUE self)
{
  BPath *arg0;
  const char *result;
  VALUE vresult;

  Get_BInstance(Path, self, arg0);
  result = arg0->Path();
  vresult = rb_str_new2(result);
  return vresult;
}


static VALUE
rb_path_Leaf(VALUE self)
{
  BPath *arg0;
  const char *result;
  VALUE vresult;

  Get_BInstance(Path, self, arg0);
  result = arg0->Leaf();
  vresult = rb_str_new2(result);
  return vresult;
}


static VALUE
rb_path_GetParent(VALUE self)
{
  BPath *arg0, arg1, *copy;
  status_t result;
  VALUE vresult;

  Get_BInstance(Path, self, arg0);
  result = arg0->GetParent(&arg1);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  NEW(copy, BPath(arg1));
  vresult = Data_Wrap_Struct(cPath, 0, delete_path, copy);
  return vresult;
}


void
Init_Path()
{
  cPath = rb_define_class_under(mBe, "Path", cFlattenable);
  rb_define_singleton_method(cPath, "new", RUBY_METHOD_FUNC(rb_path_s_new), -1);
  rb_define_method(cPath, "initialize", RUBY_METHOD_FUNC(rb_path_initialize), -1);

  rb_define_method(cPath, "InitCheck", RUBY_METHOD_FUNC(rb_path_InitCheck), 0);

  rb_define_method(cPath, "SetTo", RUBY_METHOD_FUNC(rb_path_SetTo), -1);
  rb_define_method(cPath, "Append", RUBY_METHOD_FUNC(rb_path_Append), -1);
  rb_define_method(cPath, "Unset", RUBY_METHOD_FUNC(rb_path_Unset), 0);

  rb_define_method(cPath, "Path", RUBY_METHOD_FUNC(rb_path_Path), 0);
  rb_define_method(cPath, "Leaf", RUBY_METHOD_FUNC(rb_path_Leaf), 0);

  rb_define_method(cPath, "GetParent", RUBY_METHOD_FUNC(rb_path_GetParent), 0);
}
