/*
 *  Point.cc
 */

#include <Rect.h>
#include "rbPoint.h"
#include "my_be.h"


VALUE cPoint;


void
delete_point(BPoint *arg0)
{
  delete arg0;
}


static VALUE
rb_point_s_new(int argc, VALUE *argv, VALUE klass)
{
  VALUE vresult = my_class_s_new(argc, argv, klass, delete_point);
  return vresult;
}


static VALUE
rb_point_initialize(int argc, VALUE *argv, VALUE self)
{
  BPoint *result;
  VALUE varg1, varg2;

  Data_To_Null(self, BPoint);

  switch(rb_scan_args(argc, argv, "02", &varg1, &varg2)){
    case 0:
      result = new Point();
      break;

    case 1:
      {
        BPoint *arg1;

        Get_BInstance(Point, varg1, arg1);
        result = new Point(*arg1);
      }
      break;

    case 2:
      {
        float arg1, arg2;

        arg1 = NUM2DBL(varg1);
        arg2 = NUM2DBL(varg2);
        result = new Point(arg1, arg2);
      }
      break;

    default:
      rb_bug("rb_point_initialize");
  }
  DATA_PTR(self) = result;
  return self;
}


static VALUE
rb_point_x_set(VALUE self, VALUE varg1)
{
  BPoint *arg0;
  float arg1;

  Get_BInstance(Point, self, arg0);
  arg1 = NUM2DBL(varg1);
  arg0->x = arg1;
  return varg1;
}


static VALUE
rb_point_x_ref(VALUE self)
{
  BPoint *arg0;
  float result;
  VALUE vresult;

  Get_BInstance(Point, self, arg0);
  result = arg0->x;
  vresult = rb_float_new(result);
  return vresult;
}


static VALUE
rb_point_y_set(VALUE self, VALUE varg1)
{
  BPoint *arg0;
  float arg1;

  Get_BInstance(Point, self, arg0);
  arg1 = NUM2DBL(varg1);
  arg0->y = arg1;
  return varg1;
}


static VALUE
rb_point_y_ref(VALUE self)
{
  BPoint *arg0;
  float result;
  VALUE vresult;

  Get_BInstance(Point, self, arg0);
  result = arg0->y;
  vresult = rb_float_new(result);
  return vresult;
}


static VALUE
rb_point_Set(VALUE self, VALUE varg1, VALUE varg2)
{
  BPoint *arg0;
  float arg1, arg2;

  Get_BInstance(Point, self, arg0);
  arg1 = NUM2DBL(varg1);
  arg2 = NUM2DBL(varg2);
  arg0->Set(arg1, arg2);
  return Qnil;
}


static VALUE
rb_point_ConstrainTo(VALUE self, VALUE varg1)
{
  BPoint *arg0;
  BRect *arg1;

  Get_BInstance(Point, self, arg0);
  Get_BInstance(Rect, varg1, arg1);
  arg0->ConstrainTo(*arg1);
  return Qnil;
}


static VALUE
rb_point_PrintToStream(VALUE self)
{
  BPoint *arg0;

  Get_BInstance(Point, self, arg0);
  arg0->PrintToStream();
  return Qnil;
}


static VALUE
rb_point_equal(VALUE self, VALUE varg1)
{
  BPoint *arg0, *arg1;

  if (!rb_obj_is_kind_of(varg1, cPoint)) {
    return Qfalse;
  }

  Get_BInstance(Point, self, arg0);
  Get_BInstance(Point, varg1, arg1);

  if (*arg0 == *arg1) {
    return Qtrue;
  }
  return Qfalse;
}


static VALUE
rb_point_not_equal(VALUE self, VALUE varg1)
{
  BPoint *arg0, *arg1;

  if (!rb_obj_is_kind_of(varg1, cPoint)) {
    return Qtrue;
  }

  Get_BInstance(Point, self, arg0);
  Get_BInstance(Point, varg1, arg1);

  if (*arg0 != *arg1) {
    return Qtrue;
  }
  return Qfalse;
}


static VALUE
rb_point_plus(VALUE self, VALUE varg1)
{
  BPoint *arg0, *arg1, *result;

  Get_BInstance(Point, self, arg0);
  Get_BInstance(Point, varg1, arg1);

  result = new Point((*arg0 + *arg1));
  VALUE vresult = Data_Wrap_Struct(cPoint, 0, delete_point, result);
  return vresult;
}


static VALUE
rb_point_minus(VALUE self, VALUE varg1)
{
  BPoint *arg0, *arg1, *result;

  Get_BInstance(Point, self, arg0);
  Get_BInstance(Point, varg1, arg1);

  result = new Point((*arg0 - *arg1));
  VALUE vresult = Data_Wrap_Struct(cPoint, 0, delete_point, result);
  return vresult;
}


static VALUE
rb_point_inspect(VALUE self)
{
  BPoint *arg0;
  VALUE str;
  char *c;

  Get_BInstance(Point, self, arg0);

  c = rb_class2name(CLASS_OF(self));
  str = rb_str_new2("#<");
  str = rb_str_cat2(str, c);

  str = rb_str_cat2(str, " x=");
  str = rb_str_concat(str, rb_obj_as_string(rb_float_new(arg0->x)));
  str = rb_str_cat2(str, " y=");
  str = rb_str_concat(str, rb_obj_as_string(rb_float_new(arg0->y)));

  str = rb_str_cat2(str, ">");
  return str;
}


void
Init_Point()
{
  cPoint = rb_define_class_under(mBe, "Point", rb_cObject);
  rb_define_singleton_method(cPoint, "new", RUBY_METHOD_FUNC(rb_point_s_new), -1);
  rb_define_method(cPoint, "initialize", RUBY_METHOD_FUNC(rb_point_initialize), -1);

  rb_define_method(cPoint, "x=", RUBY_METHOD_FUNC(rb_point_x_set), 1);
  rb_define_method(cPoint, "x", RUBY_METHOD_FUNC(rb_point_x_ref), 0);
  rb_define_method(cPoint, "y=", RUBY_METHOD_FUNC(rb_point_y_set), 1);
  rb_define_method(cPoint, "y", RUBY_METHOD_FUNC(rb_point_y_ref), 0);
  rb_define_method(cPoint, "Set", RUBY_METHOD_FUNC(rb_point_Set), 2);
  rb_define_method(cPoint, "ConstrainTo", RUBY_METHOD_FUNC(rb_point_ConstrainTo), 1);
  rb_define_method(cPoint, "PrintToStream", RUBY_METHOD_FUNC(rb_point_PrintToStream), 0);

  rb_define_method(cPoint, "==", RUBY_METHOD_FUNC(rb_point_equal), 1);
  rb_define_method(cPoint, "!=", RUBY_METHOD_FUNC(rb_point_not_equal), 1);

  rb_define_method(cPoint, "+", RUBY_METHOD_FUNC(rb_point_plus), 1);
  rb_define_method(cPoint, "-", RUBY_METHOD_FUNC(rb_point_minus), 1);

  rb_define_const(mBe, "ORIGIN", Data_Wrap_Struct(cPoint, 0, 0, (BPoint *)&B_ORIGIN));

  rb_define_method(cPoint, "inspect", RUBY_METHOD_FUNC(rb_point_inspect), 0);
}
