/*
 *  Query.cc
 */

#include <Query.h>
#include <Messenger.h>
#include "my_be.h"


VALUE cQuery;


void
delete_query(BQuery *arg0)
{
  delete arg0;
}


static VALUE
rb_query_s_new(int argc, VALUE *argv, VALUE klass)
{
  VALUE vresult = my_class_s_new(argc, argv, klass, delete_query);
  return vresult;
}


static VALUE
rb_query_initialize(VALUE self)
{
  BQuery *result;

  Data_To_Null(self, BQuery);
  result = my_create_be_obj<BQuery>();
  DATA_PTR(self) = result;
  return self;
}


static VALUE
rb_query_Clear(VALUE self)
{
  BQuery *arg0;
  status_t result;

  Get_BInstance(Query, self, arg0);
  result = arg0->Clear();
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_query_PushAttr(VALUE self, VALUE varg1)
{
  BQuery *arg0;
  char *arg1;

  Get_BInstance(Query, self, arg0);
  arg1 = STR2CSTR(varg1);
  arg0->PushAttr(arg1);
  return Qnil;
}


static VALUE
rb_query_PushOp(VALUE self, VALUE varg1)
{
  BQuery *arg0;
  query_op arg1;

  Get_BInstance(Query, self, arg0);
  arg1 = (query_op)NUM2LONG(varg1);
  arg0->PushOp(arg1);
  return Qnil;
}


static VALUE
rb_query_PushUInt32(VALUE self, VALUE varg1)
{
  BQuery *arg0;
  uint32 arg1;

  Get_BInstance(Query, self, arg0);
  arg1 = NUM2ULONG(varg1);
  arg0->PushUInt32(arg1);
  return Qnil;
}


static VALUE
rb_query_PushInt32(VALUE self, VALUE varg1)
{
  BQuery *arg0;
  int32 arg1;

  Get_BInstance(Query, self, arg0);
  arg1 = NUM2LONG(varg1);
  arg0->PushInt32(arg1);
  return Qnil;
}


static VALUE
rb_query_PushUInt64(VALUE self, VALUE varg1)
{
  BQuery *arg0;
  uint64 arg1;

  Get_BInstance(Query, self, arg0);
  arg1 = NUM2ULLONG(varg1);
  arg0->PushUInt64(arg1);
  return Qnil;
}


static VALUE
rb_query_PushInt64(VALUE self, VALUE varg1)
{
  BQuery *arg0;
  int64 arg1;

  Get_BInstance(Query, self, arg0);
  arg1 = NUM2LLONG(varg1);
  arg0->PushInt64(arg1);
  return Qnil;
}


static VALUE
rb_query_PushFloat(VALUE self, VALUE varg1)
{
  BQuery *arg0;
  float arg1;

  Get_BInstance(Query, self, arg0);
  arg1 = NUM2DBL(varg1);
  arg0->PushFloat(arg1);
  return Qnil;
}


static VALUE
rb_query_PushDouble(VALUE self, VALUE varg1)
{
  BQuery *arg0;
  double arg1;

  Get_BInstance(Query, self, arg0);
  arg1 = NUM2DBL(varg1);
  arg0->PushDouble(arg1);
  return Qnil;
}


static VALUE
rb_query_PushString(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1, varg2;
  BQuery *arg0;

  Get_BInstance(Query, self, arg0);
  switch (rb_scan_args(argc, argv, "11", &varg1, &varg2)) {
    case 1:
      {
        char *arg1;

        arg1 = STR2CSTR(varg1);
        arg0->PushString(arg1);
      }
      break;

    case 2:
      {
        char *arg1;
        bool arg2;

        arg1 = STR2CSTR(varg1);
        arg2 = RTEST(varg2);
        arg0->PushString(arg1, arg2);
      }
      break;

    default:
      rb_bug("rb_query_PushString");
  }
  return Qnil;
}


static VALUE
rb_query_PushDate(VALUE self, VALUE varg1)
{
  BQuery *arg0;
  char *arg1;
  status_t result;

  Get_BInstance(Query, self, arg0);
  arg1 = STR2CSTR(varg1);
  result = arg0->PushDate(arg1);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_query_SetVolume(VALUE self, VALUE varg1)
{
  BQuery *arg0;
  BVolume *arg1;
  status_t result;

  Get_BInstance(Query, self, arg0);
  Get_BInstance(Volume, varg1, arg1);
  result = arg0->SetVolume(arg1);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_query_SetPredicate(VALUE self, VALUE varg1)
{
  BQuery *arg0;
  char *arg1;
  status_t result;

  Get_BInstance(Query, self, arg0);
  arg1 = STR2CSTR(varg1);
  result = arg0->SetPredicate(arg1);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_query_SetTarget(VALUE self, VALUE varg1)
{
  BQuery *arg0;
  BMessenger *arg1;
  status_t result;

  Get_BInstance(Query, self, arg0);
  Get_BInstance(Messenger, varg1, arg1);
  result = arg0->SetTarget(*arg1);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_query_IsLive(VALUE self)
{
  BQuery *arg0;
  bool result;
  VALUE vresult;

  Get_BInstance(Query, self, arg0);
  result = arg0->IsLive();
  vresult = (result ? Qtrue:Qfalse);
  return vresult;
}


static VALUE
rb_query_GetPredicate(VALUE self)
{
  BQuery *arg0;
  char *arg1;
  size_t arg2;
  status_t result;
  VALUE vresult;

  Get_BInstance(Query, self, arg0);

  arg2 = arg0->PredicateLength();
  arg1 = ALLOCA_N(char, arg2);

  result = arg0->GetPredicate(arg1, arg2);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  vresult = rb_str_new2(arg1);
  return vresult;
}


static VALUE
rb_query_PredicateLength(VALUE self)
{
  BQuery *arg0;
  size_t result;
  VALUE vresult;

  Get_BInstance(Query, self, arg0);
  result = arg0->PredicateLength();
  vresult = LONG2NUM(result);
  return vresult;
}


static VALUE
rb_query_TargetDevice(VALUE self)
{
  BQuery *arg0;
  dev_t result;
  VALUE vresult;

  Get_BInstance(Query, self, arg0);
  result = arg0->TargetDevice();
  vresult = LONG2NUM(result);
  return vresult;
}


static VALUE
rb_query_Fetch(VALUE self)
{
  BQuery *arg0;
  status_t result;

  Get_BInstance(Query, self, arg0);
  result = arg0->Fetch();
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


void
Init_Query()
{
  cQuery = rb_define_class_under(mBe, "Query", cEntryList);
  rb_undef_method(CLASS_OF(cQuery), "Rewind");
  rb_undef_method(CLASS_OF(cQuery), "CountEntries");

  rb_define_singleton_method(cQuery, "new", RUBY_METHOD_FUNC(rb_query_s_new), -1);
  rb_define_method(cQuery, "initialize", RUBY_METHOD_FUNC(rb_query_initialize), 0);

  rb_define_method(cQuery, "Clear", RUBY_METHOD_FUNC(rb_query_Clear), 0);

  rb_define_method(cQuery, "PushAttr", RUBY_METHOD_FUNC(rb_query_PushAttr), 1);
  rb_define_method(cQuery, "PushOp", RUBY_METHOD_FUNC(rb_query_PushOp), 1);

  rb_define_method(cQuery, "PushUInt32", RUBY_METHOD_FUNC(rb_query_PushUInt32), 1);
  rb_define_method(cQuery, "PushInt32", RUBY_METHOD_FUNC(rb_query_PushInt32), 1);
  rb_define_method(cQuery, "PushUInt64", RUBY_METHOD_FUNC(rb_query_PushUInt64), 1);
  rb_define_method(cQuery, "PushInt64", RUBY_METHOD_FUNC(rb_query_PushInt64), 1);
  rb_define_method(cQuery, "PushFloat", RUBY_METHOD_FUNC(rb_query_PushFloat), 1);
  rb_define_method(cQuery, "PushDouble", RUBY_METHOD_FUNC(rb_query_PushDouble), 1);
  rb_define_method(cQuery, "PushString", RUBY_METHOD_FUNC(rb_query_PushString), -1);
  rb_define_method(cQuery, "PushDate", RUBY_METHOD_FUNC(rb_query_PushDate), 1);

  rb_define_method(cQuery, "SetVolume", RUBY_METHOD_FUNC(rb_query_SetVolume), 1);
  rb_define_method(cQuery, "SetPredicate", RUBY_METHOD_FUNC(rb_query_SetPredicate), 1);
  rb_define_method(cQuery, "SetTarget", RUBY_METHOD_FUNC(rb_query_SetTarget), 1);

  rb_define_method(cQuery, "IsLive", RUBY_METHOD_FUNC(rb_query_IsLive), 0);

  rb_define_method(cQuery, "GetPredicate", RUBY_METHOD_FUNC(rb_query_GetPredicate), 0);
  rb_define_method(cQuery, "PredicateLength", RUBY_METHOD_FUNC(rb_query_PredicateLength), 0);

  rb_define_method(cQuery, "TargetDevice", RUBY_METHOD_FUNC(rb_query_TargetDevice), 0);

  rb_define_method(cQuery, "Fetch", RUBY_METHOD_FUNC(rb_query_Fetch), 0);

  rb_define_const(mBe, "INVALID_OP", ULONG2NUM(B_INVALID_OP));
  rb_define_const(mBe, "EQ", ULONG2NUM(B_EQ));
  rb_define_const(mBe, "GT", ULONG2NUM(B_GT));
  rb_define_const(mBe, "GE", ULONG2NUM(B_GE));
  rb_define_const(mBe, "LT", ULONG2NUM(B_LT));
  rb_define_const(mBe, "LE", ULONG2NUM(B_LE));
  rb_define_const(mBe, "NE", ULONG2NUM(B_NE));
  rb_define_const(mBe, "CONTAINS", ULONG2NUM(B_CONTAINS));
  rb_define_const(mBe, "BEGINS_WITH", ULONG2NUM(B_BEGINS_WITH));
  rb_define_const(mBe, "ENDS_WITH", ULONG2NUM(B_ENDS_WITH));
  rb_define_const(mBe, "AND", ULONG2NUM(B_AND));
  rb_define_const(mBe, "OR", ULONG2NUM(B_OR));
  rb_define_const(mBe, "NOT", ULONG2NUM(B_NOT));
}
