/*
 *  StringView.cc
 */

#include <Message.h>
#include "rbStringView.h"
#include "my_be.h"


VALUE cStringView;


void
delete_stringview(BStringView *arg0)
{
  delete arg0;
}


static VALUE
rb_stringview_s_new(int argc, VALUE *argv, VALUE klass)
{
  VALUE vresult = my_class_s_new(argc, argv, klass, delete_stringview);
  return vresult;
}


static VALUE
rb_stringview_initialize(int argc, VALUE *argv, VALUE self)
{
  StringView *result;

  Data_To_Null(self, BStringView);

  if (argc > 1) {
    VALUE varg1, varg2, varg3, varg4, varg5;

    switch (rb_scan_args(argc, argv, "32", &varg1, &varg2, &varg3, &varg4, &varg5)) {
      case 3:
        {
          BRect *arg1;
          char *arg2, *arg3;

          Get_BInstance(Rect, varg1, arg1)
          arg2 = STR2CSTR(varg2);
          arg3 = STR2CSTR(varg3);
          result = new StringView(*arg1, arg2, arg3);
        }
        break;

      case 4:
        {
          BRect *arg1;
          char *arg2, *arg3;
          uint32 arg4;

          Get_BInstance(Rect, varg1, arg1)
          arg2 = STR2CSTR(varg2);
          arg3 = STR2CSTR(varg3);
          arg4 = NUM2ULONG(varg4);
          result = new StringView(*arg1, arg2, arg3, arg4);
        }
        break;

      case 5:
        {
          BRect *arg1;
          char *arg2, *arg3;
          uint32 arg4, arg5;

          Get_BInstance(Rect, varg1, arg1)
          arg2 = STR2CSTR(varg2);
          arg3 = STR2CSTR(varg3);
          arg4 = NUM2ULONG(varg4);
          arg5 = NUM2ULONG(varg5);

          result = new StringView(*arg1, arg2, arg3, arg4, arg5);
        }
        break;

      default:
        rb_bug("rb_stringview_s_new");
    }
  }
  else {
    VALUE varg1;
    BMessage *arg1;

    rb_scan_args(argc, argv, "1", &varg1);
    Get_BInstance(Message, varg1, arg1)

    result = new StringView(arg1);
  }
  result->myself = self;
  DATA_PTR(self) = result;
  return self;
}


static VALUE
rb_stringview_SetText(VALUE self, VALUE varg1)
{
  BStringView *arg0;
  char *arg1;

  Get_BInstance(StringView, self, arg0);
  arg1 = STR2CSTR(varg1);
  arg0->SetText(arg1);
  return Qnil;
}


static VALUE
rb_stringview_Text(VALUE self)
{
  BStringView *arg0;
  const char *result;
  VALUE vresult;

  Get_BInstance(StringView, self, arg0);
  result = arg0->Text();
  vresult = rb_str_new2(result);
  return vresult;
}


static VALUE
rb_stringview_SetAlignment(VALUE self, VALUE varg1)
{
  BStringView *arg0;
  alignment arg1;

  Get_BInstance(StringView, self, arg0);
  arg1 = (alignment)NUM2ULONG(varg1);
  arg0->SetAlignment(arg1);
  return Qnil;
}


static VALUE
rb_stringview_Alignment(VALUE self)
{
  BStringView *arg0;
  alignment result;
  VALUE vresult;

  Get_BInstance(StringView, self, arg0);
  result = arg0->Alignment();
  vresult = ULONG2NUM(result);
  return vresult;
}


static VALUE
rb_stringview_AttachedToWindow(VALUE self)
{
  BStringView *arg0;

  Get_BInstance(StringView, self, arg0);
  arg0->BStringView::AttachedToWindow();
  return Qnil;
}


static VALUE
rb_stringview_Draw(VALUE self, VALUE varg1)
{
  BStringView *arg0;
  BRect *arg1;

  Get_BInstance(StringView, self, arg0);
  Get_BInstance(Rect, varg1, arg1);
  arg0->BStringView::Draw(*arg1);
  return Qnil;
}


static VALUE
rb_stringview_MessageReceived(VALUE self, VALUE varg1)
{
  BStringView *arg0;
  BMessage *arg1;

  Get_BInstance(StringView, self, arg0);
  Get_BInstance(Message, varg1, arg1);
  arg0->BStringView::MessageReceived(arg1);
  return Qnil;
}


static VALUE
rb_stringview_MouseDown(VALUE self, VALUE varg1)
{
  BStringView *arg0;
  BPoint *arg1;

  Get_BInstance(StringView, self, arg0);
  Get_BInstance(Point, varg1, arg1);
  arg0->BStringView::MouseDown(*arg1);
  return Qnil;
}


static VALUE
rb_stringview_MouseUp(VALUE self, VALUE varg1)
{
  BStringView *arg0;
  BPoint *arg1;

  Get_BInstance(StringView, self, arg0);
  Get_BInstance(Point, varg1, arg1);
  arg0->BStringView::MouseUp(*arg1);
  return Qnil;
}


static VALUE
rb_stringview_MouseMoved(VALUE self, VALUE varg1, VALUE varg2, VALUE varg3)
{
  BStringView *arg0;
  BPoint *arg1;
  uint32 arg2;
  BMessage *arg3 = NULL;

  Get_BInstance(StringView, self, arg0);
  Get_BInstance(Point, varg1, arg1);
  arg2 = NUM2ULONG(varg2);
  if (!NIL_P(varg3)) {
    Get_BInstance(Message, varg3, arg3);
  }
  arg0->BStringView::MouseMoved(*arg1, arg2, arg3);
  return Qnil;
}


static VALUE
rb_stringview_DetachedFromWindow(VALUE self)
{
  BStringView *arg0;

  Get_BInstance(StringView, self, arg0);
  arg0->BStringView::DetachedFromWindow();
  return Qnil;
}


static VALUE
rb_stringview_FrameMoved(VALUE self, VALUE varg1)
{
  BStringView *arg0;
  BPoint *arg1;

  Get_BInstance(StringView, self, arg0);
  Get_BInstance(Point, varg1, arg1);
  arg0->BStringView::FrameMoved(*arg1);
  return Qnil;
}


static VALUE
rb_stringview_FrameResized(VALUE self, VALUE varg1, VALUE varg2)
{
  BStringView *arg0;
  float arg1;
  float arg2;

  Get_BInstance(StringView, self, arg0);
  arg1 = NUM2DBL(varg1);
  arg2 = NUM2DBL(varg2);
  arg0->BStringView::FrameResized(arg1, arg2);
  return Qnil;
}


static VALUE
rb_stringview_ResolveSpecifier(VALUE self, VALUE varg1, VALUE varg2, VALUE varg3, VALUE varg4, VALUE varg5)
{
  BStringView *arg0;
  BMessage *arg1,  *arg3;
  int32 arg2, arg4;
  char *arg5;
  BHandler *result;
  VALUE vresult;

  Get_BInstance(StringView, self, arg0);
  Get_BInstance(Message, varg1, arg1);
  arg2 = NUM2LONG(varg2);
  Get_BInstance(Message, varg3, arg3);
  arg4 = NUM2LONG(varg4);
  arg5 = STR2CSTR(varg5);

  result = arg0->BStringView::ResolveSpecifier(arg1, arg2, arg3, arg4, arg5);
  vresult = ((result) ? Data_Wrap_Struct(cHandler, 0, 0, result):Qnil);
  return vresult;
}


static VALUE
rb_stringview_ResizeToPreferred(VALUE self)
{
  BStringView *arg0;

  Get_BInstance(StringView, self, arg0);
  arg0->BStringView::ResizeToPreferred();
  return Qnil;
}


static VALUE
rb_stringview_GetPreferredSize(int argc, VALUE *argv, VALUE self)
{
  BStringView *arg0;
  float arg1 = 0.0, arg2 = 0.0;
  VALUE vresult;

  Get_BInstance(StringView, self, arg0);
  arg0->BStringView::GetPreferredSize(&arg1, &arg2);
  vresult = rb_ary_new3(2, rb_float_new(arg1), rb_float_new(arg2));
  return vresult;
}


static VALUE
rb_stringview_MakeFocus(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1;
  BStringView *arg0;

  Get_BInstance(StringView, self, arg0);
  switch (rb_scan_args(argc, argv, "01", &varg1)) {
    case 0:
      arg0->BStringView::MakeFocus();
      break;

    case 1:
      {
        bool arg1;

        arg1 = RTEST(varg1);
        arg0->BStringView::MakeFocus(arg1);
      }
      break;

    default:
      rb_bug("rb_stringview_MakeFocus");
  }
  return Qnil;
}


static VALUE
rb_stringview_AllAttached(VALUE self)
{
  BStringView *arg0;

  Get_BInstance(StringView, self, arg0);
  arg0->BStringView::AllAttached();
  return Qnil;
}


static VALUE
rb_stringview_AllDetached(VALUE self)
{
  BStringView *arg0;

  Get_BInstance(StringView, self, arg0);
  arg0->BStringView::AllDetached();
  return Qnil;
}


static VALUE
rb_stringview_GetSupportedSuites(VALUE self, VALUE varg1)
{
  BStringView *arg0;
  BMessage *arg1;
  status_t result;

  Get_BInstance(StringView, self, arg0);
  Get_BInstance(Message, varg1, arg1);
  result = arg0->BStringView::GetSupportedSuites(arg1);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


void
Init_StringView()
{
  cStringView = rb_define_class_under(mBe, "StringView", cView);
  rb_define_singleton_method(cStringView, "new", RUBY_METHOD_FUNC(rb_stringview_s_new), -1);
  rb_define_method(cStringView, "initialize", RUBY_METHOD_FUNC(rb_stringview_initialize), -1);

  rb_define_method(cStringView, "SetText", RUBY_METHOD_FUNC(rb_stringview_SetText), 1);
  rb_define_method(cStringView, "Text", RUBY_METHOD_FUNC(rb_stringview_Text), 0);
  rb_define_method(cStringView, "SetAlignment", RUBY_METHOD_FUNC(rb_stringview_SetAlignment), 1);
  rb_define_method(cStringView, "Alignment", RUBY_METHOD_FUNC(rb_stringview_Alignment), 0);

  rb_define_method(cStringView, "AttachedToWindow", RUBY_METHOD_FUNC(rb_stringview_AttachedToWindow), 0);
  rb_define_method(cStringView, "Draw", RUBY_METHOD_FUNC(rb_stringview_Draw), 1);

  rb_define_method(cStringView, "MessageReceived", RUBY_METHOD_FUNC(rb_stringview_MessageReceived), 1);
  rb_define_method(cStringView, "MouseDown", RUBY_METHOD_FUNC(rb_stringview_MouseDown), 1);
  rb_define_method(cStringView, "MouseUp", RUBY_METHOD_FUNC(rb_stringview_MouseUp), 1);
  rb_define_method(cStringView, "MouseMoved", RUBY_METHOD_FUNC(rb_stringview_MouseMoved), 3);
  rb_define_method(cStringView, "DetachedFromWindow", RUBY_METHOD_FUNC(rb_stringview_DetachedFromWindow), 0);
  rb_define_method(cStringView, "FrameMoved", RUBY_METHOD_FUNC(rb_stringview_FrameMoved), 1);
  rb_define_method(cStringView, "FrameResized", RUBY_METHOD_FUNC(rb_stringview_FrameResized), 2);

  rb_define_method(cStringView, "ResolveSpecifier", RUBY_METHOD_FUNC(rb_stringview_ResolveSpecifier), 5);

  rb_define_method(cStringView, "ResizeToPreferred", RUBY_METHOD_FUNC(rb_stringview_ResizeToPreferred), 0);
  rb_define_method(cStringView, "GetPreferredSize", RUBY_METHOD_FUNC(rb_stringview_GetPreferredSize), -1);
  rb_define_method(cStringView, "MakeFocus", RUBY_METHOD_FUNC(rb_stringview_MakeFocus), -1);
  rb_define_method(cStringView, "AllAttached", RUBY_METHOD_FUNC(rb_stringview_AllAttached), 0);
  rb_define_method(cStringView, "AllDetached", RUBY_METHOD_FUNC(rb_stringview_AllDetached), 0);
  rb_define_method(cStringView, "GetSupportedSuites", RUBY_METHOD_FUNC(rb_stringview_GetSupportedSuites), 1);
}
