/*
 *  UTF8.cc
 */

#include <UTF8.h>
#include "my_be.h"


static VALUE
rb_convert_to_utf8(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1, varg2, varg3;
  status_t result;
  VALUE vresult;

  switch (rb_scan_args(argc, argv, "21", &varg1, &varg2, &varg3)) {
    case 2:
      {
        uint32 arg1;
        char *arg2, *dst;
        int32 arg3 = 0, dstLen, state = 0;

        arg1 = NUM2ULONG(varg1);
        arg2 = rb_str2cstr(varg2, reinterpret_cast<int *>(&arg3));
        dstLen = (int)(arg3*1.5+0.5);
        dst = ALLOCA_N(char, dstLen);

        result = convert_to_utf8(arg1, arg2, &arg3, dst, &dstLen, &state);
        vresult = rb_str_new(dst, dstLen);
      }
      break;

    case 3:
      {
        uint32 arg1;
        char *arg2, *dst, arg7;
        int32 arg3 = 0, dstLen, state = 0;

        arg1 = NUM2ULONG(varg1);
        arg2 = rb_str2cstr(varg2, reinterpret_cast<int *>(&arg3));
        dstLen = (int)(arg3*1.5+0.5);
        dst = ALLOCA_N(char, dstLen);
        arg7 = NUM2CHR(varg3);

        result = convert_to_utf8(arg1, arg2, &arg3, dst, &dstLen, &state, arg7);
        vresult = rb_str_new(dst, dstLen);
      }
      break;

    default:
      rb_bug("rb_convert_to_utf8");
  }
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return vresult;
}


static VALUE
rb_convert_from_utf8(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1, varg2, varg3;
  status_t result;
  VALUE vresult;

  switch (rb_scan_args(argc, argv, "21", &varg1, &varg2, &varg3)) {
    case 2:
      {
        uint32 arg1;
        char *arg2, *dst;
        int32 arg3 = 0, dstLen, state = 0;

        arg1 = NUM2ULONG(varg1);
        arg2 = rb_str2cstr(varg2, reinterpret_cast<int *>(&arg3));
        dst = ALLOCA_N(char, arg3);
        dstLen = arg3;

        result = convert_from_utf8(arg1, arg2, &arg3, dst, &dstLen, &state);
        vresult = rb_str_new(dst, dstLen);
      }
      break;

    case 3:
      {
        uint32 arg1;
        char *arg2, *dst, arg7;
        int32 arg3 = 0, dstLen, state = 0;

        arg1 = NUM2ULONG(varg1);
        arg2 = rb_str2cstr(varg2, reinterpret_cast<int *>(&arg3));
        dst = ALLOCA_N(char, arg3);
        dstLen = arg3;
        arg7 = NUM2CHR(varg3);

        result = convert_from_utf8(arg1, arg2, &arg3, dst, &dstLen, &state, arg7);
        vresult = rb_str_new(dst, dstLen);
      }
      break;

    default:
      rb_bug("rb_convert_from_utf8");
  }
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return vresult;
}


void
Init_UTF8()
{
  rb_define_module_function(mBe, "convert_to_utf8", RUBY_METHOD_FUNC(rb_convert_to_utf8), -1);
  rb_define_module_function(mBe, "convert_from_utf8", RUBY_METHOD_FUNC(rb_convert_from_utf8), -1);

  rb_define_const(mBe, "ISO1_CONVERSION", ULONG2NUM(B_ISO1_CONVERSION));
  rb_define_const(mBe, "ISO2_CONVERSION", ULONG2NUM(B_ISO2_CONVERSION));
  rb_define_const(mBe, "ISO3_CONVERSION", ULONG2NUM(B_ISO3_CONVERSION));
  rb_define_const(mBe, "ISO4_CONVERSION", ULONG2NUM(B_ISO4_CONVERSION));
  rb_define_const(mBe, "ISO5_CONVERSION", ULONG2NUM(B_ISO5_CONVERSION));
  rb_define_const(mBe, "ISO6_CONVERSION", ULONG2NUM(B_ISO6_CONVERSION));
  rb_define_const(mBe, "ISO7_CONVERSION", ULONG2NUM(B_ISO7_CONVERSION));
  rb_define_const(mBe, "ISO8_CONVERSION", ULONG2NUM(B_ISO8_CONVERSION));
  rb_define_const(mBe, "ISO9_CONVERSION", ULONG2NUM(B_ISO9_CONVERSION));
  rb_define_const(mBe, "ISO10_CONVERSION", ULONG2NUM(B_ISO10_CONVERSION));
  rb_define_const(mBe, "MAC_ROMAN_CONVERSION", ULONG2NUM(B_MAC_ROMAN_CONVERSION));
  rb_define_const(mBe, "SJIS_CONVERSION", ULONG2NUM(B_SJIS_CONVERSION));
  rb_define_const(mBe, "EUC_CONVERSION", ULONG2NUM(B_EUC_CONVERSION));
  rb_define_const(mBe, "JIS_CONVERSION", ULONG2NUM(B_JIS_CONVERSION));
  rb_define_const(mBe, "MS_WINDOWS_CONVERSION", ULONG2NUM(B_MS_WINDOWS_CONVERSION));
  rb_define_const(mBe, "UNICODE_CONVERSION", ULONG2NUM(B_UNICODE_CONVERSION));
  rb_define_const(mBe, "KOI8R_CONVERSION", ULONG2NUM(B_KOI8R_CONVERSION));
  rb_define_const(mBe, "MS_WINDOWS_1251_CONVERSION", ULONG2NUM(B_MS_WINDOWS_1251_CONVERSION));
  rb_define_const(mBe, "MS_DOS_866_CONVERSION", ULONG2NUM(B_MS_DOS_866_CONVERSION));
  rb_define_const(mBe, "MS_DOS_CONVERSION", ULONG2NUM(B_MS_DOS_CONVERSION));
  rb_define_const(mBe, "EUC_KR_CONVERSION", ULONG2NUM(B_EUC_KR_CONVERSION));
  rb_define_const(mBe, "ISO13_CONVERSION", ULONG2NUM(B_ISO13_CONVERSION));
  rb_define_const(mBe, "ISO14_CONVERSION", ULONG2NUM(B_ISO14_CONVERSION));
  rb_define_const(mBe, "ISO15_CONVERSION", ULONG2NUM(B_ISO15_CONVERSION));
}
