/*
 *  View.cc
 */

#include <Region.h>
#include "rbPoint.h"
#include "rbRect.h"
#include "rbView.h"
#include "my_be.h"


VALUE cView;


void
delete_view(BView *arg0)
{
  delete arg0;
}


static VALUE
rb_view_s_new(int argc, VALUE *argv, VALUE klass)
{
  VALUE vresult = my_class_s_new(argc, argv, klass, delete_view);
  return vresult;
}


static VALUE
rb_view_initialize(int argc, VALUE *argv, VALUE self)
{
  View *result;
  VALUE varg1, varg2, varg3, varg4;

  Data_To_Null(self, BView);

  if (argc > 1) {
    BRect *arg1;
    char *arg2;
    uint32 arg3, arg4;

    rb_scan_args(argc, argv, "4", &varg1, &varg2, &varg3, &varg4);

    Get_BInstance(Rect, varg1, arg1);
    arg2 = STR2CSTR(varg2);
    arg3 = NUM2ULONG(varg3);
    arg4 = NUM2ULONG(varg4);

    result = new View(*arg1, arg2, arg3, arg4);
  }
  else {
    BMessage *arg1;

    rb_scan_args(argc, argv, "1", &varg1);
    Get_BInstance(Message, varg1, arg1);
    result = new View(arg1);
  }
  rb_ivar_set(self, id_relatives, Qnil);
  result->myself = self;
  DATA_PTR(self) = result;
  return self;
}


static VALUE
rb_view_AttachedToWindow(VALUE self)
{
  BView *arg0;

  Get_BInstance(View, self, arg0);
  arg0->BView::AttachedToWindow();
  return Qnil;
}


static VALUE
rb_view_AllAttached(VALUE self)
{
  BView *arg0;

  Get_BInstance(View, self, arg0);
  arg0->BView::AllAttached();
  return Qnil;
}


static VALUE
rb_view_DetachedFromWindow(VALUE self)
{
  BView *arg0;

  Get_BInstance(View, self, arg0);
  arg0->BView::DetachedFromWindow();
  return Qnil;
}


static VALUE
rb_view_AllDetached(VALUE self)
{
  BView *arg0;

  Get_BInstance(View, self, arg0);
  arg0->BView::AllDetached();
  return Qnil;
}


static VALUE
rb_view_MessageReceived(VALUE self, VALUE varg1)
{
  BView *arg0;
  BMessage *arg1;

  Get_BInstance(View, self, arg0);
  Get_BInstance(Message, varg1, arg1);
  arg0->BView::MessageReceived(arg1);
  return Qnil;
}


static VALUE
rb_view_AddChild(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1, varg2;
  BView *arg0;

  Get_BInstance(View, self, arg0);
  switch (rb_scan_args(argc, argv, "11", &varg1, &varg2)) {
    case 1:
      {
        BView *arg1;

        Get_BInstance(View, varg1, arg1);
        arg0->AddChild(arg1);
      }
      break;

    case 2:
      {
        BView *arg1, *arg2;

        Get_BInstance(View, varg1, arg1);
        Get_BInstance(View, varg2, arg2);

        arg0->AddChild(arg1, arg2);
      }
      break;

    default:
      rb_bug("rb_view_AddChild");
  }
  RDATA(varg1)->dfree = 0;
  my_add_relative(self, varg1);
  return Qnil;
}


static VALUE
rb_view_RemoveChild(VALUE self, VALUE varg1)
{
  BView *arg0, *arg1;
  bool result;
  VALUE vresult;

  Get_BInstance(View, self, arg0);
  Get_BInstance(View, varg1, arg1);
  result = arg0->RemoveChild(arg1);
  vresult = (result ? Qtrue:Qfalse);
  return vresult;
}


static VALUE
rb_view_CountChildren(VALUE self)
{
  BView *arg0;
  int32 result;
  VALUE vresult;

  Get_BInstance(View, self, arg0);
  result = arg0->CountChildren();
  vresult = LONG2NUM(result);
  return vresult;
}


static VALUE
rb_view_ChildAt(VALUE self, VALUE varg1)
{
  BView *arg0, *result;
  int32 arg1;
  VALUE vresult;

  Get_BInstance(View, self, arg0);
  arg1 = NUM2LONG(varg1);
  result = arg0->ChildAt(arg1);
  vresult = ((result) ? Data_Wrap_Struct(cView, 0, 0, result):Qnil);
  return vresult;
}


static VALUE
rb_view_NextSibling(VALUE self)
{
  BView *arg0, *result;
  VALUE vresult;

  Get_BInstance(View, self, arg0);
  result = arg0->NextSibling();
  vresult = ((result) ? Data_Wrap_Struct(cView, 0, 0, result):Qnil);
  return vresult;
}


static VALUE
rb_view_PreviousSibling(VALUE self)
{
  BView *arg0, *result;
  VALUE vresult;

  Get_BInstance(View, self, arg0);
  result = arg0->PreviousSibling();
  vresult = ((result) ? Data_Wrap_Struct(cView, 0, 0, result):Qnil);
  return vresult;
}


static VALUE
rb_view_RemoveSelf(VALUE self)
{
  BView *arg0;
  bool result;
  VALUE vresult;

  Get_BInstance(View, self, arg0);
  result = arg0->RemoveSelf();
  vresult = (result ? Qtrue:Qfalse);
  return vresult;
}


static VALUE
rb_view_Window(VALUE self)
{
  BView *arg0;
  BWindow *result;
  VALUE vresult;

  Get_BInstance(View, self, arg0);
  result = arg0->Window();
  vresult = ((result) ? Data_Wrap_Struct(cWindow, 0, 0, result):Qnil);
  return vresult;
}


static VALUE
rb_view_Draw(VALUE self, VALUE varg1)
{
  BView *arg0;
  BRect *arg1;

  Get_BInstance(View, self, arg0);
  Get_BInstance(Rect, varg1, arg1);
  arg0->BView::Draw(*arg1);
  return Qnil;
}


static VALUE
rb_view_MouseDown(VALUE self, VALUE varg1)
{
  BView *arg0;
  BPoint *arg1;

  Get_BInstance(View, self, arg0);
  Get_BInstance(Point, varg1, arg1);
  arg0->BView::MouseDown(*arg1);
  return Qnil;
}


static VALUE
rb_view_MouseUp(VALUE self, VALUE varg1)
{
  BView *arg0;
  BPoint *arg1;

  Get_BInstance(View, self, arg0);
  Get_BInstance(Point, varg1, arg1);
  arg0->BView::MouseUp(*arg1);
  return Qnil;
}


static VALUE
rb_view_MouseMoved(VALUE self, VALUE varg1, VALUE varg2, VALUE varg3)
{
  BView *arg0;
  BPoint *arg1;
  uint32 arg2;
  BMessage *arg3 = NULL;

  Get_BInstance(View, self, arg0);
  Get_BInstance(Point, varg1, arg1);
  arg2 = NUM2ULONG(varg2);
  if (!NIL_P(varg3)) {
    Get_BInstance(Message, varg3, arg3);
  }
  arg0->BView::MouseMoved(*arg1, arg2, arg3);
  return Qnil;
}


static VALUE
rb_view_WindowActivated(VALUE self, VALUE varg1)
{
  BView *arg0;
  bool arg1;

  Get_BInstance(View, self, arg0);
  arg1 = RTEST(varg1);
  arg0->BView::WindowActivated(arg1);
  return Qnil;
}


static VALUE
rb_view_KeyDown(VALUE self, VALUE varg1)
{
  BView *arg0;
  char *arg1;
  int arg2 = 0;

  Get_BInstance(View, self, arg0);
  arg1 = rb_str2cstr(varg1, &arg2);
  arg0->BView::KeyDown(arg1, arg2);
  return Qnil;
}


static VALUE
rb_view_KeyUp(VALUE self, VALUE varg1)
{
  BView *arg0;
  char *arg1;
  int arg2 = 0;

  Get_BInstance(View, self, arg0);
  arg1 = rb_str2cstr(varg1, &arg2);
  arg0->BView::KeyUp(arg1, arg2);
  return Qnil;
}


static VALUE
rb_view_Pulse(VALUE self)
{
  BView *arg0;

  Get_BInstance(View, self, arg0);
  arg0->BView::Pulse();
  return Qnil;
}


static VALUE
rb_view_FrameMoved(VALUE self, VALUE varg1)
{
  BView *arg0;
  BPoint *arg1;

  Get_BInstance(View, self, arg0);
  Get_BInstance(Point, varg1, arg1);
  arg0->BView::FrameMoved(*arg1);
  return Qnil;
}


static VALUE
rb_view_FrameResized(VALUE self, VALUE varg1, VALUE varg2)
{
  BView *arg0;
  float arg1, arg2;

  Get_BInstance(View, self, arg0);
  arg1 = NUM2DBL(varg1);
  arg2 = NUM2DBL(varg2);
  arg0->BView::FrameResized(arg1, arg2);
  return Qnil;
}


static VALUE
rb_view_TargetedByScrollView(VALUE self, VALUE varg1)
{
  BView *arg0;
  BScrollView *arg1;

  Get_BInstance(View, self, arg0);
  Get_BInstance(ScrollView, varg1, arg1);
  arg0->BView::TargetedByScrollView(arg1);
  return Qnil;
}


static VALUE
rb_view_BeginRectTracking(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1, varg2;
  BView *arg0;

  Get_BInstance(View, self, arg0);
  switch (rb_scan_args(argc, argv, "11", &varg1, &varg2)) {
    case 1:
      {
        BRect *arg1;

        Get_BInstance(Rect, varg1, arg1);
        arg0->BeginRectTracking(*arg1);
      }
      break;

    case 2:
      {
        BRect *arg1;
        uint32 arg2;

        Get_BInstance(Rect, varg1, arg1);
        arg2 = NUM2ULONG(varg2);

        arg0->BeginRectTracking(*arg1, arg2);
      }
      break;

    default:
      rb_bug("rb_view_BeginRectTracking");
  }
  return Qnil;
}


static VALUE
rb_view_EndRectTracking(VALUE self)
{
  BView *arg0;

  Get_BInstance(View, self, arg0);
  arg0->EndRectTracking();
  return Qnil;
}


static VALUE
rb_view_GetMouse(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1, vresult;
  BView *arg0;

  Get_BInstance(View, self, arg0);
  switch (rb_scan_args(argc, argv, "01", &varg1)) {
    case 0:
      {
        BPoint arg1, *copy;
        uint32 arg2;

        arg0->GetMouse(&arg1, &arg2);
        copy = new Point(arg1);
        vresult = rb_ary_new3(2,
                              Data_Wrap_Struct(cPoint, 0, delete_point, copy),
                              ULONG2NUM(arg2));
      }
      break;

    case 2:
      {
        BPoint arg1, *copy;
        uint32 arg2;
        bool arg3;

        arg3 = RTEST(varg1);

        arg0->GetMouse(&arg1, &arg2, arg3);
        copy = new Point(arg1);
        vresult = rb_ary_new3(2,
                              Data_Wrap_Struct(cPoint, 0, delete_point, copy),
                              ULONG2NUM(arg2));
      }
      break;

    default:
      rb_bug("rb_view_GetMouse");
  }
  return vresult;
}


static VALUE
rb_view_DragMessage(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1, varg2, varg3, varg4, varg5;
  BView *arg0;

  Get_BInstance(View, self, arg0);
  switch (rb_scan_args(argc, argv, "23", &varg1, &varg2, &varg3, &varg4, &varg5)) {
    case 2:
      {
        BMessage *arg1;
        BRect *arg2;

        Get_BInstance(Message, varg1, arg1);
        Get_BInstance(Rect, varg2, arg2);

        arg0->DragMessage(arg1, *arg2);
      }
      break;

    case 3:
      {
        BMessage *arg1;
        Get_BInstance(Message, varg1, arg1);

        if (rb_obj_is_kind_of(varg3, cPoint)) {
          BBitmap *arg2;
          BPoint *arg3;

          Get_BInstance(Bitmap, varg2, arg2);
          Get_BInstance(Point, varg3, arg3);

          arg0->DragMessage(arg1, arg2, *arg3);
        }
        else {
          BRect *arg2;
          BHandler *arg3;

          Get_BInstance(Rect, varg2, arg2);
          Get_BInstance(Handler, varg3, arg3);

          arg0->DragMessage(arg1, *arg2, arg3);
        }
      }
      break;

    case 4:
      {
        BMessage *arg1;
        BBitmap *arg2;

        Get_BInstance(Message, varg1, arg1);
        Get_BInstance(Bitmap, varg2, arg2);

        if (rb_obj_is_kind_of(varg3, cPoint)) {
          BPoint *arg3;
          BHandler *arg4;

          Get_BInstance(Point, varg3, arg3);
          Get_BInstance(Handler, varg4, arg4);

          arg0->DragMessage(arg1, arg2, *arg3, arg4);
        }
        else {
          drawing_mode arg3;
          BPoint *arg4;

          arg3 = (drawing_mode)NUM2ULONG(varg3);
          Get_BInstance(Point, varg4, arg4);

          arg0->DragMessage(arg1, arg2, arg3, *arg4);
        }
      }
      break;

    case 5:
      {
        BMessage *arg1;
        BBitmap *arg2;
        drawing_mode arg3;
        BPoint *arg4;
        BHandler *arg5;

        Get_BInstance(Message, varg1, arg1);
        Get_BInstance(Bitmap, varg2, arg2);
        arg3 = (drawing_mode)NUM2ULONG(varg3);
        Get_BInstance(Point, varg4, arg4);
        Get_BInstance(Handler, varg5, arg5);

        arg0->DragMessage(arg1, arg2, arg3, *arg4, arg5);
      }
      break;

    default:
      rb_bug("rb_view_DragMessage");
  }
  return Qnil;
}


static VALUE
rb_view_FindView(VALUE self, VALUE varg1)
{
  BView *arg0, *result;
  char *arg1;
  VALUE vresult;

  Get_BInstance(View, self, arg0);
  arg1 = STR2CSTR(varg1);
  result = arg0->FindView(arg1);
  vresult = ((result) ? Data_Wrap_Struct(cView, 0, 0, result):Qnil);
  return vresult;
}


static VALUE
rb_view_Parent(VALUE self)
{
  BView *arg0, *result;
  VALUE vresult;

  Get_BInstance(View, self, arg0);
  result = arg0->Parent();
  vresult = ((result) ? Data_Wrap_Struct(cView, 0, 0, result):Qnil);
  return vresult;
}


static VALUE
rb_view_Bounds(VALUE self)
{
  BView *arg0;
  BRect *result;
  VALUE vresult;

  Get_BInstance(View, self, arg0);
  result = new Rect(arg0->Bounds());
  vresult = Data_Wrap_Struct(cRect, 0, delete_rect, result);
  return vresult;
}


static VALUE
rb_view_Frame(VALUE self)
{
  BView *arg0;
  BRect *result;
  VALUE vresult;

  Get_BInstance(View, self, arg0);
  result = new Rect(arg0->Frame());
  vresult = Data_Wrap_Struct(cRect, 0, delete_rect, result);
  return vresult;
}


static VALUE
rb_view_ConvertToScreen(VALUE self, VALUE varg1)
{
  BView *arg0;
  VALUE vresult;

  Get_BInstance(View, self, arg0);
  if (rb_obj_is_kind_of(varg1, cPoint)) {
    BPoint *arg1, *result;

    Get_BInstance(Point, varg1, arg1);
    result = new Point(arg0->ConvertToScreen(*arg1));
    vresult = Data_Wrap_Struct(cPoint, 0, delete_point, result);
  }
  else {
    BRect *arg1, *result;

    Get_BInstance(Rect, varg1, arg1);
    result = new Rect(arg0->ConvertToScreen(*arg1));
    vresult = Data_Wrap_Struct(cRect, 0, delete_rect, result);
  }
  return vresult;
}


static VALUE
rb_view_ConvertFromScreen(VALUE self, VALUE varg1)
{
  BView *arg0;
  VALUE vresult;

  Get_BInstance(View, self, arg0);
  if (rb_obj_is_kind_of(varg1, cPoint)) {
    BPoint *arg1, *result;

    Get_BInstance(Point, varg1, arg1);
    result = new Point(arg0->ConvertFromScreen(*arg1));
    vresult = Data_Wrap_Struct(cPoint, 0, delete_point, result);
  }
  else {
    BRect *arg1, *result;

    Get_BInstance(Rect, varg1, arg1);
    result = new Rect(arg0->ConvertFromScreen(*arg1));
    vresult = Data_Wrap_Struct(cRect, 0, delete_rect, result);
  }
  return vresult;
}


static VALUE
rb_view_ConvertToParent(VALUE self, VALUE varg1)
{
  BView *arg0;
  VALUE vresult;

  Get_BInstance(View, self, arg0);
  if (rb_obj_is_kind_of(varg1, cPoint)) {
    BPoint *arg1, *result;

    Get_BInstance(Point, varg1, arg1);
    result = new Point(arg0->ConvertToParent(*arg1));
    vresult = Data_Wrap_Struct(cPoint, 0, delete_point, result);
  }
  else {
    BRect *arg1, *result;

    Get_BInstance(Rect, varg1, arg1);
    result = new Rect(arg0->ConvertToParent(*arg1));
    vresult = Data_Wrap_Struct(cRect, 0, delete_rect, result);
  }
  return vresult;
}


static VALUE
rb_view_ConvertFromParent(VALUE self, VALUE varg1)
{
  BView *arg0;
  VALUE vresult;

  Get_BInstance(View, self, arg0);
  if (rb_obj_is_kind_of(varg1, cPoint)) {
    BPoint *arg1, *result;

    Get_BInstance(Point, varg1, arg1);
    result = new Point(arg0->ConvertFromParent(*arg1));
    vresult = Data_Wrap_Struct(cPoint, 0, delete_point, result);
  }
  else {
    BRect *arg1, *result;

    Get_BInstance(Rect, varg1, arg1);
    result = new Rect(arg0->ConvertFromParent(*arg1));
    vresult = Data_Wrap_Struct(cRect, 0, delete_rect, result);
  }
  return vresult;
}


static VALUE
rb_view_LeftTop(VALUE self)
{
  BView *arg0;
  BPoint *result;
  VALUE vresult;

  Get_BInstance(View, self, arg0);
  result = new Point(arg0->LeftTop());
  vresult = Data_Wrap_Struct(cPoint, 0, delete_point, result);
  return vresult;
}


static VALUE
rb_view_GetClippingRegion(VALUE self)
{
  BView *arg0;
  BRegion arg1, *result;
  VALUE vresult;

  Get_BInstance(View, self, arg0);
  arg0->GetClippingRegion(&arg1);
  NEW(result, BRegion(arg1));
  vresult = Data_Wrap_Struct(cRegion, 0, delete_region, result);
  return vresult;
}


static VALUE
rb_view_ConstrainClippingRegion(VALUE self, VALUE varg1)
{
  BView *arg0;
  BRegion *arg1;

  Get_BInstance(View, self, arg0);
  Get_BInstance(Region, varg1, arg1);
  arg0->BView::ConstrainClippingRegion(arg1);
  return Qnil;
}


static VALUE
rb_view_SetDrawingMode(VALUE self, VALUE varg1)
{
  BView *arg0;
  drawing_mode arg1;

  Get_BInstance(View, self, arg0);
  arg1 = (drawing_mode)NUM2ULONG(varg1);
  arg0->BView::SetDrawingMode(arg1);
  return Qnil;
}


static VALUE
rb_view_SetPenSize(VALUE self, VALUE varg1)
{
  BView *arg0;
  float arg1;

  Get_BInstance(View, self, arg0);
  arg1 = NUM2DBL(varg1);
  arg0->BView::SetPenSize(arg1);
  return Qnil;
}


static VALUE
rb_view_SetViewColor(VALUE self, VALUE varg1)
{
  BView *arg0;
  rgb_color arg1;

  Get_BInstance(View, self, arg0);
  arg1 = my_hash2rgb_color(varg1);
  arg0->BView::SetViewColor(arg1);
  return Qnil;
}


static VALUE
rb_view_SetHighColor(VALUE self, VALUE varg1)
{
  BView *arg0;
  rgb_color arg1;

  Get_BInstance(View, self, arg0);
  arg1 = my_hash2rgb_color(varg1);
  arg0->BView::SetHighColor(arg1);
  return Qnil;
}


static VALUE
rb_view_SetLowColor(VALUE self, VALUE varg1)
{
  BView *arg0;
  rgb_color arg1;

  Get_BInstance(View, self, arg0);
  arg1 = my_hash2rgb_color(varg1);
  arg0->BView::SetLowColor(arg1);
  return Qnil;
}


static VALUE
rb_view_MovePenTo(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1, varg2;
  BView *arg0;

  Get_BInstance(View, self, arg0);
  switch (rb_scan_args(argc, argv, "11", &varg1, &varg2)) {
    case 1:
      {
        BPoint *arg1;

        Get_BInstance(Point, varg1, arg1);
        arg0->MovePenTo(*arg1);
      }
      break;

    case 2:
      {
        float arg1, arg2;

        arg1 = NUM2DBL(varg1);
        arg2 = NUM2DBL(varg2);
        arg0->MovePenTo(arg1, arg2);
      }
      break;

    default:
      rb_bug("rb_view_MovePenTo");
  }
  return Qnil;
}


static VALUE
rb_view_FillRect(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1, varg2;
  BView *arg0;

  Get_BInstance(View, self, arg0);
  switch (rb_scan_args(argc, argv, "11", &varg1, &varg2)) {
    case 1:
      {
        BRect *arg1;

        Get_BInstance(Rect, varg1, arg1);
        arg0->FillRect(*arg1);
      }
      break;

    case 2:
      {
        BRect *arg1;
        char *pat;
        pattern arg2;

        Get_BInstance(Rect, varg1, arg1);
        pat = STR2CSTR(varg2);
        for (int32 i=0; i < 8; i++) {
          arg2.data[i] = pat[i];
        }
        arg0->FillRect(*arg1, arg2);
      }
      break;

    default:
      rb_bug("rb_view_FillRect");
  }
  return Qnil;
}


static VALUE
rb_view_DrawString(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1, varg2, varg3, varg4;
  BView *arg0;

  Get_BInstance(View, self, arg0);
  switch (rb_scan_args(argc, argv, "13", &varg1, &varg2, &varg3, &varg4)) {
    case 1:
      {
        char *arg1;

        arg1 = STR2CSTR(varg1);
        arg0->DrawString(arg1);
      }
      break;

    case 2:
      {
        char *arg1;
        arg1 = STR2CSTR(varg1);

        if (rb_obj_is_kind_of(varg2, cPoint)) {
          BPoint *arg2;

          Get_BInstance(Point, varg2, arg2);
          arg0->DrawString(arg1, *arg2);
        }
        else if (rb_obj_is_kind_of(varg2, rb_cInteger)) {
          int32 arg2;

          arg2 = NUM2LONG(varg2);
          arg0->DrawString(arg1, arg2);
        }
        else {
          escapement_delta arg2;

          Check_Type(varg2, T_ARRAY);
          Check_Is_Kind_Of(RARRAY(varg2)->ptr[0], rb_cNumeric);
          Check_Is_Kind_Of(RARRAY(varg2)->ptr[1], rb_cNumeric);

          arg2.nonspace = NUM2DBL(RARRAY(varg2)->ptr[0]);
          arg2.space    = NUM2DBL(RARRAY(varg2)->ptr[1]);
          arg0->DrawString(arg1, &arg2);
        }
      }
      break;

    case 3:
      {
        char *arg1;
        arg1 = STR2CSTR(varg1);

        if (rb_obj_is_kind_of(varg2, cPoint)) {
          BPoint *arg2;
          escapement_delta arg3;

          Get_BInstance(Point, varg2, arg2);
          Check_Type(varg3, T_ARRAY);
          Check_Is_Kind_Of(RARRAY(varg3)->ptr[0], rb_cNumeric);
          Check_Is_Kind_Of(RARRAY(varg3)->ptr[1], rb_cNumeric);

          arg3.nonspace = NUM2DBL(RARRAY(varg3)->ptr[0]);
          arg3.space    = NUM2DBL(RARRAY(varg3)->ptr[1]);
          arg0->DrawString(arg1, *arg2, &arg3);
        }
        else {
          int32 arg2;
          arg2 = NUM2LONG(varg2);

          if (rb_obj_is_kind_of(varg3, cPoint)) {
            BPoint *arg3;

            Get_BInstance(Point, varg3, arg3);
            arg0->DrawString(arg1, arg2, *arg3);
          }
          else {
            escapement_delta arg3;

            Check_Type(varg3, T_ARRAY);
            Check_Is_Kind_Of(RARRAY(varg3)->ptr[0], rb_cNumeric);
            Check_Is_Kind_Of(RARRAY(varg3)->ptr[1], rb_cNumeric);

            arg3.nonspace = NUM2DBL(RARRAY(varg3)->ptr[0]);
            arg3.space    = NUM2DBL(RARRAY(varg3)->ptr[1]);
            arg0->DrawString(arg1, arg2, &arg3);
          }
        }
      }
      break;

    case 4:
      {
        char *arg1;
        int32 arg2;
        BPoint *arg3;
        escapement_delta arg4;

        arg1 = STR2CSTR(varg1);
        arg2 = NUM2LONG(varg2);
        Get_BInstance(Point, varg3, arg3);

        Check_Type(varg4, T_ARRAY);
        Check_Is_Kind_Of(RARRAY(varg4)->ptr[0], rb_cNumeric);
        Check_Is_Kind_Of(RARRAY(varg4)->ptr[1], rb_cNumeric);
        arg4.nonspace = NUM2DBL(RARRAY(varg4)->ptr[0]);
        arg4.space    = NUM2DBL(RARRAY(varg4)->ptr[1]);

        arg0->DrawString(arg1, arg2, *arg3, &arg4);
      }
      break;

    default:
      rb_bug("rb_view_DrawString");
  }
  return Qnil;
}


static VALUE
rb_view_SetFont(int argc, VALUE *argv, VALUE self)
{
  BView *arg0;
  VALUE varg1, varg2;

  Get_BInstance(View, self, arg0);
  switch (rb_scan_args(argc, argv, "11", &varg1, &varg2)) {
    case 1:
      {
        BFont *arg1;

        Get_BInstance(Font, varg1, arg1);
        arg0->BView::SetFont(arg1);
      }
      break;

    case 2:
      {
        BFont *arg1;
        uint32 arg2;

        Get_BInstance(Font, varg1, arg1);
        arg2 = NUM2ULONG(varg2);
        arg0->BView::SetFont(arg1, arg2);
      }
      break;

    default:
      rb_bug("rb_view_SetFont");
  }
  return Qnil;
}


static VALUE
rb_view_SetFontSize(VALUE self, VALUE varg1)
{
  BView *arg0;
  float arg1;

  Get_BInstance(View, self, arg0);
  arg1 = NUM2DBL(varg1);
  arg0->SetFontSize(arg1);
  return Qnil;
}


static VALUE
rb_view_Invalidate(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1;
  BView *arg0;

  Get_BInstance(View, self, arg0);
  switch (rb_scan_args(argc, argv, "01", &varg1)) {
    case 0:
      arg0->Invalidate();
      break;

    case 1:
      if (rb_obj_is_kind_of(varg1, cRect)) {
        BRect *arg1;

        Get_BInstance(Rect, varg1, arg1);
        arg0->Invalidate(*arg1);
      }
      else {
        BRegion *arg1;

        Get_BInstance(Region, varg1, arg1);
        arg0->Invalidate(arg1);
      }
      break;

    default:
      rb_bug("rb_view_Invalidate");
  }
  return Qnil;
}


static VALUE
rb_view_SetMouseEventMask(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1, varg2;
  BView *arg0;
  status_t result;

  Get_BInstance(View, self, arg0);
  switch (rb_scan_args(argc, argv, "11", &varg1, &varg2)) {
    case 1:
      {
        uint32 arg1;

        arg1 = NUM2ULONG(varg1);
        result = arg0->SetMouseEventMask(arg1);
      }
      break;

    case 2:
      {
        uint32 arg1, arg2;

        arg1 = NUM2ULONG(varg1);
        arg2 = NUM2ULONG(varg2);
        result = arg0->SetMouseEventMask(arg1, arg2);
      }
      break;

    default:
      rb_bug("rb_view_SetMouseEventMask");
  }
/* -- ?
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
*/
  return Qnil;
}


static VALUE
rb_view_SetFlags(VALUE self, VALUE varg1)
{
  BView *arg0;
  uint32 arg1;

  Get_BInstance(View, self, arg0);
  arg1 = NUM2ULONG(varg1);
  arg0->BView::SetFlags(arg1);
  return Qnil;
}


static VALUE
rb_view_SetResizingMode(VALUE self, VALUE varg1)
{
  BView *arg0;
  uint32 arg1;

  Get_BInstance(View, self, arg0);
  arg1 = NUM2ULONG(varg1);
  arg0->BView::SetResizingMode(arg1);
  return Qnil;
}


static VALUE
rb_view_ScrollTo(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1, varg2;
  BView *arg0;

  Get_BInstance(View, self, arg0);
  switch (rb_scan_args(argc, argv, "11", &varg1, &varg2)) {
    case 1:
      {
        BPoint *arg1;

        Get_BInstance(Point, varg1, arg1);
        arg0->BView::ScrollTo(*arg1);
      }
      break;

    case 2:
      {
        float arg1, arg2;

        arg1 = NUM2DBL(varg1);
        arg2 = NUM2DBL(varg2);
        arg0->ScrollTo(arg1, arg2);
      }
      break;

    default:
      rb_bug("rb_view_ScrollTo");
  }
  return Qnil;
}


static VALUE
rb_view_MakeFocus(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1;
  BView *arg0;

  Get_BInstance(View, self, arg0);
  switch (rb_scan_args(argc, argv, "01", &varg1)) {
    case 0:
      arg0->BView::MakeFocus();
      break;

    case 1:
      {
        bool arg1;

        arg1 = RTEST(varg1);
        arg0->BView::MakeFocus(arg1);
      }
      break;

    default:
      rb_bug("rb_view_MakeFocus");
  }
  return Qnil;
}


static VALUE
rb_view_Show(VALUE self)
{
  BView *arg0;

  Get_BInstance(View, self, arg0);
  arg0->BView::Show();
  return Qnil;
}


static VALUE
rb_view_Hide(VALUE self)
{
  BView *arg0;

  Get_BInstance(View, self, arg0);
  arg0->BView::Hide();
  return Qnil;
}


static VALUE
rb_view_GetPreferredSize(int argc, VALUE *argv, VALUE self)
{
  BView *arg0;
  float arg1 = 0.0, arg2 = 0.0;
  VALUE vresult;

  Get_BInstance(View, self, arg0);
  arg0->BView::GetPreferredSize(&arg1, &arg2);
  vresult = rb_ary_new3(2, rb_float_new(arg1), rb_float_new(arg2));
  return vresult;
}


static VALUE
rb_view_ResizeToPreferred(VALUE self)
{
  BView *arg0;

  Get_BInstance(View, self, arg0);
  arg0->BView::ResizeToPreferred();
  return Qnil;
}


static VALUE
rb_view_ResolveSpecifier(VALUE self, VALUE varg1, VALUE varg2, VALUE varg3, VALUE varg4, VALUE varg5)
{
  BView *arg0;
  BMessage *arg1,  *arg3;
  int32 arg2, arg4;
  char *arg5;
  BHandler *result;
  VALUE vresult;

  Get_BInstance(View, self, arg0);
  Get_BInstance(Message, varg1, arg1);
  arg2 = NUM2LONG(varg2);
  Get_BInstance(Message, varg3, arg3);
  arg4 = NUM2LONG(varg4);
  arg5 = STR2CSTR(varg5);

  result = arg0->BView::ResolveSpecifier(arg1, arg2, arg3, arg4, arg5);
  vresult = ((result) ? Data_Wrap_Struct(cHandler, 0, 0, result):Qnil);
  return vresult;
}


static VALUE
rb_view_GetSupportedSuites(VALUE self, VALUE varg1)
{
  BView *arg0;
  BMessage *arg1;
  status_t result;

  Get_BInstance(View, self, arg0);
  Get_BInstance(Message, varg1, arg1);
  result = arg0->BView::GetSupportedSuites(arg1);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


void
Init_View()
{
  cView = rb_define_class_under(mBe, "View", cHandler);
  rb_define_singleton_method(cView, "new", RUBY_METHOD_FUNC(rb_view_s_new), -1);
  rb_define_method(cView, "initialize", RUBY_METHOD_FUNC(rb_view_initialize), -1);

  rb_define_method(cView, "AttachedToWindow", RUBY_METHOD_FUNC(rb_view_AttachedToWindow), 0);
  rb_define_method(cView, "AllAttached", RUBY_METHOD_FUNC(rb_view_AllAttached), 0);
  rb_define_method(cView, "DetachedFromWindow", RUBY_METHOD_FUNC(rb_view_DetachedFromWindow), 0);
  rb_define_method(cView, "AllDetached", RUBY_METHOD_FUNC(rb_view_AllDetached), 0);

  rb_define_method(cView, "MessageReceived", RUBY_METHOD_FUNC(rb_view_MessageReceived), 1);

  rb_define_method(cView, "AddChild", RUBY_METHOD_FUNC(rb_view_AddChild), -1);
  rb_define_method(cView, "RemoveChild", RUBY_METHOD_FUNC(rb_view_RemoveChild), 1);
  rb_define_method(cView, "CountChildren", RUBY_METHOD_FUNC(rb_view_CountChildren), 0);
  rb_define_method(cView, "ChildAt", RUBY_METHOD_FUNC(rb_view_ChildAt), 1);
  rb_define_method(cView, "NextSibling", RUBY_METHOD_FUNC(rb_view_NextSibling), 0);
  rb_define_method(cView, "PreviousSibling", RUBY_METHOD_FUNC(rb_view_PreviousSibling), 0);
  rb_define_method(cView, "RemoveSelf", RUBY_METHOD_FUNC(rb_view_RemoveSelf), 0);

  rb_define_method(cView, "Window", RUBY_METHOD_FUNC(rb_view_Window), 0);

  rb_define_method(cView, "Draw", RUBY_METHOD_FUNC(rb_view_Draw), 1);
  rb_define_method(cView, "MouseDown", RUBY_METHOD_FUNC(rb_view_MouseDown), 1);
  rb_define_method(cView, "MouseUp", RUBY_METHOD_FUNC(rb_view_MouseUp), 1);
  rb_define_method(cView, "MouseMoved", RUBY_METHOD_FUNC(rb_view_MouseMoved), 3);
  rb_define_method(cView, "WindowActivated", RUBY_METHOD_FUNC(rb_view_WindowActivated), 1);
  rb_define_method(cView, "KeyDown", RUBY_METHOD_FUNC(rb_view_KeyDown), 1);
  rb_define_method(cView, "KeyUp", RUBY_METHOD_FUNC(rb_view_KeyUp), 1);
  rb_define_method(cView, "Pulse", RUBY_METHOD_FUNC(rb_view_Pulse), 0);
  rb_define_method(cView, "FrameMoved", RUBY_METHOD_FUNC(rb_view_FrameMoved), 1);
  rb_define_method(cView, "FrameResized", RUBY_METHOD_FUNC(rb_view_FrameResized), 2);

  rb_define_method(cView, "TargetedByScrollView", RUBY_METHOD_FUNC(rb_view_TargetedByScrollView), 1);
  rb_define_method(cView, "BeginRectTracking", RUBY_METHOD_FUNC(rb_view_BeginRectTracking), -1);
  rb_define_method(cView, "EndRectTracking", RUBY_METHOD_FUNC(rb_view_EndRectTracking), 0);

  rb_define_method(cView, "GetMouse", RUBY_METHOD_FUNC(rb_view_GetMouse), -1);

  rb_define_method(cView, "DragMessage", RUBY_METHOD_FUNC(rb_view_DragMessage), -1);

  rb_define_method(cView, "FindView", RUBY_METHOD_FUNC(rb_view_FindView), 1);
  rb_define_method(cView, "Parent", RUBY_METHOD_FUNC(rb_view_Parent), 0);
  rb_define_method(cView, "Bounds", RUBY_METHOD_FUNC(rb_view_Bounds), 0);
  rb_define_method(cView, "Frame", RUBY_METHOD_FUNC(rb_view_Frame), 0);
  rb_define_method(cView, "ConvertToScreen", RUBY_METHOD_FUNC(rb_view_ConvertToScreen), 1);
  rb_define_method(cView, "ConvertFromScreen", RUBY_METHOD_FUNC(rb_view_ConvertFromScreen), 1);
  rb_define_method(cView, "ConvertToParent", RUBY_METHOD_FUNC(rb_view_ConvertToParent), 1);
  rb_define_method(cView, "ConvertFromParent", RUBY_METHOD_FUNC(rb_view_ConvertFromParent), 1);
  rb_define_method(cView, "LeftTop", RUBY_METHOD_FUNC(rb_view_LeftTop), 0);

  rb_define_method(cView, "GetClippingRegion", RUBY_METHOD_FUNC(rb_view_GetClippingRegion), 1);
  rb_define_method(cView, "ConstrainClippingRegion", RUBY_METHOD_FUNC(rb_view_ConstrainClippingRegion), 1);
  rb_define_method(cView, "SetDrawingMode", RUBY_METHOD_FUNC(rb_view_SetDrawingMode), 1);
  rb_define_method(cView, "SetPenSize", RUBY_METHOD_FUNC(rb_view_SetPenSize), 1);
  rb_define_method(cView, "SetViewColor", RUBY_METHOD_FUNC(rb_view_SetViewColor), 1);
  rb_define_method(cView, "SetHighColor", RUBY_METHOD_FUNC(rb_view_SetHighColor), 1);
  rb_define_method(cView, "SetLowColor", RUBY_METHOD_FUNC(rb_view_SetLowColor), 1);

  rb_define_method(cView, "MovePenTo", RUBY_METHOD_FUNC(rb_view_MovePenTo), -1);
  rb_define_method(cView, "FillRect", RUBY_METHOD_FUNC(rb_view_FillRect), -1);
  rb_define_method(cView, "DrawString", RUBY_METHOD_FUNC(rb_view_DrawString), -1);

  rb_define_method(cView, "SetFont", RUBY_METHOD_FUNC(rb_view_SetFont), -1);
  rb_define_method(cView, "SetFontSize", RUBY_METHOD_FUNC(rb_view_SetFontSize), 1);
  rb_define_method(cView, "Invalidate", RUBY_METHOD_FUNC(rb_view_Invalidate), -1);

  rb_define_method(cView, "SetMouseEventMask", RUBY_METHOD_FUNC(rb_view_SetMouseEventMask), -1);

  rb_define_method(cView, "SetFlags", RUBY_METHOD_FUNC(rb_view_SetFlags), 1);
  rb_define_method(cView, "SetResizingMode", RUBY_METHOD_FUNC(rb_view_SetResizingMode), 1);
  rb_define_method(cView, "ScrollTo", RUBY_METHOD_FUNC(rb_view_ScrollTo), -1);
  rb_define_method(cView, "MakeFocus", RUBY_METHOD_FUNC(rb_view_MakeFocus), -1);

  rb_define_method(cView, "Show", RUBY_METHOD_FUNC(rb_view_Show), 0);
  rb_define_method(cView, "Hide", RUBY_METHOD_FUNC(rb_view_Hide), 0);

  rb_define_method(cView, "GetPreferredSize", RUBY_METHOD_FUNC(rb_view_GetPreferredSize), -1);
  rb_define_method(cView, "ResizeToPreferred", RUBY_METHOD_FUNC(rb_view_ResizeToPreferred), 0);

  rb_define_method(cView, "ResolveSpecifier", RUBY_METHOD_FUNC(rb_view_ResolveSpecifier), 5);
  rb_define_method(cView, "GetSupportedSuites", RUBY_METHOD_FUNC(rb_view_GetSupportedSuites), 1);

  rb_define_const(mBe, "PRIMARY_MOUSE_BUTTON", ULONG2NUM(B_PRIMARY_MOUSE_BUTTON));
  rb_define_const(mBe, "SECONDARY_MOUSE_BUTTON", ULONG2NUM(B_SECONDARY_MOUSE_BUTTON));
  rb_define_const(mBe, "TERTIARY_MOUSE_BUTTON", ULONG2NUM(B_TERTIARY_MOUSE_BUTTON));

  rb_define_const(mBe, "ENTERED_VIEW", ULONG2NUM(B_ENTERED_VIEW));
  rb_define_const(mBe, "INSIDE_VIEW", ULONG2NUM(B_INSIDE_VIEW));
  rb_define_const(mBe, "EXITED_VIEW", ULONG2NUM(B_EXITED_VIEW));
  rb_define_const(mBe, "OUTSIDE_VIEW", ULONG2NUM(B_OUTSIDE_VIEW));

  rb_define_const(mBe, "POINTER_EVENTS", ULONG2NUM(B_POINTER_EVENTS));
  rb_define_const(mBe, "KEYBOARD_EVENTS", ULONG2NUM(B_KEYBOARD_EVENTS));

  rb_define_const(mBe, "LOCK_WINDOW_FOCUS", ULONG2NUM(B_LOCK_WINDOW_FOCUS));
  rb_define_const(mBe, "SUSPEND_VIEW_FOCUS", ULONG2NUM(B_SUSPEND_VIEW_FOCUS));
  rb_define_const(mBe, "NO_POINTER_HISTORY", ULONG2NUM(B_NO_POINTER_HISTORY));

  rb_define_const(mBe, "TRACK_WHOLE_RECT", ULONG2NUM(B_TRACK_WHOLE_RECT));
  rb_define_const(mBe, "TRACK_RECT_CORNER", ULONG2NUM(B_TRACK_RECT_CORNER));

  rb_define_const(mBe, "FONT_FAMILY_AND_STYLE", ULONG2NUM(B_FONT_FAMILY_AND_STYLE));
  rb_define_const(mBe, "FONT_SIZE", ULONG2NUM(B_FONT_SIZE));
  rb_define_const(mBe, "FONT_SHEAR", ULONG2NUM(B_FONT_SHEAR));
  rb_define_const(mBe, "FONT_ROTATION", ULONG2NUM(B_FONT_ROTATION));
  rb_define_const(mBe, "FONT_SPACING", ULONG2NUM(B_FONT_SPACING));
  rb_define_const(mBe, "FONT_ENCODING", ULONG2NUM(B_FONT_ENCODING));
  rb_define_const(mBe, "FONT_FACE", ULONG2NUM(B_FONT_FACE));
  rb_define_const(mBe, "FONT_FLAGS", ULONG2NUM(B_FONT_FLAGS));
  rb_define_const(mBe, "FONT_ALL", ULONG2NUM(B_FONT_ALL));

  rb_define_const(mBe, "FULL_UPDATE_ON_RESIZE", ULONG2NUM(B_FULL_UPDATE_ON_RESIZE));
  rb_define_const(mBe, "WILL_DRAW", ULONG2NUM(B_WILL_DRAW));
  rb_define_const(mBe, "PULSE_NEEDED", ULONG2NUM(B_PULSE_NEEDED));
  rb_define_const(mBe, "NAVIGABLE_JUMP", ULONG2NUM(B_NAVIGABLE_JUMP));
  rb_define_const(mBe, "FRAME_EVENTS", ULONG2NUM(B_FRAME_EVENTS));
  rb_define_const(mBe, "NAVIGABLE", ULONG2NUM(B_NAVIGABLE));
  rb_define_const(mBe, "SUBPIXEL_PRECISE", ULONG2NUM(B_SUBPIXEL_PRECISE));
  rb_define_const(mBe, "DRAW_ON_CHILDREN", ULONG2NUM(B_DRAW_ON_CHILDREN));
  rb_define_const(mBe, "INPUT_METHOD_AWARE", ULONG2NUM(B_INPUT_METHOD_AWARE));

  rb_define_const(mBe, "FOLLOW_NONE", ULONG2NUM(B_FOLLOW_NONE));
  rb_define_const(mBe, "FOLLOW_ALL_SIDES", ULONG2NUM(B_FOLLOW_ALL_SIDES));
  rb_define_const(mBe, "FOLLOW_ALL", ULONG2NUM(B_FOLLOW_ALL));

  rb_define_const(mBe, "FOLLOW_LEFT", ULONG2NUM(B_FOLLOW_LEFT));
  rb_define_const(mBe, "FOLLOW_RIGHT", ULONG2NUM(B_FOLLOW_RIGHT));
  rb_define_const(mBe, "FOLLOW_LEFT_RIGHT", ULONG2NUM(B_FOLLOW_LEFT_RIGHT));
  rb_define_const(mBe, "FOLLOW_H_CENTER", ULONG2NUM(B_FOLLOW_H_CENTER));

  rb_define_const(mBe, "FOLLOW_TOP", ULONG2NUM(B_FOLLOW_TOP));
  rb_define_const(mBe, "FOLLOW_BOTTOM", ULONG2NUM(B_FOLLOW_BOTTOM));
  rb_define_const(mBe, "FOLLOW_TOP_BOTTOM", ULONG2NUM(B_FOLLOW_TOP_BOTTOM));
  rb_define_const(mBe, "FOLLOW_V_CENTER", ULONG2NUM(B_FOLLOW_V_CENTER));
}
