/*
 *  Volume.cc
 */

#include <Volume.h>
#include <Bitmap.h>
#include <Directory.h>
#include "my_be.h"


VALUE cVolume;


void
delete_volume(BVolume *arg0)
{
  delete arg0;
}


static VALUE
rb_volume_s_new(int argc, VALUE *argv, VALUE klass)
{
  VALUE vresult = my_class_s_new(argc, argv, klass, delete_volume);
  return vresult;
}


static VALUE
rb_volume_initialize(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1;
  BVolume *result;

  Data_To_Null(self, BVolume);

  switch (rb_scan_args(argc, argv, "01", &varg1)) {
    case 0:
      result = my_create_be_obj<BVolume>();
      break;

    case 1:
      if (rb_obj_is_kind_of(varg1, cVolume)) {
        BVolume *arg1;

        Get_BInstance(Volume, varg1, arg1);
        NEW(result, BVolume(*arg1));
      }
      else {
        dev_t arg1;

        arg1 = NUM2LONG(varg1);
        NEW(result, BVolume(arg1));
      }
      break;

    default:
      rb_bug("rb_volume_initialize");
  }
  DATA_PTR(self) = result;
  return self;
}


static VALUE
rb_volume_InitCheck(VALUE self)
{
  BVolume *arg0;
  status_t result;

  Get_BInstance(Volume, self, arg0);
  result = arg0->InitCheck();
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_volume_SetTo(VALUE self, VALUE varg1)
{
  BVolume *arg0;
  dev_t arg1;
  status_t result;

  Get_BInstance(Volume, self, arg0);
  arg1 = NUM2LONG(varg1);
  result = arg0->SetTo(arg1);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_volume_Unset(VALUE self)
{
  BVolume *arg0;

  Get_BInstance(Volume, self, arg0);
  arg0->Unset();
  return Qnil;
}


static VALUE
rb_volume_Device(VALUE self)
{
  BVolume *arg0;
  dev_t result;
  VALUE vresult;

  Get_BInstance(Volume, self, arg0);
  result = arg0->Device();
  vresult = LONG2NUM(result);
  return vresult;
}


static VALUE
rb_volume_GetRootDirectory(VALUE self)
{
  BVolume *arg0;
  BDirectory arg1, *copy;
  status_t result;
  VALUE vresult;

  Get_BInstance(Volume, self, arg0);
  result = arg0->GetRootDirectory(&arg1);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  NEW(copy, BDirectory(arg1));
  vresult = Data_Wrap_Struct(cDirectory, 0, delete_directory, copy);
  return vresult;
}


static VALUE
rb_volume_Capacity(VALUE self)
{
  BVolume *arg0;
  off_t result;
  VALUE vresult;

  Get_BInstance(Volume, self, arg0);
  result = arg0->Capacity();
  vresult = LLONG2NUM(result);
  return vresult;
}


static VALUE
rb_volume_FreeBytes(VALUE self)
{
  BVolume *arg0;
  off_t result;
  VALUE vresult;

  Get_BInstance(Volume, self, arg0);
  result = arg0->FreeBytes();
  vresult = LLONG2NUM(result);
  return vresult;
}


static VALUE
rb_volume_GetName(VALUE self)
{
  BVolume *arg0;
  char arg1[B_FILE_NAME_LENGTH];
  status_t result;
  VALUE vresult;

  Get_BInstance(Volume, self, arg0);
  result = arg0->GetName(arg1);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  vresult = rb_str_new2(arg1);
  return vresult;
}


static VALUE
rb_volume_SetName(VALUE self, VALUE varg1)
{
  BVolume *arg0;
  char *arg1;
  status_t result;

  Get_BInstance(Volume, self, arg0);
  arg1 = STR2CSTR(varg1);
  result = arg0->SetName(arg1);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


/*
static VALUE
rb_volume_GetIcon(VALUE self, VALUE varg1)
{
  BVolume *arg0;
  icon_size arg2;
  status_t result;
  VALUE vresult;

  Get_BInstance(Volume, self, arg0);
  arg2 = (icon_size)NUM2ULONG(varg1);
//  NEW(arg1, BBitmap(BRect(0, 0, arg2-1, arg2-1), B_COLOR_8_BIT));
//  result = arg0->GetIcon(arg1, arg2);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  vresult = Data_Wrap_Struct(cBitmap, 0, delete_bitmap, arg1);
  return vresult;
}
*/


static VALUE
rb_volume_IsRemovable(VALUE self)
{
  BVolume *arg0;
  bool result;
  VALUE vresult;

  Get_BInstance(Volume, self, arg0);
  result = arg0->IsRemovable();
  vresult = (result ? Qtrue:Qfalse);
  return vresult;
}


static VALUE
rb_volume_IsReadOnly(VALUE self)
{
  BVolume *arg0;
  bool result;
  VALUE vresult;

  Get_BInstance(Volume, self, arg0);
  result = arg0->IsReadOnly();
  vresult = (result ? Qtrue:Qfalse);
  return vresult;
}


static VALUE
rb_volume_IsPersistent(VALUE self)
{
  BVolume *arg0;
  bool result;
  VALUE vresult;

  Get_BInstance(Volume, self, arg0);
  result = arg0->IsPersistent();
  vresult = (result ? Qtrue:Qfalse);
  return vresult;
}


static VALUE
rb_volume_IsShared(VALUE self)
{
  BVolume *arg0;
  bool result;
  VALUE vresult;

  Get_BInstance(Volume, self, arg0);
  result = arg0->IsShared();
  vresult = (result ? Qtrue:Qfalse);
  return vresult;
}


static VALUE
rb_volume_KnowsMime(VALUE self)
{
  BVolume *arg0;
  bool result;
  VALUE vresult;

  Get_BInstance(Volume, self, arg0);
  result = arg0->KnowsMime();
  vresult = (result ? Qtrue:Qfalse);
  return vresult;
}


static VALUE
rb_volume_KnowsAttr(VALUE self)
{
  BVolume *arg0;
  bool result;
  VALUE vresult;

  Get_BInstance(Volume, self, arg0);
  result = arg0->KnowsAttr();
  vresult = (result ? Qtrue:Qfalse);
  return vresult;
}


static VALUE
rb_volume_KnowsQuery(VALUE self)
{
  BVolume *arg0;
  bool result;
  VALUE vresult;

  Get_BInstance(Volume, self, arg0);
  result = arg0->KnowsQuery();
  vresult = (result ? Qtrue:Qfalse);
  return vresult;
}


void
Init_Volume()
{
  cVolume = rb_define_class_under(mBe, "Volume", rb_cObject);
  rb_define_singleton_method(cVolume, "new", RUBY_METHOD_FUNC(rb_volume_s_new), -1);
  rb_define_method(cVolume, "initialize", RUBY_METHOD_FUNC(rb_volume_initialize), -1);

  rb_define_method(cVolume, "InitCheck", RUBY_METHOD_FUNC(rb_volume_InitCheck), 0);

  rb_define_method(cVolume, "SetTo", RUBY_METHOD_FUNC(rb_volume_SetTo), 1);
  rb_define_method(cVolume, "Unset", RUBY_METHOD_FUNC(rb_volume_Unset), 0);

  rb_define_method(cVolume, "Device", RUBY_METHOD_FUNC(rb_volume_Device), 0);

  rb_define_method(cVolume, "GetRootDirectory", RUBY_METHOD_FUNC(rb_volume_GetRootDirectory), 0);

  rb_define_method(cVolume, "Capacity", RUBY_METHOD_FUNC(rb_volume_Capacity), 0);
  rb_define_method(cVolume, "FreeBytes", RUBY_METHOD_FUNC(rb_volume_FreeBytes), 0);

  rb_define_method(cVolume, "GetName", RUBY_METHOD_FUNC(rb_volume_GetName), 0);
  rb_define_method(cVolume, "SetName", RUBY_METHOD_FUNC(rb_volume_SetName), 1);

//  rb_define_method(cVolume, "GetIcon", RUBY_METHOD_FUNC(rb_volume_GetIcon), 1);

  rb_define_method(cVolume, "IsRemovable", RUBY_METHOD_FUNC(rb_volume_IsRemovable), 0);
  rb_define_method(cVolume, "IsReadOnly", RUBY_METHOD_FUNC(rb_volume_IsReadOnly), 0);
  rb_define_method(cVolume, "IsPersistent", RUBY_METHOD_FUNC(rb_volume_IsPersistent), 0);
  rb_define_method(cVolume, "IsShared", RUBY_METHOD_FUNC(rb_volume_IsShared), 0);
  rb_define_method(cVolume, "KnowsMime", RUBY_METHOD_FUNC(rb_volume_KnowsMime), 0);
  rb_define_method(cVolume, "KnowsAttr", RUBY_METHOD_FUNC(rb_volume_KnowsAttr), 0);
  rb_define_method(cVolume, "KnowsQuery", RUBY_METHOD_FUNC(rb_volume_KnowsQuery), 0);
}
