/*
 *  Window.cc
 */

#include <Application.h>
#include "rbPoint.h"
#include "rbRect.h"
#include "rbWindow.h"
#include "my_be.h"


VALUE cWindow;


void
delete_window(BWindow *arg0)
{
  if (arg0->Lock()) {
    arg0->BWindow::Quit();
  }
}


static VALUE
rb_window_s_new(int argc, VALUE *argv, VALUE klass)
{
  VALUE vresult = my_class_s_new(argc, argv, klass, delete_window);
  return vresult;
}


static VALUE
rb_window_initialize(int argc, VALUE *argv, VALUE self)
{
  Window *result;

  Data_To_Null(self, BWindow);

  if (argc >= 4) {
    VALUE varg1, varg2, varg3, varg4, varg5, varg6;

    switch (rb_scan_args(argc, argv, "42", &varg1, &varg2, &varg3, &varg4, &varg5, &varg6)) {
      case 4:
        {
          BRect *arg1;
          char *arg2;
          window_type arg3;
          uint32 arg4;

          Get_BInstance(Rect, varg1, arg1);
          arg2 = STR2CSTR(varg2);
          arg3 = (window_type)NUM2LONG(varg3);
          arg4 = NUM2ULONG(varg4);

          result = new Window(*arg1, arg2, arg3, arg4);
        }
        break;

      case 5:
        {
          BRect *arg1;
          char *arg2;
          window_type arg3;
          uint32 arg4, arg5;

          Get_BInstance(Rect, varg1, arg1);
          arg2 = STR2CSTR(varg2);
          arg3 = (window_type)NUM2LONG(varg3);
          arg4 = NUM2ULONG(varg4);
          arg5 = NUM2ULONG(varg5);

          result = new Window(*arg1, arg2, arg3, arg4, arg5);
        }
        break;

      case 6:
        {
          BRect *arg1;
          char *arg2;
          window_look arg3;
          window_feel arg4;
          uint32 arg5, arg6;

          Get_BInstance(Rect, varg1, arg1);
          arg2 = STR2CSTR(varg2);
          arg3 = (window_look)NUM2LONG(varg3);
          arg4 = (window_feel)NUM2LONG(varg4);
          arg5 = NUM2ULONG(varg5);
          arg6 = NUM2ULONG(varg6);

          result = new Window(*arg1, arg2, arg3, arg4, arg5, arg6);
        }
        break;

      default:
        rb_bug("rb_window_s_new");
    }
  }
  else {
    VALUE varg1;
    BMessage *arg1;

    rb_scan_args(argc, argv, "1", &varg1);

    Get_BInstance(Message, varg1, arg1);
    result = new Window(arg1);
  }
  rb_ivar_set(self, id_relatives, Qnil);
  result->myself = self;
  DATA_PTR(self) = result;
  return self;
}


static VALUE
rb_window_Quit(VALUE self)
{
  BWindow *arg0;

  Get_BInstance(Window, self, arg0);
  arg0->BWindow::Quit();
  return Qnil;
}


static VALUE
rb_window_Close(VALUE self)
{
  BWindow *arg0;

  Get_BInstance(Window, self, arg0);
  arg0->Close();
  return Qnil;
}


static VALUE
rb_window_AddChild(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1, varg2;
  BWindow *arg0;

  Get_BInstance(Window, self, arg0);
  switch (rb_scan_args(argc, argv, "11", &varg1, &varg2)) {
    case 1:
      {
        BView *arg1;

        Get_BInstance(View, varg1, arg1);
        arg0->AddChild(arg1);
      }
      break;

    case 2:
      {
        BView *arg1, *arg2;

        Get_BInstance(View, varg1, arg1);
        Get_BInstance(View, varg2, arg2);

        arg0->AddChild(arg1, arg2);
      }
      break;

    default:
      rb_bug("rb_window_AddChild");
  }
  RDATA(varg1)->dfree = 0;
  my_add_relative(self, varg1);
  return Qnil;
}


static VALUE
rb_window_RemoveChild(VALUE self, VALUE varg1)
{
  BWindow *arg0;
  BView *arg1;
  bool result;
  VALUE vresult;

  Get_BInstance(Window, self, arg0);
  Get_BInstance(View, varg1, arg1);
  result = arg0->RemoveChild(arg1);
  vresult = (result ? Qtrue:Qfalse);
  return vresult;
}


static VALUE
rb_window_CountChildren(VALUE self)
{
  BWindow *arg0;
  int32 result;
  VALUE vresult;

  Get_BInstance(Window, self, arg0);
  result = arg0->CountChildren();
  vresult = LONG2NUM(result);
  return vresult;
}


static VALUE
rb_window_ChildAt(VALUE self, VALUE varg1)
{
  BWindow *arg0;
  int32 arg1;
  BView *result;
  VALUE vresult;

  Get_BInstance(Window, self, arg0);
  arg1 = NUM2LONG(varg1);
  result = arg0->ChildAt(arg1);
  vresult = ((result) ? Data_Wrap_Struct(cView, 0, 0, result):Qnil);
  return vresult;
}


static VALUE
rb_window_DispatchMessage(VALUE self, VALUE varg1, VALUE varg2)
{
  BWindow *arg0;
  BMessage *arg1;
  BHandler *arg2;

  Get_BInstance(Window, self, arg0);
  Get_BInstance(Message, varg1, arg1);
  Get_BInstance(Handler, varg2, arg2);
  arg0->BWindow::DispatchMessage(arg1, arg2);
  return Qnil;
}


static VALUE
rb_window_MessageReceived(VALUE self, VALUE varg1)
{
  BWindow *arg0;
  BMessage *arg1;

  Get_BInstance(Window, self, arg0);
  Get_BInstance(Message, varg1, arg1);

  arg0->BWindow::MessageReceived(arg1);
  return Qnil;
}


static VALUE
rb_window_FrameMoved(VALUE self, VALUE varg1)
{
  BWindow *arg0;
  BPoint *arg1;

  Get_BInstance(Window, self, arg0);
  Get_BInstance(Point, varg1, arg1);
  arg0->BWindow::FrameMoved(*arg1);
  return Qnil;
}


static VALUE
rb_window_WorkspacesChanged(VALUE self, VALUE varg1, VALUE varg2)
{
  BWindow *arg0;
  uint32 arg1;
  uint32 arg2;

  Get_BInstance(Window, self, arg0);
  arg1 = NUM2ULONG(varg1);
  arg2 = NUM2ULONG(varg2);
  arg0->BWindow::WorkspacesChanged(arg1, arg2);
  return Qnil;
}


static VALUE
rb_window_WorkspaceActivated(VALUE self, VALUE varg1, VALUE varg2)
{
  BWindow *arg0;
  int32 arg1;
  bool arg2;

  Get_BInstance(Window, self, arg0);
  arg1 = NUM2LONG(varg1);
  arg2 = RTEST(varg2);
  arg0->BWindow::WorkspaceActivated(arg1, arg2);
  return Qnil;
}


static VALUE
rb_window_FrameResized(VALUE self, VALUE varg1, VALUE varg2)
{
  BWindow *arg0;
  float arg1;
  float arg2;

  Get_BInstance(Window, self, arg0);
  arg1 = NUM2DBL(varg1);
  arg2 = NUM2DBL(varg2);
  arg0->BWindow::FrameResized(arg1, arg2);
  return Qnil;
}


static VALUE
rb_window_Minimize(VALUE self, VALUE varg1)
{
  BWindow *arg0;
  bool arg1;

  Get_BInstance(Window, self, arg0);
  arg1 = RTEST(varg1);
  arg0->BWindow::Minimize(arg1);
  return Qnil;
}


static VALUE
rb_window_Zoom(int argc, VALUE *argv, VALUE self)
{
  BWindow *arg0;

  Get_BInstance(Window, self, arg0);
  if (argc > 1) {
    VALUE varg1, varg2, varg3;
    BPoint *arg1;
    float arg2, arg3;

    rb_scan_args(argc, argv, "3", &varg1, &varg2, &varg3);

    Get_BInstance(Point, varg1, arg1);
    arg2 = NUM2DBL(varg2);
    arg3 = NUM2DBL(varg3);
    arg0->BWindow::Zoom(*arg1, arg2, arg3);
  }
  else {
    rb_scan_args(argc, argv, "0");
    arg0->Zoom();
  }
  return Qnil;
}


static VALUE
rb_window_SetZoomLimits(VALUE self, VALUE varg1, VALUE varg2)
{
  BWindow *arg0;
  float arg1, arg2;

  Get_BInstance(Window, self, arg0);
  arg1 = NUM2DBL(varg1);
  arg2 = NUM2DBL(varg2);
  arg0->SetZoomLimits(arg1, arg2);
  return Qnil;
}


static VALUE
rb_window_ScreenChanged(VALUE self, VALUE varg1, VALUE varg2)
{
  BWindow *arg0;
  BRect *arg1;
  color_space arg2;

  Get_BInstance(Window, self, arg0);
  Get_BInstance(Rect, varg1, arg1);
  arg2 = (color_space)NUM2ULONG(varg2);
  arg0->BWindow::ScreenChanged(*arg1, arg2);
  return Qnil;
}


static VALUE
rb_window_SetPulseRate(VALUE self, VALUE varg1)
{
  BWindow *arg0;
  bigtime_t arg1;

  Get_BInstance(Window, self, arg0);
  arg1 = NUM2LLONG(varg1);
  arg0->SetPulseRate(arg1);
  return Qnil;
}


static VALUE
rb_window_PulseRate(VALUE self)
{
  BWindow *arg0;
  bigtime_t result;
  VALUE vresult;

  Get_BInstance(Window, self, arg0);
  result = arg0->PulseRate();
  vresult = LLONG2NUM(result);
  return vresult;
}


static VALUE
rb_window_AddShortcut(int argc, VALUE *argv, VALUE self)
{
  BWindow *arg0;
  VALUE varg1, varg2, varg3, varg4;

  Get_BInstance(Window, self, arg0);
  switch (rb_scan_args(argc, argv, "31", &varg1, &varg2, &varg3, &varg4)) {
    case 3:
      {
        uint32 arg1, arg2;
        BMessage *arg3;

        arg1 = NUM2ULONG(varg1);
        arg2 = NUM2ULONG(varg2);
        Get_BInstance(Message, varg3, arg3);

        arg0->AddShortcut(arg1, arg2, arg3);
      }
      break;

    case 4:
      {
        uint32 arg1, arg2;
        BMessage *arg3;
        BHandler *arg4;

        arg1 = NUM2ULONG(varg1);
        arg2 = NUM2ULONG(varg2);
        Get_BInstance(Message, varg3, arg3);
        Get_BInstance(Handler, varg4, arg4);

        arg0->AddShortcut(arg1, arg2, arg3, arg4);
      }
      break;

    default:
      rb_bug("rb_window_AddShortcut");
  }
  return Qnil;
}


static VALUE
rb_window_RemoveShortcut(VALUE self, VALUE varg1, VALUE varg2)
{
  BWindow *arg0;
  uint32 arg1, arg2;

  Get_BInstance(Window, self, arg0);
  arg1 = NUM2ULONG(varg1);
  arg2 = NUM2ULONG(varg2);
  arg0->RemoveShortcut(arg1, arg2);
  return Qnil;
}


static VALUE
rb_window_SetDefaultButton(VALUE self, VALUE varg1)
{
  BWindow *arg0;
  BButton *arg1;

  Get_BInstance(Window, self, arg0);
  Get_BInstance(Button, varg1, arg1);
  arg0->SetDefaultButton(arg1);
  return Qnil;
}


static VALUE
rb_window_DefaultButton(VALUE self)
{
  BWindow *arg0;
  BButton *result;
  VALUE vresult;

  Get_BInstance(Window, self, arg0);
  result = arg0->DefaultButton();
  vresult = ((result) ? Data_Wrap_Struct(cButton, 0, 0, result):Qnil);
  return vresult;
}


static VALUE
rb_window_MenusBeginning(VALUE self)
{
  BWindow *arg0;

  Get_BInstance(Window, self, arg0);
  arg0->BWindow::MenusBeginning();
  return Qnil;
}


static VALUE
rb_window_MenusEnded(VALUE self)
{
  BWindow *arg0;

  Get_BInstance(Window, self, arg0);
  arg0->BWindow::MenusEnded();
  return Qnil;
}


static VALUE
rb_window_NeedsUpdate(VALUE self)
{
  BWindow *arg0;
  bool result;
  VALUE vresult;

  Get_BInstance(Window, self, arg0);
  result = arg0->NeedsUpdate();
  vresult = (result ? Qtrue:Qfalse);
  return vresult;
}


static VALUE
rb_window_UpdateIfNeeded(VALUE self)
{
  BWindow *arg0;

  Get_BInstance(Window, self, arg0);
  arg0->UpdateIfNeeded();
  return Qnil;
}


static VALUE
rb_window_FindView(VALUE self, VALUE varg1)
{
  BWindow *arg0;
  BView *result;
  VALUE vresult;

  Get_BInstance(Window, self, arg0);
  if (rb_obj_is_kind_of(varg1, cPoint)) {
    BPoint *arg1;

    Get_BInstance(Point, varg1, arg1);
    result = arg0->FindView(*arg1);
  }
  else {
    char *arg1;

    arg1 = STR2CSTR(varg1);
    result = arg0->FindView(arg1);
  }
  vresult = ((result) ? Data_Wrap_Struct(cView, 0, 0, result):Qnil);
  return vresult;
}


static VALUE
rb_window_CurrentFocus(VALUE self)
{
  BWindow *arg0;
  BView *result;
  VALUE vresult;

  Get_BInstance(Window, self, arg0);
  result = arg0->CurrentFocus();
  vresult = ((result) ? Data_Wrap_Struct(cView, 0, 0, result):Qnil);
  return vresult;
}


static VALUE
rb_window_Activate(int argc, VALUE *argv, VALUE self)
{
  BWindow *arg0;
  VALUE varg1;

  Get_BInstance(Window, self, arg0);
  switch (rb_scan_args(argc, argv, "01", &varg1)) {
    case 0:
      arg0->Activate();
      break;

    case 1:
      {
        bool arg1;

        arg1 = RTEST(varg1);
        arg0->Activate(arg1);
      }
      break;

    default:
      rb_bug("rb_window_Activate");
  }
  return Qnil;
}


static VALUE
rb_window_WindowActivated(VALUE self, VALUE varg1)
{
  BWindow *arg0;
  bool arg1;

  Get_BInstance(Window, self, arg0);
  arg1 = RTEST(varg1);
  arg0->BWindow::WindowActivated(arg1);
  return Qnil;
}


static VALUE
rb_window_ConvertToScreen(VALUE self, VALUE varg1)
{
  BWindow *arg0;
  VALUE vresult;

  Get_BInstance(Window, self, arg0);
  if (rb_obj_is_kind_of(varg1, cPoint)) {
    BPoint *arg1, *result;

    Get_BInstance(Point, varg1, arg1);
    result = new Point(arg0->ConvertToScreen(*arg1));
    vresult = Data_Wrap_Struct(cPoint, 0, delete_point, result);
  }
  else {
    BRect *arg1, *result;

    Get_BInstance(Rect, varg1, arg1);
    result = new Rect(arg0->ConvertToScreen(*arg1));
    vresult = Data_Wrap_Struct(cRect, 0, delete_rect, result);
  }
  return vresult;
}


static VALUE
rb_window_ConvertFromScreen(VALUE self, VALUE varg1)
{
  BWindow *arg0;
  VALUE vresult;

  Get_BInstance(Window, self, arg0);
  if (rb_obj_is_kind_of(varg1, cPoint)) {
    BPoint *arg1, *result;

    Get_BInstance(Point, varg1, arg1);
    result = new Point(arg0->ConvertFromScreen(*arg1));
    vresult = Data_Wrap_Struct(cPoint, 0, delete_point, result);
  }
  else {
    BRect *arg1, *result;

    Get_BInstance(Rect, varg1, arg1);
    result = new Rect(arg0->ConvertFromScreen(*arg1));
    vresult = Data_Wrap_Struct(cRect, 0, delete_rect, result);
  }
  return vresult;
}


static VALUE
rb_window_MoveBy(VALUE self, VALUE varg1, VALUE varg2)
{
  BWindow *arg0;
  float arg1, arg2;

  Get_BInstance(Window, self, arg0);
  arg1 = NUM2DBL(varg1);
  arg2 = NUM2DBL(varg2);
  arg0->MoveBy(arg1,arg2);
  return Qnil;
}


static VALUE
rb_window_MoveTo(int argc, VALUE *argv, VALUE self)
{
  BWindow *arg0;
  VALUE varg1, varg2;

  Get_BInstance(Window, self, arg0);
  switch (rb_scan_args(argc, argv, "11", &varg1, &varg2)) {
    case 1:
      {
        BPoint *arg1;

        Get_BInstance(Point, varg1, arg1);
        arg0->MoveTo(*arg1);
      }
      break;

    case 2:
      {
        float arg1, arg2;

        arg1 = NUM2DBL(varg1);
        arg2 = NUM2DBL(varg2);
        arg0->MoveTo(arg1, arg2);
      }
      break;

    default:
      rb_bug("rb_window_MoveTo");
  }
  return Qnil;
}


static VALUE
rb_window_ResizeBy(VALUE self, VALUE varg1, VALUE varg2)
{
  BWindow *arg0;
  float arg1, arg2;

  Get_BInstance(Window, self, arg0);
  arg1 = NUM2DBL(varg1);
  arg2 = NUM2DBL(varg2);
  arg0->ResizeBy(arg1, arg2);
  return Qnil;
}


static VALUE
rb_window_ResizeTo(VALUE self, VALUE varg1, VALUE varg2)
{
  BWindow *arg0;
  float arg1, arg2;

  Get_BInstance(Window, self, arg0);
  arg1 = NUM2DBL(varg1);
  arg2 = NUM2DBL(varg2);
  arg0->ResizeTo(arg1, arg2);
  return Qnil;
}


static VALUE
rb_window_Show(VALUE self)
{
  BWindow *arg0;

  Get_BInstance(Window, self, arg0);
  arg0->BWindow::Show();
  return Qnil;
}


static VALUE
rb_window_Hide(VALUE self)
{
  BWindow *arg0;

  Get_BInstance(Window, self, arg0);
  arg0->BWindow::Hide();
  return Qnil;
}


static VALUE
rb_window_IsHidden(VALUE self)
{
  BWindow *arg0;
  bool result;
  VALUE vresult;

  Get_BInstance(Window, self, arg0);
  result = arg0->IsHidden();
  vresult = (result ? Qtrue:Qfalse);
  return vresult;
}


static VALUE
rb_window_IsMinimized(VALUE self)
{
  BWindow *arg0;
  bool result;
  VALUE vresult;

  Get_BInstance(Window, self, arg0);
  result = arg0->IsMinimized();
  vresult = (result ? Qtrue:Qfalse);
  return vresult;
}


static VALUE
rb_window_Flush(VALUE self)
{
  BWindow *arg0;

  Get_BInstance(Window, self, arg0);
  arg0->Flush();
  return Qnil;
}


static VALUE
rb_window_Sync(VALUE self)
{
  BWindow *arg0;

  Get_BInstance(Window, self, arg0);
  arg0->Sync();
  return Qnil;
}


static VALUE
rb_window_SendBehind(VALUE self, VALUE varg1)
{
  BWindow *arg0, *arg1;
  status_t result;

  Get_BInstance(Window, self, arg0);
  Get_BInstance(Window, varg1, arg1);
  result = arg0->SendBehind(arg1);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_window_DisableUpdates(VALUE self)
{
  BWindow *arg0;

  Get_BInstance(Window, self, arg0);
  arg0->DisableUpdates();
  return Qnil;
}


static VALUE
rb_window_EnableUpdates(VALUE self)
{
  BWindow *arg0;

  Get_BInstance(Window, self, arg0);
  arg0->EnableUpdates();
  return Qnil;
}


static VALUE
rb_window_BeginViewTransaction(VALUE self)
{
  BWindow *arg0;

  Get_BInstance(Window, self, arg0);
  arg0->BeginViewTransaction();
  return Qnil;
}


static VALUE
rb_window_EndViewTransaction(VALUE self)
{
  BWindow *arg0;

  Get_BInstance(Window, self, arg0);
  arg0->EndViewTransaction();
  return Qnil;
}


static VALUE
rb_window_Bounds(VALUE self)
{
  BWindow *arg0;
  BRect *result;
  VALUE vresult;

  Get_BInstance(Window, self, arg0);
  result = new Rect(arg0->Bounds());
  vresult = Data_Wrap_Struct(cRect, 0, delete_rect, result);
  return vresult;
}


static VALUE
rb_window_Frame(VALUE self)
{
  BWindow *arg0;
  BRect *result;
  VALUE vresult;

  Get_BInstance(Window, self, arg0);
  result = new Rect(arg0->Frame());
  vresult = Data_Wrap_Struct(cRect, 0, delete_rect, result);
  return vresult;
}


static VALUE
rb_window_Title(VALUE self)
{
  BWindow *arg0;
  const char *result;
  VALUE vresult;

  Get_BInstance(Window, self, arg0);
  result = arg0->Title();
  vresult = rb_str_new2(result);
  return vresult;
}


static VALUE
rb_window_SetTitle(VALUE self, VALUE varg1)
{
  BWindow *arg0;
  char *arg1;

  Get_BInstance(Window, self, arg0);
  arg1 = STR2CSTR(varg1);
  arg0->SetTitle(arg1);
  return Qnil;
}


static VALUE
rb_window_IsFront(VALUE self)
{
  BWindow *arg0;
  bool result;
  VALUE vresult;

  Get_BInstance(Window, self, arg0);
  result = arg0->IsFront();
  vresult = (result ? Qtrue:Qfalse);
  return vresult;
}


static VALUE
rb_window_IsActive(VALUE self)
{
  BWindow *arg0;
  bool result;
  VALUE vresult;

  Get_BInstance(Window, self, arg0);
  result = arg0->IsActive();
  vresult = (result ? Qtrue:Qfalse);
  return vresult;
}


static VALUE
rb_window_SetKeyMenuBar(VALUE self, VALUE varg1)
{
  BWindow *arg0;
  BMenuBar *arg1;

  Get_BInstance(Window, self, arg0);
  Get_BInstance(MenuBar, varg1, arg1);
  arg0->SetKeyMenuBar(arg1);
  return Qnil;
}


static VALUE
rb_window_KeyMenuBar(VALUE self)
{
  BWindow *arg0;
  BMenuBar *result;
  VALUE vresult;

  Get_BInstance(Window, self, arg0);
  result = arg0->KeyMenuBar();
  vresult = Data_Wrap_Struct(cMenuBar, 0, 0, result);
  return vresult;
}


static VALUE
rb_window_SetSizeLimits(VALUE self, VALUE varg1, VALUE varg2, VALUE varg3, VALUE varg4)
{
  BWindow *arg0;
  float arg1, arg2, arg3, arg4;

  Get_BInstance(Window, self, arg0);
  arg1 = NUM2DBL(varg1);
  arg2 = NUM2DBL(varg2);
  arg3 = NUM2DBL(varg3);
  arg4 = NUM2DBL(varg4);
  arg0->SetSizeLimits(arg1, arg2, arg3, arg4);
  return Qnil;
}


static VALUE
rb_window_GetSizeLimits(VALUE self)
{
  BWindow *arg0;
  float arg1, arg2, arg3, arg4;
  VALUE vresult;

  Get_BInstance(Window, self, arg0);
  arg0->GetSizeLimits(&arg1, &arg2, &arg3, &arg4);
  vresult = rb_ary_new3(4, rb_float_new(arg1),
                           rb_float_new(arg2),
                           rb_float_new(arg3),
                           rb_float_new(arg4));
  return vresult;
}


static VALUE
rb_window_Workspaces(VALUE self)
{
  BWindow *arg0;
  uint32 result;
  VALUE vresult;

  Get_BInstance(Window, self, arg0);
  result = arg0->Workspaces();
  vresult = ULONG2NUM(result);
  return vresult;
}


static VALUE
rb_window_SetWorkspaces(VALUE self, VALUE varg1)
{
  BWindow *arg0;
  uint32 arg1;

  Get_BInstance(Window, self, arg0);
  arg1 = NUM2ULONG(varg1);
  arg0->SetWorkspaces(arg1);
  return Qnil;
}


static VALUE
rb_window_LastMouseMovedView(VALUE self)
{
  BWindow *arg0;
  BView *result;
  VALUE vresult;

  Get_BInstance(Window, self, arg0);
  result = arg0->LastMouseMovedView();
  vresult = ((result) ? Data_Wrap_Struct(cView, 0, 0, result):Qnil);
  return vresult;
}


static VALUE
rb_window_ResolveSpecifier(VALUE self, VALUE varg1, VALUE varg2, VALUE varg3, VALUE varg4, VALUE varg5)
{
  BWindow *arg0;
  BMessage *arg1,  *arg3;
  int32 arg2, arg4;
  char *arg5;
  BHandler *result;
  VALUE vresult;

  Get_BInstance(Window, self, arg0);
  Get_BInstance(Message, varg1, arg1);
  arg2 = NUM2LONG(varg2);
  Get_BInstance(Message, varg3, arg3);
  arg4 = NUM2LONG(varg4);
  arg5 = STR2CSTR(varg5);

  result = arg0->BWindow::ResolveSpecifier(arg1, arg2, arg3, arg4, arg5);
  vresult = ((result) ? Data_Wrap_Struct(cHandler, 0, 0, result):Qnil);
  return vresult;
}


static VALUE
rb_window_GetSupportedSuites(VALUE self, VALUE varg1)
{
  BWindow *arg0;
  BMessage *arg1;
  status_t result;

  Get_BInstance(Window, self, arg0);
  Get_BInstance(Message, varg1, arg1);
  result = arg0->BWindow::GetSupportedSuites(arg1);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_window_AddToSubset(VALUE self, VALUE varg1)
{
  BWindow *arg0, *arg1;
  status_t result;

  Get_BInstance(Window, self, arg0);
  Get_BInstance(Window, varg1, arg1);
  result = arg0->AddToSubset(arg1);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_window_RemoveFromSubset(VALUE self, VALUE varg1)
{
  BWindow *arg0, *arg1;
  status_t result;

  Get_BInstance(Window, self, arg0);
  Get_BInstance(Window, varg1, arg1);
  result = arg0->RemoveFromSubset(arg1);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_window_SetType(VALUE self, VALUE varg1)
{
  BWindow *arg0;
  window_type arg1;
  status_t result;

  Get_BInstance(Window, self, arg0);
  arg1 = (window_type)NUM2LONG(varg1);
  result = arg0->SetType(arg1);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_window_Type(VALUE self)
{
  BWindow *arg0;
  window_type result;
  VALUE vresult;

  Get_BInstance(Window, self, arg0);
  result = arg0->Type();
  vresult = LONG2NUM(result);
  return vresult;
}


static VALUE
rb_window_SetLook(VALUE self, VALUE varg1)
{
  BWindow *arg0;
  window_look arg1;
  status_t result;

  Get_BInstance(Window, self, arg0);
  arg1 = (window_look)NUM2LONG(varg1);
  result = arg0->SetLook(arg1);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_window_Look(VALUE self)
{
  BWindow *arg0;
  window_look result;
  VALUE vresult;

  Get_BInstance(Window, self, arg0);
  result = arg0->Look();
  vresult = LONG2NUM(result);
  return vresult;
}


static VALUE
rb_window_SetFeel(VALUE self, VALUE varg1)
{
  BWindow *arg0;
  window_feel arg1;
  status_t result;

  Get_BInstance(Window, self, arg0);
  arg1 = (window_feel)NUM2LONG(varg1);
  result = arg0->SetFeel(arg1);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_window_Feel(VALUE self)
{
  BWindow *arg0;
  window_feel result;
  VALUE vresult;

  Get_BInstance(Window, self, arg0);
  result = arg0->Feel();
  vresult = LONG2NUM(result);
  return vresult;
}


static VALUE
rb_window_SetFlags(VALUE self, VALUE varg1)
{
  BWindow *arg0;
  uint32 arg1;
  status_t result;

  Get_BInstance(Window, self, arg0);
  arg1 = NUM2ULONG(varg1);
  result = arg0->SetFlags(arg1);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_window_Flags(VALUE self)
{
  BWindow *arg0;
  uint32 result;
  VALUE vresult;

  Get_BInstance(Window, self, arg0);
  result = arg0->Flags();
  vresult = ULONG2NUM(result);
  return vresult;
}


static VALUE
rb_window_IsModal(VALUE self)
{
  BWindow *arg0;
  bool result;
  VALUE vresult;

  Get_BInstance(Window, self, arg0);
  result = arg0->IsModal();
  vresult = (result ? Qtrue:Qfalse);
  return vresult;
}


static VALUE
rb_window_IsFloating(VALUE self)
{
  BWindow *arg0;
  bool result;
  VALUE vresult;

  Get_BInstance(Window, self, arg0);
  result = arg0->IsFloating();
  vresult = (result ? Qtrue:Qfalse);
  return vresult;
}


static VALUE
rb_window_SetWindowAlignment(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1, varg2, varg3, varg4, varg5, varg6, varg7, varg8, varg9;
  BWindow *arg0;
  status_t result;

  Get_BInstance(Window, self, arg0);
  switch (rb_scan_args(argc, argv, "27", &varg1, &varg2, &varg3, &varg4, &varg5, &varg6, &varg7, &varg8, &varg9)) {
    case 2:
      {
        window_alignment arg1;
        int32 arg2;

        arg1 = (window_alignment)NUM2LONG(varg1);
        arg2 = NUM2LONG(varg2);
        result = arg0->SetWindowAlignment(arg1, arg2);
      }
      break;

    case 3:
      {
        window_alignment arg1;
        int32 arg2, arg3;

        arg1 = (window_alignment)NUM2LONG(varg1);
        arg2 = NUM2LONG(varg2);
        arg3 = NUM2LONG(varg3);
        result = arg0->SetWindowAlignment(arg1, arg2, arg3);
      }
      break;

    case 4:
      {
        window_alignment arg1;
        int32 arg2, arg3, arg4;

        arg1 = (window_alignment)NUM2LONG(varg1);
        arg2 = NUM2LONG(varg2);
        arg3 = NUM2LONG(varg3);
        arg4 = NUM2LONG(varg4);
        result = arg0->SetWindowAlignment(arg1, arg2, arg3, arg4);
      }
      break;

    case 5:
      {
        window_alignment arg1;
        int32 arg2, arg3, arg4, arg5;

        arg1 = (window_alignment)NUM2LONG(varg1);
        arg2 = NUM2LONG(varg2);
        arg3 = NUM2LONG(varg3);
        arg4 = NUM2LONG(varg4);
        arg5 = NUM2LONG(varg5);
        result = arg0->SetWindowAlignment(arg1, arg2, arg3, arg4, arg5);
      }
      break;

    case 6:
      {
        window_alignment arg1;
        int32 arg2, arg3, arg4, arg5, arg6;

        arg1 = (window_alignment)NUM2LONG(varg1);
        arg2 = NUM2LONG(varg2);
        arg3 = NUM2LONG(varg3);
        arg4 = NUM2LONG(varg4);
        arg5 = NUM2LONG(varg5);
        arg6 = NUM2LONG(varg6);
        result = arg0->SetWindowAlignment(arg1, arg2, arg3, arg4, arg5, arg6);
      }
      break;

    case 7:
      {
        window_alignment arg1;
        int32 arg2, arg3, arg4, arg5, arg6, arg7;

        arg1 = (window_alignment)NUM2LONG(varg1);
        arg2 = NUM2LONG(varg2);
        arg3 = NUM2LONG(varg3);
        arg4 = NUM2LONG(varg4);
        arg5 = NUM2LONG(varg5);
        arg6 = NUM2LONG(varg6);
        arg7 = NUM2LONG(varg7);
        result = arg0->SetWindowAlignment(arg1, arg2, arg3, arg4, arg5, arg6, arg7);
      }
      break;

    case 8:
      {
        window_alignment arg1;
        int32 arg2, arg3, arg4, arg5, arg6, arg7, arg8;

        arg1 = (window_alignment)NUM2LONG(varg1);
        arg2 = NUM2LONG(varg2);
        arg3 = NUM2LONG(varg3);
        arg4 = NUM2LONG(varg4);
        arg5 = NUM2LONG(varg5);
        arg6 = NUM2LONG(varg6);
        arg7 = NUM2LONG(varg7);
        arg8 = NUM2LONG(varg8);
        result = arg0->SetWindowAlignment(arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8);
      }
      break;

    case 9:
      {
        window_alignment arg1;
        int32 arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9;

        arg1 = (window_alignment)NUM2LONG(varg1);
        arg2 = NUM2LONG(varg2);
        arg3 = NUM2LONG(varg3);
        arg4 = NUM2LONG(varg4);
        arg5 = NUM2LONG(varg5);
        arg6 = NUM2LONG(varg6);
        arg7 = NUM2LONG(varg7);
        arg8 = NUM2LONG(varg8);
        arg9 = NUM2LONG(varg9);
        result = arg0->SetWindowAlignment(arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9);
      }
      break;

    default:
      rb_bug("rb_window_SetWindowAlignment");
  }
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  return Qnil;
}


static VALUE
rb_window_GetWindowAlignment(VALUE self)
{
  BWindow *arg0;
  window_alignment arg1;
  int32 arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9;
  status_t result;
  VALUE vresult;

  Get_BInstance(Window, self, arg0);
  result = arg0->GetWindowAlignment(&arg1, &arg2, &arg3, &arg4, &arg5, &arg6, &arg7, &arg8, &arg9);
  if (result != B_OK) {
    DEBUG_TRACE();
    my_raise(result);
  }
  vresult = rb_ary_new3(9, LONG2NUM(arg1),
                           LONG2NUM(arg2),
                           LONG2NUM(arg3),
                           LONG2NUM(arg4),
                           LONG2NUM(arg5),
                           LONG2NUM(arg6),
                           LONG2NUM(arg7),
                           LONG2NUM(arg8),
                           LONG2NUM(arg9));
  return vresult;
}


static VALUE
rb_window_QuitRequested(VALUE self)
{
  BWindow *arg0;
  bool result;
  VALUE vresult = Qnil;

  Get_BInstance(Window, self, arg0);
  result = arg0->BWindow::QuitRequested();
  vresult = (result ? Qtrue:Qfalse);

  return vresult;
}


static VALUE
rb_window_Run(VALUE self)
{
  BWindow *arg0;
  thread_id result;
  VALUE vresult;

  Get_BInstance(Window, self, arg0);
  result = arg0->BWindow::Run();
  vresult = LONG2NUM(result);
  return vresult;
}


void
Init_Window()
{
  cWindow = rb_define_class_under(mBe, "Window", cLooper);
  rb_define_singleton_method(cWindow, "new", RUBY_METHOD_FUNC(rb_window_s_new), -1);
  rb_define_method(cWindow, "initialize", RUBY_METHOD_FUNC(rb_window_initialize), -1);

  rb_define_method(cWindow, "Quit", RUBY_METHOD_FUNC(rb_window_Quit), 0);
  rb_define_method(cWindow, "Close", RUBY_METHOD_FUNC(rb_window_Close), 0);

  rb_define_method(cWindow, "AddChild", RUBY_METHOD_FUNC(rb_window_AddChild), -1);
  rb_define_method(cWindow, "RemoveChild", RUBY_METHOD_FUNC(rb_window_RemoveChild), 1);
  rb_define_method(cWindow, "CountChildren", RUBY_METHOD_FUNC(rb_window_CountChildren), 0);
  rb_define_method(cWindow, "ChildAt", RUBY_METHOD_FUNC(rb_window_ChildAt), 1);

  rb_define_method(cWindow, "DispatchMessage", RUBY_METHOD_FUNC(rb_window_DispatchMessage), 2);
  rb_define_method(cWindow, "MessageReceived", RUBY_METHOD_FUNC(rb_window_MessageReceived), 1);
  rb_define_method(cWindow, "FrameMoved", RUBY_METHOD_FUNC(rb_window_FrameMoved), 1);
  rb_define_method(cWindow, "WorkspacesChanged", RUBY_METHOD_FUNC(rb_window_WorkspacesChanged), 2);
  rb_define_method(cWindow, "WorkspaceActivated", RUBY_METHOD_FUNC(rb_window_WorkspaceActivated), 2);
  rb_define_method(cWindow, "FrameResized", RUBY_METHOD_FUNC(rb_window_FrameResized), 2);
  rb_define_method(cWindow, "Minimize", RUBY_METHOD_FUNC(rb_window_Minimize), 1);
  rb_define_method(cWindow, "Zoom", RUBY_METHOD_FUNC(rb_window_Zoom), -1);
  rb_define_method(cWindow, "SetZoomLimits", RUBY_METHOD_FUNC(rb_window_SetZoomLimits), 2);
  rb_define_method(cWindow, "ScreenChanged", RUBY_METHOD_FUNC(rb_window_ScreenChanged), 2);
  rb_define_method(cWindow, "SetPulseRate", RUBY_METHOD_FUNC(rb_window_SetPulseRate), 1);
  rb_define_method(cWindow, "PulseRate", RUBY_METHOD_FUNC(rb_window_PulseRate), 0);
  rb_define_method(cWindow, "AddShortcut", RUBY_METHOD_FUNC(rb_window_AddShortcut), -1);
  rb_define_method(cWindow, "RemoveShortcut", RUBY_METHOD_FUNC(rb_window_RemoveShortcut), 2);
  rb_define_method(cWindow, "SetDefaultButton", RUBY_METHOD_FUNC(rb_window_SetDefaultButton), 1);
  rb_define_method(cWindow, "DefaultButton", RUBY_METHOD_FUNC(rb_window_DefaultButton), 0);
  rb_define_method(cWindow, "MenusBeginning", RUBY_METHOD_FUNC(rb_window_MenusBeginning), 0);
  rb_define_method(cWindow, "MenusEnded", RUBY_METHOD_FUNC(rb_window_MenusEnded), 0);
  rb_define_method(cWindow, "NeedsUpdate", RUBY_METHOD_FUNC(rb_window_NeedsUpdate), 0);
  rb_define_method(cWindow, "UpdateIfNeeded", RUBY_METHOD_FUNC(rb_window_UpdateIfNeeded), 0);
  rb_define_method(cWindow, "FindView", RUBY_METHOD_FUNC(rb_window_FindView), 1);
  rb_define_method(cWindow, "CurrentFocus", RUBY_METHOD_FUNC(rb_window_CurrentFocus), 0);
  rb_define_method(cWindow, "Activate", RUBY_METHOD_FUNC(rb_window_Activate), -1);
  rb_define_method(cWindow, "WindowActivated", RUBY_METHOD_FUNC(rb_window_WindowActivated), 1);
  rb_define_method(cWindow, "ConvertToScreen", RUBY_METHOD_FUNC(rb_window_ConvertToScreen), 1);
  rb_define_method(cWindow, "ConvertFromScreen", RUBY_METHOD_FUNC(rb_window_ConvertFromScreen), 1);
  rb_define_method(cWindow, "MoveBy", RUBY_METHOD_FUNC(rb_window_MoveBy), 2);
  rb_define_method(cWindow, "MoveTo", RUBY_METHOD_FUNC(rb_window_MoveTo), -1);
  rb_define_method(cWindow, "ResizeBy", RUBY_METHOD_FUNC(rb_window_ResizeBy), 2);
  rb_define_method(cWindow, "ResizeTo", RUBY_METHOD_FUNC(rb_window_ResizeTo), 2);
  rb_define_method(cWindow, "Show", RUBY_METHOD_FUNC(rb_window_Show), 0);
  rb_define_method(cWindow, "Hide", RUBY_METHOD_FUNC(rb_window_Hide), 0);
  rb_define_method(cWindow, "IsHidden", RUBY_METHOD_FUNC(rb_window_IsHidden), 0);
  rb_define_method(cWindow, "IsMinimized", RUBY_METHOD_FUNC(rb_window_IsMinimized), 0);

  rb_define_method(cWindow, "Flush", RUBY_METHOD_FUNC(rb_window_Flush), 0);
  rb_define_method(cWindow, "Sync", RUBY_METHOD_FUNC(rb_window_Sync), 0);

  rb_define_method(cWindow, "SendBehind", RUBY_METHOD_FUNC(rb_window_SendBehind), 1);

  rb_define_method(cWindow, "DisableUpdates", RUBY_METHOD_FUNC(rb_window_DisableUpdates), 0);
  rb_define_method(cWindow, "EnableUpdates", RUBY_METHOD_FUNC(rb_window_EnableUpdates), 0);

  rb_define_method(cWindow, "BeginViewTransaction", RUBY_METHOD_FUNC(rb_window_BeginViewTransaction), 0);
  rb_define_method(cWindow, "EndViewTransaction", RUBY_METHOD_FUNC(rb_window_EndViewTransaction), 0);

  rb_define_method(cWindow, "Bounds", RUBY_METHOD_FUNC(rb_window_Bounds), 0);
  rb_define_method(cWindow, "Frame", RUBY_METHOD_FUNC(rb_window_Frame), 0);
  rb_define_method(cWindow, "Title", RUBY_METHOD_FUNC(rb_window_Title), 0);
  rb_define_method(cWindow, "SetTitle", RUBY_METHOD_FUNC(rb_window_SetTitle), 1);
  rb_define_method(cWindow, "IsFront", RUBY_METHOD_FUNC(rb_window_IsFront), 0);
  rb_define_method(cWindow, "IsActive", RUBY_METHOD_FUNC(rb_window_IsActive), 0);
  rb_define_method(cWindow, "SetKeyMenuBar", RUBY_METHOD_FUNC(rb_window_SetKeyMenuBar), 1);
  rb_define_method(cWindow, "KeyMenuBar", RUBY_METHOD_FUNC(rb_window_KeyMenuBar), 0);
  rb_define_method(cWindow, "SetSizeLimits", RUBY_METHOD_FUNC(rb_window_SetSizeLimits), 4);
  rb_define_method(cWindow, "GetSizeLimits", RUBY_METHOD_FUNC(rb_window_GetSizeLimits), 0);
  rb_define_method(cWindow, "Workspaces", RUBY_METHOD_FUNC(rb_window_Workspaces), 0);
  rb_define_method(cWindow, "SetWorkspaces", RUBY_METHOD_FUNC(rb_window_SetWorkspaces), 1);
  rb_define_method(cWindow, "LastMouseMovedView", RUBY_METHOD_FUNC(rb_window_LastMouseMovedView), 0);

  rb_define_method(cWindow, "ResolveSpecifier", RUBY_METHOD_FUNC(rb_window_ResolveSpecifier), 5);
  rb_define_method(cWindow, "GetSupportedSuites", RUBY_METHOD_FUNC(rb_window_GetSupportedSuites), 1);

  rb_define_method(cWindow, "AddToSubset", RUBY_METHOD_FUNC(rb_window_AddToSubset), 1);
  rb_define_method(cWindow, "RemoveFromSubset", RUBY_METHOD_FUNC(rb_window_RemoveFromSubset), 1);

  rb_define_method(cWindow, "SetType", RUBY_METHOD_FUNC(rb_window_SetType), 1);
  rb_define_method(cWindow, "Type", RUBY_METHOD_FUNC(rb_window_Type), 0);
  rb_define_method(cWindow, "SetLook", RUBY_METHOD_FUNC(rb_window_SetLook), 1);
  rb_define_method(cWindow, "Look", RUBY_METHOD_FUNC(rb_window_Look), 0);
  rb_define_method(cWindow, "SetFeel", RUBY_METHOD_FUNC(rb_window_SetFeel), 1);
  rb_define_method(cWindow, "Feel", RUBY_METHOD_FUNC(rb_window_Feel), 0);
  rb_define_method(cWindow, "SetFlags", RUBY_METHOD_FUNC(rb_window_SetFlags), 1);
  rb_define_method(cWindow, "Flags", RUBY_METHOD_FUNC(rb_window_Flags), 0);

  rb_define_method(cWindow, "IsModal", RUBY_METHOD_FUNC(rb_window_IsModal), 0);
  rb_define_method(cWindow, "IsFloating", RUBY_METHOD_FUNC(rb_window_IsFloating), 0);

  rb_define_method(cWindow, "SetWindowAlignment", RUBY_METHOD_FUNC(rb_window_SetWindowAlignment), -1);
  rb_define_method(cWindow, "GetWindowAlignment", RUBY_METHOD_FUNC(rb_window_GetWindowAlignment), 0);

  rb_define_method(cWindow, "QuitRequested", RUBY_METHOD_FUNC(rb_window_QuitRequested), 0);
  rb_define_method(cWindow, "Run", RUBY_METHOD_FUNC(rb_window_Run), 0);

  rb_define_const(mBe, "UNTYPED_WINDOW", ULONG2NUM(B_UNTYPED_WINDOW));
  rb_define_const(mBe, "TITLED_WINDOW", ULONG2NUM(B_TITLED_WINDOW));
  rb_define_const(mBe, "MODAL_WINDOW", ULONG2NUM(B_MODAL_WINDOW));
  rb_define_const(mBe, "DOCUMENT_WINDOW", ULONG2NUM(B_DOCUMENT_WINDOW));
  rb_define_const(mBe, "BORDERED_WINDOW", ULONG2NUM(B_BORDERED_WINDOW));
  rb_define_const(mBe, "FLOATING_WINDOW", ULONG2NUM(B_FLOATING_WINDOW));

  rb_define_const(mBe, "BORDERED_WINDOW_LOOK", ULONG2NUM(B_BORDERED_WINDOW_LOOK));
  rb_define_const(mBe, "NO_BORDER_WINDOW_LOOK", ULONG2NUM(B_NO_BORDER_WINDOW_LOOK));
  rb_define_const(mBe, "TITLED_WINDOW_LOOK", ULONG2NUM(B_TITLED_WINDOW_LOOK));
  rb_define_const(mBe, "DOCUMENT_WINDOW_LOOK", ULONG2NUM(B_DOCUMENT_WINDOW_LOOK));
  rb_define_const(mBe, "MODAL_WINDOW_LOOK", ULONG2NUM(B_MODAL_WINDOW_LOOK));
  rb_define_const(mBe, "FLOATING_WINDOW_LOOK", ULONG2NUM(B_FLOATING_WINDOW_LOOK));

  rb_define_const(mBe, "NORMAL_WINDOW_FEEL", ULONG2NUM(B_NORMAL_WINDOW_FEEL));
  rb_define_const(mBe, "MODAL_SUBSET_WINDOW_FEEL", ULONG2NUM(B_MODAL_SUBSET_WINDOW_FEEL));
  rb_define_const(mBe, "MODAL_APP_WINDOW_FEEL", ULONG2NUM(B_MODAL_APP_WINDOW_FEEL));
  rb_define_const(mBe, "MODAL_ALL_WINDOW_FEEL", ULONG2NUM(B_MODAL_ALL_WINDOW_FEEL));
  rb_define_const(mBe, "FLOATING_SUBSET_WINDOW_FEEL", ULONG2NUM(B_FLOATING_SUBSET_WINDOW_FEEL));
  rb_define_const(mBe, "FLOATING_APP_WINDOW_FEEL", ULONG2NUM(B_FLOATING_APP_WINDOW_FEEL));
  rb_define_const(mBe, "FLOATING_ALL_WINDOW_FEEL", ULONG2NUM(B_FLOATING_ALL_WINDOW_FEEL));

  rb_define_const(mBe, "BYTE_ALIGNMENT", ULONG2NUM(B_BYTE_ALIGNMENT));
  rb_define_const(mBe, "PIXEL_ALIGNMENT", ULONG2NUM(B_PIXEL_ALIGNMENT));

  rb_define_const(mBe, "NOT_MOVABLE", ULONG2NUM(B_NOT_MOVABLE));
  rb_define_const(mBe, "NOT_CLOSABLE", ULONG2NUM(B_NOT_CLOSABLE));
  rb_define_const(mBe, "NOT_ZOOMABLE", ULONG2NUM(B_NOT_ZOOMABLE));
  rb_define_const(mBe, "NOT_MINIMIZABLE", ULONG2NUM(B_NOT_MINIMIZABLE));
  rb_define_const(mBe, "NOT_RESIZABLE", ULONG2NUM(B_NOT_RESIZABLE));
  rb_define_const(mBe, "NOT_H_RESIZABLE", ULONG2NUM(B_NOT_H_RESIZABLE));
  rb_define_const(mBe, "NOT_V_RESIZABLE", ULONG2NUM(B_NOT_V_RESIZABLE));
  rb_define_const(mBe, "AVOID_FRONT", ULONG2NUM(B_AVOID_FRONT));
  rb_define_const(mBe, "AVOID_FOCUS", ULONG2NUM(B_AVOID_FOCUS));
  rb_define_const(mBe, "WILL_ACCEPT_FIRST_CLICK", ULONG2NUM(B_WILL_ACCEPT_FIRST_CLICK));
  rb_define_const(mBe, "OUTLINE_RESIZE", ULONG2NUM(B_OUTLINE_RESIZE));
  rb_define_const(mBe, "NO_WORKSPACE_ACTIVATION", ULONG2NUM(B_NO_WORKSPACE_ACTIVATION));
  rb_define_const(mBe, "NOT_ANCHORED_ON_ACTIVATE", ULONG2NUM(B_NOT_ANCHORED_ON_ACTIVATE));
  rb_define_const(mBe, "ASYNCHRONOUS_CONTROLS", ULONG2NUM(B_ASYNCHRONOUS_CONTROLS));
  rb_define_const(mBe, "QUIT_ON_WINDOW_CLOSE", ULONG2NUM(B_QUIT_ON_WINDOW_CLOSE));

  rb_define_const(mBe, "CURRENT_WORKSPACE", ULONG2NUM(B_CURRENT_WORKSPACE));
  rb_define_const(mBe, "ALL_WORKSPACES", ULONG2NUM(B_ALL_WORKSPACES));
}
