/*
 *  fs_attr.cc
 */

#include <fs_attr.h>
#include <errno.h>
#include <TypeConstants.h>
#include <Message.h>
#include <Messenger.h>
#include <Mime.h>
#include <Entry.h>
#include "my_be.h"


static VALUE cAttrDir;


#define My_Fs_Read_Attr(buf, count) { \
  ssize_t result;    \
  result = fs_read_attr(fd, attribute, type, pos, buf, count); \
  if (result < 0) {  \
    DEBUG_TRACE();   \
    my_raise(errno); \
  }                  \
}


static VALUE
my_fs_read_attr(int fd, const char *attribute, uint32 type, off_t pos, size_t count)
{
  VALUE vresult;

  switch (type) {
    case B_BOOL_TYPE:
      {
        bool arg5 = false;
        My_Fs_Read_Attr(&arg5, sizeof(arg5));
        vresult = (arg5 ? Qtrue:Qfalse);
      }
      break;

    case B_CHAR_TYPE:
    case B_INT8_TYPE:
      {
        char arg5 = 0;
        My_Fs_Read_Attr(&arg5, sizeof(arg5));
        vresult = CHR2FIX(arg5);
      }
      break;

    case B_DOUBLE_TYPE:
      {
        double arg5 = 0;
        My_Fs_Read_Attr(&arg5, sizeof(arg5));
        vresult = rb_float_new(arg5);
      }
      break;

    case B_FLOAT_TYPE:
      {
        float arg5 = 0;
        My_Fs_Read_Attr(&arg5, sizeof(arg5));
        vresult = rb_float_new(arg5);
      }
      break;

    case B_INT64_TYPE:
      {
        int64 arg5 = 0;
        My_Fs_Read_Attr(&arg5, sizeof(arg5));
        vresult = LLONG2NUM(arg5);
      }
      break;

    case B_INT32_TYPE:
      {
        int32 arg5 = 0;
        My_Fs_Read_Attr(&arg5, sizeof(arg5));
        vresult = LONG2NUM(arg5);
      }
      break;

    case B_INT16_TYPE:
      {
        int16 arg5 = 0;
        My_Fs_Read_Attr(&arg5, sizeof(arg5));
        vresult = INT2FIX(arg5);
      }
      break;

    case B_MESSAGE_TYPE:
      {
        BMessage arg5, *copy;
        My_Fs_Read_Attr(&arg5, sizeof(arg5));
        NEW(copy, BMessage(arg5));
        vresult = Data_Wrap_Struct(cMessage, 0, delete_message, copy);
      }
      break;

    case B_MESSENGER_TYPE:
      {
        BMessenger arg5, *copy;
        My_Fs_Read_Attr(&arg5, sizeof(arg5));
        NEW(copy, BMessenger(arg5));
        vresult = Data_Wrap_Struct(cMessenger, 0, delete_messenger, copy);
      }
      break;

    case B_MIME_TYPE:
    case B_RAW_TYPE:
      {
        char *arg5 = ALLOCA_N(char, count);
        My_Fs_Read_Attr(arg5, count);
        if (memcmp(arg5, "text/", 5) == 0) { // supertype "text/*" ?
          vresult = rb_str_new2(arg5);
        }
        else {
          vresult = rb_str_new(arg5, count);
        }
      }
      break;

    case B_MONOCHROME_1_BIT_TYPE:
      {
        uchar arg5 = 0;
        My_Fs_Read_Attr(&arg5, sizeof(arg5));
        vresult = INT2FIX(arg5 & 0x01);
      }
      break;

    case B_OBJECT_TYPE:
      {
        BMessage *arg5, *copy;
        My_Fs_Read_Attr(&arg5, sizeof(arg5));
        NEW(copy, BMessage(*arg5));
        vresult = Data_Wrap_Struct(cMessage, 0, delete_message, copy);
      }
      break;

    case B_OFF_T_TYPE:
      {
        off_t arg5 = 0;
        My_Fs_Read_Attr(&arg5, sizeof(arg5));
        vresult = LLONG2NUM(arg5);
      }
      break;

    case B_PATTERN_TYPE:
      {
        pattern arg5;
        My_Fs_Read_Attr(&arg5, sizeof(arg5));
        vresult = rb_str_new(reinterpret_cast<char *>(arg5.data), 8);
      }
      break;

    case B_POINTER_TYPE:
      {
        void *arg5;
        My_Fs_Read_Attr(&arg5, sizeof(arg5));
        vresult = Data_Wrap_Struct(rb_cData, 0, 0, arg5);
      }
      break;

    case B_POINT_TYPE:
      {
        BPoint arg5, *copy;
        My_Fs_Read_Attr(&arg5, sizeof(arg5));
        NEW(copy, BPoint(arg5));
        vresult = Data_Wrap_Struct(cPoint, 0, delete_point, copy);
      }
      break;

    case B_RECT_TYPE:
      {
        BRect arg5, *copy;
        My_Fs_Read_Attr(&arg5, sizeof(arg5));
        NEW(copy, BRect(arg5));
        vresult = Data_Wrap_Struct(cRect, 0, delete_rect, copy);
      }
      break;

    case B_REF_TYPE:
      {
        entry_ref arg5, *copy;
        My_Fs_Read_Attr(&arg5, sizeof(arg5));
        NEW(copy, entry_ref(arg5));
        vresult = Data_Wrap_Struct(cEntry_ref, 0, delete_entry_ref, copy);
      }
      break;

    case B_RGB_COLOR_TYPE:
      {
        rgb_color arg5;
        My_Fs_Read_Attr(&arg5, sizeof(arg5));
        vresult = my_rgb_color2hash(arg5);
      }
      break;

    case B_SIZE_T_TYPE:
      {
        size_t arg5 = 0;
        My_Fs_Read_Attr(&arg5, sizeof(arg5));
        vresult = ULONG2NUM(arg5);
      }
      break;

    case B_SSIZE_T_TYPE:
      {
        ssize_t arg5 = 0;
        My_Fs_Read_Attr(&arg5, sizeof(arg5));
        vresult = LONG2NUM(arg5);
      }
      break;

    case B_STRING_TYPE:
    case B_MIME_STRING_TYPE:
      {
        char *arg5 = ALLOCA_N(char, count);
        My_Fs_Read_Attr(arg5, count);
        vresult = rb_str_new2(arg5);
      }
      break;

    case B_TIME_TYPE:
      {
        bigtime_t arg5 = 0;
        My_Fs_Read_Attr(&arg5, sizeof(arg5));
        vresult = LLONG2NUM(arg5);
      }
      break;

    case B_UINT64_TYPE:
      {
        uint64 arg5 = 0;
        My_Fs_Read_Attr(&arg5, sizeof(arg5));
        vresult = ULLONG2NUM(arg5);
      }
      break;

    case B_RGB_32_BIT_TYPE:
    case B_UINT32_TYPE:
      {
        uint32 arg5 = 0;
        My_Fs_Read_Attr(&arg5, sizeof(arg5));
        vresult = ULONG2NUM(arg5);
      }
      break;

    case B_UINT16_TYPE:
      {
        uint16 arg5 = 0;
        My_Fs_Read_Attr(&arg5, sizeof(arg5));
        vresult = INT2FIX(arg5);
      }
      break;

    case B_COLOR_8_BIT_TYPE:
    case B_GRAYSCALE_8_BIT_TYPE:
    case B_UINT8_TYPE:
      {
        uchar arg5 = 0;
        My_Fs_Read_Attr(&arg5, sizeof(arg5));
        vresult = UCHR2FIX(arg5);
      }
      break;

    case B_ANY_TYPE:
    case B_MEDIA_PARAMETER_TYPE:
    case B_MEDIA_PARAMETER_WEB_TYPE:
    case B_MEDIA_PARAMETER_GROUP_TYPE:
      {
        char *arg5 = ALLOCA_N(char, count);
        My_Fs_Read_Attr(arg5, count);
        vresult = rb_str_new(arg5, count);
      }

    default:
      {
        char *arg5 = ALLOCA_N(char, count);
        My_Fs_Read_Attr(arg5, count);
        vresult = rb_str_new(arg5, count);
      }
  }
  return vresult;
}


static VALUE
rb_fs_read_attr(VALUE self, VALUE varg1, VALUE varg2, VALUE varg4)
{
  int arg1;
  char *arg2;
  uint32 arg3;
  off_t arg4;
  size_t arg6;
  ssize_t result;
  VALUE vresult;
  attr_info attr_i;

  arg1 = NUM2LONG(varg1);
  arg2 = STR2CSTR(varg2);

  result = fs_stat_attr(arg1, arg2, &attr_i);
  if (result < 0) {
    DEBUG_TRACE();
    my_raise(errno);
  }

  arg3 = attr_i.type;
  arg4 = NUM2LLONG(varg4);
  arg6 = attr_i.size;

  vresult = my_fs_read_attr(arg1, arg2, arg3, arg4, arg6);
  return vresult;
}


#define My_Fs_Write_Attr(buf, count) { \
  result = fs_write_attr(fd, attribute, type, pos, buf, count); \
  if (result < 0) {  \
    DEBUG_TRACE();   \
    my_raise(errno); \
  }                  \
}


static VALUE
my_fs_write_attr(int fd, const char *attribute, uint32 type, off_t pos, VALUE varg5)
{
  ssize_t result;
  VALUE vresult;

  switch (type) {
    case B_BOOL_TYPE:
      {
        bool arg5 = RTEST(varg5);
        My_Fs_Write_Attr(&arg5, sizeof(arg5));
      }
      break;

    case B_CHAR_TYPE:
    case B_INT8_TYPE:
      {
        char arg5 = NUM2CHR(varg5);
        My_Fs_Write_Attr(&arg5, sizeof(arg5));
      }
      break;

    case B_DOUBLE_TYPE:
      {
        double arg5 = NUM2DBL(varg5);
        My_Fs_Write_Attr(&arg5, sizeof(arg5));
      }
      break;

    case B_FLOAT_TYPE:
      {
        float arg5 = NUM2DBL(varg5);
        My_Fs_Write_Attr(&arg5, sizeof(arg5));
      }
      break;

    case B_INT64_TYPE:
      {
        int64 arg5 = NUM2LLONG(varg5);
        My_Fs_Write_Attr(&arg5, sizeof(arg5));
      }
      break;

    case B_INT32_TYPE:
      {
        int32 arg5 = NUM2LONG(varg5);
        My_Fs_Write_Attr(&arg5, sizeof(arg5));
      }
      break;

    case B_INT16_TYPE:
      {
        int16 arg5 = NUM2SHRT(varg5);
        My_Fs_Write_Attr(&arg5, sizeof(arg5));
      }
      break;

    case B_MESSAGE_TYPE:
      {
        BMessage *arg5, copy;
        Get_BInstance(Message, varg5, arg5);
        copy = *arg5;
        My_Fs_Write_Attr(&copy, sizeof(copy));
      }
      break;

    case B_MESSENGER_TYPE:
      {
        BMessenger *arg5, copy;
        Get_BInstance(Messenger, varg5, arg5);
        copy = *arg5;
        My_Fs_Write_Attr(&copy, sizeof(copy));
      }
      break;

    case B_MIME_TYPE:
    case B_RAW_TYPE:
      {
        char *arg5;
        int len = 0;
        arg5 = rb_str2cstr(varg5, &len);
        My_Fs_Write_Attr(arg5, len);
      }
      break;

    case B_MONOCHROME_1_BIT_TYPE:
      {
        uchar arg5 = NUM2UCHR(varg5);
        arg5 = (arg5 & 0x01);
        My_Fs_Write_Attr(&arg5, sizeof(arg5));
      }
      break;

    case B_OBJECT_TYPE:
      {
        BMessage *arg5;
        Get_BInstance(Message, varg5, arg5);
        My_Fs_Write_Attr(&arg5, sizeof(arg5));
      }
      break;

    case B_OFF_T_TYPE:
      {
        off_t arg5 = NUM2LLONG(varg5);
        My_Fs_Write_Attr(&arg5, sizeof(arg5));
      }
      break;

    case B_PATTERN_TYPE:
      {
        char *str;
        int len = 0;
        pattern arg5;
        str = rb_str2cstr(varg5, &len);
        memcpy(arg5.data, str, 8);
        My_Fs_Write_Attr(&arg5, sizeof(arg5));
      }
      break;

    case B_POINTER_TYPE:
      {
        void *arg5 = DATA_PTR(varg5);
        My_Fs_Write_Attr(&arg5, sizeof(arg5));
      }
      break;

    case B_POINT_TYPE:
      {
        BPoint *arg5, copy;
        Get_BInstance(Point, varg5, arg5);
        copy = *arg5;
        My_Fs_Write_Attr(&copy, sizeof(copy));
      }
      break;

    case B_RECT_TYPE:
      {
        BRect *arg5, copy;
        Get_BInstance(Rect, varg5, arg5);
        copy = *arg5;
        My_Fs_Write_Attr(&copy, sizeof(copy));
      }
      break;

    case B_REF_TYPE:
      {
        entry_ref *arg5, copy;
        Get_BInstance2(Entry_ref, entry_ref, varg5, arg5);
        copy = *arg5;
        My_Fs_Write_Attr(&copy, sizeof(copy));
      }
      break;

    case B_RGB_COLOR_TYPE:
      {
        rgb_color arg5 = my_hash2rgb_color(varg5);
        My_Fs_Write_Attr(&arg5, sizeof(arg5));
      }
      break;

    case B_SIZE_T_TYPE:
      {
        size_t arg5 = NUM2ULONG(varg5);
        My_Fs_Write_Attr(&arg5, sizeof(arg5));
      }
      break;

    case B_SSIZE_T_TYPE:
      {
        ssize_t arg5 = NUM2LONG(varg5);
        My_Fs_Write_Attr(&arg5, sizeof(arg5));
      }
      break;

    case B_STRING_TYPE:
    case B_MIME_STRING_TYPE:
      {
        char *arg5;
        int len = 0;
        arg5 = rb_str2cstr(varg5, &len);
        My_Fs_Write_Attr(arg5, len);
      }
      break;

    case B_TIME_TYPE:
      {
        bigtime_t arg5 = NUM2LLONG(varg5);
        My_Fs_Write_Attr(&arg5, sizeof(arg5));
      }
      break;

    case B_UINT64_TYPE:
      {
        uint64 arg5 = NUM2ULLONG(varg5);
        My_Fs_Write_Attr(&arg5, sizeof(arg5));
      }
      break;

    case B_RGB_32_BIT_TYPE:
    case B_UINT32_TYPE:
      {
        uint32 arg5 = NUM2ULONG(varg5);
        My_Fs_Write_Attr(&arg5, sizeof(arg5));
      }
      break;

    case B_UINT16_TYPE:
      {
        uint16 arg5 = NUM2USHRT(varg5);
        My_Fs_Write_Attr(&arg5, sizeof(arg5));
      }
      break;

    case B_COLOR_8_BIT_TYPE:
    case B_GRAYSCALE_8_BIT_TYPE:
    case B_UINT8_TYPE:
      {
        uchar arg5 = NUM2UCHR(varg5);
        My_Fs_Write_Attr(&arg5, sizeof(arg5));
      }
      break;

    case B_ANY_TYPE:
    case B_MEDIA_PARAMETER_TYPE:
    case B_MEDIA_PARAMETER_WEB_TYPE:
    case B_MEDIA_PARAMETER_GROUP_TYPE:
      {
        char *arg5;
        int len = 0;
        arg5 = rb_str2cstr(varg5, &len);
        My_Fs_Write_Attr(arg5, len);
      }

    default:
      {
        char *arg5;
        int len = 0;
        arg5 = rb_str2cstr(varg5, &len);
        My_Fs_Write_Attr(arg5, len);
      }
  }
  vresult = LONG2NUM(result);
  return vresult;
}


static VALUE
rb_fs_write_attr(VALUE self, VALUE varg1, VALUE varg2, VALUE varg3, VALUE varg4, VALUE varg5)
{
  int arg1;
  char *arg2;
  uint32 arg3;
  off_t arg4;
  VALUE vresult;

  arg1 = NUM2LONG(varg1);
  arg2 = STR2CSTR(varg2);
  arg3 = NUM2ULONG(varg3);
  arg4 = NUM2LLONG(varg4);

  vresult = my_fs_write_attr(arg1, arg2, arg3, arg4, varg5);
  return vresult;
}


static VALUE
rb_fs_remove_attr(VALUE self, VALUE varg1, VALUE varg2)
{
  int arg1;
  char *arg2;
  int result;

  arg1 = NUM2LONG(varg1);
  arg2 = STR2CSTR(varg2);
  result = fs_remove_attr(arg1, arg2);
  if (result < 0) {
    DEBUG_TRACE();
    my_raise(errno);
  }
  return Qnil;
}


static void
close_attr_dir(DIR *dir)
{
  if (dir) {
    int result;
    result = fs_close_attr_dir(dir);
    if (result < 0) {
      DEBUG_TRACE();
      rb_warn("%s", strerror(errno));
    }
  }
}


static VALUE
rb_fs_close_attr_dir(VALUE self, VALUE varg1)
{
  DIR *arg1;
  int result;

  Get_BInstance2(AttrDir, DIR, varg1, arg1)
  result = fs_close_attr_dir(arg1);
  DATA_PTR(varg1) = NULL;
  if (result < 0) {
    DEBUG_TRACE();
    my_raise(errno);
  }
  return Qnil;
}


static VALUE
yld_fs_close_attr_dir(VALUE self)
{
  return rb_fs_close_attr_dir(cAttrDir, self);
}


static VALUE
rb_attr_dir_close(VALUE self)
{
  return rb_fs_close_attr_dir(cAttrDir, self);
}


static VALUE
rb_fs_open_attr_dir(VALUE self, VALUE varg1)
{
  char *arg1;
  DIR *result;
  VALUE vresult;

  arg1 = STR2CSTR(varg1);
  result = fs_open_attr_dir(arg1);
  if (!result) {
    DEBUG_TRACE();
    my_raise(errno);
  }
  vresult = Data_Wrap_Struct(cAttrDir, 0, close_attr_dir, result);
  if (rb_block_given_p()) {
    vresult = rb_ensure(RUBY_METHOD_FUNC(rb_yield), vresult,
                        RUBY_METHOD_FUNC(yld_fs_close_attr_dir), vresult);
  }
  return vresult;
}


static VALUE
rb_fs_fopen_attr_dir(VALUE self, VALUE varg1)
{
  int arg1;
  DIR *result;
  VALUE vresult;

  arg1 = NUM2LONG(varg1);
  result = fs_fopen_attr_dir(arg1);
  if (!result) {
    DEBUG_TRACE();
    my_raise(errno);
  }
  vresult = Data_Wrap_Struct(cAttrDir, 0, close_attr_dir, result);
  if (rb_block_given_p()) {
    vresult = rb_ensure(RUBY_METHOD_FUNC(rb_yield), vresult,
                        RUBY_METHOD_FUNC(yld_fs_close_attr_dir), vresult);
  }
  return vresult;
}


static VALUE
rb_fs_read_attr_dir(VALUE self, VALUE varg1)
{
  DIR *arg1;
  dirent_t *result;
  VALUE vresult = Qnil;

  Get_BInstance2(AttrDir, DIR, varg1, arg1)
  if (rb_block_given_p()) {
    while ((result = fs_read_attr_dir(arg1))) {
      vresult = rb_yield(my_dirent_t2hash(*result));
    }
  }
  else {
    result = fs_read_attr_dir(arg1);
    if (!result) {
      DEBUG_TRACE();
      my_raise(errno);
    }
    vresult = my_dirent_t2hash(*result);
  }
  return vresult;
}


static VALUE
rb_attr_dir_read(VALUE self)
{
  return rb_fs_read_attr_dir(cAttrDir, self);
}


static VALUE
rb_fs_rewind_attr_dir(VALUE self, VALUE varg1)
{
  DIR *arg1;

  Get_BInstance2(AttrDir, DIR, self, arg1)
  fs_rewind_attr_dir(arg1);
  return Qnil;
}


static VALUE
rb_attr_dir_rewind(VALUE self)
{
  return rb_fs_rewind_attr_dir(cAttrDir, self);
}


static VALUE
my_attr_info2hash(attr_info &attr)
{
  VALUE vresult = rb_hash_new();
  rb_hash_aset(vresult, rb_str_new2("type"), ULONG2NUM(attr.type));
  rb_hash_aset(vresult, rb_str_new2("size"), LLONG2NUM(attr.size));
  return vresult;
}


static VALUE
rb_fs_stat_attr(VALUE self, VALUE varg1, VALUE varg2)
{
  int arg1;
  char *arg2;
  attr_info arg3;
  int result;
  VALUE vresult;

  arg1 = NUM2LONG(varg1);
  arg2 = STR2CSTR(varg2);
  result = fs_stat_attr(arg1, arg2, &arg3);
  if (result < 0) {
    DEBUG_TRACE();
    my_raise(errno);
  }
  vresult = my_attr_info2hash(arg3);
  return vresult;
}


void
Init_fs_attr()
{
  cAttrDir = rb_define_class_under(mBe, "AttrDir", rb_cObject);
  rb_undef_method(CLASS_OF(cAttrDir), "new");
  rb_define_method(cAttrDir, "close", RUBY_METHOD_FUNC(rb_attr_dir_close), 0);
  rb_define_method(cAttrDir, "read", RUBY_METHOD_FUNC(rb_attr_dir_read), 0);
  rb_define_method(cAttrDir, "rewind", RUBY_METHOD_FUNC(rb_attr_dir_rewind), 0);

  rb_define_module_function(mBe, "fs_read_attr", RUBY_METHOD_FUNC(rb_fs_read_attr), 3);
  rb_define_module_function(mBe, "fs_write_attr", RUBY_METHOD_FUNC(rb_fs_write_attr), 5);
  rb_define_module_function(mBe, "fs_remove_attr", RUBY_METHOD_FUNC(rb_fs_remove_attr), 2);

  rb_define_module_function(mBe, "fs_open_attr_dir", RUBY_METHOD_FUNC(rb_fs_open_attr_dir), 1);
  rb_define_module_function(mBe, "fs_fopen_attr_dir", RUBY_METHOD_FUNC(rb_fs_fopen_attr_dir), 1);
  rb_define_module_function(mBe, "fs_close_attr_dir", RUBY_METHOD_FUNC(rb_fs_close_attr_dir), 1);
  rb_define_module_function(mBe, "fs_read_attr_dir", RUBY_METHOD_FUNC(rb_fs_read_attr_dir), 1);
  rb_define_module_function(mBe, "fs_rewind_attr_dir", RUBY_METHOD_FUNC(rb_fs_rewind_attr_dir), 1);

  rb_define_module_function(mBe, "fs_stat_attr", RUBY_METHOD_FUNC(rb_fs_stat_attr), 2);
}
