/*
 *  fs_info.cc
 */

#include <fs_info.h>
#include <errno.h>
#include "my_be.h"


static VALUE
rb_dev_for_path(VALUE self, VALUE varg1)
{
  char *arg1;
  dev_t result;
  VALUE vresult;

  arg1 = STR2CSTR(varg1);
  result = dev_for_path(arg1);
  if (result < 0) {
    DEBUG_TRACE();
    my_raise(result);
  }
  vresult = LONG2NUM(result);
  return vresult;
}


static VALUE
rb_next_dev(int argc, VALUE *argv, VALUE self)
{
  int32 arg1 = 0;
  dev_t result;
  VALUE varg1, vresult = Qnil;

  if (rb_block_given_p()) {
    rb_scan_args(argc, argv, "01", &varg1);
    if (argc > 0) {
      arg1 = NUM2LONG(varg1);
    }

    while ((result = next_dev(&arg1)) >= 0) {
      vresult = rb_yield(LONG2NUM(result));
    }
  }
  else {
    rb_scan_args(argc, argv, "1", &varg1);
    arg1 = NUM2LONG(varg1);

    result = next_dev(&arg1);
    if (result < 0) {
      DEBUG_TRACE();
      my_raise(result);
    }
    vresult = rb_ary_new3(2, LONG2NUM(result), LONG2NUM(arg1));
  }
  return vresult;
}


static VALUE
my_fs_info2hash(fs_info &fsi)
{
  VALUE vresult = rb_hash_new();
  rb_hash_aset(vresult, rb_str_new2("dev"),          LONG2NUM(fsi.dev));
  rb_hash_aset(vresult, rb_str_new2("root"),         LLONG2NUM(fsi.root));
  rb_hash_aset(vresult, rb_str_new2("flags"),        ULONG2NUM(fsi.flags));
  rb_hash_aset(vresult, rb_str_new2("block_size"),   LLONG2NUM(fsi.block_size));
  rb_hash_aset(vresult, rb_str_new2("io_size"),      LLONG2NUM(fsi.io_size));
  rb_hash_aset(vresult, rb_str_new2("total_blocks"), LLONG2NUM(fsi.total_blocks));
  rb_hash_aset(vresult, rb_str_new2("free_blocks"),  LLONG2NUM(fsi.free_blocks));
  rb_hash_aset(vresult, rb_str_new2("total_nodes"),  LLONG2NUM(fsi.total_nodes));
  rb_hash_aset(vresult, rb_str_new2("free_nodes"),   LLONG2NUM(fsi.free_nodes));
  rb_hash_aset(vresult, rb_str_new2("device_name"),  rb_str_new2(fsi.device_name));
  rb_hash_aset(vresult, rb_str_new2("volume_name"),  rb_str_new2(fsi.volume_name));
  rb_hash_aset(vresult, rb_str_new2("fsh_name"),     rb_str_new2(fsi.fsh_name));
  return vresult;
}


static VALUE
rb_fs_stat_dev(VALUE self, VALUE varg1)
{
  dev_t arg1;
  fs_info arg2;
  int result;
  VALUE vresult;

  arg1 = NUM2LONG(varg1);
  result = fs_stat_dev(arg1, &arg2);
  if (result < 0) {
    DEBUG_TRACE();
    my_raise(errno);
  }
  vresult = my_fs_info2hash(arg2);
  return vresult;
}


void
Init_fs_info()
{
  rb_define_module_function(mBe, "dev_for_path", RUBY_METHOD_FUNC(rb_dev_for_path), 1);
  rb_define_module_function(mBe, "next_dev", RUBY_METHOD_FUNC(rb_next_dev), -1);
  rb_define_module_function(mBe, "fs_stat_dev", RUBY_METHOD_FUNC(rb_fs_stat_dev), 1);

  rb_define_const(mBe, "FS_IS_READONLY", ULONG2NUM(B_FS_IS_READONLY));
  rb_define_const(mBe, "FS_IS_REMOVABLE", ULONG2NUM(B_FS_IS_REMOVABLE));
  rb_define_const(mBe, "FS_IS_PERSISTENT", ULONG2NUM(B_FS_IS_PERSISTENT));
  rb_define_const(mBe, "FS_IS_SHARED", ULONG2NUM(B_FS_IS_SHARED));
  rb_define_const(mBe, "FS_HAS_MIME", ULONG2NUM(B_FS_HAS_MIME));
  rb_define_const(mBe, "FS_HAS_ATTR", ULONG2NUM(B_FS_HAS_ATTR));
  rb_define_const(mBe, "FS_HAS_QUERY", ULONG2NUM(B_FS_HAS_QUERY));
}
