/*
 *  my_be.cc
 */

#include <List.h>
#include "my_be.h"


VALUE mBe;
VALUE be_object_list;

ID id_relatives;


VALUE
my_class_s_new(int argc, VALUE *argv, VALUE klass, void func(void*))
{
  VALUE vresult = Data_Wrap_Struct(klass, 0, func, 0);
  rb_obj_call_init(vresult, argc, argv);
  return vresult;
}


struct klass_and_delete_func {
  VALUE klass;
  void (*delete_func)(void*);
};


VALUE
my_data_wrap_object(VALUE klass, void *obj)
{
  struct klass_and_delete_func klasses[] = {
    /* app */
    { cApplication,   delete_application },
    { cApp_info,      delete_app_info },
    { cClipboard,     delete_clipboard },
    { cCursor,        delete_cursor },
    { cHandler,       delete_handler },
    { cInvoker,       delete_invoker },
    { cLooper,        delete_looper },
    { cMessage,       delete_message },
    { cMessageFilter, delete_messagefilter },
    { cMessageQueue,  delete_messagequeue },
    { cMessageRunner, delete_messagerunner },
    { cMessenger,     delete_messenger },
    { cRoster,        delete_roster },
    /* interface */
    { cAlert,         0 },
    { cBitmap,        delete_bitmap },
    { cButton,        delete_button },
    { cControl,       delete_control },
    { cFont,          delete_font },
    { cMenu,          delete_menu },
    { cMenuBar,       delete_menubar },
    { cMenuItem,      delete_menuitem },
    { cPicture,       delete_picture },
    { cPoint,         delete_point },
    { cPolygon,       delete_polygon },
    { cRect,          delete_rect },
    { cScrollView,    delete_scrollview },
    { cSeparatorItem, delete_separatoritem },
    { cStringView,    delete_stringview },
    { cTextView,      delete_textview },
    { cView,          delete_view },
    { cWindow,        delete_window },
    /* mail */
    { cMailMessage,   delete_mailmessage },
    /* storage */
    { cDirectory,     delete_directory },
    { cEntry,         delete_entry },
    { cEntry_ref,     delete_entry_ref },
    { cEntryList,     delete_entrylist },
    { cNode,          delete_node },
    { cNode_ref,      delete_node_ref },
    { cPath,          delete_path },
    { cQuery,         delete_query },
    { cResources,     delete_resources },
    { cStatable,      delete_statable },
    { cVolume,        delete_volume },
    { cVolumeRoster,  delete_volumeroster },
    /* support */
    { cArchivable,    delete_archivable },
    { cDataIO,        delete_dataio },
    { cFlattenable,   delete_flattenable },
    { cList,          delete_list },
    { cStopWatch,     delete_stopwatch },
    { cString,        delete_string },
    /* translation */
    { cTranslatorRoster, delete_translatorroster },
    { 0,              0 }
  };

  for (int32 i=0; klasses[i].klass; i++) {
    if (rb_funcall(klasses[i].klass, rb_intern(">="), 1, klass)) {
      return ((obj) ? Data_Wrap_Struct(klass, 0, klasses[i].delete_func, obj):Qnil);
    }
  }

  DEBUG_TRACE();
  rb_raise(rb_eTypeError, "not a Be object");
  return Qnil; /* not reached */
}


void
my_add_relative(VALUE obj, VALUE relative)
{
  VALUE hash = rb_ivar_get(obj, id_relatives);

  if (NIL_P(hash) || !HASH_P(hash)) {
    hash = rb_hash_new();
    rb_ivar_set(obj, id_relatives, hash);
  }
  rb_hash_aset(hash, ULONG2NUM(relative), relative);
}


void
my_clear_relative(VALUE obj, VALUE relative)
{
  VALUE hash = rb_ivar_get(obj, id_relatives);

  if (NIL_P(hash) || !HASH_P(hash)) {
    hash = rb_hash_new();
    rb_ivar_set(obj, id_relatives, hash);
  }
  rb_hash_aset(hash, ULONG2NUM(relative), Qnil);
}


VALUE
my_rgb_color2hash(rgb_color &rgba)
{
  VALUE hash = rb_hash_new();
  rb_hash_aset(hash, rb_str_new2("red"),   UCHR2FIX(rgba.red));
  rb_hash_aset(hash, rb_str_new2("green"), UCHR2FIX(rgba.green));
  rb_hash_aset(hash, rb_str_new2("blue"),  UCHR2FIX(rgba.blue));
  rb_hash_aset(hash, rb_str_new2("alpha"), UCHR2FIX(rgba.alpha));
  return hash;
}

rgb_color
my_hash2rgb_color(VALUE hash)
{
  rgb_color agba;

  Check_Type(hash, T_HASH);

  agba.red   = NUM2UCHR(rb_hash_aref(hash, rb_str_new2("red")));
  agba.green = NUM2UCHR(rb_hash_aref(hash, rb_str_new2("green")));
  agba.blue  = NUM2UCHR(rb_hash_aref(hash, rb_str_new2("blue")));
  agba.alpha = NUM2UCHR(rb_hash_aref(hash, rb_str_new2("alpha")));
  return agba;
}


VALUE
my_dirent_t2hash(dirent_t &dir)
{
  VALUE hash = rb_hash_new();
  rb_hash_aset(hash, rb_str_new2("d_dev"),    LONG2NUM(dir.d_dev));
  rb_hash_aset(hash, rb_str_new2("d_pdev"),   LONG2NUM(dir.d_pdev));
  rb_hash_aset(hash, rb_str_new2("d_ino"),    LLONG2NUM(dir.d_ino));
  rb_hash_aset(hash, rb_str_new2("d_pino"),   LLONG2NUM(dir.d_pino));
  rb_hash_aset(hash, rb_str_new2("d_reclen"), ULONG2NUM(dir.d_reclen));
  rb_hash_aset(hash, rb_str_new2("d_name"),   rb_str_new2(dir.d_name));
  return hash;
}


char**
my_rarray2argv(VALUE ary)
{
  char **argv=0;
  int32 len;

  Check_Type(ary, T_ARRAY);

  len = RARRAY(ary)->len;
  argv = ALLOC_N(char*, len);
  for (int32 i=0; i < len; i++) {
    argv[i] = STR2CSTR(RARRAY(ary)->ptr[i]);
  }
  return argv;
}


VALUE
my_filter_list2rarry(BList &list)
{
  VALUE vresult, vfilter;
  int32 n;
  BMessageFilter *filter;

  vresult = rb_ary_new();

  n = list.CountItems();
  for (int32 i=0; i < n; i++) {
    filter = reinterpret_cast<BMessageFilter *>(list.ItemAt(i));
    vfilter = Data_Wrap_Struct(cMessageFilter, 0, 0, filter);
    rb_ary_push(vresult, vfilter);
  }
  return vresult;
}


extern "C" void
Init_be()
{
  id_relatives = rb_intern("relatives");

  rb_global_variable(&be_object_list);
  be_object_list = rb_hash_new();

  mBe = rb_define_module("Be");

  Init_Archivable();
    Init_Bitmap();
    Init_Cursor();
      Init_AppDefs();
    Init_Handler();
      Init_Looper();
        Init_Application();
        Init_Window();
          Init_Alert();
      Init_View();
        Init_ScrollView();
        Init_StringView();
        Init_TextView();
        Init_Control();
          Init_Button();
        Init_Menu();
          Init_MenuBar();
    Init_Picture();
    Init_Translator();
    Init_TranslatorRoster();

  Init_EntryList();
    Init_Query();

  Init_Flattenable();
    Init_Path();

  Init_Invoker();
    Init_MenuItem();
      Init_SeparatorItem();

  Init_Statable();
    Init_Entry();
    Init_Node();
      Init_Directory();

  Init_Background();
  Init_Beep();
  Init_Clipboard();
  Init_DataIO();
  Init_E_mail();
  Init_Errors();
  Init_Font();
  Init_GraphicsDefs();
  Init_InterfaceDefs();
  Init_List();
  Init_MediaDefs();
  Init_Message();
  Init_MessageFilter();
  Init_MessageQueue();
  Init_MessageRunner();
  Init_Messenger();
  Init_MidiDefs();
  Init_Mime();
  Init_NetPositive();
  Init_NodeMonitor();
  Init_OS();
  Init_Point();
  Init_Polygon();
  Init_Rect();
  Init_Region();
  Init_Resources();
  Init_Roster();
  Init_scheduler();
  Init_StopWatch();
  Init_String();
  Init_TranslationDefs();
  Init_TranslationErrors();
  Init_TranslatorFormats();
  Init_TypeConstants();
  Init_UTF8();
  Init_Volume();
  Init_VolumeRoster();
  Init_fs_attr();
  Init_fs_index();
  Init_fs_info();
}
