/*
 *  my_bignum.cc
 *
 *  original code is
 *   Ruby bignum.c
 */

#include <ruby.h>
#include "my_be.h"


bool
my_check_range_long(VALUE x)
{
  if (rb_funcall(LONG2NUM(LONG_MIN), rb_intern(">"), 1, x)
      || rb_funcall(LONG2NUM(LONG_MAX), rb_intern("<"), 1, x)) {
    return false;
  }
  return true;
}


bool
my_check_range_ulong(VALUE x)
{
  if (rb_funcall(INT2FIX(0), rb_intern(">"), 1, x)
      || rb_funcall(ULONG2NUM(ULONG_MAX), rb_intern("<"), 1, x)) {
    return false;
  }
  return true;
}


bool
my_check_range_llong(VALUE x)
{
  if (rb_funcall(LLONG2NUM(LONGLONG_MIN), rb_intern(">"), 1, x)
      || rb_funcall(LLONG2NUM(LONGLONG_MAX), rb_intern("<"), 1, x)) {
    return false;
  }
  return true;
}


bool
my_check_range_ullong(VALUE x)
{
  if (rb_funcall(INT2FIX(0), rb_intern(">"), 1, x)
      || rb_funcall(ULLONG2NUM(ULONGLONG_MAX), rb_intern("<"), 1, x)) {
    return false;
  }
  return true;
}


typedef unsigned int BDIGIT;
typedef unsigned long long BDIGIT_DBL;
typedef long long BDIGIT_DBL_SIGNED;


#define BDIGITS(x) ((BDIGIT*)RBIGNUM(x)->digits)
#define BITSPERDIG (sizeof(BDIGIT)*CHAR_BIT)
#define BIGRAD ((BDIGIT_DBL)1 << BITSPERDIG)
#define DIGSPERLONG ((long)(sizeof(BDIGIT_DBL)/sizeof(BDIGIT)))
#define BIGUP(x) ((BDIGIT_DBL)(x) << BITSPERDIG)
#define BIGDN(x) RSHIFT(x,BITSPERDIG)
#define BIGLO(x) ((BDIGIT)((x) & (BIGRAD-1)))


static VALUE
my_bignew(long len, char sign)
{
  NEWOBJ(big, struct RBignum);
  OBJSETUP(big, rb_cBignum, T_BIGNUM);
  big->sign = sign;
  big->len = len;
  big->digits = ALLOC_N(BDIGIT, len);

  return (VALUE)big;
}


VALUE
my_ullong2big(BDIGIT_DBL n)
{
  BDIGIT_DBL num = n;
  long i = 0;
  BDIGIT *digits;
  VALUE big;

  i = 0;
  big = my_bignew(DIGSPERLONG, 1);
  digits = BDIGITS(big);
  while (i < DIGSPERLONG) {
    digits[i++] = BIGLO(num);
    num = BIGDN(num);
  }

  i = DIGSPERLONG;
  while (i-- && !digits[i]) ;
  RBIGNUM(big)->len = i+1;
  return big;
}


VALUE
my_llong2big(BDIGIT_DBL_SIGNED n)
{
  long neg = 0;
  VALUE big;

  if (n < 0) {
    n = -n;
    neg = 1;
  }
  big = my_ullong2big(n);
  if (neg) {
    RBIGNUM(big)->sign = 0;
  }
  return big;
}


static BDIGIT_DBL
big2ullong(VALUE x)
{
  long len = RBIGNUM(x)->len;
  BDIGIT_DBL num;
  BDIGIT *ds;

  ds = BDIGITS(x);
  num = 0;
  while (len--) {
    num = BIGUP(num);
    num += ds[len];
  }
  return num;
}


BDIGIT_DBL
my_big2ullong(VALUE x)
{
  BDIGIT_DBL num;

  Check_Range_Ullong(x);
  num = big2ullong(x);
  return num;
}


BDIGIT_DBL_SIGNED
my_big2llong(VALUE x)
{
  BDIGIT_DBL num;

  Check_Range_Llong(x);
  num = big2ullong(x);
  if (!RBIGNUM(x)->sign) {
    return -(BDIGIT_DBL_SIGNED)num;
  }
  return num;
}
