/*
 *  rbAlert.cc
 */

#include <Application.h>
#include "rbAlert.h"
#include "my_be.h"


Alert::Alert(const char *title,
             const char *text,
             const char *button1,
             const char *button2 = NULL,
             const char *button3 = NULL,
             button_width width = B_WIDTH_AS_USUAL,
             alert_type type = B_INFO_ALERT)
  : BAlert(title, text, button1, button2, button3, width, type)
{
  myself = Qnil;
  sync = false;
}


Alert::Alert(const char *title,
             const char *text,
             const char *button1,
             const char *button2,
             const char *button3,
             button_width width,
             button_spacing spacing,
             alert_type type = B_INFO_ALERT)
  : BAlert(title, text, button1, button2, button3, width, spacing, type)
{
  myself = Qnil;
  sync = false;
}


Alert::Alert(BMessage *data)
  : BAlert(data)
{
  myself = Qnil;
  sync = false;
}


void
Alert::MessageReceived(BMessage *message)
{
  VALUE varg1;

  switch (message->what) {
    case 'ALTB':
      BAlert::MessageReceived(message);
      break;

    default:
      if (sync) {
        varg1 = Data_Wrap_Struct(cMessage, 0, 0, message);
        rb_funcall(myself, id_MessageReceived, 1, varg1);
      }
      else {
        this->Unlock();
        if (be_app->Lock() && this->Lock()) {
          varg1 = Data_Wrap_Struct(cMessage, 0, 0, message);

          rb_funcall(myself, id_MessageReceived, 1, varg1);
          be_app->Unlock();
        }
      }
  }
}


void
Alert::FrameResized(float new_width,
                    float new_height)
{
  VALUE varg1, varg2;

  if (sync) {
    varg1 = rb_float_new(new_width);
    varg2 = rb_float_new(new_height);
    rb_funcall(myself, rb_intern("FrameResized"), 2, varg1, varg2);
  }
  else {
    this->Unlock();
    if (be_app->Lock() && this->Lock()) {
      varg1 = rb_float_new(new_width);
      varg2 = rb_float_new(new_height);

      rb_funcall(myself, rb_intern("FrameResized"), 2, varg1, varg2);
      be_app->Unlock();
    }
  }
}


BHandler*
Alert::ResolveSpecifier(BMessage *msg,
                        int32 index,
                        BMessage *specifier,
                        int32 form,
                        const char *property)
{
  VALUE varg1, varg2, varg3, varg4, varg5, vresult;
  BHandler *result = NULL;

  if (sync || be_app->Lock()) {
    varg1 = Data_Wrap_Struct(cMessage, 0, 0, msg);
    varg2 = LONG2NUM(index);
    varg3 = Data_Wrap_Struct(cMessage, 0, 0, specifier);
    varg4 = LONG2NUM(form);
    varg5 = rb_str_new2(property);

    vresult = rb_funcall(myself, rb_intern("ResolveSpecifier"), 5,
                         varg1,
                         varg2,
                         varg3,
                         varg4,
                         varg5);
    Get_BInstance(Handler, vresult, result);
    if (!sync) be_app->Unlock();
  }
  return result;
}


status_t
Alert::GetSupportedSuites(BMessage *data)
{
  VALUE varg1;

  if (sync || be_app->Lock()) {
    varg1 = Data_Wrap_Struct(cMessage, 0, 0, data);
    rb_funcall(myself, rb_intern("GetSupportedSuites"), 1, varg1);
    if (!sync) be_app->Unlock();
  }
  return B_OK;
}


void
Alert::DispatchMessage(BMessage *message,
                       BHandler *handler)
{
  VALUE varg1, varg2;

  switch (message->what) {
    case 'ALTB':
    case _QUIT_:
      BAlert::DispatchMessage(message, handler);
      break;

    default:
      if (sync) {
        varg1 = Data_Wrap_Struct(cMessage, 0, 0, message);
        varg2 = Data_Wrap_Struct(cHandler, 0, 0, handler);

        rb_funcall(myself, id_DispatchMessage, 2, varg1, varg2);
      }
      else {
        this->Unlock();
        if (be_app->Lock() && this->Lock()) {
          varg1 = Data_Wrap_Struct(cMessage, 0, 0, message);
          varg2 = Data_Wrap_Struct(cHandler, 0, 0, handler);

          rb_funcall(myself, id_DispatchMessage, 2, varg1, varg2);
          be_app->Unlock();
        }
      }
  }
}


void
Alert::Quit()
{
  if (find_thread(NULL) == this->Thread()) {
    while (be_app->IsLocked()) {
      be_app->Unlock();
    }
    BAlert::Quit();
    /* not reached */
  }
  else {
    BAlert::Quit();
  }
}


bool
Alert::QuitRequested()
{
  bool result = true;

  if (find_thread(NULL) == this->Thread()) {
    if (sync || be_app->Lock()) {
      VALUE vresult;
      vresult = rb_funcall2(myself, rb_intern("QuitRequested"), 0, 0);
      result = RTEST(vresult);
      if (!sync) be_app->Unlock();
    }
  }
  else {
    VALUE vresult;

    vresult = rb_funcall2(myself, rb_intern("QuitRequested"), 0, 0);
    result = RTEST(vresult);
  }
  return result;
}


void*
Alert::operator new(size_t size)
{
  return xmalloc(size);
}


void
Alert::operator delete(void *ptr)
{
  free(ptr);
}
