/*
 *  rbApplication.cc
 */

#include "rbApplication.h"
#include "my_be.h"


Application::Application(const char *signature)
  : BApplication(signature)
{
}


Application::Application(const char *signature,
                         status_t *error)
  : BApplication(signature, error)
{
}


Application::Application(BMessage *data)
  : BApplication(data)
{
}


thread_id
Application::Run()
{
  VALUE vresult;
  thread_id result;

  vresult = rb_funcall(my_app, rb_intern("Run"), 0, 0);
  result = NUM2LONG(vresult);
  return result;
}


void
Application::Quit()
{
  rb_funcall(my_app, rb_intern("Quit"), 0, 0);
}


bool
Application::QuitRequested()
{
  VALUE vresult;
  bool result;

  vresult = rb_funcall(my_app, rb_intern("QuitRequested"), 0, 0);
  result = RTEST(vresult);
  return result;
}


void
Application::Pulse()
{
  rb_funcall(my_app, rb_intern("Pulse"), 0, 0);
}


void
Application::ReadyToRun()
{
  rb_funcall(my_app, rb_intern("ReadyToRun"), 0, 0);
}


void
Application::MessageReceived(BMessage *message)
{
  VALUE varg1;
  varg1 = Data_Wrap_Struct(cMessage, 0, 0, message);
  rb_funcall(my_app, id_MessageReceived, 1, varg1);
}


static VALUE
my_argv2rarray(int argc, char** argv)
{
  VALUE args;

  args = rb_ary_new();
  for (int32 i=0; i < argc; i++) {
    rb_ary_push(args, rb_str_new2(argv[i]));
  }
  return args;
}


void
Application::ArgvReceived(int32 argc,
                          char **argv)
{
  VALUE varg1;
  varg1 = my_argv2rarray(argc, argv);
  rb_funcall2(my_app, rb_intern("ArgvReceived"), argc, RARRAY(varg1)->ptr);
}


void
Application::AppActivated(bool active)
{
  VALUE varg1;
  varg1 = (active ? Qtrue:Qfalse);
  rb_funcall(my_app, rb_intern("AppActivated"), 1, varg1);
}


void
Application::RefsReceived(BMessage *a_message)
{
  VALUE varg1;
  varg1 = Data_Wrap_Struct(cMessage, 0, 0, a_message);
  rb_funcall(my_app, rb_intern("RefsReceived"), 1, varg1);
}


void
Application::AboutRequested()
{
  rb_funcall(my_app, rb_intern("AboutRequested"), 0, 0);
}


BHandler*
Application::ResolveSpecifier(BMessage *msg,
                              int32 index,
                              BMessage *specifier,
                              int32 form,
                              const char *property)
{
  VALUE varg1, varg2, varg3, varg4, varg5, vresult;
  BHandler *result;

  varg1 = Data_Wrap_Struct(cMessage, 0, 0, msg);
  varg2 = LONG2NUM(index);
  varg3 = Data_Wrap_Struct(cMessage, 0, 0, specifier);
  varg4 = LONG2NUM(form);
  varg5 = rb_str_new2(property);

  vresult = rb_funcall(my_app, rb_intern("ResolveSpecifier"), 5,
                       varg1,
                       varg2,
                       varg3,
                       varg4,
                       varg5);
  Get_BInstance(Handler, vresult, result);
  return result;
}


void
Application::DispatchMessage(BMessage *message,
                             BHandler *handler)
{
  VALUE varg1, varg2;

  varg1 = Data_Wrap_Struct(cMessage, 0, 0, message);
  varg2 = Data_Wrap_Struct(cHandler, 0, 0, handler);
  rb_funcall(my_app, id_DispatchMessage, 2, varg1, varg2);
}


status_t
Application::GetSupportedSuites(BMessage *data)
{
  VALUE varg1, vresult;
  status_t result = B_OK;

  varg1 = Data_Wrap_Struct(cMessage, 0, 0, data);
  vresult = rb_funcall(my_app, rb_intern("GetSupportedSuites"), 1, varg1);
  if (!NIL_P(vresult)) {
    result = NUM2LONG(vresult);
  }
  return result;
}


void*
Application::operator new(size_t size)
{
  return xmalloc(size);
}


void
Application::operator delete(void *ptr)
{
  free(ptr);
}
