/*
 *  rbButton.cc
 */

#include "rbButton.h"
#include "my_be.h"


Button::Button(BRect frame,
                   const char *name,
                   const char *label,
                   BMessage *message,
                   uint32 resizeMask = B_FOLLOW_LEFT | B_FOLLOW_TOP,
                   uint32 flags = B_WILL_DRAW | B_NAVIGABLE)
  : BButton(frame, name, label, message, resizeMask, flags)
{
  myself = Qnil;
}


Button::Button(BMessage *data)
  : BButton(data)
{
  myself = Qnil;
}


void
Button::Draw(BRect updateRect)
{
  VALUE varg1;
  varg1 = Data_Wrap_Struct(cRect, 0, 0, &updateRect);
  rb_funcall(myself, rb_intern("Draw"), 1, varg1);
}


void
Button::MouseDown(BPoint where)
{
  VALUE varg1;
  varg1 = Data_Wrap_Struct(cPoint, 0, 0, &where);
  rb_funcall(myself, rb_intern("MouseDown"), 1, varg1);
}


void
Button::AttachedToWindow()
{
  rb_funcall2(myself, rb_intern("AttachedToWindow"), 0, 0);
}


void
Button::KeyDown(const char *bytes, int32 numBytes)
{
  VALUE varg1;
  varg1 = rb_str_new(bytes, numBytes);
  rb_funcall(myself, rb_intern("KeyDown"), 1, varg1);
}


void
Button::MakeDefault(bool state)
{
  VALUE varg1;
  varg1 = (state ? Qtrue:Qfalse);
  rb_funcall(myself, rb_intern("MakeDefault"), 1, varg1);
}


void
Button::SetLabel(const char *text)
{
  VALUE varg1;
  varg1 = rb_str_new2(text);
  rb_funcall(myself, rb_intern("SetLabel"), 1, varg1);
}


void
Button::MessageReceived(BMessage *message)
{
  VALUE varg1;
  varg1 = Data_Wrap_Struct(cMessage, 0, 0, message);
  rb_funcall(myself, id_MessageReceived, 1, varg1);
}


void
Button::WindowActivated(bool state)
{
  VALUE varg1;
  varg1 = (state ? Qtrue:Qfalse);
  rb_funcall(myself, rb_intern("WindowActivated"), 1, varg1);
}


void
Button::MouseUp(BPoint pt)
{
  VALUE varg1;
  varg1 = Data_Wrap_Struct(cPoint, 0, 0, &pt);
  rb_funcall(myself, rb_intern("MouseUp"), 1, varg1);
}


void
Button::MouseMoved(BPoint where,
                   uint32 code,
                   const BMessage *message)
{
  VALUE varg1, varg2, varg3;

  varg1 = Data_Wrap_Struct(cPoint, 0, 0, &where);
  varg2 = ULONG2NUM(code);
  varg3 = ((message) ? Data_Wrap_Struct(cMessage, 0, 0, const_cast<BMessage *>(message)):Qnil);
  rb_funcall(myself, rb_intern("MouseMoved"), 3, varg1, varg2, varg3);
}


void
Button::DetachedFromWindow()
{
  rb_funcall2(myself, rb_intern("DetachedFromWindow"), 0, 0);
}


void
Button::SetValue(int32 value)
{
  VALUE varg1;
  varg1 = LONG2NUM(value);
  rb_funcall(myself, rb_intern("SetValue"), 1, varg1);
}


void
Button::GetPreferredSize(float *width,
                         float *height)
{
  VALUE varg1, varg2, vresult;
  varg1 = rb_float_new(*width);
  varg2 = rb_float_new(*height);
  vresult = rb_funcall(myself, rb_intern("GetPreferredSize"), 2, varg1, varg2);

  Check_Type(vresult, T_ARRAY);
  Check_Is_Kind_Of(RARRAY(vresult)->ptr[0], rb_cNumeric);
  Check_Is_Kind_Of(RARRAY(vresult)->ptr[1], rb_cNumeric);

  *width = NUM2DBL(RARRAY(vresult)->ptr[0]);
  *height = NUM2DBL(RARRAY(vresult)->ptr[1]);
}


void
Button::ResizeToPreferred()
{
  rb_funcall2(myself, rb_intern("ResizeToPreferred"), 0, 0);
}


status_t
Button::Invoke(BMessage *message = NULL)
{
  VALUE varg1, vresult;
  status_t result = B_OK;

  if (message) {
    varg1 = Data_Wrap_Struct(cMessage, 0, 0, message);
    vresult = rb_funcall(myself, rb_intern("Invoke"), 1, varg1);
  }
  else {
    vresult = rb_funcall2(myself, rb_intern("Invoke"), 0, 0);
  }

  if (!NIL_P(vresult)) {
    result = NUM2LONG(vresult);
  }
  return result;
}


void
Button::FrameMoved(BPoint new_position)
{
  VALUE varg1;
  varg1 = Data_Wrap_Struct(cPoint, 0, 0, &new_position);
  rb_funcall(myself, rb_intern("FrameMoved"), 1, varg1);
}


void
Button::FrameResized(float new_width, float new_height)
{
  VALUE varg1, varg2;
  varg1 = rb_float_new(new_width);
  varg2 = rb_float_new(new_height);
  rb_funcall(myself, rb_intern("FrameResized"), 2, varg1, varg2);
}


void
Button::MakeFocus(bool state=true)
{
  VALUE varg1;
  varg1 = (state ? Qtrue:Qfalse);
  rb_funcall(myself, rb_intern("MakeFocus"), 1, varg1);
}


void
Button::AllAttached()
{
  rb_funcall2(myself, rb_intern("AllAttached"), 0, 0);
}


void
Button::AllDetached()
{
  rb_funcall2(myself, rb_intern("AllDetached"), 0, 0);
}


BHandler*
Button::ResolveSpecifier(BMessage *msg,
                         int32 index,
                         BMessage *specifier,
                         int32 form,
                         const char *property)
{
  VALUE varg1, varg2, varg3, varg4, varg5, vresult;
  BHandler *result;

  varg1 = Data_Wrap_Struct(cMessage, 0, 0, msg);
  varg2 = LONG2NUM(index);
  varg3 = Data_Wrap_Struct(cMessage, 0, 0, specifier);
  varg4 = LONG2NUM(form);
  varg5 = rb_str_new2(property);

  vresult = rb_funcall(myself, rb_intern("ResolveSpecifier"), 5,
                       varg1,
                       varg2,
                       varg3,
                       varg4,
                       varg5);
  Get_BInstance(Handler, vresult, result);
  return result;
}


status_t
Button::GetSupportedSuites(BMessage *data)
{
  VALUE varg1, vresult;
  status_t result = B_OK;

  varg1 = Data_Wrap_Struct(cMessage, 0, 0, data);
  vresult = rb_funcall(myself, rb_intern("GetSupportedSuites"), 1, varg1);
  if (!NIL_P(vresult)) {
    result = NUM2LONG(vresult);
  }
  return result;
}


void*
Button::operator new(size_t size)
{
  return xmalloc(size);
}


void
Button::operator delete(void *ptr)
{
  free(ptr);
}
