/*
 *  rbHandler.cc
 */

#include "rbHandler.h"


Handler::Handler(const char *name=NULL)
  : BHandler(name)
{
  myself = Qnil;
}


Handler::Handler(BMessage *data)
  : BHandler(data)
{
  myself = Qnil;
}


void
Handler::MessageReceived(BMessage *message)
{
  VALUE varg1;
  varg1 = Data_Wrap_Struct(cMessage, 0, 0, message);
  rb_funcall(myself, id_MessageReceived, 1, varg1);
}


void
Handler::SetNextHandler(BHandler *handler)
{
  VALUE varg1;
  varg1 = Data_Wrap_Struct(cHandler, 0, 0, handler);
  rb_funcall(myself, rb_intern("SetNextHandler"), 1, varg1);
}


void
Handler::AddFilter(BMessageFilter *filter)
{
  VALUE varg1;
  varg1 = Data_Wrap_Struct(cMessageFilter, 0, 0, filter);
  rb_funcall(myself, rb_intern("AddFilter"), 1, varg1);
}


bool
Handler::RemoveFilter(BMessageFilter *filter)
{
  VALUE varg1, vresult;
  bool result;

  varg1 = Data_Wrap_Struct(cMessageFilter, 0, 0, filter);
  vresult = rb_funcall(myself, rb_intern("RemoveFilter"), 1, varg1);

  result = RTEST(vresult);
  return result;
}


void
Handler::SetFilterList(BList *filters)
{
  VALUE varg1;
  varg1 = Data_Wrap_Struct(cList, 0, 0, filters);
  rb_funcall(myself, rb_intern("SetFilterList"), 1, varg1);
}


BHandler*
Handler::ResolveSpecifier(BMessage *msg,
                            int32 index,
                            BMessage *specifier,
                            int32 form,
                            const char *property)
{
  VALUE varg1, varg2, varg3, varg4, varg5, vresult;
  BHandler *result;

  varg1 = Data_Wrap_Struct(cMessage, 0, 0, msg);
  varg2 = LONG2NUM(index);
  varg3 = Data_Wrap_Struct(cMessage, 0, 0, specifier);
  varg4 = LONG2NUM(form);
  varg5 = rb_str_new2(property);

  vresult = rb_funcall(myself, rb_intern("ResolveSpecifier"), 5,
                       varg1,
                       varg2,
                       varg3,
                       varg4,
                       varg5);
  Get_BInstance(Handler, vresult, result);
  return result;
}


status_t
Handler::GetSupportedSuites(BMessage *data)
{
  VALUE varg1;

  varg1 = Data_Wrap_Struct(cMessage, 0, 0, data);
  rb_funcall(myself, rb_intern("GetSupportedSuites"), 1, varg1);
  return B_OK;
}


void
Handler::SendNotices(uint32 what,
                       const BMessage *msg=0)
{
  VALUE varg1, varg2;

  varg1 = ULONG2NUM(what);
  varg2 = ((msg) ? Data_Wrap_Struct(cMessage, 0, 0, (BMessage *)msg):Qnil);
  rb_funcall(myself, rb_intern("SendNotices"), 2, varg1, varg2);
}


void*
Handler::operator new(size_t size)
{
  return xmalloc(size);
}


void
Handler::operator delete(void *ptr)
{
  free(ptr);
}
