/*
 *  rbLooper.cc
 */

#include <Application.h>
#include "rbLooper.h"
#include "my_be.h"


Looper::Looper(const char *name=NULL,
               int32 priority=B_NORMAL_PRIORITY,
               int32 port_capacity=B_LOOPER_PORT_DEFAULT_CAPACITY)
  : BLooper(name, priority, port_capacity)
{
  myself = Qnil;
}


Looper::Looper(BMessage *data)
  : BLooper(data)
{
  myself = Qnil;
}


void
Looper::DispatchMessage(BMessage *message,
                        BHandler *handler)
{
  this->Unlock();
  if (be_app->Lock() && this->Lock()) {
    VALUE varg1, varg2;

    varg1 = Data_Wrap_Struct(cMessage, 0, 0, message);
    varg2 = Data_Wrap_Struct(cHandler, 0, 0, handler);

    rb_funcall(myself, id_DispatchMessage, 2, varg1, varg2);
    be_app->Unlock();
  }
}


void
Looper::MessageReceived(BMessage *message)
{
  this->Unlock();
  if (be_app->Lock() && this->Lock()) {
    VALUE varg1 = Data_Wrap_Struct(cMessage, 0, 0, message);
    rb_funcall(myself, id_MessageReceived, 1, varg1);
    be_app->Unlock();
  }
}


thread_id
Looper::Run()
{
  VALUE vresult;
  thread_id result;

  vresult = rb_funcall(myself, rb_intern("Run"), 0, 0);
  result = NUM2LONG(vresult);

  return result;
}


void
Looper::Quit()
{
  if (find_thread(NULL) == this->Thread()) {
    while (be_app->IsLocked()) {
      be_app->Unlock();
    }
    BLooper::Quit();
    /* not reached */
  }
  else {
    BLooper::Quit();
  }
}


bool
Looper::QuitRequested()
{
  bool result = true;

  if (find_thread(NULL) == this->Thread()) {
    this->Unlock();
    if (be_app->Lock() && this->Lock()) {
      VALUE vresult;
      vresult = rb_funcall2(myself, rb_intern("QuitRequested"), 0, 0);
      result = RTEST(vresult);
      be_app->Unlock();
    }
  }
  else {
    VALUE vresult;

    vresult = rb_funcall2(myself, rb_intern("QuitRequested"), 0, 0);
    result = RTEST(vresult);
  }
  return result;
}


BHandler*
Looper::ResolveSpecifier(BMessage *msg,
                         int32 index,
                         BMessage *specifier,
                         int32 form,
                         const char *property)
{
  VALUE varg1, varg2, varg3, varg4, varg5, vresult;
  BHandler *result = NULL;

  this->Unlock();
  if (be_app->Lock() && this->Lock()) {
    varg1 = Data_Wrap_Struct(cMessage, 0, 0, msg);
    varg2 = LONG2NUM(index);
    varg3 = Data_Wrap_Struct(cMessage, 0, 0, specifier);
    varg4 = LONG2NUM(form);
    varg5 = rb_str_new2(property);

    vresult = rb_funcall(myself, rb_intern("ResolveSpecifier"), 5,
                         varg1,
                         varg2,
                         varg3,
                         varg4,
                         varg5);
    Get_BInstance(Handler, vresult, result);
    be_app->Unlock();
  }
  return result;
}


status_t
Looper::GetSupportedSuites(BMessage *data)
{
  this->Unlock();
  if (be_app->Lock() && this->Lock()) {
    VALUE varg1 = Data_Wrap_Struct(cMessage, 0, 0, data);
    rb_funcall(myself, rb_intern("GetSupportedSuites"), 1, varg1);
    be_app->Unlock();
  }
  return B_OK;
}


void
Looper::AddCommonFilter(BMessageFilter *filter)
{
  this->Unlock();
  if (be_app->Lock() && this->Lock()) {
    VALUE varg1 = Data_Wrap_Struct(cMessageFilter, 0, 0, filter);
    rb_funcall(myself, rb_intern("AddCommonFilter"), 1, varg1);
    be_app->Unlock();
  }
}


bool
Looper::RemoveCommonFilter(BMessageFilter *filter)
{
  VALUE varg1, vresult = Qfalse;

  this->Unlock();
  if (be_app->Lock() && this->Lock()) {
    varg1 = Data_Wrap_Struct(cMessageFilter, 0, 0, filter);
    vresult = rb_funcall(myself, rb_intern("RemoveCommonFilter"), 1, varg1);
    be_app->Unlock();
  }
  return vresult;
}


void
Looper::SetCommonFilterList(BList *filters)
{
  this->Unlock();
  if (be_app->Lock() && this->Lock()) {
    VALUE varg1 = Data_Wrap_Struct(cList, 0, 0, filters);
    rb_funcall(myself, rb_intern("SetCommonFilterList"), 1, varg1);
    be_app->Unlock();
  }
}


void*
Looper::operator new(size_t size)
{
  return xmalloc(size);
}


void
Looper::operator delete(void *ptr)
{
  free(ptr);
}
