/*
 *  rbMenuBar.cc
 */

#include "rbMenuBar.h"


MenuBar::MenuBar(BRect frame,
                 const char *title,
                 uint32 resizeMask = B_FOLLOW_LEFT_RIGHT | B_FOLLOW_TOP,
                 menu_layout layout = B_ITEMS_IN_ROW,
                 bool resizeToFit = true)
  : BMenuBar(frame, title, resizeMask, layout, resizeToFit)
{
  myself = Qnil;
}


MenuBar::MenuBar(BMessage *data)
  : BMenuBar(data)
{
  myself = Qnil;
}


void
MenuBar::SetBorder(menu_bar_border border)
{
  VALUE varg1;
  varg1 = LONG2NUM(border);
  rb_funcall(myself, rb_intern("SetBorder"), 1, varg1);
}


void
MenuBar::Draw(BRect updateRect)
{
  VALUE varg1;
  varg1 = Data_Wrap_Struct(cRect, 0, 0, &updateRect);
  rb_funcall(myself, rb_intern("Draw"), 1, varg1);
}


void
MenuBar::AttachedToWindow()
{
  rb_funcall2(myself, rb_intern("AttachedToWindow"), 0, 0);
}


void
MenuBar::DetachedFromWindow()
{
  rb_funcall2(myself, rb_intern("DetachedFromWindow"), 0, 0);
}


void
MenuBar::MessageReceived(BMessage *message)
{
  VALUE varg1;
  varg1 = Data_Wrap_Struct(cMessage, 0, 0, message);
  rb_funcall(myself, id_MessageReceived, 1, varg1);
}


void
MenuBar::MouseDown(BPoint where)
{
  VALUE varg1;
  varg1 = Data_Wrap_Struct(cPoint, 0, 0, &where);
  rb_funcall(myself, rb_intern("MouseDown"), 1, varg1);
}


void
MenuBar::WindowActivated(bool state)
{
  VALUE varg1;
  varg1 = (state ? Qtrue:Qfalse);
  rb_funcall(myself, rb_intern("WindowActivated"), 1, varg1);
}


void
MenuBar::MouseUp(BPoint where)
{
  VALUE varg1;
  varg1 = Data_Wrap_Struct(cPoint, 0, 0, &where);
  rb_funcall(myself, rb_intern("MouseUp"), 1, varg1);
}


void
MenuBar::FrameMoved(BPoint new_position)
{
  VALUE varg1;
  varg1 = Data_Wrap_Struct(cPoint, 0, 0, &new_position);
  rb_funcall(myself, rb_intern("FrameMoved"), 1, varg1);
}


void
MenuBar::FrameResized(float new_width,
                      float new_height)
{
  VALUE varg1, varg2;
  varg1 = rb_float_new(new_width);
  varg2 = rb_float_new(new_height);
  rb_funcall(myself, rb_intern("FrameResized"), 2, varg1, varg2);
}


void
MenuBar::Show()
{
  rb_funcall2(myself, rb_intern("Show"), 0, 0);
}


void
MenuBar::Hide()
{
  rb_funcall2(myself, rb_intern("Hide"), 0, 0);
}


BHandler*
MenuBar::ResolveSpecifier(BMessage *msg,
                          int32 index,
                          BMessage *specifier,
                          int32 form,
                          const char *property)
{
  VALUE varg1, varg2, varg3, varg4, varg5, vresult;
  BHandler *result;

  varg1 = Data_Wrap_Struct(cMessage, 0, 0, msg);
  varg2 = LONG2NUM(index);
  varg3 = Data_Wrap_Struct(cMessage, 0, 0, specifier);
  varg4 = LONG2NUM(form);
  varg5 = rb_str_new2(property);

  vresult = rb_funcall(myself, rb_intern("ResolveSpecifier"), 5,
                       varg1,
                       varg2,
                       varg3,
                       varg4,
                       varg5);
  Get_BInstance(Handler, vresult, result);
  return result;
}


status_t
MenuBar::GetSupportedSuites(BMessage *data)
{
  VALUE varg1, vresult;
  status_t result = B_OK;

  varg1 = Data_Wrap_Struct(cMessage, 0, 0, data);
  vresult = rb_funcall(myself, rb_intern("GetSupportedSuites"), 1, varg1);
  if (!NIL_P(vresult)) {
    result = NUM2LONG(vresult);
  }
  return result;
}


void
MenuBar::ResizeToPreferred()
{
  rb_funcall2(myself, rb_intern("ResizeToPreferred"), 0, 0);
}


void
MenuBar::GetPreferredSize(float *width,
                          float *height)
{
  VALUE varg1, varg2, vresult;
  varg1 = rb_float_new(*width);
  varg2 = rb_float_new(*height);
  vresult = rb_funcall(myself, rb_intern("GetPreferredSize"), 2, varg1, varg2);

  Check_Type(vresult, T_ARRAY);
  Check_Is_Kind_Of(RARRAY(vresult)->ptr[0], rb_cNumeric);
  Check_Is_Kind_Of(RARRAY(vresult)->ptr[1], rb_cNumeric);

  *width = NUM2DBL(RARRAY(vresult)->ptr[0]);
  *height = NUM2DBL(RARRAY(vresult)->ptr[1]);
}


void
MenuBar::MakeFocus(bool state = true)
{
  VALUE varg1;
  varg1 = (state ? Qtrue:Qfalse);
  rb_funcall(myself, rb_intern("MakeFocus"), 1, varg1);
}


void
MenuBar::AllAttached()
{
  rb_funcall2(myself, rb_intern("AllAttached"), 0, 0);
}


void
MenuBar::AllDetached()
{
  rb_funcall2(myself, rb_intern("AllDetached"), 0, 0);
}


void*
MenuBar::operator new(size_t size)
{
  return xmalloc(size);
}


void
MenuBar::operator delete(void *ptr)
{
  free(ptr);
}
