/*
 *  rbStringView.cc
 */

#include "rbMessage.h"
#include "rbStringView.h"
#include "my_be.h"


StringView::StringView(BRect bounds,
                       const char *name,
                       const char *text,
                       uint32 resizeFlags=B_FOLLOW_LEFT | B_FOLLOW_TOP,
                       uint32 flags=B_WILL_DRAW)
  : BStringView(bounds, name, text, resizeFlags, flags)
{
  myself = Qnil;
}


StringView::StringView(BMessage *data)
  : BStringView(data)
{
  myself = Qnil;
}


void
StringView::AttachedToWindow()
{
  rb_funcall2(myself, rb_intern("AttachedToWindow"), 0, 0);
}


void
StringView::Draw(BRect bounds)
{
  VALUE varg1;
  varg1 = Data_Wrap_Struct(cRect, 0, 0, &bounds);
  rb_funcall(myself, rb_intern("Draw"), 1, varg1);
}


void
StringView::MessageReceived(BMessage *message)
{
  VALUE varg1;
  varg1 = Data_Wrap_Struct(cMessage, 0, 0, message);
  rb_funcall(myself, id_MessageReceived, 1, varg1);
}


void
StringView::MouseDown(BPoint pt)
{
  VALUE varg1;
  varg1 = Data_Wrap_Struct(cPoint, 0, 0, &pt);
  rb_funcall(myself, rb_intern("MouseDown"), 1, varg1);
}


void
StringView::MouseUp(BPoint pt)
{
  VALUE varg1;
  varg1 = Data_Wrap_Struct(cPoint, 0, 0, &pt);
  rb_funcall(myself, rb_intern("MouseUp"), 1, varg1);
}


void
StringView::MouseMoved(BPoint pt, uint32 code, const BMessage *message)
{
  VALUE varg1, varg2, varg3;

  varg1 = Data_Wrap_Struct(cPoint, 0, 0, &pt);
  varg2 = ULONG2NUM(code);
  varg3 = ((message) ? Data_Wrap_Struct(cMessage, 0, 0, const_cast<BMessage *>(message)):Qnil);
  rb_funcall(myself, rb_intern("MouseMoved"), 3, varg1, varg2, varg3);
}


void
StringView::DetachedFromWindow()
{
  rb_funcall2(myself, rb_intern("DetachedFromWindow"), 0, 0);
}


void
StringView::FrameMoved(BPoint new_position)
{
  VALUE varg1;
  varg1 = Data_Wrap_Struct(cPoint, 0, 0, &new_position);
  rb_funcall(myself, rb_intern("MouseUp"), 1, varg1);
}


void
StringView::FrameResized(float new_width, float new_height)
{
  VALUE varg1, varg2;
  varg1 = rb_float_new(new_width);
  varg2 = rb_float_new(new_height);
  rb_funcall(myself, rb_intern("FrameResized"), 2, varg1, varg2);
}


BHandler*
StringView::ResolveSpecifier(BMessage *msg,
                               int32 index,
                               BMessage *specifier,
                               int32 form,
                               const char *property)
{
  VALUE varg1, varg2, varg3, varg4, varg5, vresult;
  BHandler *result;

  varg1 = Data_Wrap_Struct(cMessage, 0, 0, msg);
  varg2 = LONG2NUM(index);
  varg3 = Data_Wrap_Struct(cMessage, 0, 0, specifier);
  varg4 = LONG2NUM(form);
  varg5 = rb_str_new2(property);

  vresult = rb_funcall(myself, rb_intern("ResolveSpecifier"), 5,
                       varg1,
                       varg2,
                       varg3,
                       varg4,
                       varg5);
  Get_BInstance(Handler, vresult, result);
  return result;
}


void
StringView::ResizeToPreferred()
{
  rb_funcall2(myself, rb_intern("ResizeToPreferred"), 0, 0);
}


void
StringView::GetPreferredSize(float *width, float *height)
{
  VALUE varg1, varg2, vresult;
  varg1 = rb_float_new(*width);
  varg2 = rb_float_new(*height);
  vresult = rb_funcall(myself, rb_intern("GetPreferredSize"), 2, varg1, varg2);

  Check_Type(vresult, T_ARRAY);
  Check_Is_Kind_Of(RARRAY(vresult)->ptr[0], rb_cNumeric);
  Check_Is_Kind_Of(RARRAY(vresult)->ptr[1], rb_cNumeric);

  *width = NUM2DBL(RARRAY(vresult)->ptr[0]);
  *height = NUM2DBL(RARRAY(vresult)->ptr[1]);
}


void
StringView::MakeFocus(bool state=true)
{
  VALUE varg1;
  varg1 = (state ? Qtrue:Qfalse);
  rb_funcall(myself, rb_intern("MakeFocus"), 1, varg1);
}


void
StringView::AllAttached()
{
  rb_funcall2(myself, rb_intern("AllAttached"), 0, 0);
}


void
StringView::AllDetached()
{
  rb_funcall2(myself, rb_intern("AllDetached"), 0, 0);
}


status_t
StringView::GetSupportedSuites(BMessage *data)
{
  VALUE varg1, vresult;
  status_t result = B_OK;

  varg1 = Data_Wrap_Struct(cMessage, 0, 0, data);
  vresult = rb_funcall(myself, rb_intern("GetSupportedSuites"), 1, varg1);
  if (!NIL_P(vresult)) {
    result = NUM2LONG(vresult);
  }
  return result;
}


void*
StringView::operator new(size_t size)
{
  return xmalloc(size);
}


void
StringView::operator delete(void *ptr)
{
  free(ptr);
}
