/*
 *  rbView.cc
 */

#include "rbView.h"
#include "my_be.h"


View::View(BRect frame,
           const char *name,
           uint32 resizeMask,
           uint32 flags)
  : BView(frame, name, resizeMask, flags)
{
  myself = Qnil;
}


View::View(BMessage *data)
  : BView(data)
{
  myself = Qnil;
}


void
View::AttachedToWindow()
{
  rb_funcall2(myself, rb_intern("AttachedToWindow"), 0, 0);
}


void
View::AllAttached()
{
  rb_funcall2(myself, rb_intern("AllAttached"), 0, 0);
}


void
View::DetachedFromWindow()
{
  rb_funcall2(myself, rb_intern("DetachedFromWindow"), 0, 0);
}


void
View::AllDetached()
{
  rb_funcall2(myself, rb_intern("AllDetached"), 0, 0);
}


void
View::MessageReceived(BMessage *message)
{
  VALUE varg1;
  varg1 = Data_Wrap_Struct(cMessage, 0, 0, message);
  rb_funcall(myself, id_MessageReceived, 1, varg1);
}


void
View::Draw(BRect updateRect)
{
  VALUE varg1;
  varg1 = Data_Wrap_Struct(cRect, 0, 0, &updateRect);
  rb_funcall(myself, rb_intern("Draw"), 1, varg1);
}


void
View::MouseDown(BPoint where)
{
  VALUE varg1;
  varg1 = Data_Wrap_Struct(cPoint, 0, 0, &where);
  rb_funcall(myself, rb_intern("MouseDown"), 1, varg1);
}


void
View::MouseUp(BPoint where)
{
  VALUE varg1;
  varg1 = Data_Wrap_Struct(cPoint, 0, 0, &where);
  rb_funcall(myself, rb_intern("MouseUp"), 1, varg1);
}


void
View::MouseMoved(BPoint where,
                 uint32 code,
                 const BMessage *message)
{
  VALUE varg1, varg2, varg3;

  varg1 = Data_Wrap_Struct(cPoint, 0, 0, &where);
  varg2 = ULONG2NUM(code);
  varg3 = ((message) ? Data_Wrap_Struct(cMessage, 0, 0, const_cast<BMessage *>(message)):Qnil);
  rb_funcall(myself, rb_intern("MouseMoved"), 3, varg1, varg2, varg3);
}


void
View::WindowActivated(bool state)
{
  VALUE varg1;
  varg1 = (state ? Qtrue:Qfalse);
  rb_funcall(myself, rb_intern("WindowActivated"), 1, varg1);
}


void
View::KeyDown(const char *bytes,
              int32 numBytes)
{
  VALUE varg1;
  varg1 = rb_str_new(bytes, numBytes);
  rb_funcall(myself, rb_intern("KeyDown"), 1, varg1);
}


void
View::KeyUp(const char *bytes,
            int32 numBytes)
{
  VALUE varg1;
  varg1 = rb_str_new(bytes, numBytes);
  rb_funcall(myself, rb_intern("KeyUp"), 1, varg1);
}


void
View::Pulse()
{
  rb_funcall2(myself, rb_intern("Pulse"), 0, 0);
}


void
View::FrameMoved(BPoint new_position)
{
  VALUE varg1;
  varg1 = Data_Wrap_Struct(cPoint, 0, 0, &new_position);
  rb_funcall(myself, rb_intern("FrameMoved"), 1, varg1);
}


void
View::FrameResized(float new_width,
                   float new_height)
{
  VALUE varg1, varg2;
  varg1 = rb_float_new(new_width);
  varg2 = rb_float_new(new_height);
  rb_funcall(myself, rb_intern("FrameResized"), 2, varg1, varg2);
}


void
View::TargetedByScrollView(BScrollView *scroll_view)
{
  VALUE varg1;
  varg1 = Data_Wrap_Struct(cScrollView, 0, 0, scroll_view);
  rb_funcall(myself, rb_intern("TargetedByScrollView"), 1, varg1);
}


void
View::ConstrainClippingRegion(BRegion *region)
{
  VALUE varg1;
  varg1 = Data_Wrap_Struct(cRegion, 0, 0, region);
  rb_funcall(myself, rb_intern("ConstrainClippingRegion"), 1, varg1);
}


void
View::SetDrawingMode(drawing_mode mode)
{
  VALUE varg1;
  varg1 = ULONG2NUM(mode);
  rb_funcall(myself, rb_intern("SetDrawingMode"), 1, varg1);
}


void
View::SetPenSize(float size)
{
  VALUE varg1;
  varg1 = rb_float_new(size);
  rb_funcall(myself, rb_intern("SetPenSize"), 1, varg1);
}


void
View::SetViewColor(rgb_color c)
{
  VALUE varg1, varg2, varg3, varg4;

  varg1 = UCHR2FIX(c.red);
  varg2 = UCHR2FIX(c.green);
  varg3 = UCHR2FIX(c.blue);
  varg4 = UCHR2FIX(c.alpha);
  rb_funcall(myself, rb_intern("SetViewColor"), 4, varg1, varg2, varg3, varg4);
}


void
View::SetHighColor(rgb_color a_color)
{
  VALUE varg1, varg2, varg3, varg4;

  varg1 = UCHR2FIX(a_color.red);
  varg2 = UCHR2FIX(a_color.green);
  varg3 = UCHR2FIX(a_color.blue);
  varg4 = UCHR2FIX(a_color.alpha);
  rb_funcall(myself, rb_intern("SetHighColor"), 4, varg1, varg2, varg3, varg4);
}


void
View::SetLowColor(rgb_color a_color)
{
  VALUE varg1, varg2, varg3, varg4;

  varg1 = UCHR2FIX(a_color.red);
  varg2 = UCHR2FIX(a_color.green);
  varg3 = UCHR2FIX(a_color.blue);
  varg4 = UCHR2FIX(a_color.alpha);
  rb_funcall(myself, rb_intern("SetLowColor"), 4, varg1, varg2, varg3, varg4);
}


void
View::SetFont(const BFont *font,
              uint32 mask=B_FONT_ALL)
{
  VALUE varg1, varg2;
  varg1 = Data_Wrap_Struct(cFont, 0, 0, const_cast<BFont *>(font));
  varg2 = ULONG2NUM(mask);
  rb_funcall(myself, rb_intern("SetFont"), 2, varg1, varg2);
}


void
View::SetFlags(uint32 flags)
{
  VALUE varg1;
  varg1 = ULONG2NUM(flags);
  rb_funcall(myself, rb_intern("SetFlags"), 1, varg1);
}


void
View::SetResizingMode(uint32 mode)
{
  VALUE varg1;
  varg1 = ULONG2NUM(mode);
  rb_funcall(myself, rb_intern("SetResizingMode"), 1, varg1);
}


void
View::ScrollTo(BPoint where)
{
  VALUE varg1;
  varg1 = Data_Wrap_Struct(cPoint, 0, 0, &where);
  rb_funcall(myself, rb_intern("ScrollTo"), 1, varg1);
}


void
View::MakeFocus(bool focusState=true)
{
  VALUE varg1;
  varg1 = (focusState ? Qtrue:Qfalse);
  rb_funcall(myself, rb_intern("MakeFocus"), 1, varg1);
}


void
View::Show()
{
  rb_funcall2(myself, rb_intern("Show"), 0, 0);
}


void
View::Hide()
{
  rb_funcall2(myself, rb_intern("Hide"), 0, 0);
}


void
View::GetPreferredSize(float *width,
                       float *height)
{
  VALUE varg1, varg2, vresult;
  varg1 = rb_float_new(*width);
  varg2 = rb_float_new(*height);
  vresult = rb_funcall(myself, rb_intern("GetPreferredSize"), 2, varg1, varg2);

  Check_Type(vresult, T_ARRAY);
  Check_Is_Kind_Of(RARRAY(vresult)->ptr[0], rb_cNumeric);
  Check_Is_Kind_Of(RARRAY(vresult)->ptr[1], rb_cNumeric);

  *width = NUM2DBL(RARRAY(vresult)->ptr[0]);
  *height = NUM2DBL(RARRAY(vresult)->ptr[1]);
}


void
View::ResizeToPreferred()
{
  rb_funcall2(myself, rb_intern("ResizeToPreferred"), 0, 0);
}


BHandler*
View::ResolveSpecifier(BMessage *msg,
                       int32 index,
                       BMessage *specifier,
                       int32 form,
                       const char *property)
{
  VALUE varg1, varg2, varg3, varg4, varg5, vresult;
  BHandler *result;

  varg1 = Data_Wrap_Struct(cMessage, 0, 0, msg);
  varg2 = LONG2NUM(index);
  varg3 = Data_Wrap_Struct(cMessage, 0, 0, specifier);
  varg4 = LONG2NUM(form);
  varg5 = rb_str_new2(property);

  vresult = rb_funcall(myself, rb_intern("ResolveSpecifier"), 5,
                       varg1,
                       varg2,
                       varg3,
                       varg4,
                       varg5);
  Get_BInstance(Handler, vresult, result);
  return result;
}


status_t
View::GetSupportedSuites(BMessage *data)
{
  VALUE varg1, vresult;
  status_t result = B_OK;

  varg1 = Data_Wrap_Struct(cMessage, 0, 0, data);
  vresult = rb_funcall(myself, rb_intern("GetSupportedSuites"), 1, varg1);
  if (!NIL_P(vresult)) {
    result = NUM2LONG(vresult);
  }
  return result;
}


void*
View::operator new(size_t size)
{
  return xmalloc(size);
}


void
View::operator delete(void *ptr)
{
  free(ptr);
}
