/*
 *  rbWindow.cc
 */

#include <Application.h>
#include "rbWindow.h"
#include "my_be.h"


Window::Window(BRect frame,
               const char *title,
               window_type type,
               uint32 flags,
               uint32 workspace=B_CURRENT_WORKSPACE)
  : BWindow(frame, title, type, flags, workspace)
{
  myself = Qnil;
  quit = false;
}


Window::Window(BRect frame,
               const char *title,
               window_look look,
               window_feel feel,
               uint32 flags,
               uint32 workspace=B_CURRENT_WORKSPACE)
 : BWindow(frame, title, look, feel, flags, workspace)
{
  myself = Qnil;
  quit = false;
}


Window::Window(BMessage *data)
 : BWindow(data)
{
  myself = Qnil;
}


void
Window::Quit()
{
  if (find_thread(NULL) == this->Thread()) {
    quit = true;
    BWindow::Quit();
    /* not reached */
  }
  else {
    BWindow::Quit();
  }
}


void
Window::DispatchMessage(BMessage *message, BHandler *handler)
{
  switch (message->what) {
    case _QUIT_:
      BWindow::DispatchMessage(message, handler);
      break;

    default:
      this->Unlock();
      if (be_app->Lock() && this->Lock()) {
        VALUE varg1, varg2;

        varg1 = Data_Wrap_Struct(cMessage, 0, 0, message);
        varg2 = Data_Wrap_Struct(cHandler, 0, 0, handler);

        rb_funcall(myself, id_DispatchMessage, 2, varg1, varg2);
        be_app->Unlock();
      }
  }
}


void
Window::MessageReceived(BMessage *message)
{
  this->Unlock();
  if (be_app->Lock() && this->Lock()) {
    VALUE varg1 = Data_Wrap_Struct(cMessage, 0, 0, message);
    rb_funcall(myself, id_MessageReceived, 1, varg1);
    be_app->Unlock();
  }
}


void
Window::FrameMoved(BPoint new_position)
{
  this->Unlock();
  if (be_app->Lock() && this->Lock()) {
    VALUE varg1 = Data_Wrap_Struct(cPoint, 0, 0, &new_position);
    rb_funcall(myself, rb_intern("FrameMoved"), 1, varg1);
    be_app->Unlock();
  }
}


void
Window::WorkspacesChanged(uint32 old_ws,
                          uint32 new_ws)
{
  this->Unlock();
  if (be_app->Lock() && this->Lock()) {
    VALUE varg1, varg2;
    varg1 = ULONG2NUM(old_ws);
    varg2 = ULONG2NUM(new_ws);
    rb_funcall(myself, rb_intern("WorkspacesChanged"), 2, varg1, varg2);
    be_app->Unlock();
  }
}


void
Window::WorkspaceActivated(int32 ws,
                           bool state)
{
  this->Unlock();
  if (be_app->Lock() && this->Lock()) {
    VALUE varg1, varg2;
    varg1 = LONG2NUM(ws);
    varg2 = (state ? Qtrue:Qfalse);
    rb_funcall(myself, rb_intern("WorkspaceActivated"), 2, varg1, varg2);
    be_app->Unlock();
  }
}


void
Window::FrameResized(float new_width,
                     float new_height)
{
  this->Unlock();
  if (be_app->Lock() && this->Lock()) {
    VALUE varg1, varg2;
    varg1 = rb_float_new(new_width);
    varg2 = rb_float_new(new_height);
    rb_funcall(myself, rb_intern("FrameResized"), 2, varg1, varg2);
    be_app->Unlock();
  }
}


void
Window::Minimize(bool minimize)
{
  this->Unlock();
  if (be_app->Lock() && this->Lock()) {
    VALUE varg1;
    varg1 = (minimize ? Qtrue:Qfalse);
    rb_funcall(myself, rb_intern("Minimize"), 1, varg1);
    be_app->Unlock();
  }
}


void
Window::Zoom(BPoint rec_position,
             float rec_width,
             float rec_height)
{
  this->Unlock();
  if (be_app->Lock() && this->Lock()) {
    VALUE varg1, varg2, varg3;
    varg1 = Data_Wrap_Struct(cPoint, 0, 0, &rec_position);
    varg2 = rb_float_new(rec_width);
    varg3 = rb_float_new(rec_height);
    rb_funcall(myself, rb_intern("Zoom"), 3, varg1, varg2, varg3);
    be_app->Unlock();
  }
}


void
Window::ScreenChanged(BRect screen_size,
                      color_space depth)
{
  this->Unlock();
  if (be_app->Lock() && this->Lock()) {
    VALUE varg1, varg2;
    varg1 = Data_Wrap_Struct(cRect, 0, 0, &screen_size);
    varg2 = ULONG2NUM(depth);
    rb_funcall(myself, rb_intern("ScreenChanged"), 2, varg1, varg2);
    be_app->Unlock();
  }
}


void
Window::MenusBeginning()
{
  this->Unlock();
  if (be_app->Lock() && this->Lock()) {
    rb_funcall2(myself, rb_intern("MenusBeginning"), 0, 0);
    be_app->Unlock();
  }
}


void
Window::MenusEnded()
{
  this->Unlock();
  if (be_app->Lock() && this->Lock()) {
    rb_funcall2(myself, rb_intern("MenusEnded"), 0, 0);
    be_app->Unlock();
  }
}


void
Window::WindowActivated(bool state)
{
  this->Unlock();
  if (be_app->Lock() && this->Lock()) {
    VALUE varg1;
    varg1 = (state ? Qtrue:Qfalse);
    rb_funcall(myself, rb_intern("WindowActivated"), 1, varg1);
    be_app->Unlock();
  }
}


void
Window::Show()
{
  this->Unlock();
  if (be_app->Lock() && this->Lock()) {
    rb_funcall2(myself, rb_intern("Show"), 0, 0);
    be_app->Unlock();
  }
}


void
Window::Hide()
{
  if (find_thread(NULL) == this->Thread()) {
    if (quit) {
      while (be_app->IsLocked()) {
        be_app->Unlock();
      }
      BWindow::Hide();
    }
    else {
      this->Unlock();
      if (be_app->Lock() && this->Lock()) {
        rb_funcall2(myself, rb_intern("Hide"), 0, 0);
        be_app->Unlock();
      }
    }
  }
  else {
    rb_funcall2(myself, rb_intern("Hide"), 0, 0);
  }
}


BHandler*
Window::ResolveSpecifier(BMessage *msg,
                         int32 index,
                         BMessage *specifier,
                         int32 form,
                         const char *property)
{
  VALUE varg1, varg2, varg3, varg4, varg5, vresult;
  BHandler *result;

  varg1 = Data_Wrap_Struct(cMessage, 0, 0, msg);
  varg2 = LONG2NUM(index);
  varg3 = Data_Wrap_Struct(cMessage, 0, 0, specifier);
  varg4 = LONG2NUM(form);
  varg5 = rb_str_new2(property);

  vresult = rb_funcall(myself, rb_intern("ResolveSpecifier"), 5,
                       varg1,
                       varg2,
                       varg3,
                       varg4,
                       varg5);
  Get_BInstance(Handler, vresult, result);
  return result;
}


status_t
Window::GetSupportedSuites(BMessage *data)
{
  VALUE varg1;

  varg1 = Data_Wrap_Struct(cMessage, 0, 0, data);
  rb_funcall(myself, rb_intern("GetSupportedSuites"), 1, varg1);
  return B_OK;
}


bool
Window::QuitRequested()
{
  bool result = true;

  if (find_thread(NULL) == this->Thread()) {
    this->Unlock();
    if (be_app->Lock() && this->Lock()) {
      VALUE vresult = rb_funcall2(myself, rb_intern("QuitRequested"), 0, 0);
      result = RTEST(vresult);
      be_app->Unlock();
    }
  }
  else {
    VALUE vresult = rb_funcall2(myself, rb_intern("QuitRequested"), 0, 0);
    result = RTEST(vresult);
  }
  return result;
}


thread_id
Window::Run()
{
  VALUE vresult;
  thread_id result;

  vresult = rb_funcall(myself, rb_intern("Run"), 0, 0);
  result = NUM2LONG(vresult);
  my_add_relative(my_app, myself);

  return result;
}


void*
Window::operator new(size_t size)
{
  return xmalloc(size);
}


void
Window::operator delete(void *ptr)
{
  free(ptr);
}
