#! /boot/home/config/bin/ruby -d

#
# DoubleClick.rb
#

require "be.so"
include Be


signature = "application/x-vnd.Be.DoubleClick"


class DoubleClickView < View
  def initialize(frame, name, resizeMask, flags)
    super(frame, name, resizeMask, flags)

    @LastButton = 0
    @ClickCount = 0
    @Text = "No clicks so far"
  end

  def Draw(updateRect)
    DrawString(@Text, Point.new(20, 30))
  end

  def MouseDown(where)
    msg = Window().CurrentMessage

    button = msg.FindInt32("buttons")

    case button
      when Be::SECONDARY_MOUSE_BUTTON
        buttonName = "Secondary button"
      when Be::TERTIARY_MOUSE_BUTTON
        buttonName = "Tertiary button"
      else
        buttonName = "Primary button"
    end

    clicks = msg.FindInt32("clicks")

    if (button == @LastButton) and (clicks > 1)
      @ClickCount += 1
    else
      @ClickCount = 1
    end

    case @ClickCount
      when 1
        clickName = "single click"
      when 2
        clickName = "double click"
      when 3
        clickName = "triple click"
      else
        clickName = "many clicks!"
    end

    @LastButton = button
    @Text = "#{buttonName}; #{clickName}"

    Invalidate()
  end

end


class DoubleClickWin < Be::Window
  def initialize
    super(Rect.new(50, 50, 250, 100), "DoubleClick", TITLED_WINDOW,
          NOT_ZOOMABLE | NOT_MINIMIZABLE | NOT_RESIZABLE | ASYNCHRONOUS_CONTROLS)

    view = DoubleClickView.new(Bounds(), "DoubleClickView",
                               FOLLOW_NONE, WILL_DRAW)
    AddChild(view)
  end

  def QuitRequested()
    $be_app.PostMessage(QUIT_REQUESTED)
    return true
  end
end


class DoubleClickApp < Application
  def ReadyToRun
    win = DoubleClickWin.new
    win.Show
  end
end


DoubleClickApp.new(signature)
$be_app.Run
