#! /boot/home/config/bin/ruby -d

#
# messageworld.rb
#

require "to_integer.rb"
require "be.so"
include Be


# Application's signature
APP_SIGNATURE         = "application/x-vnd.Be-MessageWorld"

# Messages for window registry with application
WINDOW_REGISTRY_ADD   = "WRad".to_integer
WINDOW_REGISTRY_SUB   = "WRsb".to_integer
WINDOW_REGISTRY_ADDED = "WRdd".to_integer

# Messages for menu commands
MENU_FILE_NEW         = "MFnw".to_integer
MENU_FILE_OPEN        = "MFop".to_integer
MENU_FILE_CLOSE       = "MFcl".to_integer
MENU_FILE_SAVE        = "MFsv".to_integer
MENU_FILE_SAVEAS      = "MFsa".to_integer
MENU_FILE_PAGESETUP   = "MFps".to_integer
MENU_FILE_PRINT       = "MFpr".to_integer
MENU_FILE_QUIT        = "MFqu".to_integer

MENU_OPT_HELLO        = "MOhl".to_integer

STRING_HELLO          = "Hello World!"
STRING_GOODBYE        = "Goodbye World!"


class HelloView < View
  def initialize(frame)
    super(frame, "HelloView", FOLLOW_ALL_SIDES, WILL_DRAW)

    @message = ""
    SetString(STRING_HELLO)
  end

  def Draw(updateRect)
    MovePenTo(Point.new(20, 75))
    DrawString(@message)
  end

  def SetString(s)
    @message = s
  end
end

class HelloWindow < Window
  def initialize(frame)
    super(frame, "Untitled ", TITLED_WINDOW, NOT_RESIZABLE | NOT_ZOOMABLE)

    # Add the menu bar

    r = Bounds()
    @menubar = MenuBar.new(r, "menu_bar")
    AddChild(@menubar)

    # Add File menu to menu bar

    menu = Menu.new("File")
    menu.AddItem(MenuItem.new("New", Message.new(MENU_FILE_NEW), ?N))
    menu.AddItem(MenuItem.new("Open" + UTF8_ELLIPSIS, Message.new(MENU_FILE_OPEN), ?O))
    menu.AddItem(MenuItem.new("Close", Message.new(MENU_FILE_CLOSE), ?W))
    menu.AddSeparatorItem

    menu.AddItem(MenuItem.new("Save", Message.new(MENU_FILE_SAVE), ?S))
    menu.AddItem(MenuItem.new("Save as" + UTF8_ELLIPSIS, Message.new(MENU_FILE_SAVEAS)))
    menu.AddSeparatorItem

    menu.AddItem(MenuItem.new("Page Setup" + UTF8_ELLIPSIS, Message.new(MENU_FILE_PAGESETUP)))
    menu.AddItem(MenuItem.new("Print" + UTF8_ELLIPSIS, Message.new(MENU_FILE_PRINT), ?P))
    menu.AddSeparatorItem

    menu.AddItem(MenuItem.new("Quit", Message.new(MENU_FILE_QUIT), ?Q))
    @menubar.AddItem(menu)

    # Add Options menu to menu bar

    menu = Menu.new("Options")
    item = MenuItem.new("Say Hello", Message.new(MENU_OPT_HELLO))
    item.SetMarked(true)
    menu.AddItem(item)
    @menubar.AddItem(menu)

    # Add the drawing view

    r.top = @menubar.Bounds().bottom + 1
    AddChild(@helloview = HelloView.new(r))

    # Tell the application that there's one more window
    # and get the number for this untitled window.

    Register(true)
    Show()
  end

  def QuitRequested
    Unregister()
    return true
  end

  def MessageReceived(message)
    case message.what
      when WINDOW_REGISTRY_ADDED
        id = message.FindInt32("new_window_number")
        SetTitle("Untitled #{id.to_s}")

      when MENU_FILE_NEW
        r = Frame()
        r.OffsetBy(20, 20)
        HelloWindow.new(r)

      when MENU_FILE_CLOSE
        PostMessage(QUIT_REQUESTED)

      when MENU_FILE_QUIT
        $be_app.PostMessage(QUIT_REQUESTED)

      when MENU_OPT_HELLO
        item = message.FindPointer("source", MenuItem)
        if item.IsMarked
          s = STRING_GOODBYE
          mark = false
        else
          s = STRING_HELLO
          mark = true
        end
        @helloview.SetString(s)
        item.SetMarked(mark)
        @helloview.Invalidate

      else
        super(message)
    end
  end

  def Register(need_id)
    messenger = Messenger.new(APP_SIGNATURE)
    message = Message.new(WINDOW_REGISTRY_ADD)

    message.AddBool("need_id", need_id)
    messenger.SendMessage(message, self)
  end
  private :Register

  def Unregister
    messenger = Messenger.new(APP_SIGNATURE)
    messenger.SendMessage(Message.new(WINDOW_REGISTRY_SUB))
  end
  private :Unregister
end

class HelloApp < Application
  def initialize
    super(APP_SIGNATURE)

    windowRect = Rect.new(50, 50, 349, 399)
    @window_count = 0          # No windows yet
    @next_untitled_number = 1  # Next window is "Untitled 1"

    HelloWindow.new(windowRect)
  end

  def MessageReceived(message)
    case message.what
      when WINDOW_REGISTRY_ADD
        need_id = message.FindBool("need_id")
        if need_id
          reply = Message.new(WINDOW_REGISTRY_ADDED)

          reply.AddInt32("new_window_number", @next_untitled_number)
          message.SendReply(reply)
          @next_untitled_number += 1
        end
        @window_count += 1

      when WINDOW_REGISTRY_SUB
        @window_count -= 1
        if @window_count <= 0
          Quit()
        end

      else
        super(message)
    end
  end
end


HelloApp.new
$be_app.Run
