/*
 *  scheduler.cc
 */

#include <scheduler.h>
#include "my_be.h"


static VALUE
rb_suggest_thread_priority(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1, varg2, varg3, varg4;
  int32 result;
  VALUE vresult;

  switch (rb_scan_args(argc, argv, "04", &varg1, &varg2, &varg3, &varg4)) {
    case 0:
      result = suggest_thread_priority();
      break;

    case 1:
      {
        uint32 arg1;

        arg1 = NUM2ULONG(varg1);
        result = suggest_thread_priority(arg1);
      }
      break;

    case 2:
      {
        uint32 arg1;
        int32 arg2;

        arg1 = NUM2ULONG(varg1);
        arg2 = NUM2LONG(varg2);
        result = suggest_thread_priority(arg1, arg2);
      }
      break;

    case 3:
      {
        uint32 arg1;
        int32 arg2;
        bigtime_t arg3;

        arg1 = NUM2ULONG(varg1);
        arg2 = NUM2LONG(varg2);
        arg3 = NUM2LLONG(varg3);
        result = suggest_thread_priority(arg1, arg2, arg3);
      }
      break;

    case 4:
      {
        uint32 arg1;
        int32 arg2;
        bigtime_t arg3, arg4;

        arg1 = NUM2ULONG(varg1);
        arg2 = NUM2LONG(varg2);
        arg3 = NUM2LLONG(varg3);
        arg4 = NUM2LLONG(varg4);
        result = suggest_thread_priority(arg1, arg2, arg3, arg4);
      }
      break;

    default:
      rb_bug("rb_suggest_thread_priority");
  }
  vresult = LONG2NUM(result);
  return vresult;
}


static VALUE
rb_estimate_max_scheduling_latency(int argc, VALUE *argv, VALUE self)
{
  VALUE varg1;
  bigtime_t result;
  VALUE vresult;

  switch (rb_scan_args(argc, argv, "01", &varg1)) {
    case 0:
      result = estimate_max_scheduling_latency();
      break;

    case 1:
      {
        thread_id arg1;

        arg1 = NUM2LONG(varg1);
        result = estimate_max_scheduling_latency(arg1);
      }
      break;

    default:
      rb_bug("rb_estimate_max_scheduling_latency");
  }
  vresult = LLONG2NUM(result);
  return vresult;
}


void
Init_scheduler()
{
  rb_define_module_function(mBe, "suggest_thread_priority", RUBY_METHOD_FUNC(rb_suggest_thread_priority), -1);
  rb_define_module_function(mBe, "estimate_max_scheduling_latency", RUBY_METHOD_FUNC(rb_estimate_max_scheduling_latency), -1);

  rb_define_const(mBe, "DEFAULT_MEDIA_PRIORITY", ULONG2NUM(B_DEFAULT_MEDIA_PRIORITY));
  rb_define_const(mBe, "OFFLINE_PROCESSING", ULONG2NUM(B_OFFLINE_PROCESSING));
  rb_define_const(mBe, "STATUS_RENDERING", ULONG2NUM(B_STATUS_RENDERING));
  rb_define_const(mBe, "USER_INPUT_HANDLING", ULONG2NUM(B_USER_INPUT_HANDLING));
  rb_define_const(mBe, "LIVE_VIDEO_MANIPULATION", ULONG2NUM(B_LIVE_VIDEO_MANIPULATION));
  rb_define_const(mBe, "VIDEO_PLAYBACK", ULONG2NUM(B_VIDEO_PLAYBACK));
  rb_define_const(mBe, "VIDEO_RECORDING", ULONG2NUM(B_VIDEO_RECORDING));
  rb_define_const(mBe, "LIVE_AUDIO_MANIPULATION", ULONG2NUM(B_LIVE_AUDIO_MANIPULATION));
  rb_define_const(mBe, "AUDIO_PLAYBACK", ULONG2NUM(B_AUDIO_PLAYBACK));
  rb_define_const(mBe, "AUDIO_RECORDING", ULONG2NUM(B_AUDIO_RECORDING));
  rb_define_const(mBe, "LIVE_3D_RENDERING", ULONG2NUM(B_LIVE_3D_RENDERING));
  rb_define_const(mBe, "NUMBER_CRUNCHING", ULONG2NUM(B_NUMBER_CRUNCHING));
  rb_define_const(mBe, "MIDI_PROCESSING", ULONG2NUM(B_MIDI_PROCESSING));
}
