/*
 *  TestNumeric.cc
 */

#include "../../my_be.h"
#include "../../my_bignum.cc"
#include "./rb_assert.cc"

VALUE cNumTest;


/*

// -- BeOS
  #define SCHAR_MIN               (-127-1)
  #define SCHAR_MAX               (127)
  #define UCHAR_MAX               (255U)

  #define SHRT_MIN                (-32767-1)
  #define SHRT_MAX                (32767)
  #define USHRT_MAX               (65535U)

  #define LONG_MIN                (-2147483647L-1)
  #define LONG_MAX                (2147483647L)
  #define ULONG_MAX               (4294967295U)

  #define LONGLONG_MIN    (-9223372036854775807LL - 1)  // these are Be specific
  #define LONGLONG_MAX    (9223372036854775807LL)
  #define ULONGLONG_MAX   (0xffffffffffffffffULL)

// -- Ruby
  #define FIXNUM_MAX (LONG_MAX>>1)
  #define FIXNUM_MIN RSHIFT((long)LONG_MIN,1)


  CHR2FIX(n);
  UCHR2FIX(n);

  LONG2NUM(n);
  ULONG2NUM(n);

  LLONG2NUM(n);
  ULLONG2NUM(n);

  Check_Range_Long(x);
  Check_Range_Ulong(x);

  Check_Range_Llong(x);
  Check_Range_Ullong(x);

  NUM2CHR(x);
  NUM2UCHR(x);

  NUM2SHRT(x);
  NUM2USHRT(x);

  NUM2LONG(x);
  NUM2ULONG(x);

  NUM2LLONG(x);
  NUM2ULLONG(x);

  Check_Range_Char(x);
  Check_Range_Uchar(x);
*/



// -- setup & teardown
static VALUE
setup(VALUE self)
{
  return Qnil;
}
static VALUE
teardown(VALUE self)
{
  return Qnil;
}



// -- CHR2FIX
static VALUE
test_000_CHR2FIX(VALUE self)
{
  VALUE val;

  val = CHR2FIX(SCHAR_MIN);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("-128"));
  return Qnil;
}
static VALUE
test_001_CHR2FIX(VALUE self)
{
  VALUE val;

  val = CHR2FIX(SCHAR_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("127"));
  return Qnil;
}


// -- UCHR2FIX
static VALUE
test_010_UCHR2FIX(VALUE self)
{
  VALUE val;

  val = UCHR2FIX(SCHAR_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("127"));
  return Qnil;
}
static VALUE
test_011_UCHR2FIX(VALUE self)
{
  VALUE val;

  val = UCHR2FIX(UCHAR_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("255"));
  return Qnil;
}


// -- LONG2NUM
static VALUE
test_020_LONG2NUM(VALUE self)
{
  VALUE val;

  val = LONG2NUM(SCHAR_MIN);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("-128"));
  return Qnil;
}
static VALUE
test_021_LONG2NUM(VALUE self)
{
  VALUE val;

  val = LONG2NUM(SCHAR_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("127"));
  return Qnil;
}
static VALUE
test_022_LONG2NUM(VALUE self)
{
  VALUE val;

  val = LONG2NUM(UCHAR_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("255"));
  return Qnil;
}
static VALUE
test_023_LONG2NUM(VALUE self)
{
  VALUE val;

  val = LONG2NUM(SHRT_MIN);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("-32768"));
  return Qnil;
}
static VALUE
test_024_LONG2NUM(VALUE self)
{
  VALUE val;

  val = LONG2NUM(SHRT_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("32767"));
  return Qnil;
}
static VALUE
test_025_LONG2NUM(VALUE self)
{
  VALUE val;

  val = LONG2NUM(USHRT_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("65535"));
  return Qnil;
}
static VALUE
test_026_LONG2NUM(VALUE self)
{
  VALUE val;

  val = LONG2NUM(FIXNUM_MIN);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("-1073741824"));
  return Qnil;
}
static VALUE
test_027_LONG2NUM(VALUE self)
{
  VALUE val;

  val = LONG2NUM(FIXNUM_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("1073741823"));
  return Qnil;
}
static VALUE
test_028_LONG2NUM(VALUE self)
{
  VALUE val;

  val = LONG2NUM(LONG_MIN);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("-2147483648"));
  return Qnil;
}
static VALUE
test_029_LONG2NUM(VALUE self)
{
  VALUE val;

  val = LONG2NUM(LONG_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("2147483647"));
  return Qnil;
}


// -- ULONG2NUM
static VALUE
test_030_ULONG2NUM(VALUE self)
{
  VALUE val;

  val = ULONG2NUM(SCHAR_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("127"));
  return Qnil;
}
static VALUE
test_031_ULONG2NUM(VALUE self)
{
  VALUE val;

  val = ULONG2NUM(UCHAR_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("255"));
  return Qnil;
}
static VALUE
test_032_ULONG2NUM(VALUE self)
{
  VALUE val;

  val = ULONG2NUM(SHRT_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("32767"));
  return Qnil;
}
static VALUE
test_033_ULONG2NUM(VALUE self)
{
  VALUE val;

  val = ULONG2NUM(USHRT_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("65535"));
  return Qnil;
}
static VALUE
test_034_ULONG2NUM(VALUE self)
{
  VALUE val;

  val = ULONG2NUM(FIXNUM_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("1073741823"));
  return Qnil;
}
static VALUE
test_035_ULONG2NUM(VALUE self)
{
  VALUE val;

  val = ULONG2NUM(LONG_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("2147483647"));
  return Qnil;
}
static VALUE
test_036_ULONG2NUM(VALUE self)
{
  VALUE val;

  val = ULONG2NUM(ULONG_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("4294967295"));
  return Qnil;
}


// -- LLONG2NUM
static VALUE
test_040_LLONG2NUM(VALUE self)
{
  VALUE val;

  val = LLONG2NUM(SCHAR_MIN);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("-128"));
  return Qnil;
}
static VALUE
test_041_LLONG2NUM(VALUE self)
{
  VALUE val;

  val = LLONG2NUM(SCHAR_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("127"));
  return Qnil;
}
static VALUE
test_042_LLONG2NUM(VALUE self)
{
  VALUE val;

  val = LLONG2NUM(UCHAR_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("255"));
  return Qnil;
}
static VALUE
test_043_LLONG2NUM(VALUE self)
{
  VALUE val;

  val = LLONG2NUM(SHRT_MIN);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("-32768"));
  return Qnil;
}
static VALUE
test_044_LLONG2NUM(VALUE self)
{
  VALUE val;

  val = LLONG2NUM(SHRT_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("32767"));
  return Qnil;
}
static VALUE
test_045_LLONG2NUM(VALUE self)
{
  VALUE val;

  val = LLONG2NUM(USHRT_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("65535"));
  return Qnil;
}
static VALUE
test_046_LLONG2NUM(VALUE self)
{
  VALUE val;

  val = LLONG2NUM(FIXNUM_MIN);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("-1073741824"));
  return Qnil;
}
static VALUE
test_047_LLONG2NUM(VALUE self)
{
  VALUE val;

  val = LLONG2NUM(FIXNUM_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("1073741823"));
  return Qnil;
}
static VALUE
test_048_LLONG2NUM(VALUE self)
{
  VALUE val;

  val = LLONG2NUM(LONG_MIN);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("-2147483648"));
  return Qnil;
}
static VALUE
test_049_LLONG2NUM(VALUE self)
{
  VALUE val;

  val = LLONG2NUM(LONG_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("2147483647"));
  return Qnil;
}
static VALUE
test_04a_LLONG2NUM(VALUE self)
{
  VALUE val;

  val = LLONG2NUM(ULONG_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("4294967295"));
  return Qnil;
}
static VALUE
test_04b_LLONG2NUM(VALUE self)
{
  VALUE val;

  val = LLONG2NUM(LONGLONG_MIN);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("-9223372036854775808"));
  return Qnil;
}
static VALUE
test_04c_LLONG2NUM(VALUE self)
{
  VALUE val;

  val = LLONG2NUM(LONGLONG_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("9223372036854775807"));
  return Qnil;
}


// -- ULLONG2NUM
static VALUE
test_050_ULLONG2NUM(VALUE self)
{
  VALUE val;

  val = ULLONG2NUM(SCHAR_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("127"));
  return Qnil;
}
static VALUE
test_051_ULLONG2NUM(VALUE self)
{
  VALUE val;

  val = ULLONG2NUM(UCHAR_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("255"));
  return Qnil;
}
static VALUE
test_052_ULLONG2NUM(VALUE self)
{
  VALUE val;

  val = ULLONG2NUM(SHRT_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("32767"));
  return Qnil;
}
static VALUE
test_053_ULLONG2NUM(VALUE self)
{
  VALUE val;

  val = ULLONG2NUM(USHRT_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("65535"));
  return Qnil;
}
static VALUE
test_054_ULLONG2NUM(VALUE self)
{
  VALUE val;

  val = ULLONG2NUM(FIXNUM_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("1073741823"));
  return Qnil;
}
static VALUE
test_055_ULLONG2NUM(VALUE self)
{
  VALUE val;

  val = ULLONG2NUM(LONG_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("2147483647"));
  return Qnil;
}
static VALUE
test_056_ULLONG2NUM(VALUE self)
{
  VALUE val;

  val = ULLONG2NUM(ULONG_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("4294967295"));
  return Qnil;
}
static VALUE
test_057_ULLONG2NUM(VALUE self)
{
  VALUE val;

  val = ULLONG2NUM(LONGLONG_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("9223372036854775807"));
  return Qnil;
}
static VALUE
test_058_ULLONG2NUM(VALUE self)
{
  VALUE val;

  val = ULLONG2NUM(ULONGLONG_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("18446744073709551615"));
  return Qnil;
}


// -- Check_Range_Long
static VALUE
test_060_RangeCheck_Long(VALUE self)
{
  VALUE val;

  val = LONG2NUM(SCHAR_MIN);
  Check_Range_Long(val);
  return Qnil;
}
static VALUE
test_061_RangeCheck_Long(VALUE self)
{
  VALUE val;

  val = LONG2NUM(SCHAR_MAX);
  Check_Range_Long(val);
  return Qnil;
}
static VALUE
test_062_RangeCheck_Long(VALUE self)
{
  VALUE val;

  val = ULONG2NUM(UCHAR_MAX);
  Check_Range_Long(val);
  return Qnil;
}
static VALUE
test_063_RangeCheck_Long(VALUE self)
{
  VALUE val;

  val = LONG2NUM(SHRT_MIN);
  Check_Range_Long(val);
  return Qnil;
}
static VALUE
test_064_RangeCheck_Long(VALUE self)
{
  VALUE val;

  val = LONG2NUM(SHRT_MAX);
  Check_Range_Long(val);
  return Qnil;
}
static VALUE
test_065_RangeCheck_Long(VALUE self)
{
  VALUE val;

  val = ULONG2NUM(USHRT_MAX);
  Check_Range_Long(val);
  return Qnil;
}
static VALUE
test_066_RangeCheck_Long(VALUE self)
{
  VALUE val;

  val = LONG2NUM(FIXNUM_MIN);
  Check_Range_Long(val);
  return Qnil;
}
static VALUE
test_067_RangeCheck_Long(VALUE self)
{
  VALUE val;

  val = LONG2NUM(FIXNUM_MAX);
  Check_Range_Long(val);
  return Qnil;
}
static VALUE
test_068_RangeCheck_Long(VALUE self)
{
  VALUE val;

  val = LONG2NUM(LONG_MIN);
  Check_Range_Long(val);
  return Qnil;
}
static VALUE
test_069_RangeCheck_Long(VALUE self)
{
  VALUE val;

  val = LONG2NUM(LONG_MAX);
  Check_Range_Long(val);
  return Qnil;
}


// -- Check_Range_Ulong
static VALUE
test_070_RangeCheck_Ulong(VALUE self)
{
  VALUE val;

  val = LONG2NUM(SCHAR_MAX);
  Check_Range_Ulong(val);
  return Qnil;
}
static VALUE
test_071_RangeCheck_Ulong(VALUE self)
{
  VALUE val;

  val = ULONG2NUM(UCHAR_MAX);
  Check_Range_Ulong(val);
  return Qnil;
}
static VALUE
test_072_RangeCheck_Ulong(VALUE self)
{
  VALUE val;

  val = LONG2NUM(SHRT_MAX);
  Check_Range_Ulong(val);
  return Qnil;
}
static VALUE
test_073_RangeCheck_Ulong(VALUE self)
{
  VALUE val;

  val = ULONG2NUM(USHRT_MAX);
  Check_Range_Ulong(val);
  return Qnil;
}
static VALUE
test_074_RangeCheck_Ulong(VALUE self)
{
  VALUE val;

  val = LONG2NUM(FIXNUM_MAX);
  Check_Range_Ulong(val);
  return Qnil;
}
static VALUE
test_075_RangeCheck_Ulong(VALUE self)
{
  VALUE val;

  val = LONG2NUM(LONG_MAX);
  Check_Range_Ulong(val);
  return Qnil;
}
static VALUE
test_076_RangeCheck_Ulong(VALUE self)
{
  VALUE val;

  val = ULONG2NUM(ULONG_MAX);
  Check_Range_Ulong(val);
  return Qnil;
}


// -- Check_Range_Long
static VALUE
test_080_RangeCheck_Llong(VALUE self)
{
  VALUE val;

  val = LONG2NUM(SCHAR_MIN);
  Check_Range_Llong(val);
  return Qnil;
}
static VALUE
test_081_RangeCheck_Llong(VALUE self)
{
  VALUE val;

  val = LONG2NUM(SCHAR_MAX);
  Check_Range_Llong(val);
  return Qnil;
}
static VALUE
test_082_RangeCheck_Llong(VALUE self)
{
  VALUE val;

  val = ULONG2NUM(UCHAR_MAX);
  Check_Range_Llong(val);
  return Qnil;
}
static VALUE
test_083_RangeCheck_Llong(VALUE self)
{
  VALUE val;

  val = LONG2NUM(SHRT_MIN);
  Check_Range_Llong(val);
  return Qnil;
}
static VALUE
test_084_RangeCheck_Llong(VALUE self)
{
  VALUE val;

  val = LONG2NUM(SHRT_MAX);
  Check_Range_Llong(val);
  return Qnil;
}
static VALUE
test_085_RangeCheck_Llong(VALUE self)
{
  VALUE val;

  val = ULONG2NUM(USHRT_MAX);
  Check_Range_Llong(val);
  return Qnil;
}
static VALUE
test_086_RangeCheck_Llong(VALUE self)
{
  VALUE val;

  val = LONG2NUM(FIXNUM_MIN);
  Check_Range_Llong(val);
  return Qnil;
}
static VALUE
test_087_RangeCheck_Llong(VALUE self)
{
  VALUE val;

  val = LONG2NUM(FIXNUM_MAX);
  Check_Range_Llong(val);
  return Qnil;
}
static VALUE
test_088_RangeCheck_Llong(VALUE self)
{
  VALUE val;

  val = LONG2NUM(LONG_MIN);
  Check_Range_Llong(val);
  return Qnil;
}
static VALUE
test_089_RangeCheck_Llong(VALUE self)
{
  VALUE val;

  val = LONG2NUM(LONG_MAX);
  Check_Range_Llong(val);
  return Qnil;
}
static VALUE
test_08a_RangeCheck_Llong(VALUE self)
{
  VALUE val;

  val = ULONG2NUM(ULONG_MAX);
  Check_Range_Llong(val);
  return Qnil;
}
static VALUE
test_08b_RangeCheck_Llong(VALUE self)
{
  VALUE val;

  val = LLONG2NUM(LONGLONG_MIN);
  Check_Range_Llong(val);
  return Qnil;
}
static VALUE
test_08c_RangeCheck_Llong(VALUE self)
{
  VALUE val;

  val = LLONG2NUM(LONGLONG_MAX);
  Check_Range_Llong(val);
  return Qnil;
}


// -- Check_Range_Ullong
static VALUE
test_090_RangeCheck_Ullong(VALUE self)
{
  VALUE val;

  val = LONG2NUM(SCHAR_MAX);
  Check_Range_Ullong(val);
  return Qnil;
}
static VALUE
test_091_RangeCheck_Ullong(VALUE self)
{
  VALUE val;

  val = ULONG2NUM(UCHAR_MAX);
  Check_Range_Ullong(val);
  return Qnil;
}
static VALUE
test_092_RangeCheck_Ullong(VALUE self)
{
  VALUE val;

  val = LONG2NUM(SHRT_MAX);
  Check_Range_Ullong(val);
  return Qnil;
}
static VALUE
test_093_RangeCheck_Ullong(VALUE self)
{
  VALUE val;

  val = ULONG2NUM(USHRT_MAX);
  Check_Range_Ullong(val);
  return Qnil;
}
static VALUE
test_094_RangeCheck_Ullong(VALUE self)
{
  VALUE val;

  val = LONG2NUM(FIXNUM_MAX);
  Check_Range_Ullong(val);
  return Qnil;
}
static VALUE
test_095_RangeCheck_Ullong(VALUE self)
{
  VALUE val;

  val = LONG2NUM(LONG_MAX);
  Check_Range_Ullong(val);
  return Qnil;
}
static VALUE
test_096_RangeCheck_Ullong(VALUE self)
{
  VALUE val;

  val = ULONG2NUM(ULONG_MAX);
  Check_Range_Ullong(val);
  return Qnil;
}
static VALUE
test_097_RangeCheck_Ullong(VALUE self)
{
  VALUE val;

  val = LLONG2NUM(LONGLONG_MAX);
  Check_Range_Ullong(val);
  return Qnil;
}
static VALUE
test_098_RangeCheck_Ullong(VALUE self)
{
  VALUE val;

  val = ULLONG2NUM(ULONGLONG_MAX);
  Check_Range_Ullong(val);
  return Qnil;
}


static VALUE
res_RangeError(VALUE self, VALUE err)
{
  assert_instance_of(self, rb_eRangeError, err);
  return Qtrue;
}


// -- NUM2CHR
static VALUE
test_0a0_NUM2CHR(VALUE self)
{
  VALUE val;
  char c;

  val = CHR2FIX(SCHAR_MIN);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("-128"));

  c = NUM2CHR(val);
  ASSERT_EQUAL(self, c, SCHAR_MIN);

  val = CHR2FIX(c);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("-128"));

  return Qnil;
}
static VALUE
test_0a1_NUM2CHR(VALUE self)
{
  VALUE val;
  char c;

  val = CHR2FIX(SCHAR_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("127"));

  c = NUM2CHR(val);
  ASSERT_EQUAL(self, c, SCHAR_MAX);

  val = CHR2FIX(c);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("127"));

  return Qnil;
}
static VALUE
beg_NUM2CHR(VALUE val)
{
  char c;
  c = NUM2CHR(val);
  return Qnil;
}
static VALUE
test_0a2_NUM2CHR(VALUE self)
{
  VALUE val;
  VALUE ret;

  val = UCHR2FIX(UCHAR_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("255"));

  ret = rb_rescue2(RUBY_METHOD_FUNC(beg_NUM2CHR), val,
                   RUBY_METHOD_FUNC(res_RangeError), self,
                   rb_eRangeError, 0);
  assert(self, ret);

  return Qnil;
}
static VALUE
test_0a3_NUM2CHR(VALUE self)
{
  VALUE val;
  VALUE ret;

  val = INT2FIX(SHRT_MIN);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("-32768"));

  ret = rb_rescue2(RUBY_METHOD_FUNC(beg_NUM2CHR), val,
                   RUBY_METHOD_FUNC(res_RangeError), self,
                   rb_eRangeError, 0);
  assert(self, ret);

  return Qnil;
}
static VALUE
test_0a4_NUM2CHR(VALUE self)
{
  VALUE val;
  VALUE ret;

  val = INT2FIX(SHRT_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("32767"));

  ret = rb_rescue2(RUBY_METHOD_FUNC(beg_NUM2CHR), val,
                   RUBY_METHOD_FUNC(res_RangeError), self,
                   rb_eRangeError, 0);
  assert(self, ret);

  return Qnil;
}
static VALUE
test_0a5_NUM2CHR(VALUE self)
{
  VALUE val;
  VALUE ret;

  val = INT2FIX(USHRT_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("65535"));

  ret = rb_rescue2(RUBY_METHOD_FUNC(beg_NUM2CHR), val,
                   RUBY_METHOD_FUNC(res_RangeError), self,
                   rb_eRangeError, 0);
  assert(self, ret);

  return Qnil;
}
static VALUE
test_0a6_NUM2CHR(VALUE self)
{
  VALUE val;
  VALUE ret;

  val = LONG2NUM(FIXNUM_MIN);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("-1073741824"));

  ret = rb_rescue2(RUBY_METHOD_FUNC(beg_NUM2CHR), val,
                   RUBY_METHOD_FUNC(res_RangeError), self,
                   rb_eRangeError, 0);
  assert(self, ret);

  return Qnil;
}
static VALUE
test_0a7_NUM2CHR(VALUE self)
{
  VALUE val;
  VALUE ret;

  val = LONG2NUM(FIXNUM_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("1073741823"));

  ret = rb_rescue2(RUBY_METHOD_FUNC(beg_NUM2CHR), val,
                   RUBY_METHOD_FUNC(res_RangeError), self,
                   rb_eRangeError, 0);
  assert(self, ret);

  return Qnil;
}
static VALUE
test_0a8_NUM2CHR(VALUE self)
{
  VALUE val;
  VALUE ret;

  val = LONG2NUM(LONG_MIN);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("-2147483648"));

  ret = rb_rescue2(RUBY_METHOD_FUNC(beg_NUM2CHR), val,
                   RUBY_METHOD_FUNC(res_RangeError), self,
                   rb_eRangeError, 0);
  assert(self, ret);

  return Qnil;
}
static VALUE
test_0a9_NUM2CHR(VALUE self)
{
  VALUE val;
  VALUE ret;

  val = LONG2NUM(LONG_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("2147483647"));

  ret = rb_rescue2(RUBY_METHOD_FUNC(beg_NUM2CHR), val,
                   RUBY_METHOD_FUNC(res_RangeError), self,
                   rb_eRangeError, 0);
  assert(self, ret);

  return Qnil;
}
static VALUE
test_0aa_NUM2CHR(VALUE self)
{
  VALUE val;
  VALUE ret;

  val = ULONG2NUM(ULONG_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("4294967295"));

  ret = rb_rescue2(RUBY_METHOD_FUNC(beg_NUM2CHR), val,
                   RUBY_METHOD_FUNC(res_RangeError), self,
                   rb_eRangeError, 0);
  assert(self, ret);

  return Qnil;
}
static VALUE
test_0ab_NUM2CHR(VALUE self)
{
  VALUE val;
  VALUE ret;

  val = LLONG2NUM(LONGLONG_MIN);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("-9223372036854775808"));

  ret = rb_rescue2(RUBY_METHOD_FUNC(beg_NUM2CHR), val,
                   RUBY_METHOD_FUNC(res_RangeError), self,
                   rb_eRangeError, 0);
  assert(self, ret);

  return Qnil;
}
static VALUE
test_0ac_NUM2CHR(VALUE self)
{
  VALUE val;
  VALUE ret;

  val = LLONG2NUM(LONGLONG_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("9223372036854775807"));

  ret = rb_rescue2(RUBY_METHOD_FUNC(beg_NUM2CHR), val,
                   RUBY_METHOD_FUNC(res_RangeError), self,
                   rb_eRangeError, 0);
  assert(self, ret);

  return Qnil;
}
static VALUE
test_0ad_NUM2CHR(VALUE self)
{
  VALUE val;
  VALUE ret;

  val = ULLONG2NUM(ULONGLONG_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("18446744073709551615"));

  ret = rb_rescue2(RUBY_METHOD_FUNC(beg_NUM2CHR), val,
                   RUBY_METHOD_FUNC(res_RangeError), self,
                   rb_eRangeError, 0);
  assert(self, ret);

  return Qnil;
}


// -- NUM2UCHR
static VALUE
beg_NUM2UCHR(VALUE val)
{
  char c;
  c = NUM2UCHR(val);
  return Qnil;
}
static VALUE
test_0b0_NUM2UCHR(VALUE self)
{
  VALUE val;
  VALUE ret;

  val = CHR2FIX(SCHAR_MIN);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("-128"));

  ret = rb_rescue2(RUBY_METHOD_FUNC(beg_NUM2UCHR), val,
                   RUBY_METHOD_FUNC(res_RangeError), self,
                   rb_eRangeError, 0);
  assert(self, ret);

  return Qnil;
}
static VALUE
test_0b1_NUM2UCHR(VALUE self)
{
  VALUE val;
  unsigned char c;

  val = UCHR2FIX(SCHAR_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("127"));

  c = NUM2UCHR(val);
  ASSERT_EQUAL(self, c, SCHAR_MAX);

  val = UCHR2FIX(c);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("127"));

  return Qnil;
}
static VALUE
test_0b2_NUM2UCHR(VALUE self)
{
  VALUE val;
  unsigned char c;

  val = UCHR2FIX(UCHAR_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("255"));

  c = NUM2UCHR(val);
  ASSERT_EQUAL(self, c, UCHAR_MAX);

  val = UCHR2FIX(c);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("255"));

  return Qnil;
}
static VALUE
test_0b3_NUM2UCHR(VALUE self)
{
  VALUE val;
  VALUE ret;

  val = INT2FIX(SHRT_MIN);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("-32768"));

  ret = rb_rescue2(RUBY_METHOD_FUNC(beg_NUM2UCHR), val,
                   RUBY_METHOD_FUNC(res_RangeError), self,
                   rb_eRangeError, 0);
  assert(self, ret);

  return Qnil;
}
static VALUE
test_0b4_NUM2UCHR(VALUE self)
{
  VALUE val;
  VALUE ret;

  val = INT2FIX(SHRT_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("32767"));

  ret = rb_rescue2(RUBY_METHOD_FUNC(beg_NUM2UCHR), val,
                   RUBY_METHOD_FUNC(res_RangeError), self,
                   rb_eRangeError, 0);
  assert(self, ret);

  return Qnil;
}
static VALUE
test_0b5_NUM2UCHR(VALUE self)
{
  VALUE val;
  VALUE ret;

  val = INT2FIX(USHRT_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("65535"));

  ret = rb_rescue2(RUBY_METHOD_FUNC(beg_NUM2UCHR), val,
                   RUBY_METHOD_FUNC(res_RangeError), self,
                   rb_eRangeError, 0);
  assert(self, ret);

  return Qnil;
}
static VALUE
test_0b6_NUM2UCHR(VALUE self)
{
  VALUE val;
  VALUE ret;

  val = LONG2NUM(FIXNUM_MIN);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("-1073741824"));

  ret = rb_rescue2(RUBY_METHOD_FUNC(beg_NUM2UCHR), val,
                   RUBY_METHOD_FUNC(res_RangeError), self,
                   rb_eRangeError, 0);
  assert(self, ret);

  return Qnil;
}
static VALUE
test_0b7_NUM2UCHR(VALUE self)
{
  VALUE val;
  VALUE ret;

  val = LONG2NUM(FIXNUM_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("1073741823"));

  ret = rb_rescue2(RUBY_METHOD_FUNC(beg_NUM2UCHR), val,
                   RUBY_METHOD_FUNC(res_RangeError), self,
                   rb_eRangeError, 0);
  assert(self, ret);

  return Qnil;
}
static VALUE
test_0b8_NUM2UCHR(VALUE self)
{
  VALUE val;
  VALUE ret;

  val = LONG2NUM(LONG_MIN);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("-2147483648"));

  ret = rb_rescue2(RUBY_METHOD_FUNC(beg_NUM2UCHR), val,
                   RUBY_METHOD_FUNC(res_RangeError), self,
                   rb_eRangeError, 0);
  assert(self, ret);

  return Qnil;
}
static VALUE
test_0b9_NUM2UCHR(VALUE self)
{
  VALUE val;
  VALUE ret;

  val = LONG2NUM(LONG_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("2147483647"));

  ret = rb_rescue2(RUBY_METHOD_FUNC(beg_NUM2UCHR), val,
                   RUBY_METHOD_FUNC(res_RangeError), self,
                   rb_eRangeError, 0);
  assert(self, ret);

  return Qnil;
}
static VALUE
test_0ba_NUM2UCHR(VALUE self)
{
  VALUE val;
  VALUE ret;

  val = ULONG2NUM(ULONG_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("4294967295"));

  ret = rb_rescue2(RUBY_METHOD_FUNC(beg_NUM2UCHR), val,
                   RUBY_METHOD_FUNC(res_RangeError), self,
                   rb_eRangeError, 0);
  assert(self, ret);

  return Qnil;
}
static VALUE
test_0bb_NUM2UCHR(VALUE self)
{
  VALUE val;
  VALUE ret;

  val = LLONG2NUM(LONGLONG_MIN);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("-9223372036854775808"));

  ret = rb_rescue2(RUBY_METHOD_FUNC(beg_NUM2UCHR), val,
                   RUBY_METHOD_FUNC(res_RangeError), self,
                   rb_eRangeError, 0);
  assert(self, ret);

  return Qnil;
}
static VALUE
test_0bc_NUM2UCHR(VALUE self)
{
  VALUE val;
  VALUE ret;

  val = LLONG2NUM(LONGLONG_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("9223372036854775807"));

  ret = rb_rescue2(RUBY_METHOD_FUNC(beg_NUM2UCHR), val,
                   RUBY_METHOD_FUNC(res_RangeError), self,
                   rb_eRangeError, 0);
  assert(self, ret);

  return Qnil;
}
static VALUE
test_0bd_NUM2UCHR(VALUE self)
{
  VALUE val;
  VALUE ret;

  val = ULLONG2NUM(ULONGLONG_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("18446744073709551615"));

  ret = rb_rescue2(RUBY_METHOD_FUNC(beg_NUM2UCHR), val,
                   RUBY_METHOD_FUNC(res_RangeError), self,
                   rb_eRangeError, 0);
  assert(self, ret);

  return Qnil;
}


// -- NUM2SHRT
static VALUE
test_0c0_NUM2SHRT(VALUE self)
{
  VALUE val;
  short c;

  val = CHR2FIX(SCHAR_MIN);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("-128"));

  c = NUM2SHRT(val);
  ASSERT_EQUAL(self, c, SCHAR_MIN);

  val = CHR2FIX(c);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("-128"));

  return Qnil;
}
static VALUE
test_0c1_NUM2SHRT(VALUE self)
{
  VALUE val;
  short c;

  val = CHR2FIX(SCHAR_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("127"));

  c = NUM2SHRT(val);
  ASSERT_EQUAL(self, c, SCHAR_MAX);

  val = CHR2FIX(c);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("127"));

  return Qnil;
}
static VALUE
test_0c2_NUM2SHRT(VALUE self)
{
  VALUE val;
  short c;

  val = UCHR2FIX(UCHAR_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("255"));

  c = NUM2SHRT(val);
  ASSERT_EQUAL(self, c, UCHAR_MAX);

  val = UCHR2FIX(c);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("255"));

  return Qnil;
}
static VALUE
test_0c3_NUM2SHRT(VALUE self)
{
  VALUE val;
  short c;

  val = INT2FIX(SHRT_MIN);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("-32768"));

  c = NUM2SHRT(val);
  ASSERT_EQUAL(self, c, SHRT_MIN);

  val = INT2FIX(c);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("-32768"));

  return Qnil;
}
static VALUE
test_0c4_NUM2SHRT(VALUE self)
{
  VALUE val;
  short c;

  val = INT2FIX(SHRT_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("32767"));

  c = NUM2SHRT(val);
  ASSERT_EQUAL(self, c, SHRT_MAX);

  val = INT2FIX(c);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("32767"));

  return Qnil;
}
static VALUE
beg_NUM2SHRT(VALUE val)
{
  char c;
  c = NUM2SHRT(val);
  return Qnil;
}
static VALUE
test_0c5_NUM2SHRT(VALUE self)
{
  VALUE val;
  VALUE ret;

  val = INT2FIX(USHRT_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("65535"));

  ret = rb_rescue2(RUBY_METHOD_FUNC(beg_NUM2SHRT), val,
                   RUBY_METHOD_FUNC(res_RangeError), self,
                   rb_eRangeError, 0);
  assert(self, ret);

  return Qnil;
}
static VALUE
test_0c6_NUM2SHRT(VALUE self)
{
  VALUE val;
  VALUE ret;

  val = LONG2NUM(FIXNUM_MIN);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("-1073741824"));

  ret = rb_rescue2(RUBY_METHOD_FUNC(beg_NUM2SHRT), val,
                   RUBY_METHOD_FUNC(res_RangeError), self,
                   rb_eRangeError, 0);
  assert(self, ret);

  return Qnil;
}
static VALUE
test_0c7_NUM2SHRT(VALUE self)
{
  VALUE val;
  VALUE ret;

  val = LONG2NUM(FIXNUM_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("1073741823"));

  ret = rb_rescue2(RUBY_METHOD_FUNC(beg_NUM2SHRT), val,
                   RUBY_METHOD_FUNC(res_RangeError), self,
                   rb_eRangeError, 0);
  assert(self, ret);

  return Qnil;
}
static VALUE
test_0c8_NUM2SHRT(VALUE self)
{
  VALUE val;
  VALUE ret;

  val = LONG2NUM(LONG_MIN);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("-2147483648"));

  ret = rb_rescue2(RUBY_METHOD_FUNC(beg_NUM2SHRT), val,
                   RUBY_METHOD_FUNC(res_RangeError), self,
                   rb_eRangeError, 0);
  assert(self, ret);

  return Qnil;
}
static VALUE
test_0c9_NUM2SHRT(VALUE self)
{
  VALUE val;
  VALUE ret;

  val = LONG2NUM(LONG_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("2147483647"));

  ret = rb_rescue2(RUBY_METHOD_FUNC(beg_NUM2SHRT), val,
                   RUBY_METHOD_FUNC(res_RangeError), self,
                   rb_eRangeError, 0);
  assert(self, ret);

  return Qnil;
}
static VALUE
test_0ca_NUM2SHRT(VALUE self)
{
  VALUE val;
  VALUE ret;

  val = ULONG2NUM(ULONG_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("4294967295"));

  ret = rb_rescue2(RUBY_METHOD_FUNC(beg_NUM2SHRT), val,
                   RUBY_METHOD_FUNC(res_RangeError), self,
                   rb_eRangeError, 0);
  assert(self, ret);

  return Qnil;
}
static VALUE
test_0cb_NUM2SHRT(VALUE self)
{
  VALUE val;
  VALUE ret;

  val = LLONG2NUM(LONGLONG_MIN);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("-9223372036854775808"));

  ret = rb_rescue2(RUBY_METHOD_FUNC(beg_NUM2SHRT), val,
                   RUBY_METHOD_FUNC(res_RangeError), self,
                   rb_eRangeError, 0);
  assert(self, ret);

  return Qnil;
}
static VALUE
test_0cc_NUM2SHRT(VALUE self)
{
  VALUE val;
  VALUE ret;

  val = LLONG2NUM(LONGLONG_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("9223372036854775807"));

  ret = rb_rescue2(RUBY_METHOD_FUNC(beg_NUM2SHRT), val,
                   RUBY_METHOD_FUNC(res_RangeError), self,
                   rb_eRangeError, 0);
  assert(self, ret);

  return Qnil;
}
static VALUE
test_0cd_NUM2SHRT(VALUE self)
{
  VALUE val;
  VALUE ret;

  val = ULLONG2NUM(ULONGLONG_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("18446744073709551615"));

  ret = rb_rescue2(RUBY_METHOD_FUNC(beg_NUM2SHRT), val,
                   RUBY_METHOD_FUNC(res_RangeError), self,
                   rb_eRangeError, 0);
  assert(self, ret);

  return Qnil;
}


// -- NUM2USHRT
static VALUE
beg_NUM2USHRT(VALUE val)
{
  char c;
  c = NUM2USHRT(val);
  return Qnil;
}
static VALUE
test_0d0_NUM2USHRT(VALUE self)
{
  VALUE val;
  VALUE ret;

  val = CHR2FIX(SCHAR_MIN);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("-128"));

  ret = rb_rescue2(RUBY_METHOD_FUNC(beg_NUM2USHRT), val,
                   RUBY_METHOD_FUNC(res_RangeError), self,
                   rb_eRangeError, 0);
  assert(self, ret);

  return Qnil;
}
static VALUE
test_0d1_NUM2USHRT(VALUE self)
{
  VALUE val;
  unsigned short c;

  val = UCHR2FIX(SCHAR_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("127"));

  c = NUM2USHRT(val);
  ASSERT_EQUAL(self, c, SCHAR_MAX);

  val = UCHR2FIX(c);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("127"));

  return Qnil;
}
static VALUE
test_0d2_NUM2USHRT(VALUE self)
{
  VALUE val;
  unsigned short c;

  val = UCHR2FIX(UCHAR_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("255"));

  c = NUM2USHRT(val);
  ASSERT_EQUAL(self, c, UCHAR_MAX);

  val = UCHR2FIX(c);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("255"));

  return Qnil;
}
static VALUE
test_0d3_NUM2USHRT(VALUE self)
{
  VALUE val;
  VALUE ret;

  val = INT2FIX(SHRT_MIN);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("-32768"));

  ret = rb_rescue2(RUBY_METHOD_FUNC(beg_NUM2USHRT), val,
                   RUBY_METHOD_FUNC(res_RangeError), self,
                   rb_eRangeError, 0);
  assert(self, ret);

  return Qnil;
}
static VALUE
test_0d4_NUM2USHRT(VALUE self)
{
  VALUE val;
  unsigned short c;

  val = INT2FIX(SHRT_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("32767"));

  c = NUM2USHRT(val);
  ASSERT_EQUAL(self, c, SHRT_MAX);

  val = INT2FIX(c);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("32767"));

  return Qnil;
}
static VALUE
test_0d5_NUM2USHRT(VALUE self)
{
  VALUE val;
  unsigned short c;

  val = INT2FIX(USHRT_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("65535"));

  c = NUM2USHRT(val);
  ASSERT_EQUAL(self, c, USHRT_MAX);

  val = INT2FIX(c);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("65535"));

  return Qnil;
}
static VALUE
test_0d6_NUM2USHRT(VALUE self)
{
  VALUE val;
  VALUE ret;

  val = LONG2NUM(FIXNUM_MIN);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("-1073741824"));

  ret = rb_rescue2(RUBY_METHOD_FUNC(beg_NUM2USHRT), val,
                   RUBY_METHOD_FUNC(res_RangeError), self,
                   rb_eRangeError, 0);
  assert(self, ret);

  return Qnil;
}
static VALUE
test_0d7_NUM2USHRT(VALUE self)
{
  VALUE val;
  VALUE ret;

  val = LONG2NUM(FIXNUM_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("1073741823"));

  ret = rb_rescue2(RUBY_METHOD_FUNC(beg_NUM2USHRT), val,
                   RUBY_METHOD_FUNC(res_RangeError), self,
                   rb_eRangeError, 0);
  assert(self, ret);

  return Qnil;
}
static VALUE
test_0d8_NUM2USHRT(VALUE self)
{
  VALUE val;
  VALUE ret;

  val = LONG2NUM(LONG_MIN);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("-2147483648"));

  ret = rb_rescue2(RUBY_METHOD_FUNC(beg_NUM2USHRT), val,
                   RUBY_METHOD_FUNC(res_RangeError), self,
                   rb_eRangeError, 0);
  assert(self, ret);

  return Qnil;
}
static VALUE
test_0d9_NUM2USHRT(VALUE self)
{
  VALUE val;
  VALUE ret;

  val = LONG2NUM(LONG_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("2147483647"));

  ret = rb_rescue2(RUBY_METHOD_FUNC(beg_NUM2USHRT), val,
                   RUBY_METHOD_FUNC(res_RangeError), self,
                   rb_eRangeError, 0);
  assert(self, ret);

  return Qnil;
}
static VALUE
test_0da_NUM2USHRT(VALUE self)
{
  VALUE val;
  VALUE ret;

  val = ULONG2NUM(ULONG_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("4294967295"));

  ret = rb_rescue2(RUBY_METHOD_FUNC(beg_NUM2USHRT), val,
                   RUBY_METHOD_FUNC(res_RangeError), self,
                   rb_eRangeError, 0);
  assert(self, ret);

  return Qnil;
}
static VALUE
test_0db_NUM2USHRT(VALUE self)
{
  VALUE val;
  VALUE ret;

  val = LLONG2NUM(LONGLONG_MIN);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("-9223372036854775808"));

  ret = rb_rescue2(RUBY_METHOD_FUNC(beg_NUM2USHRT), val,
                   RUBY_METHOD_FUNC(res_RangeError), self,
                   rb_eRangeError, 0);
  assert(self, ret);

  return Qnil;
}
static VALUE
test_0dc_NUM2USHRT(VALUE self)
{
  VALUE val;
  VALUE ret;

  val = LLONG2NUM(LONGLONG_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("9223372036854775807"));

  ret = rb_rescue2(RUBY_METHOD_FUNC(beg_NUM2USHRT), val,
                   RUBY_METHOD_FUNC(res_RangeError), self,
                   rb_eRangeError, 0);
  assert(self, ret);

  return Qnil;
}
static VALUE
test_0dd_NUM2USHRT(VALUE self)
{
  VALUE val;
  VALUE ret;

  val = ULLONG2NUM(ULONGLONG_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("18446744073709551615"));

  ret = rb_rescue2(RUBY_METHOD_FUNC(beg_NUM2USHRT), val,
                   RUBY_METHOD_FUNC(res_RangeError), self,
                   rb_eRangeError, 0);
  assert(self, ret);

  return Qnil;
}


// -- NUM2LONG
static VALUE
test_0e0_NUM2LONG(VALUE self)
{
  VALUE val;
  long c;

  val = CHR2FIX(SCHAR_MIN);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("-128"));

  c = NUM2LONG(val);
  ASSERT_EQUAL(self, c, SCHAR_MIN);

  val = CHR2FIX(c);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("-128"));

  return Qnil;
}
static VALUE
test_0e1_NUM2LONG(VALUE self)
{
  VALUE val;
  long c;

  val = CHR2FIX(SCHAR_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("127"));

  c = NUM2LONG(val);
  ASSERT_EQUAL(self, c, SCHAR_MAX);

  val = CHR2FIX(c);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("127"));

  return Qnil;
}
static VALUE
test_0e2_NUM2LONG(VALUE self)
{
  VALUE val;
  long c;

  val = UCHR2FIX(UCHAR_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("255"));

  c = NUM2LONG(val);
  ASSERT_EQUAL(self, c, UCHAR_MAX);

  val = UCHR2FIX(c);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("255"));

  return Qnil;
}
static VALUE
test_0e3_NUM2LONG(VALUE self)
{
  VALUE val;
  long c;

  val = LONG2NUM(SHRT_MIN);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("-32768"));

  c = NUM2LONG(val);
  ASSERT_EQUAL(self, c, SHRT_MIN);

  val = LONG2NUM(c);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("-32768"));

  return Qnil;
}
static VALUE
test_0e4_NUM2LONG(VALUE self)
{
  VALUE val;
  long c;

  val = LONG2NUM(SHRT_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("32767"));

  c = NUM2LONG(val);
  ASSERT_EQUAL(self, c, SHRT_MAX);

  val = LONG2NUM(c);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("32767"));

  return Qnil;
}
static VALUE
test_0e5_NUM2LONG(VALUE self)
{
  VALUE val;
  long c;

  val = LONG2NUM(USHRT_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("65535"));

  c = NUM2LONG(val);
  ASSERT_EQUAL(self, c, USHRT_MAX);

  val = LONG2NUM(c);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("65535"));

  return Qnil;
}
static VALUE
test_0e6_NUM2LONG(VALUE self)
{
  VALUE val;
  long c;

  val = LONG2NUM(FIXNUM_MIN);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("-1073741824"));

  c = NUM2LONG(val);
  ASSERT_EQUAL(self, c, FIXNUM_MIN);

  val = LONG2NUM(c);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("-1073741824"));

  return Qnil;
}
static VALUE
test_0e7_NUM2LONG(VALUE self)
{
  VALUE val;
  long c;

  val = LONG2NUM(FIXNUM_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("1073741823"));

  c = NUM2LONG(val);
  ASSERT_EQUAL(self, c, FIXNUM_MAX);

  val = LONG2NUM(c);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("1073741823"));

  return Qnil;
}
static VALUE
test_0e8_NUM2LONG(VALUE self)
{
  VALUE val;
  long c;

  val = LONG2NUM(LONG_MIN);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("-2147483648"));

  c = NUM2LONG(val);
  ASSERT_EQUAL(self, c, LONG_MIN);

  val = LONG2NUM(c);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("-2147483648"));

  return Qnil;
}
static VALUE
test_0e9_NUM2LONG(VALUE self)
{
  VALUE val;
  long c;

  val = LONG2NUM(LONG_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("2147483647"));

  c = NUM2LONG(val);
  ASSERT_EQUAL(self, c, LONG_MAX);

  val = LONG2NUM(c);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("2147483647"));

  return Qnil;
}
static VALUE
beg_NUM2LONG(VALUE val)
{
  char c;
  c = NUM2LONG(val);
  return Qnil;
}
static VALUE
test_0ea_NUM2LONG(VALUE self)
{
  VALUE val;
  VALUE ret;

  val = ULONG2NUM(ULONG_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("4294967295"));

  ret = rb_rescue2(RUBY_METHOD_FUNC(beg_NUM2LONG), val,
                   RUBY_METHOD_FUNC(res_RangeError), self,
                   rb_eRangeError, 0);
  assert(self, ret);

  return Qnil;
}
static VALUE
test_0eb_NUM2LONG(VALUE self)
{
  VALUE val;
  VALUE ret;

  val = LLONG2NUM(LONGLONG_MIN);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("-9223372036854775808"));

  ret = rb_rescue2(RUBY_METHOD_FUNC(beg_NUM2LONG), val,
                   RUBY_METHOD_FUNC(res_RangeError), self,
                   rb_eRangeError, 0);
  assert(self, ret);

  return Qnil;
}
static VALUE
test_0ec_NUM2LONG(VALUE self)
{
  VALUE val;
  VALUE ret;

  val = LLONG2NUM(LONGLONG_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("9223372036854775807"));

  ret = rb_rescue2(RUBY_METHOD_FUNC(beg_NUM2LONG), val,
                   RUBY_METHOD_FUNC(res_RangeError), self,
                   rb_eRangeError, 0);
  assert(self, ret);

  return Qnil;
}
static VALUE
test_0ed_NUM2LONG(VALUE self)
{
  VALUE val;
  VALUE ret;

  val = ULLONG2NUM(ULONGLONG_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("18446744073709551615"));

  ret = rb_rescue2(RUBY_METHOD_FUNC(beg_NUM2LONG), val,
                   RUBY_METHOD_FUNC(res_RangeError), self,
                   rb_eRangeError, 0);
  assert(self, ret);

  return Qnil;
}


// -- NUM2ULONG
static VALUE
beg_NUM2ULONG(VALUE val)
{
  char c;
  c = NUM2ULONG(val);
  return Qnil;
}
static VALUE
test_0f0_NUM2ULONG(VALUE self)
{
  VALUE val;
  VALUE ret;

  val = CHR2FIX(SCHAR_MIN);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("-128"));

  ret = rb_rescue2(RUBY_METHOD_FUNC(beg_NUM2ULONG), val,
                   RUBY_METHOD_FUNC(res_RangeError), self,
                   rb_eRangeError, 0);
  assert(self, ret);

  return Qnil;
}
static VALUE
test_0f1_NUM2ULONG(VALUE self)
{
  VALUE val;
  unsigned long c;

  val = UCHR2FIX(SCHAR_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("127"));

  c = NUM2ULONG(val);
  ASSERT_EQUAL(self, c, SCHAR_MAX);

  val = UCHR2FIX(c);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("127"));

  return Qnil;
}
static VALUE
test_0f2_NUM2ULONG(VALUE self)
{
  VALUE val;
  unsigned long c;

  val = UCHR2FIX(UCHAR_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("255"));

  c = NUM2ULONG(val);
  ASSERT_EQUAL(self, c, UCHAR_MAX);

  val = UCHR2FIX(c);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("255"));

  return Qnil;
}
static VALUE
test_0f3_NUM2ULONG(VALUE self)
{
  VALUE val;
  VALUE ret;

  val = INT2FIX(SHRT_MIN);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("-32768"));

  ret = rb_rescue2(RUBY_METHOD_FUNC(beg_NUM2ULONG), val,
                   RUBY_METHOD_FUNC(res_RangeError), self,
                   rb_eRangeError, 0);
  assert(self, ret);

  return Qnil;
}
static VALUE
test_0f4_NUM2ULONG(VALUE self)
{
  VALUE val;
  unsigned long c;

  val = LONG2NUM(SHRT_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("32767"));

  c = NUM2ULONG(val);
  ASSERT_EQUAL(self, c, SHRT_MAX);

  val = LONG2NUM(c);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("32767"));

  return Qnil;
}
static VALUE
test_0f5_NUM2ULONG(VALUE self)
{
  VALUE val;
  unsigned long c;

  val = LONG2NUM(USHRT_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("65535"));

  c = NUM2ULONG(val);
  ASSERT_EQUAL(self, c, USHRT_MAX);

  val = LONG2NUM(c);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("65535"));

  return Qnil;
}
static VALUE
test_0f6_NUM2ULONG(VALUE self)
{
  VALUE val;
  VALUE ret;

  val = LONG2NUM(FIXNUM_MIN);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("-1073741824"));

  ret = rb_rescue2(RUBY_METHOD_FUNC(beg_NUM2ULONG), val,
                   RUBY_METHOD_FUNC(res_RangeError), self,
                   rb_eRangeError, 0);
  assert(self, ret);

  return Qnil;
}
static VALUE
test_0f7_NUM2ULONG(VALUE self)
{
  VALUE val;
  unsigned long c;

  val = LONG2NUM(FIXNUM_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("1073741823"));

  c = NUM2ULONG(val);
  ASSERT_EQUAL(self, c, FIXNUM_MAX);

  val = LONG2NUM(c);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("1073741823"));

  return Qnil;
}
static VALUE
test_0f8_NUM2ULONG(VALUE self)
{
  VALUE val;
  VALUE ret;

  val = LONG2NUM(LONG_MIN);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("-2147483648"));

  ret = rb_rescue2(RUBY_METHOD_FUNC(beg_NUM2ULONG), val,
                   RUBY_METHOD_FUNC(res_RangeError), self,
                   rb_eRangeError, 0);
  assert(self, ret);

  return Qnil;
}
static VALUE
test_0f9_NUM2ULONG(VALUE self)
{
  VALUE val;
  unsigned long c;

  val = LONG2NUM(LONG_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("2147483647"));

  c = NUM2ULONG(val);
  ASSERT_EQUAL(self, c, LONG_MAX);

  val = LONG2NUM(c);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("2147483647"));

  return Qnil;
}
static VALUE
test_0fa_NUM2ULONG(VALUE self)
{
  VALUE val;
  unsigned long c;

  val = ULONG2NUM(ULONG_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("4294967295"));

  c = NUM2ULONG(val);
  ASSERT_EQUAL(self, c, ULONG_MAX);

  val = ULONG2NUM(c);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("4294967295"));

  return Qnil;
}
static VALUE
test_0fb_NUM2ULONG(VALUE self)
{
  VALUE val;
  VALUE ret;

  val = LLONG2NUM(LONGLONG_MIN);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("-9223372036854775808"));

  ret = rb_rescue2(RUBY_METHOD_FUNC(beg_NUM2ULONG), val,
                   RUBY_METHOD_FUNC(res_RangeError), self,
                   rb_eRangeError, 0);
  assert(self, ret);

  return Qnil;
}
static VALUE
test_0fc_NUM2ULONG(VALUE self)
{
  VALUE val;
  VALUE ret;

  val = LLONG2NUM(LONGLONG_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("9223372036854775807"));

  ret = rb_rescue2(RUBY_METHOD_FUNC(beg_NUM2ULONG), val,
                   RUBY_METHOD_FUNC(res_RangeError), self,
                   rb_eRangeError, 0);
  assert(self, ret);

  return Qnil;
}
static VALUE
test_0fd_NUM2ULONG(VALUE self)
{
  VALUE val;
  VALUE ret;

  val = ULLONG2NUM(ULONGLONG_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("18446744073709551615"));

  ret = rb_rescue2(RUBY_METHOD_FUNC(beg_NUM2ULONG), val,
                   RUBY_METHOD_FUNC(res_RangeError), self,
                   rb_eRangeError, 0);
  assert(self, ret);

  return Qnil;
}


// -- NUM2LLONG
static VALUE
test_0g0_NUM2LLONG(VALUE self)
{
  VALUE val;
  long long c;

  val = CHR2FIX(SCHAR_MIN);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("-128"));

  c = NUM2LLONG(val);
  ASSERT_EQUAL(self, c, SCHAR_MIN);

  val = CHR2FIX(c);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("-128"));

  return Qnil;
}
static VALUE
test_0g1_NUM2LLONG(VALUE self)
{
  VALUE val;
  long long c;

  val = CHR2FIX(SCHAR_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("127"));

  c = NUM2LLONG(val);
  ASSERT_EQUAL(self, c, SCHAR_MAX);

  val = CHR2FIX(c);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("127"));

  return Qnil;
}
static VALUE
test_0g2_NUM2LLONG(VALUE self)
{
  VALUE val;
  long long c;

  val = UCHR2FIX(UCHAR_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("255"));

  c = NUM2LLONG(val);
  ASSERT_EQUAL(self, c, UCHAR_MAX);

  val = UCHR2FIX(c);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("255"));

  return Qnil;
}
static VALUE
test_0g3_NUM2LLONG(VALUE self)
{
  VALUE val;
  long long c;

  val = LONG2NUM(SHRT_MIN);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("-32768"));

  c = NUM2LLONG(val);
  ASSERT_EQUAL(self, c, SHRT_MIN);

  val = LONG2NUM(c);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("-32768"));

  return Qnil;
}
static VALUE
test_0g4_NUM2LLONG(VALUE self)
{
  VALUE val;
  long long c;

  val = LONG2NUM(SHRT_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("32767"));

  c = NUM2LLONG(val);
  ASSERT_EQUAL(self, c, SHRT_MAX);

  val = LONG2NUM(c);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("32767"));

  return Qnil;
}
static VALUE
test_0g5_NUM2LLONG(VALUE self)
{
  VALUE val;
  long long c;

  val = LONG2NUM(USHRT_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("65535"));

  c = NUM2LLONG(val);
  ASSERT_EQUAL(self, c, USHRT_MAX);

  val = LONG2NUM(c);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("65535"));

  return Qnil;
}
static VALUE
test_0g6_NUM2LLONG(VALUE self)
{
  VALUE val;
  long long c;

  val = LONG2NUM(FIXNUM_MIN);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("-1073741824"));

  c = NUM2LLONG(val);
  ASSERT_EQUAL(self, c, FIXNUM_MIN);

  val = LONG2NUM(c);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("-1073741824"));

  return Qnil;
}
static VALUE
test_0g7_NUM2LLONG(VALUE self)
{
  VALUE val;
  long long c;

  val = LONG2NUM(FIXNUM_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("1073741823"));

  c = NUM2LLONG(val);
  ASSERT_EQUAL(self, c, FIXNUM_MAX);

  val = LONG2NUM(c);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("1073741823"));

  return Qnil;
}
static VALUE
test_0g8_NUM2LLONG(VALUE self)
{
  VALUE val;
  long long c;

  val = LONG2NUM(LONG_MIN);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("-2147483648"));

  c = NUM2LLONG(val);
  ASSERT_EQUAL(self, c, LONG_MIN);

  val = LONG2NUM(c);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("-2147483648"));

  return Qnil;
}
static VALUE
test_0g9_NUM2LLONG(VALUE self)
{
  VALUE val;
  long long c;

  val = LONG2NUM(LONG_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("2147483647"));

  c = NUM2LLONG(val);
  ASSERT_EQUAL(self, c, LONG_MAX);

  val = LONG2NUM(c);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("2147483647"));

  return Qnil;
}
static VALUE
test_0ga_NUM2LLONG(VALUE self)
{
  VALUE val;
  long long c;

  val = ULONG2NUM(ULONG_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("4294967295"));

  c = NUM2LLONG(val);
  ASSERT_EQUAL(self, c, ULONG_MAX);

  val = ULONG2NUM(c);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("4294967295"));

  return Qnil;
}
static VALUE
test_0gb_NUM2LLONG(VALUE self)
{
  VALUE val;
  long long c;

  val = LLONG2NUM(LONGLONG_MIN);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("-9223372036854775808"));

  c = NUM2LLONG(val);
  ASSERT_EQUAL(self, c, LONGLONG_MIN);

  val = LLONG2NUM(c);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("-9223372036854775808"));

  return Qnil;
}
static VALUE
test_0gc_NUM2LLONG(VALUE self)
{
  VALUE val;
  long long c;

  val = LLONG2NUM(LONGLONG_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("9223372036854775807"));

  c = NUM2LLONG(val);
  ASSERT_EQUAL(self, c, LONGLONG_MAX);

  val = LLONG2NUM(c);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("9223372036854775807"));

  return Qnil;
}
static VALUE
beg_NUM2LLONG(VALUE val)
{
  char c;
  c = NUM2LLONG(val);
  return Qnil;
}
static VALUE
test_0gd_NUM2LLONG(VALUE self)
{
  VALUE val;
  VALUE ret;

  val = ULLONG2NUM(ULONGLONG_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("18446744073709551615"));

  ret = rb_rescue2(RUBY_METHOD_FUNC(beg_NUM2LLONG), val,
                   RUBY_METHOD_FUNC(res_RangeError), self,
                   rb_eRangeError, 0);
  assert(self, ret);

  return Qnil;
}


// -- NUM2ULLONG
static VALUE
beg_NUM2ULLONG(VALUE val)
{
  char c;
  c = NUM2ULLONG(val);
  return Qnil;
}
static VALUE
test_0h0_NUM2ULLONG(VALUE self)
{
  VALUE val;
  VALUE ret;

  val = CHR2FIX(SCHAR_MIN);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("-128"));

  ret = rb_rescue2(RUBY_METHOD_FUNC(beg_NUM2ULLONG), val,
                   RUBY_METHOD_FUNC(res_RangeError), self,
                   rb_eRangeError, 0);
  assert(self, ret);

  return Qnil;
}
static VALUE
test_0h1_NUM2ULLONG(VALUE self)
{
  VALUE val;
  unsigned long long c;

  val = UCHR2FIX(SCHAR_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("127"));

  c = NUM2ULLONG(val);
  ASSERT_EQUAL(self, c, SCHAR_MAX);

  val = UCHR2FIX(c);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("127"));

  return Qnil;
}
static VALUE
test_0h2_NUM2ULLONG(VALUE self)
{
  VALUE val;
  unsigned long long c;

  val = UCHR2FIX(UCHAR_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("255"));

  c = NUM2ULLONG(val);
  ASSERT_EQUAL(self, c, UCHAR_MAX);

  val = UCHR2FIX(c);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("255"));

  return Qnil;
}
static VALUE
test_0h3_NUM2ULLONG(VALUE self)
{
  VALUE val;
  VALUE ret;

  val = INT2FIX(SHRT_MIN);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("-32768"));

  ret = rb_rescue2(RUBY_METHOD_FUNC(beg_NUM2ULLONG), val,
                   RUBY_METHOD_FUNC(res_RangeError), self,
                   rb_eRangeError, 0);
  assert(self, ret);

  return Qnil;
}
static VALUE
test_0h4_NUM2ULLONG(VALUE self)
{
  VALUE val;
  unsigned long long c;

  val = LONG2NUM(SHRT_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("32767"));

  c = NUM2ULLONG(val);
  ASSERT_EQUAL(self, c, SHRT_MAX);

  val = LONG2NUM(c);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("32767"));

  return Qnil;
}
static VALUE
test_0h5_NUM2ULLONG(VALUE self)
{
  VALUE val;
  unsigned long long c;

  val = LONG2NUM(USHRT_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("65535"));

  c = NUM2ULLONG(val);
  ASSERT_EQUAL(self, c, USHRT_MAX);

  val = LONG2NUM(c);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("65535"));

  return Qnil;
}
static VALUE
test_0h6_NUM2ULLONG(VALUE self)
{
  VALUE val;
  VALUE ret;

  val = LONG2NUM(FIXNUM_MIN);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("-1073741824"));

  ret = rb_rescue2(RUBY_METHOD_FUNC(beg_NUM2ULLONG), val,
                   RUBY_METHOD_FUNC(res_RangeError), self,
                   rb_eRangeError, 0);
  assert(self, ret);

  return Qnil;
}
static VALUE
test_0h7_NUM2ULLONG(VALUE self)
{
  VALUE val;
  unsigned long long c;

  val = LONG2NUM(FIXNUM_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("1073741823"));

  c = NUM2ULLONG(val);
  ASSERT_EQUAL(self, c, FIXNUM_MAX);

  val = LONG2NUM(c);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("1073741823"));

  return Qnil;
}
static VALUE
test_0h8_NUM2ULLONG(VALUE self)
{
  VALUE val;
  VALUE ret;

  val = LONG2NUM(LONG_MIN);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("-2147483648"));

  ret = rb_rescue2(RUBY_METHOD_FUNC(beg_NUM2ULLONG), val,
                   RUBY_METHOD_FUNC(res_RangeError), self,
                   rb_eRangeError, 0);
  assert(self, ret);

  return Qnil;
}
static VALUE
test_0h9_NUM2ULLONG(VALUE self)
{
  VALUE val;
  unsigned long long c;

  val = LONG2NUM(LONG_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("2147483647"));

  c = NUM2ULLONG(val);
  ASSERT_EQUAL(self, c, LONG_MAX);

  val = LONG2NUM(c);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("2147483647"));

  return Qnil;
}
static VALUE
test_0ha_NUM2ULLONG(VALUE self)
{
  VALUE val;
  unsigned long long c;

  val = ULONG2NUM(ULONG_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("4294967295"));

  c = NUM2ULLONG(val);
  ASSERT_EQUAL(self, c, ULONG_MAX);

  val = ULONG2NUM(c);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("4294967295"));

  return Qnil;
}
static VALUE
test_0hb_NUM2ULLONG(VALUE self)
{
  VALUE val;
  VALUE ret;

  val = LLONG2NUM(LONGLONG_MIN);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("-9223372036854775808"));

  ret = rb_rescue2(RUBY_METHOD_FUNC(beg_NUM2ULLONG), val,
                   RUBY_METHOD_FUNC(res_RangeError), self,
                   rb_eRangeError, 0);
  assert(self, ret);

  return Qnil;
}
static VALUE
test_0hc_NUM2ULLONG(VALUE self)
{
  VALUE val;
  unsigned long long c;

  val = LLONG2NUM(LONGLONG_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("9223372036854775807"));

  c = NUM2ULLONG(val);
  ASSERT_EQUAL(self, c, LONGLONG_MAX);

  val = LLONG2NUM(c);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("9223372036854775807"));

  return Qnil;
}
static VALUE
test_0hd_NUM2ULLONG(VALUE self)
{
  VALUE val;
  unsigned long long c;

  val = ULLONG2NUM(ULONGLONG_MAX);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("18446744073709551615"));

  c = NUM2ULLONG(val);
  ASSERT_EQUAL(self, c, ULONGLONG_MAX);

  val = ULLONG2NUM(c);
  assert_equal(self, rb_obj_as_string(val), rb_str_new2("18446744073709551615"));

  return Qnil;
}



int
main(int argc, char **argv)
{
  extern VALUE ruby_verbose, ruby_debug;

  ruby_init();
  ruby_init_loadpath();

  if (argc > 1) {
    ruby_verbose = Qtrue;
    ruby_debug   = Qtrue;
  }


  rb_require("runit/testcase");
  cRUNIT    = my_const_get(rb_cObject, RUNIT);
  cTestCase = my_const_get(cRUNIT, TestCase);



#define define_NumTest(method) define_test_method(cNumTest, method)

  cNumTest = rb_define_class("NumericTest", cTestCase);
  define_NumTest(setup);
  define_NumTest(teardown);

  define_NumTest(test_000_CHR2FIX);
  define_NumTest(test_001_CHR2FIX);

  define_NumTest(test_010_UCHR2FIX);
  define_NumTest(test_011_UCHR2FIX);

  define_NumTest(test_020_LONG2NUM);
  define_NumTest(test_021_LONG2NUM);
  define_NumTest(test_022_LONG2NUM);
  define_NumTest(test_023_LONG2NUM);
  define_NumTest(test_024_LONG2NUM);
  define_NumTest(test_025_LONG2NUM);
  define_NumTest(test_026_LONG2NUM);
  define_NumTest(test_027_LONG2NUM);
  define_NumTest(test_028_LONG2NUM);
  define_NumTest(test_029_LONG2NUM);

  define_NumTest(test_030_ULONG2NUM);
  define_NumTest(test_031_ULONG2NUM);
  define_NumTest(test_032_ULONG2NUM);
  define_NumTest(test_033_ULONG2NUM);
  define_NumTest(test_034_ULONG2NUM);
  define_NumTest(test_035_ULONG2NUM);
  define_NumTest(test_036_ULONG2NUM);

  define_NumTest(test_040_LLONG2NUM);
  define_NumTest(test_041_LLONG2NUM);
  define_NumTest(test_042_LLONG2NUM);
  define_NumTest(test_043_LLONG2NUM);
  define_NumTest(test_044_LLONG2NUM);
  define_NumTest(test_045_LLONG2NUM);
  define_NumTest(test_046_LLONG2NUM);
  define_NumTest(test_047_LLONG2NUM);
  define_NumTest(test_048_LLONG2NUM);
  define_NumTest(test_049_LLONG2NUM);
  define_NumTest(test_04a_LLONG2NUM);
  define_NumTest(test_04b_LLONG2NUM);
  define_NumTest(test_04c_LLONG2NUM);

  define_NumTest(test_050_ULLONG2NUM);
  define_NumTest(test_051_ULLONG2NUM);
  define_NumTest(test_052_ULLONG2NUM);
  define_NumTest(test_053_ULLONG2NUM);
  define_NumTest(test_054_ULLONG2NUM);
  define_NumTest(test_055_ULLONG2NUM);
  define_NumTest(test_056_ULLONG2NUM);
  define_NumTest(test_057_ULLONG2NUM);
  define_NumTest(test_058_ULLONG2NUM);

  define_NumTest(test_060_RangeCheck_Long);
  define_NumTest(test_061_RangeCheck_Long);
  define_NumTest(test_062_RangeCheck_Long);
  define_NumTest(test_063_RangeCheck_Long);
  define_NumTest(test_064_RangeCheck_Long);
  define_NumTest(test_065_RangeCheck_Long);
  define_NumTest(test_066_RangeCheck_Long);
  define_NumTest(test_067_RangeCheck_Long);
  define_NumTest(test_068_RangeCheck_Long);
  define_NumTest(test_069_RangeCheck_Long);

  define_NumTest(test_070_RangeCheck_Ulong);
  define_NumTest(test_071_RangeCheck_Ulong);
  define_NumTest(test_072_RangeCheck_Ulong);
  define_NumTest(test_073_RangeCheck_Ulong);
  define_NumTest(test_074_RangeCheck_Ulong);
  define_NumTest(test_075_RangeCheck_Ulong);
  define_NumTest(test_076_RangeCheck_Ulong);

  define_NumTest(test_080_RangeCheck_Llong);
  define_NumTest(test_081_RangeCheck_Llong);
  define_NumTest(test_082_RangeCheck_Llong);
  define_NumTest(test_083_RangeCheck_Llong);
  define_NumTest(test_084_RangeCheck_Llong);
  define_NumTest(test_085_RangeCheck_Llong);
  define_NumTest(test_086_RangeCheck_Llong);
  define_NumTest(test_087_RangeCheck_Llong);
  define_NumTest(test_088_RangeCheck_Llong);
  define_NumTest(test_089_RangeCheck_Llong);
  define_NumTest(test_08a_RangeCheck_Llong);
  define_NumTest(test_08b_RangeCheck_Llong);
  define_NumTest(test_08c_RangeCheck_Llong);

  define_NumTest(test_090_RangeCheck_Ullong);
  define_NumTest(test_091_RangeCheck_Ullong);
  define_NumTest(test_092_RangeCheck_Ullong);
  define_NumTest(test_093_RangeCheck_Ullong);
  define_NumTest(test_094_RangeCheck_Ullong);
  define_NumTest(test_095_RangeCheck_Ullong);
  define_NumTest(test_096_RangeCheck_Ullong);
  define_NumTest(test_097_RangeCheck_Ullong);
  define_NumTest(test_098_RangeCheck_Ullong);

  define_NumTest(test_0a0_NUM2CHR);
  define_NumTest(test_0a1_NUM2CHR);
  define_NumTest(test_0a2_NUM2CHR);
  define_NumTest(test_0a3_NUM2CHR);
  define_NumTest(test_0a4_NUM2CHR);
  define_NumTest(test_0a5_NUM2CHR);
  define_NumTest(test_0a6_NUM2CHR);
  define_NumTest(test_0a7_NUM2CHR);
  define_NumTest(test_0a8_NUM2CHR);
  define_NumTest(test_0a9_NUM2CHR);
  define_NumTest(test_0aa_NUM2CHR);
  define_NumTest(test_0ab_NUM2CHR);
  define_NumTest(test_0ac_NUM2CHR);
  define_NumTest(test_0ad_NUM2CHR);

  define_NumTest(test_0b0_NUM2UCHR);
  define_NumTest(test_0b1_NUM2UCHR);
  define_NumTest(test_0b2_NUM2UCHR);
  define_NumTest(test_0b3_NUM2UCHR);
  define_NumTest(test_0b4_NUM2UCHR);
  define_NumTest(test_0b5_NUM2UCHR);
  define_NumTest(test_0b6_NUM2UCHR);
  define_NumTest(test_0b7_NUM2UCHR);
  define_NumTest(test_0b8_NUM2UCHR);
  define_NumTest(test_0b9_NUM2UCHR);
  define_NumTest(test_0ba_NUM2UCHR);
  define_NumTest(test_0bb_NUM2UCHR);
  define_NumTest(test_0bc_NUM2UCHR);
  define_NumTest(test_0bd_NUM2UCHR);

  define_NumTest(test_0c0_NUM2SHRT);
  define_NumTest(test_0c1_NUM2SHRT);
  define_NumTest(test_0c2_NUM2SHRT);
  define_NumTest(test_0c3_NUM2SHRT);
  define_NumTest(test_0c4_NUM2SHRT);
  define_NumTest(test_0c5_NUM2SHRT);
  define_NumTest(test_0c6_NUM2SHRT);
  define_NumTest(test_0c7_NUM2SHRT);
  define_NumTest(test_0c8_NUM2SHRT);
  define_NumTest(test_0c9_NUM2SHRT);
  define_NumTest(test_0ca_NUM2SHRT);
  define_NumTest(test_0cb_NUM2SHRT);
  define_NumTest(test_0cc_NUM2SHRT);
  define_NumTest(test_0cd_NUM2SHRT);

  define_NumTest(test_0d0_NUM2USHRT);
  define_NumTest(test_0d1_NUM2USHRT);
  define_NumTest(test_0d2_NUM2USHRT);
  define_NumTest(test_0d3_NUM2USHRT);
  define_NumTest(test_0d4_NUM2USHRT);
  define_NumTest(test_0d5_NUM2USHRT);
  define_NumTest(test_0d6_NUM2USHRT);
  define_NumTest(test_0d7_NUM2USHRT);
  define_NumTest(test_0d8_NUM2USHRT);
  define_NumTest(test_0d9_NUM2USHRT);
  define_NumTest(test_0da_NUM2USHRT);
  define_NumTest(test_0db_NUM2USHRT);
  define_NumTest(test_0dc_NUM2USHRT);
  define_NumTest(test_0dd_NUM2USHRT);

  define_NumTest(test_0e0_NUM2LONG);
  define_NumTest(test_0e1_NUM2LONG);
  define_NumTest(test_0e2_NUM2LONG);
  define_NumTest(test_0e3_NUM2LONG);
  define_NumTest(test_0e4_NUM2LONG);
  define_NumTest(test_0e5_NUM2LONG);
  define_NumTest(test_0e6_NUM2LONG);
  define_NumTest(test_0e7_NUM2LONG);
  define_NumTest(test_0e8_NUM2LONG);
  define_NumTest(test_0e9_NUM2LONG);
  define_NumTest(test_0ea_NUM2LONG);
  define_NumTest(test_0eb_NUM2LONG);
  define_NumTest(test_0ec_NUM2LONG);
  define_NumTest(test_0ed_NUM2LONG);

  define_NumTest(test_0f0_NUM2ULONG);
  define_NumTest(test_0f1_NUM2ULONG);
  define_NumTest(test_0f2_NUM2ULONG);
  define_NumTest(test_0f3_NUM2ULONG);
  define_NumTest(test_0f4_NUM2ULONG);
  define_NumTest(test_0f5_NUM2ULONG);
  define_NumTest(test_0f6_NUM2ULONG);
  define_NumTest(test_0f7_NUM2ULONG);
  define_NumTest(test_0f8_NUM2ULONG);
  define_NumTest(test_0f9_NUM2ULONG);
  define_NumTest(test_0fa_NUM2ULONG);
  define_NumTest(test_0fb_NUM2ULONG);
  define_NumTest(test_0fc_NUM2ULONG);
  define_NumTest(test_0fd_NUM2ULONG);

  define_NumTest(test_0g0_NUM2LLONG);
  define_NumTest(test_0g1_NUM2LLONG);
  define_NumTest(test_0g2_NUM2LLONG);
  define_NumTest(test_0g3_NUM2LLONG);
  define_NumTest(test_0g4_NUM2LLONG);
  define_NumTest(test_0g5_NUM2LLONG);
  define_NumTest(test_0g6_NUM2LLONG);
  define_NumTest(test_0g7_NUM2LLONG);
  define_NumTest(test_0g8_NUM2LLONG);
  define_NumTest(test_0g9_NUM2LLONG);
  define_NumTest(test_0ga_NUM2LLONG);
  define_NumTest(test_0gb_NUM2LLONG);
  define_NumTest(test_0gc_NUM2LLONG);
  define_NumTest(test_0gd_NUM2LLONG);

  define_NumTest(test_0h0_NUM2ULLONG);
  define_NumTest(test_0h1_NUM2ULLONG);
  define_NumTest(test_0h2_NUM2ULLONG);
  define_NumTest(test_0h3_NUM2ULLONG);
  define_NumTest(test_0h4_NUM2ULLONG);
  define_NumTest(test_0h5_NUM2ULLONG);
  define_NumTest(test_0h6_NUM2ULLONG);
  define_NumTest(test_0h7_NUM2ULLONG);
  define_NumTest(test_0h8_NUM2ULLONG);
  define_NumTest(test_0h9_NUM2ULLONG);
  define_NumTest(test_0ha_NUM2ULLONG);
  define_NumTest(test_0hb_NUM2ULLONG);
  define_NumTest(test_0hc_NUM2ULLONG);
  define_NumTest(test_0hd_NUM2ULLONG);



  rb_require("runit/cui/testrunner");
  cTestRunner = my_const_get2(cRUNIT, CUI, TestRunner);

  VALUE suite = rb_funcall(cNumTest, rb_intern("suite"), 0, 0);
  VALUE result = rb_funcall(cTestRunner, rb_intern("run"), 1, suite);

  result = rb_funcall(result, rb_intern("succeed?"), 0, 0);
  return (result ? 0:1);
}
