/*
 *  rb_assert.cc
 */

#include <ruby.h>


VALUE cRUNIT;
VALUE cTestCase;
VALUE cTestRunner;


#define empty_string rb_str_new2("")


#define define_test_method(klass, method) (                     \
  rb_define_method(klass, #method, RUBY_METHOD_FUNC(method), 0) \
)

#define my_const_get(klass, c1) (     \
  rb_const_get(klass, rb_intern(#c1)) \
)

#define my_const_get2(klass, c1, c2) (                              \
  rb_const_get(rb_const_get(klass, rb_intern(#c1)), rb_intern(#c2)) \
)


VALUE
assert(VALUE rcv, VALUE condition, VALUE message=empty_string)
{
  return rb_funcall(rcv, rb_intern("assert"), 2, condition, message);
}

VALUE
assert_equal(VALUE rcv, VALUE expected, VALUE actual, VALUE message=empty_string)
{
  return rb_funcall(rcv, rb_intern("assert_equal"), 3, expected, actual, message);
}

VALUE
assert_equal_float(VALUE rcv, VALUE expected, VALUE actual, VALUE e, VALUE message=empty_string)
{
  return rb_funcall(rcv, rb_intern("assert_equal_float"), 4, expected, actual, e, message);
}

VALUE
assert_same(VALUE rcv, VALUE expected, VALUE actual, VALUE message=empty_string)
{
  return rb_funcall(rcv, rb_intern("assert_same"), 3, expected, actual, message);
}

VALUE
assert_nil(VALUE rcv, VALUE obj, VALUE message=empty_string)
{
  return rb_funcall(rcv, rb_intern("assert_nil"), 2, obj, message);
}

VALUE
assert_not_nil(VALUE rcv, VALUE obj, VALUE message=empty_string)
{
  return rb_funcall(rcv, rb_intern("assert_not_nil"), 2, obj, message);
}

VALUE
assert_respond_to(VALUE rcv, VALUE method, VALUE obj, VALUE message=empty_string)
{
  return rb_funcall(rcv, rb_intern("assert_respond_to"), 3, method, obj, message);
}

VALUE
assert_kind_of(VALUE rcv, VALUE c, VALUE obj, VALUE message=empty_string)
{
  return rb_funcall(rcv, rb_intern("assert_kind_of"), 3, c, obj, message);
}

VALUE
assert_instance_of(VALUE rcv, VALUE c, VALUE obj, VALUE message=empty_string)
{
  return rb_funcall(rcv, rb_intern("assert_instance_of"), 3, c, obj, message);
}

VALUE
assert_match(VALUE rcv, VALUE str, VALUE re, VALUE message=empty_string)
{
  return rb_funcall(rcv, rb_intern("assert_match"), 3, str, re, message);
}

VALUE
assert_not_match(VALUE rcv, VALUE str, VALUE re, VALUE message=empty_string)
{
  return rb_funcall(rcv, rb_intern("assert_not_match"), 3, str, re, message);
}

VALUE
assert_exception(VALUE rcv, VALUE exception, VALUE message=empty_string)
{
  return rb_funcall(rcv, rb_intern("assert_exception"), 2, exception, message);
}

VALUE
assert_no_exception(VALUE rcv, VALUE args)
{
  return rb_apply(rcv, rb_intern("assert_no_exception"), args);
}

VALUE
assert_fail(VALUE rcv, VALUE message, VALUE level=INT2FIX(2))
{
  return rb_funcall(rcv, rb_intern("assert_fail"), 2, message, level);
}


#define ASSERT_EQUAL(rcv, expected, actual) ( \
  ((expected) != (actual))                    \
    ? assert_fail((rcv),                      \
                  rb_str_plus(rb_str_new2("File: "__FILE__", Line: "), \
                              rb_obj_as_string(LONG2NUM(__LINE__))))   \
    : (int)1                                  \
)
