#! /boot/home/config/bin/ruby -d -Ku

#
# watcher_mount.rb
#
# ex.)
#   ruby watcher_mount.rb
#

require "be.so"
include Be


class MyApplication < Application
  def initialize
    super("application/x-vnd.s_nojo-watcher_mount")
    @dev = {}
  end

  def MessageReceived(message)
    case message.what
      when NODE_MONITOR
        node_monitor(message)
      else
        super(message)
    end
  end

  private
  def node_monitor(msg)
    opcode = msg.FindInt32("opcode")

    case opcode
      when ENTRY_CREATED
        puts "ENTRY_CREATED"
      when ENTRY_REMOVED
        puts "ENTRY_REMOVED"
      when ENTRY_MOVED
        puts "ENTRY_MOVED"
      when STAT_CHANGED
        puts "STAT_CHANGED"
      when ATTR_CHANGED
        puts "ATTR_CHANGED"

      when DEVICE_MOUNTED
        device = msg.FindInt32("new device")
        vol = Volume.new(device)
        @dev[device] = vol.GetName
        puts "`#{@dev[device]}' mounted."

      when DEVICE_UNMOUNTED
        device = msg.FindInt32("device")
        puts "`#{@dev[device]}' unmounted."
        @dev.delete(device)

      else
        puts "unknown opcode:"
        msg.PrintToStream
    end
  end
end

class MyWindow < Window
  def initialize
    super(Rect.new(100, 80, 260, 120), "watcher_mount.rb",
          TITLED_WINDOW, NOT_RESIZABLE | NOT_ZOOMABLE)
  end
  def QuitRequested
    $be_app.PostMessage(QUIT_REQUESTED)
    return true
  end
end


MyApplication.new

vros = VolumeRoster.new
win  = MyWindow.new
win.Show


begin
  vros.StartWatching
  $be_app.Run

ensure
  vros.StopWatching
end
